<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Ideas
 * @package    Ideas_Module
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Categories module identity class
 *
 * @uses       Singular_Module_Abstract
 * @category   Ideas
 * @package    Ideas_Module
 */
class Ideas_Module extends Singular_Module_Abstract
{
    /**
     * Priority for module load order, used in module installation process.
     *
     * @ignore
     * @var int
     */
    public $loadOrder = 1;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @return void
     */
    public function init ()
    {
      $this->_info();
    }

    /**
     * Module installation method.
     *
     * @return void
     */
    public function install()
    {
        $modulesDb = Singular_Loader::get('System_Model_DbTable_Modules');
        $module = $modulesDb->findByName('ideas');

        /* types */
        $type = new System_Model_ResponseType();
        $type->save(array(
            'slug'              => 'idea',
            'title'             => 'Idea',
            'objects'           => 'ideas',
            'icon'              => 'icon_medium.png',
            'module_id'         => $module->id,
            'has_comments'      => 1,
            'has_negative_vote' => 1,
            'is_active'         => 1,
            'is_default'        => 1,
            'order'             => 1
        ));

        /* statuses */
        $status = new System_Model_ResponseStatus();
        $status->save(array(
            'title'         => 'Under Consideration',
            'slug'          => 'under_consideration',
            'is_default'    => 1,
            'is_protected'  => 1,
            'type'          => 'idea',
            'pay_attention' => 1,
            'color'         => '#fba344',
            'order'         => 1
        ));

        $status = new System_Model_ResponseStatus();
        $status->save(array(
            'title'         => 'Declined',
            'slug'          => 'declined',
            'is_default'    => 0,
            'is_protected'  => 1,
            'type'          => 'idea',
            'pay_attention' => 0,
            'color'         => '#ff7272',
            'order'         => 2
        ));

        $status = new System_Model_ResponseStatus();
        $status->save(array(
            'title'         => 'Planned',
            'slug'          => 'planned',
            'is_default'    => 0,
            'is_protected'  => 1,
            'type'          => 'idea',
            'pay_attention' => 1,
            'color'         => '#b0b5bf',
            'order'         => 3
        ));

        $status = new System_Model_ResponseStatus();
        $status->save(array(
            'title'         => 'In Progress',
            'slug'          => 'in_progress',
            'is_default'    => 0,
            'is_protected'  => 1,
            'type'          => 'idea',
            'pay_attention' => 1,
            'color'         => '#6f7379',
            'order'         => 4
        ));

        $status = new System_Model_ResponseStatus();
        $status->save(array(
            'title'         => 'Completed',
            'slug'          => 'completed',
            'is_default'    => 0,
            'is_protected'  => 1,
            'type'          => 'idea',
            'pay_attention' => 0,
            'color'         => '#40bf00',
            'order'         => 5
        ));
    }

    /**
     * Module uninstallation method.
     *
     * @return void
     */
    public function uninstall()
    {
        $typeDb = Singular_Loader::get('System_Model_DbTable_ResponsesTypes');
        $type = $typeDb->find('idea')->current();
        if ($type instanceof System_Model_ResponseType) {
            $type->delete();
        }
    }

    /**
     * Set module information to be available in code and to display on Modules
     * list at backend.
     *
     * @return void
     */
    protected function _info()
    {
        $this->getContainer()->setName('Ideas')
              ->setDescription('Helps you to find out what users think about your products or services and suggest new features')
              ->setVersion('2.1')
              ->addDependency('system', '2.3.3')
              ->addDependency('resources', '2.2')
              ->setAuthor('USWebStyle, Inc')
              ->setEmail('support@uswebstyle.com')
              ->setUrl('http://www.uswebstyle.com')
              ->setReleaseDate('2013-05-08');
    }
    
}
