<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Ideas
 * @package    Ideas_Handler
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Ideas Events Handler
 *
 * @uses       Singular_Event_Listener_Abstract
 * @category   Ideas
 * @package    Ideas_Handler
 */
class Ideas_Handler_Events extends Singular_Event_Listener_Abstract
{
    /**
     * Binds aggregates on specified events.
     *
     * @return void
     */
    public static function bindListeners ()
    {
        self::attachAggregate('rightMenu',        array(__CLASS__, 'onRightMenu'));
        self::attachAggregate('onMainPage',       array(__CLASS__, 'onMainPage'));
        self::attachAggregate('postDispatchOnce', array(__CLASS__, 'onPostDispatchOnce'));
        self::attachAggregate('buildNavigation',  array(__CLASS__, 'onNavigation'));
        self::attachAggregate('preDispatch',      array(__CLASS__, 'onPreDispatch'));
        self::attachAggregate('insert',           array(__CLASS__, 'onInsert'));
        self::attachAggregate('formInitAfter',    array(__CLASS__, 'onFormInitAfter'));
        self::attachAggregate('responseShow',     array(__CLASS__, 'onResponseShow'));
        self::attachAggregate('commentActions',   array(__CLASS__, 'onCommentActions'));
        self::attachAggregate('afterAnalyticsReportsInit', array(__CLASS__, 'afterAnalyticsReportsInit'));
    }

    /**
     * Subscribes to "commentActions" event.
     *
     * @param  stdClass $params
     * @return void
     */
    public static function onCommentActions($params)
    {
        $response = Singular_Runtime::extract("activeResponse");
        if ($response instanceof System_Model_Response
            && $response->type == 'idea'
        ) {
            $params->actions['good']['label'] = 'Good point';
        }
    }

    /**
     * Subscribes to "responseShow" event.
     *
     * @param  stdClass $data
     * @return void
     */
    public static function onResponseShow($data)
    {
        $response = Singular_Runtime::extract("activeResponse");
        if ($response instanceof System_Model_Response
            && $response->type == 'idea'
        ) {
            self::getView()->additionalModulePath('ideas');
            $data->sections['votes'] = self::getView()->render('idea_votes.phtml');
        }
    }

    /**
     * Subscribes to "responseShow" event.
     *
     * @param  System_Model_Response $object
     * @return void
     */
    public static function onInsert($object)
    {
        if ($object instanceof System_Model_Response && $object->type == 'idea') {
            $defaultStatus = $object->getDefaultStatus()->slug;
            if (isset($defaultStatus) && !empty($defaultStatus)) {
                $object->status = $defaultStatus;
            }
            else {
                $object->status = IDEAS_STATUS_UNDER_CONSIDERATION;    
            }
        }
    }

    /**
     * Subscribes to "preDispatch" event.
     *
     * @return void
     */
    public static function onPreDispatch()
    {
        if (!defined('IDEAS_STATUS_UNDER_CONSIDERATION')) {
            define('IDEAS_STATUS_UNDER_CONSIDERATION', 'under_consideration');
        }
    }

    /**
     * Subscribes to "postDispatchOnce" event.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public static function onPostDispatchOnce($request)
    {
        self::getView()->headLink()->prependStylesheet(self::getView()->stylesheetSrc('frontend.css', 'ideas'));
        self::getView()->headScript()->appendFile(self::getView()->javascriptSrc('main.js', 'ideas'));
        
        $activeResponse = Singular_Runtime::extract('activeResponse');
        
        if ($activeResponse instanceof System_Model_Response
            && $activeResponse->type == 'idea'
        ) {
            $item =
                array(
                    'id' => 'idea-' . $activeResponse->id,
                    'label' => self::getView()->truncate($activeResponse->title, '16', '...'),
                    'route' => 'responses_show',
                    'params' => array(
                        'slug' => $activeResponse->slug
                    ),
                    'pages' => array(
                        array(
                            'id' => 'idea-edit',
                            'label' => self::getView()->t('Edit'),
                            'route' => 'response_edit',
                            'params' => array(
                                'slug' => $activeResponse->slug
                            )
                        )
                    )
                );
            $navigation = self::getView()->navigation()->getContainer();
            $page = $navigation->findOneBy('id', 'idea-' . $activeResponse->status);
            if ($page instanceof Zend_Navigation_Page) {
                $page->hide_in_bread = true;
                $page->addPage($item);
            }
        }
    }
    
    /**
     * Subscribes to "navigation" event.
     *
     * Builds navigation
     *
     * @param  Singular_Navigation $navigation
     * @return void
     */
    public static function onNavigation($navigation)
    {
        $item = array(
            'id'     => 'ideas',
            'label'  => self::getView()->t('Ideas'),
            'order'  => Singular_Runtime::extract("types")->findBySlug('idea')->order,
            'route'  => 'responses_type',
            'params' => array(
                'objects' => 'ideas'
            ),
            'pages'  => array(
                'all' => array(
                    'id'    => 'idea-all',
                    'label' => self::getView()->t('All Ideas'),
                    'route' => 'responses_type',
                    'params' => array(
                        'objects' => 'ideas',
                        'status'  => 'all'
                    )
                )
            )
        );

        $statuses = Singular_Runtime::extract("statuses")
            ->findItemsByField('type', 'idea');
        
        if ($statuses->valid()) {
            foreach ($statuses as $status) {
                $item['pages'][$status->slug] = array(
                    'id'     => 'idea-' . $status->slug,
                    'label'  => self::getView()->t(ucfirst($status->title)),
                    'route'  => 'responses_type',
                    'params' => array(
                        'objects' => 'ideas',
                        'status'  => $status->slug
                    )
                );
            }
        }

        $page = $navigation->findOneBy('id', 'responses');
        if ($page instanceof Zend_Navigation_Page) {
            $page->addPage($item);
        }
    }

    /**
     * Subscribes to "rightMenu" event.
     *
     * @param  stdClass $params
     * @return void
     */
    public static function onRightMenu($params)
    {
        $menu = $params->menu;
        
        if (Singular_Runtime::inStorage('ideasCount')) {
            $ideasCount = Singular_Runtime::extract('ideasCount');
        } else {
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $ideasCount = $responsesDb->count(array('type' => 'idea'));
        }

        /** @var $page Zend_Navigation_Page */
        $page = $menu->findById("statistics");
        $page->addPage(
            array(
                 'id'     => 'ideas',
                 'order'  => Singular_Runtime::extract("types")->findBySlug('idea')->order,
                 'count'  => $ideasCount,
                 'icon'   => self::getView()->imageSrc('icon_small.png', 'ideas'),
                 'label'  => 'Ideas',
                 'route'  => 'responses_type',
                 'params' => array(
                     'objects' => 'ideas'
                 )
            )
        );
        
        $params->cacheKeys[] = 'ideas';
    }

    /**
     * Subscribes to "mainPage" event.
     *
     * @param  stdClass $sections
     * @return void
     */
    public static function onMainPage($sections)
    {
        if (
            1 != Singular_Core::_("Settings")->getSetting('homepage_content_responses')
            || 'divided_by_type' != Singular_Core::_("settings")
                ->getSetting("homepage_responses_display_variant")
        ) {
            return;
        }
        $cache = Singular_Runtime::extract("cachemanager")->getCache("default");
        $loggedUser = Singular_Runtime::extract("loggedUser");
        $cacheId = "homepageIdeas" . "_" . $loggedUser->role;
        $cacheId = preg_replace("/[^a-zA-Z0-9_]/", "", $cacheId);
        $tags = null;
        
        if ( ($output = $cache->load($cacheId)) == false ) {
            /** @var $responsesDb System_Model_DbTable_Responses */
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $ideas = $responsesDb->findTop('idea');
            
            self::getView()->ideas = $ideas;
            $ideasCount = $responsesDb->count(array('type' => 'idea'));
            $ideasLeftCount = $ideasCount - count($ideas);
            self::getView()->ideasCount = $ideasLeftCount;
            Singular_Runtime::store('ideasCount', $ideasCount);

            self::getView()->additionalModulePath('ideas');
            self::getView()->cacheTags = new StdClass();
            self::getView()->cacheTags->tags = array();
            $output = self::getView()->render('_homepage-ideas.phtml');
            $tags   = self::getView()->cacheTags->tags;
            $tags[] = 'responses';
            $tags[] = 'ideas';
            $cache->save($output, $cacheId, $tags);
        }
        Resources_Service_Votes::setUserVotes($cacheId, $tags);
        $order = Singular_Runtime::extract("types")->findBySlug('idea')->order + 10;
        $sections->sections[$order] = $output;
    }

    /**
     * Subscribes to "formInitAfter" event.
     *
     * @param  Singular_Form $form
     * @return void
     */
    public static function onFormInitAfter($form)
    {
        if ($form instanceof System_Form_Response) {
            if ($form instanceof System_Form_ResponseWidget) {
                $form->type->addMultiOption('idea', 'Idea');
            } else {
                $form->type->addMultiOption('idea', 'Share an idea');
            }
        }
        if ($form instanceof System_Form_SearchAdd) {
            $form->type->addMultiOption('idea', 'Share an idea');
            if (!$form->submit->getDescription()) {
                $form->submit->setDescription(self::getView()->t("Enter your idea and we'll find it or add it"));
            }
        }
        if ($form instanceof System_Form_WidgetFeedback || $form instanceof System_Form_WidgetSearch) {
            $form->widget_default_tab->addMultiOption('idea', 'Ideas');
        }
    }
    
    /**
     * Subscribes to "afterAnalyticsReportsInit" event.
     *
     * @param  stdClass $reportsConf
     * @return void
     */
    public static function afterAnalyticsReportsInit ($reportsConf) 
    {
        $reportsConf->reports[] = array(
            'id' => 'ideas_responses',
            'title' => self::getView()->t('Ideas'),
            'model' => 'System_Model_DbTable_Responses',
            'counter' => 'countByType',
            'order' => '25',
            'options' => array(
                'row_type' => 'default',
                'response_type' => 'idea',
                'icon' => 'url('.self::getView()->imageSrc('icon_small.png', 'ideas', false).')',
            )
        );
    }
}
