<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   LoginPlus
 * @package    LoginPlus_Module
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * LoginPlus module identity class
 *
 * @uses       Singular_Module_Abstract
 * @category   LoginPlus
 * @package    LoginPlus_Module
 */
class LoginPlus_Module extends Singular_Module_Abstract
{
    /**
     * Priority for module load order, used in module installation process.
     *
     * @ignore
     * @var int
     */
    public $loadOrder = 6;

    protected $_adapter;
    protected $_tablePrefix;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @return void
     */
    public function init ()
    {
        $this->_adapter = Zend_Db_Table::getDefaultAdapter();
        $this->_tablePrefix = Singular_Runtime::extract("config")->resources->db->params->prefix;
        $this->_info();
        $this->_settings();
    }

    /**
     * Social login installation routine
     *
     * @return void
     */
    protected function _installSocial()
    {
        $sql = "ALTER TABLE `{$this->_tablePrefix}users` ADD `facebook_id` BIGINT NOT NULL DEFAULT '0' AFTER `id`";
        $this->_adapter->query($sql);
        $sql = "ALTER TABLE `{$this->_tablePrefix}users` ADD `twitter_id` BIGINT NOT NULL DEFAULT '0' AFTER `id`";
        $this->_adapter->query($sql);
        $sql = "ALTER TABLE `{$this->_tablePrefix}users` ADD `google_id` VARCHAR( 255 ) NOT NULL DEFAULT '0' AFTER `id`";
        $this->_adapter->query($sql);

        /* customer */
        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_facebook_app_id',
            'value'       => '<default_key>',
            'type'        => 'text',
            'title'       => 'Facebook Application ID',
            'description' => 'Leave empty if you don\'t want it to use or <default_key> to use standard keys',
            'module'      => 'loginplus',
            'section'     => 'general'
        ));
        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_facebook_app_secret',
            'value'       => '<default_key>',
            'type'        => 'text',
            'title'       => 'Facebook Application Secret Key',
            'description' => 'Leave empty if you don\'t want it to use or <default_key> to use standard keys',
            'module'      => 'loginplus',
            'section'     => 'general'
        ));
        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_twitter_app_id',
            'value'       => '<default_key>',
            'type'        => 'text',
            'title'       => 'Twitter Consumer Key',
            'description' => 'Leave empty if you don\'t want it to use or <default_key> to use standard keys',
            'module'      => 'loginplus',
            'section'     => 'general'
        ));
        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_twitter_app_secret',
            'value'       => '<default_key>',
            'type'        => 'text',
            'title'       => 'Twitter Consumer Secret',
            'description' => 'Leave empty if you don\'t want it to use or <default_key> to use standard keys',
            'module'      => 'loginplus',
            'section'     => 'general'
        ));
        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_google_app_id',
            'value'       => '<default_key>',
            'type'        => 'text',
            'title'       => 'Google OAuth Consumer Key',
            'description' => 'Leave empty if you don\'t want it to use or <default_key> to use standard keys',
            'module'      => 'loginplus',
            'section'     => 'general'
        ));
        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_google_app_secret',
            'value'       => '<default_key>',
            'type'        => 'text',
            'title'       => 'Google OAuth Consumer Secret',
            'description' => 'Leave empty if you don\'t want it to use or <default_key> to use standard keys',
            'module'      => 'loginplus',
            'section'     => 'general'
        ));

        /* default */
        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_facebook_app_id_default',
            'value'       => '240564972666523',
            'type'        => 'text',
            'title'       => 'Default Facebook Application ID',
            'description' => '',
            'module'      => 'loginplus',
            'section'     => 'default'
        ));
        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_facebook_app_secret_default',
            'value'       => '91306560743fc916b268f5529bcabe0b',
            'type'        => 'text',
            'title'       => 'Default Facebook Application Secret Key',
            'description' => '',
            'module'      => 'loginplus',
            'section'     => 'default'
        ));

        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_twitter_app_id_default',
            'value'       => 'sRlrbUZMoUUU9k2PSEm6A',
            'type'        => 'text',
            'title'       => 'Default Twitter Consumer Key',
            'description' => '',
            'module'      => 'loginplus',
            'section'     => 'default'
        ));

        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_twitter_app_secret_default',
            'value'       => '6NxYiWrLcf23dQEnqmhfSlt3CdUrkVXB87lLIqTUiQ',
            'type'        => 'text',
            'title'       => 'Default Twitter Consumer Secret',
            'description' => '',
            'module'      => 'loginplus',
            'section'     => 'default'
        ));

        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_google_app_id_default',
            'value'       => 'www.useresponse.com',
            'type'        => 'text',
            'title'       => 'Default Google OAuth Consumer Key',
            'description' => '',
            'module'      => 'loginplus',
            'section'     => 'default'
        ));

        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'site_google_app_secret_default',
            'value'       => 'qDy8xa_YiuWq1--vqCwEcS8P',
            'type'        => 'text',
            'title'       => 'Default Google OAuth Consumer Secret',
            'description' => '',
            'module'      => 'loginplus',
            'section'     => 'default'
        ));
    }
   
    /**
     * Social login uninstallation routine
     *
     * @return void
     */
    protected function _uninstallSocial()
    {
        $sql = "ALTER TABLE `{$this->_tablePrefix}users` DROP `facebook_id`;";
        $this->_adapter->query($sql);
        $sql = "ALTER TABLE `{$this->_tablePrefix}users` DROP `twitter_id`;";
        $this->_adapter->query($sql);
        $sql = "ALTER TABLE `{$this->_tablePrefix}users` DROP `google_id`;";
        $this->_adapter->query($sql);

        // User-specified settings' values
        Singular_Core::_("settings")->deleteSetting("site_facebook_app_id");
        Singular_Core::_("settings")->deleteSetting("site_facebook_app_secret");
        Singular_Core::_("settings")->deleteSetting("site_twitter_app_id");
        Singular_Core::_("settings")->deleteSetting("site_twitter_app_secret");
        Singular_Core::_("settings")->deleteSetting("site_google_app_id");
        Singular_Core::_("settings")->deleteSetting("site_google_app_secret");

        // Default settings' values
        Singular_Core::_("settings")->deleteSetting("site_facebook_app_id_default");
        Singular_Core::_("settings")->deleteSetting("site_facebook_app_secret_default");
        Singular_Core::_("settings")->deleteSetting("site_twitter_app_id_default");
        Singular_Core::_("settings")->deleteSetting("site_twitter_app_secret_default");
        Singular_Core::_("settings")->deleteSetting("site_google_app_id_default");
        Singular_Core::_("settings")->deleteSetting("site_google_app_secret_default");
    }

 
    /**
     * Module installation method
     *
     * @return void
     */
    public function install()
    {
        $this->_installSocial();
    }

    /**
     * Module uninstallation method
     *
     * @return void
     */
    public function uninstall()
    {
        $this->_uninstallSocial();

    }

    /**
     * Set module information to be available in code and to display on Modules
     * list at backend.
     *
     * @return void
     */
    protected function _info()
    {
        $this->getContainer()->setName('Login Plus')
              ->setDescription('Adds support of social logins (Twitter, Facebook, etc)')
              ->setVersion('2.1')
              ->addDependency('system', '2.3.3')
              ->addDependency('resources', '2.2')
              ->setAuthor('USWebStyle, Inc')
              ->setEmail('support@uswebstyle.com')
              ->setUrl('http://www.uswebstyle.com')
              ->setReleaseDate('2013-05-08');
    }

    /**
     * Define module settings pages to be accessible from backend.
     *
     * @return void
     */
    protected function _settings()
    {
        $front = Zend_Controller_Front::getInstance();
        $this->settingUri = $front->getBaseUrl() . '/admin/login-plus';
    }
}
