<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Loginplus
 * @package    Loginplus_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Auth controller
 *
 * @uses       System_Controller_Abstract_Action
 * @category   Loginplus
 * @package    Loginplus_Controller
 */
class LoginPlus_AuthController extends System_Controller_Abstract_Auth
{
    /**
     * Initialize object.
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @return void
     */
    public function init ()
    {
        parent::init();
        
        $this->_helper->layout->disableLayout();    
        $this->socialNamespace = new Zend_Session_Namespace('social');
    }

    /**
     * Displays authorization form
     * and authorizes user by using oAuth services
     *
     * @return mixed
     */
    public function oauthLoginAction()
    {
        if ($this->_auth->hasIdentity()) {
            $this->template('success-social.phtml', null, true);
        }
        $social = $this->_getParam('social');
        $form = new Loginplus_Form_Email();
        $url = $this->_helper->url->url(array('social' => $social), 'login_oauth');
        $form->setAction($url);

        /* if first login proceed to registration */
        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            if ($form->isValid($post)) {
                $data                  = $form->getValues();
                $data[$social . '_id'] = $this->socialNamespace->user[$social . '_id'];
                $data['full_name']     = $this->socialNamespace->user['full_name'];

                // save image
                $url       = $this->socialNamespace->user['photo_url'];
                $imageName = $this->socialNamespace->user['photo_name'];

                if ($url) {
                    
                    $client = new Zend_Http_Client($url);
                    $image = $client->request()->getBody();
                    
                    $contentType = $client->request()->getHeader('Content-Type');
                    $ext = str_replace('image/', '', $contentType);
                    
                    $filename = AVATAR_PATH . DS . $imageName . '.' . $ext;
                    if (file_exists($filename)) {
                        $filter = new System_Filter_Rename(array(
                            'target'    => AVATAR_PATH,
                            'overwrite' => false,
                        ));
                        $filename = $filter->filter($filename);
                    }

                    file_put_contents($filename, $image);

                    $filter = new System_Filter_ResizeImage(array(
                        'targetDir' => dirname($filename),
                        'newSize'   => "50*50",
                        'rename'    => false
                    ));
                    $filter->filter($filename);

                    $data['photo'] = basename($filename);
                }
                
                $user = new System_Model_User();
                $register = $user->register($data);
                if ($register) {
                    $message = "Success. Please check your email for password!";
                    //clean session
                    $this->socialNamespace->unsetAll();
                    
                    if ($this->isAjax()) {
                        $this->loginForm->setDefault('email', $user->email);
                        $this->view->additionalModulePath('system');
                        $content = $this->fetchTemplate('system_auth_login.phtml');
                        $this->autoresponder->setOutput($content);
                        $this->autoresponder->setSuccess($message);
                        return;
                    } else {
                        $this->view->notify()->success($message);
                        $this->_helper->redirector->gotoRoute(
                            array('email' => $user->email), 'login'
                        );
                    }
                }
            } else {
                $this->_setFormErrorToAutoresponder($form);
            }
        } else {
            // create adapter
            $adapterName = 'Loginplus_Service_Adapter_' . ucfirst($social);
            $id = Singular_Core::_('settings')->getSetting('site_' . $social . '_app_id');
            $secret = Singular_Core::_('settings')->getSetting('site_' . $social . '_app_secret');

            if ($id != "<default_key>") {
                $redirectUrl = $this->_helper->url->url(array('social' => $social), 'login_oauth');
            } else {
                $id = Singular_Core::_('settings')->getSetting('site_' . $social . '_app_id_default');
                $secret = Singular_Core::_('settings')->getSetting('site_' . $social . '_app_secret_default');
                $redirectUrl = 'http://community.useresponse.com/loginplus.php?social=' . $social . '&redirect=' . rawurlencode(base64_encode(preg_replace("/^[^\/]*\/\//", "", $this->view->baseUrl()) . '/'));
            }
            $adapter = new $adapterName($id, $secret, $redirectUrl);

            //authentication
            $result = $this->_auth->authenticate($adapter);
            if ($result->isValid()) {
                $messages = $result->getMessages();
                if ( isset($messages['user']) && !empty($messages['user'][$social . '_id']) ) {
                    $user = $messages['user'];
                    if ($this->_loginFromSocial($social, $user[$social . '_id'])) {
                        $this->socialNamespace->unsetAll();
                        $this->redirect->unsetAll();
                        $this->template('success-social', null, true);
                    } else {
                        if (isset($user["email"]) && $user["email"]) {
                            $usersDb    = Singular_Loader::get("System_Model_DbTable_Users");
                            $userObject = $usersDb->findByEmail($user['email']);
                            
                            if ($userObject instanceof System_Model_User) {
                                if($userObject->is_active) {
                                    System_Service_Auth::loginByEmail($user['email']);
                                    $field      = $social . '_id';
                                    $userObject->$field = $user[$field];
                                    $userObject->save();
                                    $this->socialNamespace->unsetAll();
                                    $this->redirect->unsetAll();
                                    $this->template('success-social', null, true);
                                    return;
                                } else {
                                    $this->view->notify()->error("Your account is blocked!");
                                    $this->socialNamespace->unsetAll();
                                    $this->redirect->unsetAll();
                                    $this->template('error-social', null, true);
                                    return;
                                }
                            } else {
                                $url              = $user['photo_url'];
                                $imageName        = $user['photo_name'];
                                if ($url) {

                                    $client = new Zend_Http_Client($url);
                                    $image = $client->request()->getBody();

                                    $contentType = $client->request()->getHeader('Content-Type');
                                    $ext = str_replace('image/', '', $contentType);

                                    $filename = AVATAR_PATH . DS . $imageName . '.' . $ext;
                                    if (file_exists($filename)) {
                                        $filter = new System_Filter_Rename(array(
                                            'target'    => AVATAR_PATH,
                                            'overwrite' => false,
                                        ));
                                        $filename = $filter->filter($filename);
                                    }

                                    file_put_contents($filename, $image);

                                    $filter = new System_Filter_ResizeImage(array(
                                        'targetDir' => dirname($filename),
                                        'newSize'   => "50*50",
                                        'rename'    => false
                                    ));
                                    $filter->filter($filename);

                                    $user['photo'] = basename($filename);
                                }
                                $newUser = new System_Model_User();
                                $register = $newUser->register($user);
                                if ($register) {
                                    System_Service_Auth::loginByEmail($newUser->email);
                                    $this->socialNamespace->unsetAll();
                                    $this->redirect->unsetAll();
                                    $this->template('success-social', null, true);
                                    return;
                                }
                            }
                        } else {
                            // remember user
                            $this->socialNamespace->user = $user;
                        }
                    }
                } else {
                    $this->view->notify()->error("Error");
                    $this->socialNamespace->unsetAll();
                    $this->redirect->unsetAll();
                    $this->template('error-social', null, true);
                }
            } else {
                $this->view->notify()->error($result->getMessages());
                $this->socialNamespace->unsetAll();
                $this->redirect->unsetAll();
                $this->template('error-social', null, true);
            }
        }
        $this->view->form = $form;
    }

    /**
     * Authorize user in system
     * by using social networks credentials
     *
     * @param  string $socila
     * @param  int $id
     * @return bool
     */
    protected function _loginFromSocial($socila, $id)
    {
        $adapter = new Zend_Auth_Adapter_DbTable(
            Singular_Db_Table_Abstract::getDefaultAdapter()
        );
        $tablePrefix = Singular_Runtime::extract('config')
            ->resources->db->params->prefix;
        $adapter->setTableName($tablePrefix . 'users')
                ->setIdentityColumn($socila . '_id')
                ->setCredentialColumn($socila . '_id')
                ->setIdentity($id)
                ->setCredential($id)
                ->getDbSelect()->where('is_active = ?', 1);

        if ($this->_auth->authenticate($adapter)->isValid()) {
            $this->_auth->getStorage()->write($adapter->getResultRowObject());
            return true;
        }

        return false;
    }
}
