<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Loginplus
 * @package    Loginplus_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Email form
 *
 * @uses       Singular_Form
 * @category   Loginplus
 * @package    Loginplus_Form
 */
class Loginplus_Form_Email extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @return void
     */
    public function init()
    {
        parent::init();
        
        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            new Zend_Form_Decorator_ViewScript(array(
                    'viewScript' => 'forms/auth.phtml'
                )),
            'Form'
        ));

        $table = Singular_Loader::get('System_Model_DbTable_Users')->getTableName();
        $messages = array(
            'emailRequired'  => 'Email is required!',
            'emailIncorrect' => 'Email is incorrect!'
        );
        $this->addElement('text', 'email', array(
            'label'       => 'Email',
            'description' => 'This is one-time procedure',
            'required'    => true,
            'filters'     => array('StringTrim'),
            'validators'  => array(
                array('validator' => 'EmailAddress', 'options' => array(
                    'messages' => array(
                        Zend_Validate_EmailAddress::INVALID            => $messages['emailRequired'],
                        Zend_Validate_EmailAddress::INVALID_FORMAT     => $messages['emailIncorrect'],
                        Zend_Validate_EmailAddress::INVALID_LOCAL_PART => $messages['emailIncorrect'],
                        Zend_Validate_EmailAddress::INVALID_HOSTNAME   => $messages['emailIncorrect'],
                        Zend_Validate_EmailAddress::INVALID_SEGMENT    => $messages['emailIncorrect'],
                        Zend_Validate_EmailAddress::INVALID_MX_RECORD  => $messages['emailIncorrect']
                    )
                )),
                array('validator' => 'Db_NoRecordExists', 'options' => array(
                    'table' => $table,
                    'field' => 'email',
                    'messages' => array(
                        Zend_Validate_Db_NoRecordExists::ERROR_RECORD_FOUND => 'Email already in use'
                    )
                ))
            ),
            'decorators' => array(
                'viewHelper'
            )
        ));

        $this->addElement('submit', 'submit', array(
            'label'      => 'Submit',
            'ignore'     => true,
            'Attribs'    => array(
                'class'  => 'button-type1'
            )
        ));
    }
}
