<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Loginplus
 * @package    Loginplus_Service
 * @subpackage Loginplus_Service_Adapter
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Facebook auth adapter
 *
 * @uses       Zend_Auth_Adapter_Interface
 * @category   Loginplus
 * @package    Loginplus_Service
 * @subpackage Loginplus_Service_Adapter
 */
class Loginplus_Service_Adapter_Google implements Zend_Auth_Adapter_Interface
{
    /**
     * The identity value being authenticated
     *
     * @var string
     */
    private $_id = null;

    /**
     * Consumer Key
     *
     * @var string
     */
    private $_consKey = null;

    /**
     * Consumer Secret
     *
     * @var string
     */
    private $_consSecret = null;

    /**
     * Call Back Url
     *
     * @var string
     */
    private $_callbackUrl = null;

    /**
     * Scope
     *
     * @var string
     */
    private $_scope = null;

    const GOOGLE_OAUTH_REQUEST_TOKEN_URL = 'https://www.google.com/accounts/OAuthGetRequestToken';
    const GOOGLE_OAUTH_AUTHORIZE_URL     = 'https://www.google.com/accounts/OAuthAuthorizeToken';
    const GOOGLE_OAUTH_ACCESS_TOKEN_URL  = 'https://www.google.com/accounts/OAuthGetAccessToken';

    /**
     * Constructor
     *
     * @param string $consumerKey
     * @param string $consumerSecret
     * @param string $callbackUrl
     */
    public function __construct($consumerKey, $consumerSecret, $callbackUrl)
    {
        $this->_consSecret = $consumerSecret;
        $this->_consKey = $consumerKey;
        $this->_callbackUrl = $callbackUrl;
        $this->_scope = array(
            'https://www.googleapis.com/auth/userinfo.profile',
            'https://www.googleapis.com/auth/userinfo.email'
        );
    }

    /**
     * Autheticate
     *
     * @return Zend_Auth_Result
     */
    public function authenticate()
    {
        $id = $this->_id;
        $config = array(
            'requestScheme'         => Zend_Oauth::REQUEST_SCHEME_HEADER,
            'version'               => '1.0',
            'consumerKey'           => $this->_consKey,
            'consumerSecret'        => $this->_consSecret,
            'userAuthorizationUrl'  => self::GOOGLE_OAUTH_AUTHORIZE_URL,
            'callbackUrl'           => $this->_callbackUrl,
            'signatureMethod'       => 'HMAC-SHA1',
            'requestTokenUrl'       => self::GOOGLE_OAUTH_REQUEST_TOKEN_URL,
            'accessTokenUrl'        => self::GOOGLE_OAUTH_ACCESS_TOKEN_URL
        );
        $consumer = new Zend_Oauth_Consumer($config);

        $sess = new Zend_Session_Namespace('auth_google');

        if (!isset($_GET['oauth_token'])) {
            $token = $consumer->getRequestToken(array('scope' => implode(' ', $this->_scope)));
            $sess->requestToken = serialize($token);
            $consumer->redirect();
        } else {
            try {
                $requestToken = unserialize($sess->requestToken);
                if ( !($requestToken instanceof Zend_Oauth_Token_Request) ) {
                    throw new Zend_Oauth_Exception('Request token must be instance of Zend_Oauth_Token_Request');
                }
                $sess->unsetAll();
                
                $token = $consumer->getAccessToken($_GET, $requestToken);
                
                $client = $token->getHttpClient($config);
                $client->setUri('https://www.googleapis.com/oauth2/v1/userinfo');                
                $client->setParameterGet('alt', 'json');
                $client->setMethod(Zend_Http_Client::GET);
                $response = $client->request();

                $body = $response->getBody();
                $data = Zend_Json_Decoder::decode($body);

                $user = array(
                    'google_id'  => $data['id'],
                    'email'      => $data['email'],
                    'full_name'  => $data['name'],
                    'photo_url'  => isset($data['picture']) ? $data['picture'] : null,
                    'photo_name' => $data['id']
                );

                return new Zend_Auth_Result(
                    Zend_Auth_Result::SUCCESS,
                    $user['google_id'],
                    array(
                        'user'  => $user,
                        'token' => serialize($token)
                    )
                );
            } catch (Zend_Oauth_Exception $e) {
                return new Zend_Auth_Result(Zend_Auth_Result::FAILURE, $id, array("Authenticaton failed"));
            }
        }
    }
}
