<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Loginplus
 * @package    Loginplus_Service
 * @subpackage Loginplus_Service_Adapter
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Twitter auth adapter
 *
 * @uses       Zend_Auth_Adapter_Interface
 * @category   Loginplus
 * @package    Loginplus_Service
 * @subpackage Loginplus_Service_Adapter
 */
class Loginplus_Service_Adapter_Twitter implements Zend_Auth_Adapter_Interface
{
    /**
     * The identity value being authenticated
     *
     * @var string
     */
    private $_id = null;

    /**
     * Consumer Key
     *
     * @var string
     */
    private $_consKey = null;

    /**
     * Consumer Secret
     * @var string
     */
    private $_consSecret = null;

    /**
     * Call Back Url
     *
     * @var string
     */
    private $_callbackUrl = null;

    const TWITTER_OAUTH_SITEURL           = 'https://twitter.com/oauth';
    const TWITTER_OAUTH_REQUEST_TOKEN_URL = 'https://twitter.com/oauth/request_token';
    const TWITTER_OAUTH_AUTHORIZE_URL     = 'https://twitter.com/oauth/authenticate';
    const TWITTER_OAUTH_ACCESS_TOKEN_URL  = '/access_token';

    /**
     * Constructor
     *
     * @param string $consumerKey
     * @param string $consumerSecret
     * @param string $callbackUrl
     */
    public function __construct($consumerKey, $consumerSecret, $callbackUrl)
    {
        $this->_consSecret  = $consumerSecret;
        $this->_consKey     = $consumerKey;
        $this->_callbackUrl = $callbackUrl;
    }

    /**
     * Autheticate
     *
     * @return Zend_Auth_Result
     */
    public function authenticate()
    {
        $id = $this->_id;
        $config = array(
            'siteUrl'              => self::TWITTER_OAUTH_SITEURL,
            'requestMethod'        => Zend_Oauth::GET,
            'consumerKey'          => $this->_consKey,
            'consumerSecret'       => $this->_consSecret,
            'userAuthorizationUrl' => self::TWITTER_OAUTH_AUTHORIZE_URL,
            'callbackUrl'          => $this->_callbackUrl
        );
        $consumer = new Zend_Oauth_Consumer($config);
 
        $sess = new Zend_Session_Namespace('auth_twitter');
 
        if (!isset($_GET['oauth_token']) && !$sess->requestToken) {
            try {
                $token = $consumer->getRequestToken();
                $sess->requestToken = serialize($token);
                $consumer->redirect();
            } catch (Exception $e) {
                return new Zend_Auth_Result(Zend_Auth_Result::FAILURE, $id, array("Authenticaton failed"));
            }
        } else {
            try {
                $token = $consumer->getAccessToken($_GET, unserialize($sess->requestToken));
                $sess->unsetAll();

                $client = $token->getHttpClient($config);
                $client->setUri('http://api.twitter.com/1.1/account/verify_credentials.json');
                $client->setMethod(Zend_Http_Client::GET);
                $response = $client->request();
                $data = Zend_Json::decode($response->getBody());

                $user['twitter_id'] = $data['id'];
                $user['full_name']  = $data['name'];
                $user['photo_url']  = $data['profile_image_url'];
                $user['photo_name'] = $data['id'];
//                $user['photo_name'] = substr(
//                    $user['photo_url'],
//                    strrpos($user['photo_url'], '/'),
//                    strlen($user['photo_url'])
//                );
                return new Zend_Auth_Result(Zend_Auth_Result::SUCCESS, $data['id'],  array('user' => $user, 'token' => serialize($token)));
            } catch (Zend_Oauth_Exception $e) {
                return new Zend_Auth_Result(Zend_Auth_Result::FAILURE, $id, array("Authenticaton failed"));
            }
        }
    }

}
