<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Problems
 * @package    Problems_Handler
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Problems handler
 *
 * @uses       Singular_Event_Listener_Abstract
 * @category   Problems
 * @package    Problems_Handler
 */
class Problems_Handler_Events extends Singular_Event_Listener_Abstract
{
    /**
     * Binds aggregates on specified events.
     *
     * @return void
     */
    public static function bindListeners ()
    {
        self::attachAggregate('rightMenu',        array(__CLASS__, 'onRightMenu'));
        self::attachAggregate('onMainPage',       array(__CLASS__, 'onMainPage'));
        self::attachAggregate('postDispatchOnce', array(__CLASS__, 'onPostDispatchOnce'));
        self::attachAggregate('buildNavigation',  array(__CLASS__, 'onNavigation'));
        self::attachAggregate('preDispatch',      array(__CLASS__, 'onPreDispatch'));
        self::attachAggregate('insert',           array(__CLASS__, 'onInsert'));
        self::attachAggregate('formInitAfter',    array(__CLASS__, 'onFormInitAfter'));
        self::attachAggregate('responseShow',     array(__CLASS__, 'onResponseShow'));
        self::attachAggregate('commentActions',   array(__CLASS__, 'onCommentActions'));
        self::attachAggregate('afterAnalyticsReportsInit', array(__CLASS__, 'afterAnalyticsReportsInit'));
    }

    /**
     * Subscribes to "commentActions" event.
     *
     * @param  stdClass $params
     * @return void
     */
    public static function onCommentActions($params)
    {
        $response = Singular_Runtime::extract("activeResponse");
        if ($response instanceof System_Model_Response
            && $response->type == 'problem'
        ) {
            $params->actions['good']['label'] = 'Good Solution';
        }
    }

    /**
     * Subscribes to "responseShow" event.
     *
     * @param  stdClass $data
     * @return void
     */
    public static function onResponseShow($data)
    {
        $response = Singular_Runtime::extract("activeResponse");
        if ($response instanceof System_Model_Response
            && $response->type == 'problem'
        ) {
            self::getView()->additionalModulePath('problems');
            $data->sections['votes'] = self::getView()->render('problems_votes.phtml');
        }
    }

    /**
     * Subscribes to "insert" event.
     *
     * @param  System_Model_Response $object
     * @return void
     */
    public static function onInsert($object)
    {
        if ($object instanceof System_Model_Response && $object->type == 'problem') {
            $defaultStatus = $object->getDefaultStatus()->slug;
            if (isset($defaultStatus) && !empty($defaultStatus)) {
                $object->status = $defaultStatus;
            }
            else {
                $object->status = PROBLEMS_STATUS_KNOWN;
            } 
        }
    }

    /**
     * Subscribes to "preDispatch" event.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public static function onPreDispatch($request)
    {
        if (!defined('PROBLEMS_STATUS_KNOWN')) {
            define('PROBLEMS_STATUS_KNOWN', 'known');
        }
    }

    /**
     * Subscribes to "postDispatchOnce" event.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public static function onPostDispatchOnce($request)
    {
        self::getView()->headLink()->prependStylesheet(self::getView()->stylesheetSrc('frontend.css', 'problems'));
        self::getView()->headScript()->appendFile(self::getView()->javascriptSrc('main.js', 'problems'));
        
        $activeResponse = Singular_Runtime::extract('activeResponse');
        
        if ($activeResponse instanceof System_Model_Response
            && $activeResponse->type == 'problem'
        ) {
            $item =
                array(
                    'id'     => 'problem-' . $activeResponse->id,
                    'label'  => self::getView()->truncate($activeResponse->title, '16', '...'),
                    'route'  => 'responses_show',
                    'params' => array(
                        'slug'    => $activeResponse->slug
                    ),
                    'pages' => array(
                        array(
                            'id'     => 'problem-edit',
                            'label'  => self::getView()->t('Edit'),
                            'route'  => 'response_edit',
                            'params' => array(
                                'slug' => $activeResponse->slug
                            )
                        )
                    )
                );
            $navigation = self::getView()->navigation()->getContainer();
            $page = $navigation->findOneBy('id', 'problem-' . $activeResponse->status);
            if ($page instanceof Zend_Navigation_Page) {
                $page->hide_in_bread = true;
                $page->addPage($item);
            }
        }
    }
    
    /**
     * Subscribes to "navigation" event.
     *
     * Builds navigation
     *
     * @param  Singular_Navigation $navigation
     * @return void
     */
    public static function onNavigation($navigation)
    {
        $item = array(
            'id'     => 'problems',
            'label'  => self::getView()->t('Problems'),
            'order'  => Singular_Runtime::extract("types")->findBySlug('problem')->order,
            'route'  => 'responses_type',
            'params' => array(
                'objects' => 'problems'
            ),
            'pages'  => array(
                'all' => array(
                    'id' => 'idea-all',
                    'label' => self::getView()->t('All Problems'),
                    'route' => 'responses_type',
                    'params' => array(
                        'objects' => 'problems',
                        'status' => 'all'
                    )
                )
            )
        );

        $statuses = Singular_Runtime::extract("statuses")->findItemsByField('type', 'problem');
        if ($statuses->valid()) {
            foreach ($statuses as $status) {
                $item['pages'][$status->slug] = array(
                    'id'     => 'problem-' . $status->slug,
                    'label'  => self::getView()->t(ucfirst($status->title)),
                    'route'  => 'responses_type',
                    'params' => array(
                        'objects' => 'problems',
                        'status'  => $status->slug
                    )
                );
            }
        }

        $page = $navigation->findOneBy('id', 'responses');
        if ($page instanceof Zend_Navigation_Page) {
            $page->addPage($item);
        }
    }

    /**
     * Subscribes to "rightMenu" event.
     *
     * @param  stdClass $params
     * @return void
     */
    public static function onRightMenu($params)
    {
        $menu = $params->menu;
        
        if (Singular_Runtime::inStorage('problemsCount')) {
            $problemsCount = Singular_Runtime::extract('problemsCount');
        } else {
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $problemsCount = $responsesDb->count(array('type' => 'problem'));
        }

        $page = $menu->findById("statistics");
        $page->addPage(
            array(
                 'id'     => 'problems',
                 'order'  => Singular_Runtime::extract("types")->findBySlug('problem')->order,
                 'count'  => $problemsCount,
                 'icon'   => self::getView()->imageSrc('icon_small.png', 'problems'),
                 'label'  => 'Problems',
                 'route'  => 'responses_type',
                 'params' => array(
                     'objects' => 'problems'
                 )
            )
        );
        
        $params->cacheKeys[] = 'problems';
    }

    /**
     * Subscribes to "mainPage" event.
     *
     * @param  stdClass $sections
     * @return void
     */
    public static function onMainPage($sections)
    {
        if (
            1 != Singular_Core::_("Settings")->getSetting('homepage_content_responses')
            || 'divided_by_type' != Singular_Core::_("settings")
                ->getSetting("homepage_responses_display_variant")
        ) {
            return;
        }
        $cache = Singular_Runtime::extract("cachemanager")->getCache("default");
        $loggedUser = Singular_Runtime::extract("loggedUser");
        $cacheId = "homepageProblems" . "_" . $loggedUser->role;
        $cacheId = preg_replace("/[^a-zA-Z0-9_]/", "", $cacheId);
        $tags = null;
        
        if ( ($output = $cache->load($cacheId)) == false ) {
            
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $problems = $responsesDb->findTop('problem');

            self::getView()->problems = $problems;
            $problemsCount = $responsesDb->count(array('type' => 'problem'));
            $problemsLeftCount = $problemsCount - count($problems);
            self::getView()->problemsCount = $problemsLeftCount;
            Singular_Runtime::store('problemsCount', $problemsCount);

            self::getView()->additionalModulePath('problems');
            self::getView()->cacheTags = new StdClass();
            self::getView()->cacheTags->tags = array();
            $output = self::getView()->render('_homepage-problems.phtml');
            $tags = self::getView()->cacheTags->tags;
            $tags[] = 'responses';
            $cache->save($output, $cacheId, $tags);
        }
        Resources_Service_Votes::setUserVotes($cacheId, $tags);
        $order = Singular_Runtime::extract("types")->findBySlug('problem')->order + 10;
        $sections->sections[$order] = $output;
    }

    /**
     * Subscribes to "formInitAfter" event.
     *
     * @param  System_Form_Response|System_Form_SearchAdd|System_Form_WidgetFeedback|System_Form_WidgetSearch $form
     * @return void
     */
    public static function onFormInitAfter($form)
    {
        if ($form instanceof System_Form_Response) {
            if ($form instanceof System_Form_ResponseWidget) {
                $form->type->addMultiOption('problem', 'Problem');
            } else {
                $form->type->addMultiOption('problem', 'Report a problem');
            }
        }
        if ($form instanceof System_Form_SearchAdd) {
            $form->type->addMultiOption('problem', 'Report a problem');
            if (!$form->submit->getDescription()) {
                $form->submit->setDescription(self::getView()->t("Report a problem to be found or added"));
            }
        }
        if ($form instanceof System_Form_WidgetFeedback || $form instanceof System_Form_WidgetSearch) {
            $form->widget_default_tab->addMultiOption('problem', 'Problems');
        }
    }
    
    /**
     * Subscribes to "afterAnalyticsReportsInit" event.
     *
     * @param  stdClass $reportsConf
     * @return void
     */
    public static function afterAnalyticsReportsInit ($reportsConf) 
    {
        $reportsConf->reports[] = array(
            'id' => 'problems_responses',
            'title' => self::getView()->t('Problems'),
            'model' => 'System_Model_DbTable_Responses',
            'counter' => 'countByType',
            'order' => '30',
            'options' => array(
                'row_type' => 'default',
                'response_type' => 'problem',
                'icon' => 'url('.self::getView()->imageSrc('icon_small.png', 'problems', false).')',
            )
        );
    }
}
