<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Questions
 * @package    Questions_Module
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Thanks module identity class
 *
 * @uses       Singular_Module_Abstract
 * @category   Questions
 * @package    Questions_Module
 */
class Questions_Module extends Singular_Module_Abstract
{
    /**
     * Priority for module load order, used in module installation process.
     *
     * @ignore
     * @var int
     */
    public $loadOrder = 3;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @return void
     */
    public function init ()
    {
      $this->_info();
    }

    /**
     * Module installation method.
     *
     * @return void
     */
    public function install()
    {
        $modulesDb = Singular_Loader::get('System_Model_DbTable_Modules');
        $module = $modulesDb->findByName('questions');
        /* types */
        $type = new System_Model_ResponseType();
        $type->save(array(
            'slug'              => 'question',
            'title'             => 'Question',
            'objects'           => 'questions',
            'icon'              => 'icon_medium.png',
            'module_id'         => $module->id,
            'has_comments'      => 1,
            'has_negative_vote' => 0,
            'is_active'         => 1,
            'is_default'        => 0,
            'order'             => 3
        ));
        /* statuses */
        $status = new System_Model_ResponseStatus();
        $status->save(array(
            'title'         => 'Answered',
            'slug'          => 'answered',
            'is_default'    => 0,
            'is_protected'  => 1,
            'type'          => 'question',
            'pay_attention' => 0,
            'color'         => '#40bf00',
            'order'         => 1
        ));
        $status = new System_Model_ResponseStatus();
        $status->save(array(
            'title'         => 'Need Answer',
            'slug'          => 'need_answer',
            'is_default'    => 1,
            'is_protected'  => 1,
            'type'          => 'question',
            'pay_attention' => 1,
            'color'         => '#b0b5bf',
            'order'         => 2
        ));
    }

    /**
     * Module uninstallation method.
     *
     * @return void
     */
    public function uninstall()
    {
        $typeDb = Singular_Loader::get('System_Model_DbTable_ResponsesTypes');
        $type = $typeDb->find('question')->current();
        if ($type instanceof System_Model_ResponseType) {
            $type->delete();
        }
    }

    /**
     * Set module information to be available in code and to display on Modules
     * list at backend.
     *
     * @return void
     */
    protected function _info()
    {
        $this->getContainer()->setName('Questions')
              ->setDescription('Let users ask specific questions about your company, products or services to give quick and detailed response')
              ->setVersion('2.1')
              ->addDependency('system', '2.3')
              ->addDependency('resources', '2.1')
              ->setAuthor('USWebStyle, Inc')
              ->setEmail('support@uswebstyle.com')
              ->setUrl('http://www.uswebstyle.com')
              ->setReleaseDate('2013-03-15');
    }
}
