<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Thanks
 * @package    Thanks_Handler
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Thanks handler
 *
 * @uses       Singular_Event_Listener_Abstract
 * @category   Thanks
 * @package    Thanks_Handler
 */
class Questions_Handler_Events extends Singular_Event_Listener_Abstract
{
    /**
     * Binds aggregates on specified events.
     *
     * @return void
     */
    public static function bindListeners ()
    {
        self::attachAggregate('rightMenu',        array(__CLASS__, 'onRightMenu'));
        self::attachAggregate('onMainPage',       array(__CLASS__, 'onMainPage'));
        self::attachAggregate('postDispatchOnce', array(__CLASS__, 'onPostDispatchOnce'));
        self::attachAggregate('buildNavigation',  array(__CLASS__, 'onNavigation'));
        self::attachAggregate('preDispatch',      array(__CLASS__, 'onPreDispatch'));
        self::attachAggregate('insert',           array(__CLASS__, 'onInsert'));
        self::attachAggregate('formInitAfter',    array(__CLASS__, 'onFormInitAfter'));
        self::attachAggregate('responseShow',     array(__CLASS__, 'onResponseShow'));
        self::attachAggregate('commentActions',   array(__CLASS__, 'onCommentActions'));
        self::attachAggregate('afterAnalyticsReportsInit', array(__CLASS__, 'afterAnalyticsReportsInit'));
    }

    /**
     * Subscribes to "commentActions" event.
     *
     * @param  stdClass $params
     * @return void
     */
    public static function onCommentActions($params)
    {
        $response = Singular_Runtime::extract("activeResponse");
        if ($response instanceof System_Model_Response
            && $response->type == 'question'
        ) {
            $params->actions['good']['label'] = 'Good Answer';
        }
    }

    /**
     * Subscribes to "responseShow" event.
     *
     * @param  stdClass $data
     * @return void
     */
    public static function onResponseShow($data)
    {
        $response = Singular_Runtime::extract("activeResponse");
        if ($response instanceof System_Model_Response
            && $response->type == 'question'
        ) {
            self::getView()->additionalModulePath('questions');
            $data->sections['votes'] = self::getView()->render('questions_votes.phtml');
        }
    }

    /**
     * Subscribes to "responseShow" event.
     *
     * @param  System_Model_Response $object
     * @return void
     */
    public static function onInsert($object)
    {
        if ($object instanceof System_Model_Response && $object->type == 'question') {
            $defaultStatus = $object->getDefaultStatus()->slug;
            if (isset($defaultStatus) && !empty($defaultStatus)) {
                $object->status = $defaultStatus;
            }
            else {
                $object->status = QUESTIONS_STATUS_NEED_ANSWER;
            }
        }
        if ($object instanceof System_Model_ResponseAnswer) {
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $response = $responsesDb->findById($object->response_id);
            if ($response instanceof System_Model_Response) {
                if ($response->type == "question") {
                    $response->status = QUESTIONS_STATUS_ANSWERED;
                    $response->save();
                }
            }
        }
    }

    /**
     * Subscribes to "preDispatch" event.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public static function onPreDispatch($request)
    {
        if (!defined('QUESTIONS_STATUS_ANSWERED')) {
            define("QUESTIONS_STATUS_ANSWERED", 'answered');
        }
        if (!defined('QUESTIONS_STATUS_NEED_ANSWER')) {
            define("QUESTIONS_STATUS_NEED_ANSWER", 'need_answer');
        }
    }

    /**
     * Subscribes to "postDispatchOnce" event.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public static function onPostDispatchOnce($request)
    {
        self::getView()->headLink()->prependStylesheet(self::getView()->stylesheetSrc('frontend.css', 'questions'));
        self::getView()->headScript()->appendFile(self::getView()->javascriptSrc('main.js', 'questions'));
        
        $activeResponse = Singular_Runtime::extract('activeResponse');
        
        if ($activeResponse instanceof System_Model_Response
            && $activeResponse->type == 'question'
        ) {
            $item =
                array(
                    'id'     => 'question-' . $activeResponse->id,
                    'label'  => self::getView()->truncate($activeResponse->title, '16', '...'),
                    'route'  => 'responses_show',
                    'params' => array(
                        'slug'    => $activeResponse->slug
                    ),
                    'pages' => array(
                        array(
                            'id'     => 'problem-edit',
                            'label'  => self::getView()->t('Edit'),
                            'route'  => 'response_edit',
                            'params' => array(
                                'slug' => $activeResponse->slug
                            )
                        )
                    )
                );
            $navigation = self::getView()->navigation()->getContainer();
            $page = $navigation->findOneBy('id', 'question-' . $activeResponse->status);
            if ($page instanceof Zend_Navigation_Page) {
                $page->hide_in_bread = true;
                $page->addPage($item);
            }
        }
    }
    
    /**
     * Subscribes to "navigation" event.
     *
     * Builds navigation
     *
     * @param  Singular_Navigation $navigation
     * @return void
     */
    public static function onNavigation($navigation)
    {
        $item = array(
            'id'     => 'questions',
            'class'  => 'right-menu',
            'label'  => self::getView()->t('Questions'),
            'order'  => Singular_Runtime::extract("types")->findBySlug('question')->order,
            'route'  => 'responses_type',
            'params' => array(
                'objects' => 'questions'
            ),
            'pages'  => array(
                'all' => array(
                    'id'     => 'idea-all',
                    'label'  => self::getView()->t('All Questions'),
                    'route'  => 'responses_type',
                    'params' => array(
                        'objects' => 'questions',
                        'status'  => 'all'
                    )                    
                )
            )
        );

        $statuses = Singular_Runtime::extract("statuses")
                        ->findItemsByField('type', 'question');
        if ($statuses->valid()) {
            foreach ($statuses as $status) {
                $item['pages'][$status->slug] = array(
                    'id'     => 'question-' . $status->slug,
                    'label'  => self::getView()->t(ucfirst($status->title)),
                    'route'  => 'responses_type',
                    'params' => array(
                        'objects' => 'questions',
                        'status'  => $status->slug
                    )
                );
            }
        }

        $page = $navigation->findOneBy('id', 'responses');
        if ($page instanceof Zend_Navigation_Page) {
            $page->addPage($item);
        }
    }

    /**
     * Subscribes to "rightMenu" event.
     *
     * @param  StdClass $params
     * @return void
     */
    public static function onRightMenu($params)
    {
        $menu = $params->menu;
        
        if (Singular_Runtime::inStorage('questionsCount')) {
            $questionsCount = Singular_Runtime::extract('questionsCount');
        } else {
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $questionsCount = $responsesDb->count(array('type' => 'question'));
        }

        $page = $menu->findById("statistics");
        $page->addPage(
            array(
                 'id'     => 'questions',
                 'order'  => Singular_Runtime::extract("types")->findBySlug('question')->order,
                 'count'  => $questionsCount,
                 'icon'   => self::getView()->imageSrc('icon_small.png', 'questions'),
                 'label'  => 'Questions',
                 'route'  => 'responses_type',
                 'params' => array(
                     'objects' => 'questions'
                 )
            )
        );
        
        $params->cacheKeys[] = 'questions';
    }

    /**
     * Subscribes to "mainPage" event.
     *
     * @param  stdClass $sections
     * @return void
     */
    public static function onMainPage($sections)
    {
        if (
            1 != Singular_Core::_("Settings")->getSetting('homepage_content_responses')
            || 'divided_by_type' != Singular_Core::_("settings")
                ->getSetting("homepage_responses_display_variant")
        ) {
            return;
        }
        $cache = Singular_Runtime::extract("cachemanager")->getCache("default");
        $loggedUser = Singular_Runtime::extract("loggedUser");
        $cacheId = "homepageQuestions" . "_" . $loggedUser->role;
        $cacheId = preg_replace("/[^a-zA-Z0-9_]/", "", $cacheId);
        $tags = null;
        
        if ( ($output = $cache->load($cacheId)) == false ) {
            
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $questions = $responsesDb->findTop('question');

            self::getView()->questions = $questions;
            $questionsCount = $responsesDb->count(array('type' => 'question'));
            $questionsLeftCount = $questionsCount - count($questions);
            self::getView()->questionsCount = $questionsLeftCount;
            Singular_Runtime::store('questionsCount', $questionsCount);

            self::getView()->additionalModulePath('questions');
            self::getView()->cacheTags = new StdClass();
            self::getView()->cacheTags->tags = array();
            $output = self::getView()->render('_homepage-questions.phtml');
            $tags = self::getView()->cacheTags->tags;
            $tags[] = 'responses';
            $cache->save($output, $cacheId, $tags);
        }
        Resources_Service_Votes::setUserVotes($cacheId, $tags);
        $order = Singular_Runtime::extract("types")->findBySlug('question')->order + 10;
        $sections->sections[$order] = $output;
    }

    /**
     * Subscribes to "formInitAfter" event.
     *
     * @param  System_Form_Response|System_Form_SearchAdd|System_Form_WidgetFeedback|System_Form_WidgetSearch $form
     * @return void
     */
    public static function onFormInitAfter($form)
    {
        if ($form instanceof System_Form_Response) {
            if ($form instanceof System_Form_ResponseWidget) {
                $form->type->addMultiOption('question', 'Question');
            } else {
                $form->type->addMultiOption('question', 'Ask a question');
            }
        }
        if ($form instanceof System_Form_SearchAdd) {
            $form->type->addMultiOption('question', 'Ask a question');
            if (!$form->submit->getDescription()) {
                $form->submit->setDescription(self::getView()->t("Find the same question or ask us"));
            }
        }
        if ($form instanceof System_Form_WidgetFeedback || $form instanceof System_Form_WidgetSearch) {
            $form->widget_default_tab->addMultiOption('question', 'Questions');
        }
    }
    
    /**
     * Subscribes to "afterAnalyticsReportsInit" event.
     *
     * @param  stdClass $reportsConf
     * @return void
     */
    public static function afterAnalyticsReportsInit ($reportsConf) 
    {
        $reportsConf->reports[] = array(
            'id' => 'questions_responses',
            'title' => self::getView()->t('Questions'),
            'model' => 'System_Model_DbTable_Responses',
            'counter' => 'countByType',
            'order' => '35',
            'options' => array(
                'row_type' => 'default',
                'response_type' => 'question',
                'icon' => 'url('.self::getView()->imageSrc('icon_small.png', 'questions', false).')',
            )
        );
    }
}
