<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Module
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Resources module identity class
 *
 * @uses       Singular_Module_Abstract
 * @category   Resources
 * @package    Resources_Module
 */
class Resources_Module extends Singular_Module_Abstract
{
    /**
     * Priority for module load order, used in module installation process.
     *
     * @ignore
     * @var int
     */
    public $loadOrder = 8;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @return void
     */
    public function init ()
    {
      $this->_info();
      $this->setIsSystemModule(true);
    }

    /**
     * Module installation method.
     *
     * @return void
     */
    public function install ()
    {
        $db = Singular_Db_Table_Abstract::getDefaultAdapter();
        $tablePrefix = Singular_Runtime::extract("config")->resources->db->params->prefix;
        $modulesDb = new System_Model_DbTable_Modules(array('name' => $tablePrefix . 'modules'));
        $module = $modulesDb->findByName('resources');
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}emails_stack` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `time` datetime NOT NULL,
              `email` varchar(255) NOT NULL,
              `subject` varchar(255) NOT NULL,
              `body` text NOT NULL,
              PRIMARY KEY (`id`),
              KEY `email` (`email`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}email_templates` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `module_id` int(11) NOT NULL,
              `title` varchar(50) NOT NULL,
              `slug` varchar(50) NOT NULL,
              `variables` text NOT NULL,
              PRIMARY KEY (`id`),
              UNIQUE KEY `name` (`slug`)
            ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=9 ;
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}email_templates_lang` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `template_id` int(11) NOT NULL,
              `lang` varchar(2) NOT NULL,
              `subject` varchar(150) NOT NULL,
              `body` text NOT NULL,
              PRIMARY KEY (`id`),
              KEY `template_id` (`template_id`,`lang`)
            ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=17 ;
        ";
        $db->exec($sql);
        
        $email = new Resources_Model_Template();
        $email->lang = 'en';
        $email->setFromArray(array(
            'module_id' => $module->id,
            'title'     => 'Registration',
            'slug'      => 'registration',
            'subject'   => 'Your account is created in :community_name community!',
            'body'      => '<p>Hi,</p>
<p>Welcome to <a href=":community_url">:community_name</a>! Find login credentials below:</p>
<hr />
<b>Email:</b> :email <br /><b>Password:</b> :password
<hr />
<p>You can help us make our community better by expressing your opinion! <br /> You can change your settings, password, upload avatar, or track activity in <a href=":profile_url">your profile</a> </p>',
            'variables' => serialize(array(
                "community_name",
                "community_url",
                "email",
                "password",
                "login_url",
                "profile_url",
            )),
        ));
        $email->save();
        
        $emailTranslate = new Resources_Model_TemplateLang();
        $emailTranslate->setFromArray(array(
            'template_id' => $email->id,
            'lang'        => 'ru',
            'subject'     => 'Ваш аккаунт создан в :community_name сообществе!',
            'body'        => '<p>Привет,</p>
<p>Добро пожаловать в <a href=":community_url">:community_name</a>! Ниже ваш доступ в систему:</p>
<hr />
<b>Email:</b> :email <br /><b>Пароль:</b> :password
<hr />
<p>Вы можете помочь улучшить наше сообщество, выражая ваше мнение!<br /> Изменить настройки, пароль, загрузить аватар или следить за активностью в <a href=":profile_url">вашем профайле</a> </p>',
        ));
        $emailTranslate->save();
        
        $email = new Resources_Model_Template();
        $email->lang = 'en';
        $email->setFromArray(array(
            'module_id' => $module->id,
            'title'     => 'Forgot Password',
            'slug'      => 'forgot_password',
            'subject'   => 'Your New Password!',
            'body'      => '<p>Hi,</p>
<p>Your new password to <a href=":community_url">:community_name</a></p>
<hr />
<b>Email:</b> :email <br /><b>Password:</b> :password
<hr />
<p>You can change your password at your <a href=":profile_url">profile</a> any time!</p>
<p>Best,<br />:community_name</p>',
            'variables' => serialize(array(
                "community_name",
                "community_url",
                "email",
                "password",
                "login_url",
                "profile_url",
            )),
        ));
        $email->save();
        
        $emailTranslate = new Resources_Model_TemplateLang();
        $emailTranslate->setFromArray(array(
            'template_id' => $email->id,
            'lang'        => 'ru',
            'subject'     => 'Ваш новый пароль!',
            'body'        => '<p>Привет,</p>
<p>Ваш новый пароль в <a href=":community_url">:community_name</a></p>
<hr />
<b>Email:</b> :email <br /><b>Password:</b> :password
<hr />
<p>Вы можете изменить пароль в вашем <a href=":profile_url">профайле</a>в любое время!</p>
<p>С наилучшими пожеланиями,<br />:community_name</p>',
        ));
        $emailTranslate->save();
        
        $email = new Resources_Model_Template();
        $email->lang = 'en';
        $email->setFromArray(array(
            'module_id' => $module->id,
            'title'     => 'Restricted Access',
            'slug'      => 'restricted_access',
            'subject'   => 'Your access is limited to :community_name!',
            'body'      => '<p>Hi,</p>
<p>Due to our community violation rights, you have been :user_status to use all features of
our community!</p>
<p>Please contact <a href=":community_url">:community_name</a> administration to recover access.</p>
<p>Best,<br />:community_name</p>',
            'variables' => serialize(array(
                "community_name",
                "community_url",
                "user_status",
            )),
        ));
        $email->save();
        
        $emailTranslate = new Resources_Model_TemplateLang();
        $emailTranslate->setFromArray(array(
            'template_id' => $email->id,
            'lang'        => 'ru',
            'subject'     => 'Ваш доступ в систему :community_name ограничен!',
            'body'        => '<p>Привет,</p>
<p>Из-за нарушений правил использования ресурса вы :user_status в нашем сообществе!</p>
<p>Пожалуйста свяжитесь с <a href=":community_url">:community_name</a> администрацией для восстановления доступа.</p>
<p>С наилучшими пожеланиями,<br />:community_name</p>',
        ));
        $emailTranslate->save();
        
        $email = new Resources_Model_Template();
        $email->lang = 'en';
        $email->setFromArray(array(
            'module_id' => $module->id,
            'title'     => 'Recovered Access',
            'slug'      => 'recovered_access',
            'subject'   => 'Recovered Access',
            'body'      => '<p>Hi,</p>
<p>We have recovered access to :community_name! Your status is :user_status </p>
<p>If you have any questions, please contact <a href=":community_url">:community_name</a> administration.</p>
<p>Best,<br />:community_name</p',
            'variables' => serialize(array(
                "community_name",
                "community_url",
                "user_status",
            )),
        ));
        $email->save();
        
        $emailTranslate = new Resources_Model_TemplateLang();
        $emailTranslate->setFromArray(array(
            'template_id' => $email->id,
            'lang'        => 'ru',
            'subject'     => 'Ваш доступ в систему :community_name восстановлен!',
            'body'        => '<p>Привет,</p>
<p>Вы восстановили доступ в сообщество :community_name! Ваш статус :user_status </p>
<p>Если у вас есть какие-либо вопросы, пожалуйста свяжитесь с <a href=":community_url">:community_name</a> администрацией.</p>
<p>С наилучшими пожеланиями,<br />:community_name</p',
        ));
        $emailTranslate->save();
        
        $email = new Resources_Model_Template();
        $email->lang = 'en';
        $email->setFromArray(array(
            'module_id' => $module->id,
            'title'     => 'New Response',
            'slug'      => 'new_response',
            'subject'   => 'New :response_type has been added',
            'body'      => '<p>Hi,</p>
<p><a href=":created_by_url">:created_by_name</a> created a new <a href=":response_url">:response_name</a> :response_type:</p>
<hr />
:response_body
<hr />
<p><a href=":response_url">Click here</a> to see the response.</p>
<p>Best,<br />:community_name</p>
<hr />
<p><i>If you want to unsubscribe from this notification, please go to your </i> <a href=":profile_url">profile</a>!</p>',
            'variables' => serialize(array(
                "community_name",
                "community_url",
                "response_name",
                "response_type",
                "response_url",
                "response_body",
                "response_status",
                "created_by_url",
                "created_by_name",
                "profile_url",
            )),
        ));
        $email->save();
        
        $emailTranslate = new Resources_Model_TemplateLang();
        $emailTranslate->setFromArray(array(
            'template_id' => $email->id,
            'lang'        => 'ru',
            'subject'     => 'В систему добавлен(а) :response_type',
            'body'        => '<p>Привет,</p>
<p><a href=":created_by_url">:created_by_name</a> создал(а) <a href=":response_url">:response_name</a> :response_type:</p>
<hr />
:response_body
<hr />
<p><a href=":response_url">Нажмите сюда</a>, чтобы увидеть тему</p>
<p>С наилучшими пожеланиями,<br />:community_name</p>
<hr />
<p><i>Если вы хотите отписаться от уведомления, пожалуйста зайдите в ваш </i> <a href=":profile_url">профайл</a>!</p>',
        ));
        $emailTranslate->save();
        
        $email = new Resources_Model_Template();
        $email->lang = 'en';
        $email->setFromArray(array(
            'module_id' => $module->id,
            'title'     => 'Change Response Status',
            'slug'      => 'change_response_status',
            'subject'   => 'Status changed at ":response_name"',
            'body'      => '<p>Hi,</p>
<p><a href=":response_url">:response_name</a> status is changed to <b>:response_status</b></p>
<hr />
<p><i>If you want to unsubscribe from this notification, please go to your </i> <a href=":profile_url">profile</a>!</p>',
            'variables' => serialize(array(
                "community_name",
                "community_url",
                "response_name",
                "response_type",
                "response_url",
                "response_body",
                "response_status",
                "profile_url",
            )),
        ));
        $email->save();
        
        $emailTranslate = new Resources_Model_TemplateLang();
        $emailTranslate->setFromArray(array(
            'template_id' => $email->id,
            'lang'        => 'ru',
            'subject'     => 'Изменился статус у ":response_name"',
            'body'        => '<p>Привет,</p>
<p><a href=":response_url">:response_name</a> изменился статус на <b>:response_status</b></p>
<hr />
<p><i>Если вы хотите отписаться от уведомления, пожалуйста зайдите в ваш </i> <a href=":profile_url">профайл</a>!</p>',
        ));
        $emailTranslate->save();
        
        $email = new Resources_Model_Template();
        $email->lang = 'en';
        $email->setFromArray(array(
            'module_id' => $module->id,
            'title'     => 'New Comment',
            'slug'      => 'new_comment',
            'subject'   => ':response_statusNew comment on ":response_name" :response_type has been posted',
            'body'      => '<p>Hi,</p>
<p><a href=":created_by_url">:created_by_name</a> has replied to <a href=":response_url">:response_name</a> :response_type:</p>
<hr />
:comment_body
<hr />
<p><a href=":comment_url">Click here</a> to see the comment.
<p>Best,<br />:community_name</p>
<hr />
<p><i>If you want to unsubscribe from this notification, please go to your </i> <a href=":profile_url">profile</a>!</p>',
            'variables' => serialize(array(
                "community_name",
                "community_url",
                "response_name",
                "response_type",
                "response_url",
                "response_body",
                "response_status",
                "created_by_url",
                "created_by_name",
                "comment_url",
                "comment_body",
                "profile_url",
            )),
        ));
        $email->save();
        
        $emailTranslate = new Resources_Model_TemplateLang();
        $emailTranslate->setFromArray(array(
            'template_id' => $email->id,
            'lang'        => 'ru',
            'subject'     => ':response_statusДобавлен новый комментарий в ":response_name" :response_type',
            'body'        => '<p>Привет,</p>
<p><a href=":created_by_url">:created_by_name</a> ответил в <a href=":response_url">:response_name</a> :response_type:</p>
<hr />
:comment_body
<hr />
<p><a href=":comment_url">Нажмите сюда</a>, чтобы увидеть комментарий
<p>С наилучшими пожеланиями,<br />:community_name</p>
<hr />
<p><i>Если вы хотите отписаться от уведомления, пожалуйста зайдите в ваш </i> <a href=":profile_url">профайл</a>!</p>',
        ));
        $emailTranslate->save();
        
        $email = new Resources_Model_Template();
        $email->lang = 'en';
        $email->setFromArray(array(
            'module_id' => $module->id,
            'title'     => 'Official Answer',
            'slug'      => 'official_answer',
            'subject'   => ':response_statusNew official answer on ":response_name" :response_type has been posted',
            'body'      => '<p>Hi,</p>
<p><a href=":created_by_url">:created_by_name</a> has replied to <a href=":response_url">:response_name</a> :response_type:</p> with official answer
<hr />
:official_answer
<hr />
<p><a href=":response_url">Click here</a> to leave your comment</p>
<p>Best,<br />:community_name</p>
<hr />
<p><i>If you want to unsubscribe from this notification, please go to your </i> <a href=":profile_url">profile</a>!</p>',
            'variables' => serialize(array(
                "community_name",
                "community_url",
                "response_name",
                "response_type",
                "response_url",
                "response_body",
                "response_status",
                "created_by_url",
                "created_by_name",
                "official_answer",
                "profile_url",
            )),
        ));
        $email->save();
        
        $emailTranslate = new Resources_Model_TemplateLang();
        $emailTranslate->setFromArray(array(
            'template_id' => $email->id,
            'lang'        => 'ru',
            'subject'     => ':response_statusДобавлен официальный ответ в ":response_name" :response_type',
            'body'        => '<p>Привет,</p>
<p><a href=":created_by_url">:created_by_name</a> официально ответил(а) в <a href=":response_url">:response_name</a> :response_type:</p>
<hr />
:official_answer
<hr />
<p><a href=":response_url">Нажмите сюда</a>, чтобы оставить комментарий</p>
<p>С наилучшими пожеланиями,<br />:community_name</p>
<hr />
<p><i>Если вы хотите отписаться от уведомления, пожалуйста зайдите в ваш </i> <a href=":profile_url">профайл</a>!</p>',
        ));
        $emailTranslate->save();
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}response_comments` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `content` longtext CHARACTER SET utf8 COLLATE utf8_unicode_ci,
              `response_id` int(11) NOT NULL,
              `created_by_id` int(11) NOT NULL,
              `created_at` datetime NOT NULL,
              `updated_at` datetime DEFAULT NULL,
              `updated_by_id` int(11) DEFAULT NULL,
              `is_best` tinyint(1) NOT NULL DEFAULT '0',
              `best_at` datetime DEFAULT NULL,
              `is_negative` tinyint(1) NOT NULL DEFAULT '0',
              `deleted_by_id` int(11) NOT NULL DEFAULT '0',
              `deleted_at` datetime DEFAULT NULL,
              PRIMARY KEY (`id`),
              KEY `response_id` (`response_id`),
              KEY `created_at` (`created_at`),
              KEY `created_by_id` (`created_by_id`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
        ";
        $db->exec($sql); 
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}votes` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `object` varchar(40) NOT NULL,
              `object_id` int(11) NOT NULL,
              `created_by_id` int(11) NOT NULL DEFAULT '0',
              `created_at` datetime NOT NULL,
              `value` tinyint(11) NOT NULL,
              PRIMARY KEY (`id`),
              KEY `object` (`object`,`object_id`),
              KEY `created_by_id` (`created_by_id`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
        ";
        $db->exec($sql); 
    }

    /**
     * Set module information to be available in code and to display on Modules
     * list at backend.
     *
     * @return void
     */
    protected function _info ()
    {
        $this->getContainer()->setName('Resources')
              ->setDescription('System resources supported by core - comments, tags, mailing, etc...')
              ->setVersion('2.2')
              ->addDependency('system', '2.3.3')
              ->setAuthor('USWebStyle, Inc')
              ->setEmail('support@uswebstyle.com')
              ->setUrl('http://www.uswebstyle.com')
              ->setReleaseDate('2013-05-08');
    }
}
