<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * AdminTemplates controller
 *
 * @uses       System_Controller_Abstract_Admin
 * @category   Resources
 * @package    Resources_Controller
 */
class Resources_AdminTemplatesController extends System_Controller_Abstract_Admin
{
    /**
     * Templates model
     *
     * @var Resources_Model_Template
     */
    public $activeTemplate = null;

    /**
     * Initialize object.
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();

        $this->templatesDb = Singular_Loader::get('Resources_Model_DbTable_Templates');
        $languagesDb = Singular_Loader::get('System_Model_DbTable_Languages');
        $this->languages = $this->view->languages = $languagesDb->getLanguages();

        $id = $this->getRequest()->getParam('id');
        if(!empty($id)){
            $activeTemplate = $this->templatesDb->find($id)->current();
            if(!($activeTemplate instanceof Resources_Model_Template)){
                $this->view->notify()->error('Template not found');
                $this->_redirect($this->view->url(array(), 'admin_templates'));
            }
            $this->view->activeTemplate = $this->activeTemplate = $activeTemplate;
        }
        $lang = $this->getRequest()->getParam('code');
        if(!empty($lang)){
            $this->view->activeLanguage = $this->activeLanguage = $this->languages->findByCode($lang);
            $activeTemplate->lang = $lang;
        } else {
            $code = $this->settings->system->main->site_default_language->value;
            $this->view->activeLanguage = $this->activeLanguage = $this->languages->findByCode($code);
        }
        if ($this->activeLanguage == null) {
            $this->_redirect($this->view->url(array(), 'admin'));
        }
    }

    /**
     * Displays all available templates
     *
     * @return void
     */
    public function indexAction()
    {
        $this->view->items = $this->templatesDb->findAll();
        
        if ($this->isAjax()) {
            $this->_helper->layout->disableLayout();
            $output = $this->fetchTemplate('resources_admin-templates_index.phtml');
            $this->autoresponder->setOutput($output);
        }
    }

    /**
     * Updates existing template data
     *
     * @return void
     */
    public function editAction()
    {
        $form = new Resources_Form_Template();
        $form->setDefaults($this->activeTemplate->toArray());
        $form->setDefault('subject', $this->activeTemplate->getSubject());
        $form->setDefault('body', $this->activeTemplate->getBody());
        
        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            if($form->isValid($post)){
                $data = $form->getValues();
                $this->activeTemplate->setFromArray($data);
                $this->activeTemplate->save();
                $this->view->notify()->success('Email Template is updated!');
                $this->_redirect($this->view->url(array(), 'admin_templates'));
            }
        }
        $this->view->form = $form;
    }
}
