<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Comment controller
 *
 * @uses       System_Controller_Abstract_Action
 * @category   Resources
 * @package    Resources_Controller
 */
class Resources_CommentController extends System_Controller_Abstract_Action
{
    /**
     * Comments model
     *
     * @var Resources_Model_ResponseComment
     */
    protected $_activeComment;

    /**
     * Initialize object.
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();
        $this->_helper->layout()->disableLayout();
        $this->commentsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
        $id = $this->getRequest()->getParam('comment_id');
        if (isset($id)) {
            $this->_activeComment = $this->commentsDb->findById($id);
            if ( !($this->_activeComment instanceof Resources_Model_ResponseComment) ) {
                $this->view->notify()->error("Comment not found");
                $this->_helper->redirector->gotoRoute(array(), 'responses');
            }
        } else {
            $this->_activeComment = new Resources_Model_ResponseComment();
        }
        Singular_Runtime::store("activeComment", $this->_activeComment);
    }

    /**
     * Displays comments list for specific response
     *
     * @return void
     */
    public function indexAction()
    {
        $this->_helper->viewRenderer->setNeverRender();

        $responseId = $this->_getParam('response_id');
        /** @var $response System_Model_Response */
        $response = Singular_Loader::get('System_Model_DbTable_Responses')->findById($responseId);
        if ($response instanceof System_Model_Response) {

            $params = $this->_getAllParams();
            $cacheId = $this->loggedUser->role . "_comments_paginator_" . md5(var_export($params, true));
            $cacheId = preg_replace("/[^a-zA-Z0-9_]/", "", $cacheId);
            $cache = Singular_Runtime::extract('cachemanager')->getCache('default');

            $activeStatus = $this->_getParam('status', 'popular');
            $tags = null;

            if ( ($output = $cache->load($cacheId)) == false ) {
                $this->view->votes = $this->_getParam('votes');
                $this->view->activeResponse = $response;
                $this->view->cacheTags = new StdClass();
                $this->view->cacheTags->tags = array();
                $output = $this->fetchTemplate('resources_comment_index.phtml');
                $tags = $this->view->cacheTags->tags;
                $tags[] = 'comments';
                $cache->save($output, $cacheId, $tags);
            }
            $this->autoresponder->setOutput($output);
        } else {
            $this->autoresponder->setError('Response not found');
        }
    }

    /**
     * Adds new comment to specific response
     *
     * @return mixed
     * @throws Singular_Acl_Exception
     */
    public function addAction()
    {
        $redirect = new Zend_Session_Namespace('Redirect');
        if ($redirect->url == $this->view->url()) {
            $redirect->unsetAll();
        }
        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            /** @var System_Model_Response $activeResponse */
            $activeResponse = null;
            if (isset($post['response_id']) && null != $post['response_id']) {
                $activeResponse = Singular_Loader::get('System_Model_DbTable_Responses')
                    ->findById($post['response_id']);
            }

            if ($activeResponse instanceof System_Model_Response) {
                Singular_Runtime::store('activeResponse', $activeResponse);
                $form = new Resources_Form_CommentAdd();
                if ($activeResponse->canCommented()) {
                    if ($form->isValid($post)) {
                        $data = $form->getValues();

                        $statusReport = new stdClass();
                        $statusReport->isChanged = false;
                        $statusReport->newStatus = null;
                        if (isset($data['response_status']) && (bool)$data['response_status']) {
                            if (false !== $activeResponse->switchStatus($data['response_status'])) {
                                $statusReport->isChanged = true;
                                $statusReport->newStatus = true;
                            }
                        }
                        Singular_Runtime::store('statusReport', $statusReport);

                        if ($this->_activeComment->save($data)) {
                            if ($post['vote'] != 0) {
                                if ($this->isAjax()) {
                                    $this->view->negativeVotes = $activeResponse->getVotesSum()->negative;
                                    $this->view->positiveVotes = $activeResponse->getVotesSum()->positive;
                                }
                            }
                            $this->view->comment = $this->_activeComment;

                            /* output */
                            if ($this->isAjax()) {
                                $this->view->participants = $activeResponse->getParticipantsSum();
                                $output = $this->fetchTemplate('resources_comment_add.phtml');
                                $this->autoresponder->setOutput($output);
                                $this->autoresponder->setSuccess('Comment is added!');
                            } else {
                                $this->view->notify()->success('Comment is added!');
                                $this->_redirect($activeResponse->getUrl());
                            }
                            Singular_Event::dispatch("commentAdded", $this->_activeComment);
                            return;
                        } else {
                            $message = 'Db error!';
                        }
                        $redirectUrl = $activeResponse->getUrl();
                    } else {
                        if ($this->isAjax()) {
                            $this->_setFormErrorToAutoresponder($form);
                            return;
                        }
                        $message = "Error";
                        $redirectUrl = $activeResponse->getUrl();
                    }
                } else {
                    $message = 'Please authorize to add comment!';
                    $redirectUrl = $this->view->url(array(), 'login') . '?redirect=' . $activeResponse->getUrl();
                }
            } else {
                $message = "Response not found!";
                $redirectUrl = $this->getFrontController()->getBaseUrl();
            }
            if ($this->isAjax()) {
                $this->autoresponder->setError($message);
            } else {
                $this->view->notify()->error($message);
                $this->_redirect($redirectUrl);
            }
        } else {
            $this->view->notify()->error("Bab request");
            $this->_redirect($this->getFrontController()->getBaseUrl());
        }
    }

    /**
     * Edits allready existing comment for specific response
     *
     * @throws Singular_Acl_Exception
     * @return void
     */
    public function editAction()
    {
        if (!$this->_activeComment->canEdit()) {
             throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            $activeResponse = Singular_Loader::get('System_Model_DbTable_Responses')
                                ->findById($post['response_id']);
            if ($activeResponse instanceof System_Model_Response) {
                Singular_Runtime::store('activeResponse', $activeResponse);
                $form = new Resources_Form_CommentEdit();
                if ($form->isValid($post)) {
                    $data = $form->getValues();
                    if ($this->_activeComment->canEdit()) {
                        if ($this->_activeComment->save($data)) {
                            $this->view->comment = $this->_activeComment;
                            $this->view->activeResponse = $activeResponse;
                            $this->view->isBest = $this->_activeComment->is_best;
                            $content = $this->fetchTemplate("_comment.phtml");
                            $this->autoresponder->setOutput($content);
                            $this->autoresponder->setSuccess('Comment is updated!');
                        } else {
                            $this->autoresponder->setError('Db Error!');
                        }
                    } else {
                        $this->autoresponder->setError('Please authorize to manage comment!');
                    }
                } else {
                    $this->_setFormErrorToAutoresponder($form);
                }
            } else {
                $this->autoresponder->setError('Response not found');
            }
        }
    }

    /**
     * Adds "best" status to specified comment
     *
     * @throws Singular_Acl_Exception
     * @return void
     */
    public function bestAction()
    {
        if (!$this->_activeComment->canEdit()) {
             throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        $commentsDb = Singular_Loader::get("Resources_Model_DbTable_ResponsesComments");
        $comments = $commentsDb->findBestByResponseId($this->_activeComment->response_id);
        foreach ($comments as $comment) {
            $comment->unsetBest();
            $comment->save();
        }
        
        $this->_activeComment->setBest();
        $this->_activeComment->save();
        
        $this->view->notify()->success('Comment status is updated!');
        $this->_redirect($this->_activeComment->getResponse()->getUrl());
    }

    /**
     * Deletes specific comment
     *
     * @throws Singular_Acl_Exception
     * @return void
     */
    public function deleteAction()
    {
        if (!$this->_activeComment->canDelete()) {
             throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        $this->_activeComment->delete();
        $this->autoresponder->setSuccess('Comment is deleted!');
    }

    /**
     * Recoveries deleted comment from trash
     *
     * @throws Singular_Acl_Exception
     * @return void
     */
    public function recoveryAction()
    {
        if (!$this->_activeComment->canDelete()) {
             throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        $this->_activeComment->recovery();
        if ($this->isAjax()) {
            $this->autoresponder->setSuccess('Comment is restored!');
        } else {
            $this->view->notify()->success('Comment is restored!');
            $this->_redirect($this->_activeComment->getResponse()->getUrl());
        }
    }

    /**
     * Moves specific coment to trash
     *
     * @throws Singular_Acl_Exception
     * @return void
     */
    public function trashAction()
    {
        if (!$this->_activeComment->canDelete()) {
             throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        $this->_activeComment->trash();
        $this->autoresponder->setSuccess('Comment is moved to trash!');
    }

    /**
     * Adds vote to specific comment
     *
     * @return mixed
     */
    public function addVoteAction()
    {
        if ($this->loggedUser->canAddVote()) {
            /* check vote */
            if (!$this->_activeComment->isVoted($this->loggedUser->id)) {
                $vote = new Resources_Model_Vote();
                $vote->object    = $this->_activeComment->getObject();
                $vote->object_id = $this->_activeComment->id;
                $vote->value     = 1;
                $vote->save();
                if ($this->settings->system->general->anonymous_votes->value) {
                    $cookie = $this->getRequest()->getCookie('votes', '{}');
                    $cookie = Zend_Json_Decoder::decode($cookie);
                    $cookie['comment'][$this->_activeComment->id] = true;
                    setcookie("votes", Zend_Json_Encoder::encode($cookie), null, "/");
                }
                $message = "Your vote is accepted!";
                if ($this->isAjax()) {
                    $this->autoresponder->setSuccess($message);
                    return;
                } else {
                    $this->view->notify()->success($message);
                    $this->_redirect($this->_activeComment->getResponse()->getUrl());
                }
            } else {
                $message = "You have already voted for this comment!";
            }
        } else {
            $message = "You can't vote. Please authorize!";
        }
        if ($this->isAjax()) {
            $this->autoresponder->setError($message);
        } else {
            $this->view->notify()->error($message);
            $this->_redirect($this->_activeComment->getResponse()->getUrl());
        }
    }
}
