<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Comment form
 *
 * @uses       Singular_Form
 * @category   Resources
 * @package    Resources_Form
 */
class Resources_Form_Comment extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @return void
     */
    public function init()
    {
        $this->setDecorators(array(
            array('ViewScript', array('viewScript' => 'forms/system.phtml')),
            'Form',
        ));
        
        if (Singular_Core::_('theme')->getDefault() == 'mobile') {
            $this->setAttrib('data-ajax', 'false');
        }
        
        $this->addElement('hidden', 'response_id', array(
            'required' => true,
            'order' => 0,
            'decorators' => array(
                'ViewHelper'
            )
        ));

        $validator =  new System_Lib_Validate_StringLengthBbcode(array(
            'min' => 3,
            'max' => null,
        ));
        $validator->setMessages(array(
            Zend_Validate_StringLength::TOO_SHORT => 'Comment should be at least 3 characters!'
        ));
        
        $description = new System_Form_Element_Content('content', array(
            'required' => false,
            'withoutLabel' => true,
            'allowEmpty' => false,
            'filters' => array('StringTrim'),
            'validators' => array(
                $validator
            ),
            'order' => 1,
            'attribs' => array(
                'id' => 'comment_text',
                'cols' => "3",
                'rows' => "3"
            )
        ));
        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $description->setAttrib('data-role', 'none');
        }
        $this->addElement($description);
        
        if (Singular_Core::_("Settings")->getSetting("use_negative_votes")) {
            $response = Singular_Runtime::extract("activeResponse");
            if ($response instanceof System_Model_Response) {
                $type = Singular_Runtime::extract("types")
                    ->findBySlug($response->type);
                if ($type instanceof System_Model_ResponseType) {
                    if ($type->has_negative_vote) {
                        $this->addElement('checkbox', 'is_negative', array(
                            'attribs' => array(
                                'id' => $this->getAttrib("id") . '-is-negative',
                            ),
                            'label' => '<img src="' . $this->getView()->imageSrc('/negative_vote_na.png', null, false) . '" alt="I disagree with ' . $response->type . '" title="I disagree with ' . $response->type . '"/>',
                            'order' => 2,
                            'decorators' => array(
                                array('ViewScript', array('viewScript' => 'forms/elements/comment-checkbox.phtml')),
                                'ViewHelper'
                            )
                        ));
                        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
                            $this->is_negative->setAttrib('data-role', 'none');
                        }
                        $vote = Zend_Controller_Front::getInstance()->getRequest()->getParam("add-vote", 0);
                        if ($vote < 0) {
                            $this->is_negative->setValue("1");
                            $this->is_negative->setLabel('<img src="' . $this->getView()->imageSrc('/negative_vote.png', null, false) . '" alt="I disagree with ' . $response->type . '" title="I disagree with ' . $response->type . '"/>');
                        }
                    }
                }
            }
        }
        
        $this->addElement('submit', 'submit', array(
            'ignore' => true,
            'label' => 'Post Comment',
            'attribs' => array(
                'class' => 'button-type2 floatLeft'
            ),
            'order' => 5,
            'decorators' => array(
                array('ViewScript', array('viewScript' => 'forms/elements/button.phtml')),
                'ViewHelper'
            )
        ));

        $response = Singular_Runtime::extract('activeResponse');
        if ($response->isSupportStatuses()) {
            if ($response instanceof System_Model_Response && $response->canChangeStatus()) {
                $this->addElement(
                    new System_Form_Element_ResponseStatus('response_status',
                        array(
                             'order' => 6,
                        )
                    )
                );
            }
        }
    }
}
