<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * CommentAdd form
 *
 * @uses       Singular_Form
 * @category   Resources
 * @package    Resources_Form
 */
class Resources_Form_CommentAdd extends Resources_Form_Comment
{
    /**
     * Initialize form (used by extending classes)
     *
     * @return void
     */
    public function init()
    {
        $this->setAction($this->getView()->url(array(), 'comments_comment_add'));
        $this->setAttrib('id', 'comment-add');

        parent::init();

        $loggedUser = Singular_Runtime::extract('loggedUser');
        if (!($loggedUser->isAllowed('can_access_admin')
                        || $loggedUser->isAllowed('representative')
                        || $loggedUser->isAllowed('manage_responses'))
                && !$loggedUser->isGuest()) {
        
            $commentsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
            $loggedUser = Singular_Runtime::extract('loggedUser');
            $select     = $commentsDb->select();
            $select->where('created_by_id=?', $loggedUser->id)
                   ->where('created_at>?', date("Y-m-d H:i:s", time() - 60)); // -1 min

            $this->content->addValidator('Db_NoRecordExists', false, array(
                    $select,
                    'messages' => array(
                        Zend_Validate_Db_NoRecordExists::ERROR_RECORD_FOUND => 'You can add object once a minute. Please wait...'
                    )
                )
            );
        }
        
        $this->addElement('checkbox', 'notify_user_only', array(
            'attribs' => array(
                'class' => 'notify_user_only',
            ),
            'label' => '',
            'order' => 3,
            'decorators' => array(
                array('ViewScript', array('viewScript' => 'forms/elements/comment-checkbox.phtml')),
                'ViewHelper'
            )
        ));
        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $this->notify_user_only->setAttrib('data-role', 'none');
        }
        $vote = Zend_Controller_Front::getInstance()->getRequest()->getParam("add-vote", 0);

        $this->addElement('hidden', 'vote', array(
            'order'      => 9,
            'value'      => $vote,
            'decorators' => array(
                'ViewHelper'
            )
        ));
        
        $loggedUser = Singular_Runtime::extract('loggedUser');
    }
}
