<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Mailing form
 *
 * @uses       Singular_Form
 * @category   Resources
 * @package    Resources_Form
 */
class Resources_Form_Mailing extends System_Form_Settings
{
    /**
     * Initialize form (used by extending classes)
     *
     * @return void
     */
    public function init()
    {
        parent::init();

        $this->addDecorator(
            new Zend_Form_Decorator_ViewScript(array(
                'viewScript' => 'forms/mailing.phtml'
            ))
        );

        $element = $this->getElement('email_connection_type');
        if ($element !== null) {
            $element->setDisableTranslator(true);
            $element->setMultiOptions(array(
                'sendmail'  => 'Native',
                'smtp'      => 'SMTP',
            ));
            $element->setDisableTranslator(false);
            $element->setAttrib('style', 'width: 208px;');
        }

        $element = $this->getElement('email_smtp_security');
        if ($element !== null) {
            $element->setDisableTranslator(true);
            $element->setLabel($this->getView()->t($element->getLabel()));
            $element->setMultiOptions(array(
                'off' => 'Off',
                'ssl' => 'SSL',
                'tls' => 'TLS'
            ));
        }
        
        $element = $this->getElement('email_smtp_password');
        if ($element !== null) {
            $element->setRequired(false);
        }

        $this->addElement('button', 'smtp_test', array(
            'ignore' => true,
            'label' => 'Test Connection',
            'attribs' => array(
                'class' => 'button-type1'
            ),
            'decorators' => array(
                array('ViewScript', array('viewScript' => 'forms/elements/button.phtml')),
                'ViewHelper'
            )
        ));

    }
    
    /**
     * Validate the form
     *
     * @param  array $data
     * @return boolean
     */
    public function isValid($data) {
        if (empty($data['email_smtp_authentication'])) {
            $this->getElement('email_smtp_password')->setRequired(false);
            $this->getElement('email_smtp_password')->setAllowEmpty(true);
            $this->getElement('email_smtp_username')->setRequired(false);
            $this->getElement('email_smtp_username')->setAllowEmpty(true);
        }
        return parent::isValid($data);
    }
}
