<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Handler
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Resources handler
 *
 * @uses       Singular_Event_Listener_Abstract
 * @category   Resources
 * @package    Resources_Handler
 */
class Resources_Handler_Events extends Singular_Event_Listener_Abstract
{
    /**
     * Binds aggregates on specified events
     *
     * @param Singular_Event_Dispatcher $dispatcher
     */
    public static function bindListeners ()
    {
        self::attachAggregate('cronFrequently',   array(__CLASS__, 'onCronFrequently'));
        self::attachAggregate('cronDaily',        array(__CLASS__, 'onCronDaily'));
        self::attachAggregate('postDispatchOnce', array(__CLASS__, 'onPostDispatchOnce'));
        self::attachAggregate('commentAdded',     array(__CLASS__, 'onCommentAdded'));
        self::attachAggregate('afterAnalyticsReportsInit',  array(__CLASS__, 'afterAnalyticsReportsInit'));
    }

    public static function onPostDispatchOnce($request)
    {
        self::getView()->headScript()->appendFile(self::getView()->javascriptSrc('votes.js', 'resources'));
    }
    
    /**
     * Subscribes to "cronDaily" event.
     *
     * @return void
     */
    public static function onCronDaily()
    {
        /* clear temp dir */
        $iterator = new DirectoryIterator(FILES_TEMP_PATH);
        foreach ($iterator as $item) {
            $time = time() - 24 * 60 * 60;
            if (!$item->isDot() && $item->isFile() && $item->getFileInfo()->getCTime() < $time) {
                unlink($item->getRealPath());
            }
        }


    }

    /**
     * Subscribes to "cronFrequently" event.
     *
     * @return void
     */
    public static function onCronFrequently()
    {
        $emailStack = Singular_Loader::get('Resources_Model_DbTable_EmailsStack');
        $mails = $emailStack->findCurrentMails();
        foreach($mails as $mail) {
            if ($mail->send()) {
                $mail->delete();
            }
        }
    }
    
    /**
     * Subscribes to "commentAdded" event.
     *
     * @param  Resources_Model_ResponseComment $comment
     * @return void
     */
    public static function onCommentAdded (Resources_Model_ResponseComment $comment)
    {
        // Getting active objects
        /** @var System_Model_User $loggedUser */
        $loggedUser       = Singular_Runtime::extract('loggedUser');
        /** @var System_Model_Response $response */
        $response         = Singular_Runtime::extract('activeResponse');
        $request          = Zend_Controller_Front::getInstance()->getRequest();
        $useNegativeVotes = self::getView()->settings->system->responses->use_negative_votes->value;
        $subscribers      = null;
        
        // If recieved 'vote' param
        $voteParam = $request->getParam('vote', 0);
        if ($voteParam != 0) {
            if ($response->isVoted()) {
                if ($useNegativeVotes) {
                    $vote = $response->getUserVote($loggedUser->id);
                    if ($voteParam > 0 && $vote->value < 0) {
                        $vote->value = 1;
                    } elseif ($voteParam < 0 && $vote->value > 0) {
                        $vote->value = -1;
                    }
                }
            } else {
                $vote            = new Resources_Model_Vote();
                $vote->object    = $response->getObject();
                $vote->object_id = $response->id;
                $vote->value     = -1;
            }
            $vote->save();
        }

        // Build subscribers and admins listing
        $admins = Singular_Loader::get('System_Model_DbTable_Users')
            ->findByPrivileges(
                array(
                     'object:can_access_admin',
                     'object:representative'
                )
            );
        $request = Zend_Controller_Front::getInstance()->getRequest();
        if ($request->getParam('notify_user_only')) {
            $userIds = $comment->getReplyUserIds();
            if (!empty($userIds)) {
                $subscribers = Singular_Loader::get('System_Model_DbTable_Users')
                    ->findByIds($userIds);
            }
        } else {
            $subscribers = Singular_Loader::get('System_Model_DbTable_ResponsesSubscribers')
                ->findUsersByResponse($response);            
        }
        
        if ($subscribers instanceof System_Model_Rowset && $subscribers->valid()) {
            $admins->addItems($subscribers);
        }

        // Notify subscribers and admins
        $community = self::getView()->settings->system->general->site_title->value;

        $responseStatus = '';
        if (Singular_Runtime::inStorage('statusReport')) {
            $statusReport = Singular_Runtime::extract('statusReport');
            if (isset($statusReport->isChanged) && $statusReport->isChanged) {
                $responseStatus = '[' . $response->getStatus() . '] ';
            }
        }

        Resources_Service_ApplicationMailer::send($admins, 'new_comment', array(
            'community_name'  => $community,
            'community_url'   => self::getView()->baseUrl(),
            'response_name'   => $response->title,
            'response_type'   => self::getView()->t($response->type),
            'response_url'    => $response->getUrl(),
            'response_body'   => self::getView()->bbcode($response->content),
            'response_status' => $responseStatus,
            'created_by_url'  => self::getView()->url(array('user_id' => $loggedUser->id), 'profile'),
            'created_by_name' => $loggedUser->getName(),
            'comment_url'     => $response->getUrl() . "#comment-{$comment->id}",
            'comment_body'    => self::getView()->bbcode($comment->content, false),
            'profile_url'     => self::getView()->url(array(), 'profile_settings_me')
        ));
            
        // Subscribe logged user
        $comments = Singular_Loader::get(
            'Resources_Model_DbTable_ResponsesComments'
        );
        $count = $comments->count(array(
            'created_by_id' => $loggedUser->id,
            'response_id'   => $response->id
        ));
        if ($count <= 1) {
            if (!$response->isSubscribed($loggedUser->id)) {
                $response->subscribe($loggedUser->id);
            }
        }
    }
    
    /**
     * Subscribes to "afterAnalyticsReportsInit" event.
     *
     * @param  stdClass $reportsConf
     * @return void
     */
    public static function afterAnalyticsReportsInit ($reportsConf) 
    {
        $reportsConf->reports[] = array(
            'id' => 'resources_comments',
            'title' => self::getView()->t('Comments'),
            'model' => 'Resources_Model_DbTable_ResponsesComments',
            'counter' => 'countByType',
            'order' => '60',
            'options' => array(
                'row_type' => 'default',
                'response_types_assoc' => true
            ),
        );
        
        $reportsConf->reports[] = array(
            'id' => 'resources_votes',
            'title' => self::getView()->t('Votes'),
            'model' => 'Resources_Model_DbTable_Votes',
            'counter' => 'count',
            'order' => '100',
            'options' => array(
                'row_type' => 'total',
            ),
        );
    }      
}
