<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_View
 * @subpackage Resources_View_Helper
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * CommentActions view helper
 *
 * @uses       Zend_View_Helper_Abstract
 * @category   Resources
 * @package    Resources_View
 * @subpackage Resources_View_Helper
 */
class Resources_View_Helper_CommentActions extends Zend_View_Helper_Abstract
{
    /**
     * Renders comment actions links
     *
     * @param  Resources_Model_ResponseComment $comment
     * @return array
     */
    public function commentActions($comment)
    {
        $loggedUser = Singular_Runtime::extract("loggedUser");
        $actions = array();
        $response = Singular_Runtime::extract('activeResponse');
        if ($response instanceof System_Model_Response
                && !$response->lock_comments) {
            $actions['reply'] = array(
                'class' => 'comment-reply',
                'href'  => '#',
                'label' => 'Reply'
            );
        }
        $actions['good'] = array(
            'class' => "good-comment",
            'href'  => $this->view->url(array('comment_id' => $comment->id),
                                    'comments_add_vote'),
            'label' => 'Good Answer'
        );
        if ($comment->canEdit()) {
            $actions['edit'] = array(
                'class' => 'edit-comment',
                'href'  => '#',
                'label' => 'Edit'
            );
        }
        if ($comment->canDelete()) {
            $actions['trash'] = array(
                'class' => 'delete-comment padding-right',
                'href'  => $this->view->url(array('comment_id' => $comment->id),
                                            'comment_trash'),
                'label' => 'Move to Trash'
            );
        }
        if ($comment->canDelete()) {
            $actions['best'] = array(
                'class' => 'comments-best padding-right',
                'href'  => $this->view->url(array('comment_id'=>$comment->id),
                                        'comments_comment_best'),
                'label' => 'Best Answer'
            );
        }
        $params = new StdClass();
        $params->actions = $actions;
        Singular_Event::dispatch("commentActions", $params);

        return $params->actions;
    }
}
