<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Model
 * @subpackage Resources_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * email_templates table model.
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   Resources
 * @package    Resources_Model
 * @subpackage Resources_Model_DbTable
 */
class Resources_Model_DbTable_Templates extends System_Model_DbTable_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'email_templates';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'Resources_Model_Template';

    /**
     * Rowset class
     *
     * @var string
     */
    protected $_rowsetClass = 'Resources_Model_Templates';

    /**
     * Dependent Tables
     *
     * @var array
     */
    protected $_dependentTables = array(
        'Resources_Model_DbTable_TemplatesLang'
    );

    /**
     * Returns template by slug name
     *
     * @param  string $slug
     * @return Resources_Model_Template
     */
    public function findBySlug($slug)
    {
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.slug=?", $slug);
        return $this->fetchRow($select);
    }

    /**
     * Returns template by module name
     *
     * @param  int $moduleId
     * @return Resources_Model_Templates
     */
    public function findByModuleId($moduleId)
    {
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.module_id=?", $moduleId);
        return $this->fetchAll($select);
    }

    /**
     * Returns template by id and language
     * (id=? AND lang=?)
     *
     * @param  int $id
     * @param  string $lang
     * @return Resources_Model_Template
     */
    public function findByIdAndLang($id, $lang)
    {
        $select = $this->getSelect();
        $select->where("id=?", $id);
        $select->where("lang=?", $lang);
        return $this->fetchRow($select);
    }

    /**
     * Returns all available templates
     *
     * @return Resources_Model_Templates
     */
    public function findAll()
    {
        $select = $this->getSelect();
        return $this->fetchAll($select);
    }
}
