<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Model
 * @subpackage Resources_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * email_templates_lang table model.
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   Resources
 * @package    Resources_Model
 * @subpackage Resources_Model_DbTable
 */
class Resources_Model_DbTable_TemplatesLang extends System_Model_DbTable_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'email_templates_lang';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'Resources_Model_TemplateLang';

    /**
     * Reference map
     *
     * @var array
     */
    protected $_referenceMap    = array(
        'Template' => array(
            'columns'           => array('template_id'),
            'refTableClass'     => 'Resources_Model_DbTable_Templates',
            'refColumns'        => array('id'),
            'onDelete'          => self::CASCADE,
        )
    );

    /**
     * Returns all available template languages by template_id and lang
     * (template_id=? AND lang=?)
     *
     * @param  int $id
     * @param  string $lang
     * @return Resources_Model_TemplateLang
     */
    public function findByIdAndLang($id, $lang)
    {
        /** @var $select Zend_Db_Table_Select */
        $select = $this->getSelect();
        $select->where("template_id=?", $id)->where("lang=?", $lang);

        return $this->fetchRow($select);
    }
}
