<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * response_comments row model
 *
 * @uses       System_Model_Row
 * @category   Resources
 * @package    Resources_Model
 */
class Resources_Model_Template extends System_Model_Row
{
    /**
     * Table class
     *
     * @var string
     */
    protected $_tableClass = 'Resources_Model_DbTable_Templates';

    /**
     * Default site language
     *
     * @var string
     */
    public $lang = null;

    /**
     * Tamplate language row
     *
     * @var Resources_Model_TemplateLang
     */
    public $langRow = null;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();
        $this->lang = Singular_Core::_('settings')->getSetting('site_default_language');
    }

    /**
     * Returns module row
     *
     * @return System_Model_Module
     */
    public function getModule()
    {
        if ($this->getProperty('module') === null) {
            $moduleDb = Singular_Loader::get('System_Model_DbTable_Modules');
            $module = $moduleDb->find($this->module_id)->current();
            $this->setProperty('module', $module);
        }

        return $this->getProperty('module');
    }

    /**
     * Returns template variables
     *
     * @return array
     */
    public function getVariables()
    {
        return Zend_Serializer::unserialize($this->variables);
    }

    /**
     * Returns template language row
     *
     * @param  null|string $lang
     * @return null|Resources_Model_TemplateLang
     */
    public function getLangRow($lang = null)
    {
        if (empty($lang)) {
            $lang = $this->lang;
        }
        if ($this->langRow == null) {
            $templatesLangDb = Singular_Loader::get('Resources_Model_DbTable_TemplatesLang');
            if (!$this->isNew()) {
                $this->langRow = $templatesLangDb->findByIdAndLang($this->id, $lang);
            }
            if (!($this->langRow instanceof Resources_Model_TemplateLang)) {
                $this->langRow = new Resources_Model_TemplateLang();
                $this->langRow->lang = $lang;
                $this->langRow->template_id = $this->id;
                $langRowDefault = null;
                if (!$this->isNew()) {
                    $langRowDefault = $templatesLangDb->findByIdAndLang($this->id, 'en');
                }
                if ($langRowDefault instanceof Resources_Model_TemplateLang) {
                    $this->langRow->subject = $langRowDefault->subject;
                    $this->langRow->body = $langRowDefault->body;
                }
            }
        }
        return $this->langRow;
    }

    /**
     * Returns template body
     *
     * @return text
     */
    public function getBody()
    {
        return $this->getLangRow()->body;
    }

    /**
     * Returns template subject
     *
     * @return text
     */
    public function getSubject()
    {
        return $this->getLangRow()->subject;
    }

    /**
     * Populates row with data
     *
     * @param array $data
     */
    public function setFromArray(array $data)
    {
        if ($data['subject']) {
            $this->getLangRow()->subject = $data['subject'];
        }
        if ($data['body']) {
            $this->getLangRow()->body = $data['body'];
        }
        parent::setFromArray($data);
    }

    /**
     * saves the properties to the database.
     *
     * This performs an intelligent insert/update, and reloads the
     * properties with fresh data from the table on success.
     *
     * @param  array|null Data to be inserted/updated
     * @return mixed      The primary key value(s), as an associative array if the
     *                    key is compound, or a scalar if the key is single-column.
     */
    public function save($data = null)
    {
        if (parent::save($data)) {
            $this->getLangRow()->template_id = $this->id;
            $this->getLangRow()->save();
        }
    }
}
