<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Resources
 * @package    Resources_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * votes row model
 *
 * @uses       System_Model_Row
 * @category   Resources
 * @package    Resources_Model
 */
class Resources_Model_Vote extends System_Model_Row
{
    /**
     * Table class
     *
     * @var string
     */
    protected $_tableClass = 'Resources_Model_DbTable_Votes';
    
    /**
     * Allows post-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_postInsert()
     * @return void
     */
    public function _postInsert()
    {
        parent::_postInsert();
        
        $this->_cleanDefaultCache();
    }
    
    /**
     * Allows post-update logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_postUpdate()
     * @return void
     */
    public function _postUpdate()
    {
        parent::_postUpdate();
        
        $this->_cleanDefaultCache();
    }

    /**
     * Allows post-delete logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_postDelete()
     * @return void
     */
    public function _postDelete()
    {
        parent::_postDelete();
        
        $this->_cleanDefaultCache();
    }
    
    /**
     * Clean default cache
     *
     * @return void
     */
    protected function _cleanDefaultCache()
    {
        $cacheTags = array('votes');
        $types = Singular_Runtime::extract('types');
        $allowTypes = array();
        foreach ($types as $type) {
            $allowTypes[] = $type->slug;
        }
        $types->rewind();
        if (in_array($this->object, $allowTypes)) {
            $activeResponse = Singular_Runtime::extract('activeResponse');
            if ($activeResponse instanceof System_Model_Response) {
                $cacheTags[] = 'responses';
            }
        }
        if ($this->object == 'comment') {
            $activeComment = Singular_Runtime::extract('activeComment');
            if ($activeComment instanceof Resources_Model_ResponseComment) {
                $cacheTags[] = $activeComment->getResponse()->type . '_' . $activeComment->getResponse()->id;
            }
        }
        $cache = Singular_Runtime::extract('cachemanager')->getCache('default');
        $cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, $cacheTags);
        
        if (class_exists('Supercache', false)) {
            SuperCache::getInstance()->getCache()->clean(Zend_Cache::CLEANING_MODE_ALL);
        }
    }
}
