<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Module
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * System module identity class
 *
 * @uses       Singular_Module_Abstract
 * @category   System
 * @package    System_Module
 */
class System_Module extends Singular_Module_Abstract
{
    /**
     * New version value
     *
     * @var null|float|int
     */
    private static $_newVersion;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @return void
     */
    public function init()
    {
      $this->_info();
      $this->_settings();
    }

    /**
     * Module installation method.
     *
     * @return void
     */
    public function install ()
    {
        $db = Singular_Db_Table_Abstract::getDefaultAdapter();
        $tablePrefix = Singular_Runtime::extract("config")->resources->db->params->prefix;
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}activity_logs` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `object` varchar(40) NOT NULL,
              `object_id` int(11) NOT NULL,
              `action` varchar(40) NOT NULL,
              `created_at` datetime NOT NULL,
              `created_by_id` int(11) NOT NULL,
              `created_by_name` varchar(255) NOT NULL,
              `created_by_email` varchar(255) NOT NULL,
              `title` varchar(255) NOT NULL,
              `comment` text CHARACTER SET utf8 COLLATE utf8_unicode_ci NOT NULL,
              PRIMARY KEY (`id`),
              KEY `created_at` (`created_at`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}languages` (
              `id` int(3) unsigned NOT NULL AUTO_INCREMENT,
              `code` varchar(7) NOT NULL,
              `image` varchar(38) NULL,
              `name` varchar(64) NOT NULL,
              `active` int(1) unsigned NOT NULL DEFAULT '1',
              `is_system` tinyint(1) NOT NULL DEFAULT '0',
              PRIMARY KEY (`id`),
              UNIQUE KEY `code` (`code`)
            ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 AUTO_INCREMENT=3 ;
        ";
        $db->exec($sql);
        
        $sql = "
            INSERT INTO `{$tablePrefix}languages` (`id`, `code`, `image`, `name`, `active`, `is_system`) VALUES
            (1, 'en', '', 'English', 1, 1),
            (2, 'ru', 'ru.png', 'Russian', 1, 0);
        ";
        $db->exec($sql);
        
        $sql = "
             CREATE TABLE IF NOT EXISTS `{$tablePrefix}responses` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `title` varchar(150) NOT NULL,
              `slug` varchar(155) NOT NULL,
              `content` longtext NOT NULL,
              `search_index` text NOT NULL,
              `tags` varchar(255) NULL,
              `type` varchar(40) DEFAULT NULL,
              `status` varchar(40) NOT NULL,
              `created_by_id` int(11) NOT NULL DEFAULT '0',
              `created_at` datetime NOT NULL,
              `updated_at` datetime DEFAULT '0000-00-00 00:00:00',
              `updated_by_id` int(11) NOT NULL DEFAULT '0',
              `deleted_by_id` int(11) NOT NULL DEFAULT '0',
              `deleted_at` datetime DEFAULT NULL,
              `lock_comments` tinyint(1) NOT NULL DEFAULT '0',
              `views` BIGINT NOT NULL DEFAULT '0',
              `order` SMALLINT NOT NULL DEFAULT '1',
              PRIMARY KEY (`id`),
              UNIQUE KEY `slug` (`slug`),
              KEY `type` (`type`,`status`),
              KEY `created_by_id` (`created_by_id`),
              KEY `updated_by_id` (`updated_by_id`),
              FULLTEXT KEY `title` (`title`,`content`,`search_index`,`tags`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}response_answers` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `response_id` int(11) NOT NULL,
              `answer` text NOT NULL,
              `created_by_id` int(11) NOT NULL,
              `created_at` datetime NOT NULL,
              `updated_by_id` int(11) DEFAULT '0',
              `updated_at` datetime DEFAULT NULL,
              `deleted_by_id` int(11) DEFAULT '0',
              `deleted_at` datetime DEFAULT NULL,
              PRIMARY KEY (`id`),
              KEY `response_id` (`response_id`),
              FULLTEXT KEY `answer` (`answer`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}response_statuses` (
              `title` varchar(255) NOT NULL,
              `slug` varchar(255) NOT NULL,
              `is_default` tinyint(1) NOT NULL DEFAULT '0',
              `is_protected` TINYINT(1) DEFAULT '1' NOT NULL,
              `type` varchar(40) NOT NULL,
              `pay_attention` tinyint(1) NOT NULL DEFAULT '1',
              `color` varchar(7) NOT NULL,
              `text_color` varchar(7) DEFAULT '#ffffff' NOT NULL,
              `order` smallint(5) unsigned NOT NULL DEFAULT '0',
              PRIMARY KEY (`slug`),
              KEY `type` (`type`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8;
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}response_types` (
              `slug` varchar(40) NOT NULL,
              `title` varchar(255) NOT NULL,
              `objects` varchar(255) NOT NULL,
              `icon` varchar(255) NOT NULL,
              `module_id` int(11) NOT NULL,
              `has_comments` tinyint(1) NOT NULL,
              `has_negative_vote` tinyint(1) NOT NULL DEFAULT '1',
              `has_votes` tinyint(1) UNSIGNED NOT NULL DEFAULT '1',
              `is_active` tinyint(1) NOT NULL,
              `is_default` tinyint(1) NOT NULL,
              `order` smallint(5) unsigned NOT NULL DEFAULT '0',
              PRIMARY KEY (`slug`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8;
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}subscribers` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `response_id` int(11) NOT NULL,
              `user_id` int(11) NOT NULL,
              PRIMARY KEY (`id`),
              KEY `response_id` (`response_id`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}user_configs` (
              `id` int(11) NOT NULL AUTO_INCREMENT,
              `user_id` int(11) NOT NULL,
              `config_key` varchar(255) NOT NULL,
              `value` varchar(255) NOT NULL,
              PRIMARY KEY (`id`),
              KEY `user_id` (`user_id`,`config_key`)
            ) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}roles` (
              `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
              `role` varchar(100) NOT NULL,
              `parent` varchar(255) DEFAULT NULL,
              `name` varchar(255) NOT NULL DEFAULT '',
              `is_group` tinyint(1) NOT NULL DEFAULT '0',
              `is_protected` tinyint(1) NOT NULL DEFAULT '0',
              `is_active` tinyint(1) NOT NULL DEFAULT '1',
              PRIMARY KEY (`id`),
              UNIQUE KEY `role` (`role`)
            ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 CHECKSUM=1 DELAY_KEY_WRITE=1 ROW_FORMAT=DYNAMIC AUTO_INCREMENT=5 ;
        ";
        $db->exec($sql);
        
        $sql = "
            INSERT INTO `{$tablePrefix}roles` (`id`, `role`, `parent`, `name`, `is_group`, `is_protected`, `is_active`) VALUES
            (1, 'administrator', NULL, 'Admin', 0, 1, 1),
            (2, 'official-representative', NULL, 'Official Representative', 0, 1, 1),
            (3, 'user', NULL, 'User', 0, 1, 1),
            (4, 'guest', NULL, 'Guest', 0, 1, 1);
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}resources` (
              `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
              `type` enum('mvc','object','route') NOT NULL,
              `resource` varchar(255) DEFAULT NULL,
              `parent` varchar(255) DEFAULT NULL,
              `name` varchar(255) DEFAULT NULL,
              `is_protected` tinyint(1) NOT NULL DEFAULT '0',
              PRIMARY KEY (`id`)
            ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 CHECKSUM=1 DELAY_KEY_WRITE=1 ROW_FORMAT=DYNAMIC AUTO_INCREMENT=6 ;
        ";
        $db->exec($sql);
        
        $sql = "
            INSERT INTO `{$tablePrefix}resources` (`id`, `type`, `resource`, `parent`, `name`, `is_protected`) VALUES
            (1, 'object', 'object:can_access_admin', NULL, 'Admin Access', 1),
            (2, 'object', 'object:representative', NULL, 'Official Representative', 1),
            (3, 'object', 'object:manage_people', NULL, 'Manage People', 1),
            (4, 'object', 'object:manage_responses', NULL, 'Manage Responses', 1),
            (5, 'object', 'object:can_use_active_log', NULL, 'Use Activity Log', 1),
            (6, 'object', 'object:can_see_users', NULL, 'See Users', 1);
        ";
        $db->exec($sql);
        
        $sql = "
            CREATE TABLE IF NOT EXISTS `{$tablePrefix}privileges` (
              `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
              `type` enum('allow','deny') NOT NULL,
              `roles` varchar(255) DEFAULT NULL,
              `resources` varchar(255) DEFAULT NULL,
              `privileges` varchar(255) DEFAULT NULL,
              `assert` varchar(200) DEFAULT NULL,
              `is_active` tinyint(1) NOT NULL DEFAULT '1',
              `is_protected` tinyint(1) NOT NULL DEFAULT '0',
              PRIMARY KEY (`id`)
            ) ENGINE=MyISAM  DEFAULT CHARSET=utf8 CHECKSUM=1 DELAY_KEY_WRITE=1 ROW_FORMAT=DYNAMIC AUTO_INCREMENT=9 ;
        ";
        $db->exec($sql);
        
        $sql = "
            INSERT INTO `{$tablePrefix}privileges` (`id`, `type`, `roles`, `resources`, `privileges`, `assert`, `is_active`, `is_protected`) VALUES
            (1, 'allow', 'administrator', 'object:can_access_admin', NULL, NULL, 1, 1),
            (2, 'allow', 'official-representative', 'object:representative', NULL, NULL, 1, 0),
            (3, 'allow', 'administrator', 'object:representative', NULL, NULL, 1, 0),
            (4, 'allow', 'official-representative', 'object:manage_responses', NULL, NULL, 1, 0),
            (5, 'allow', 'administrator', 'object:manage_people', NULL, NULL, 1, 0),
            (6, 'allow', 'administrator', 'object:manage_responses', NULL, NULL, 1, 0),
            (7, 'allow', 'administrator', 'object:can_use_active_log', NULL, NULL, 1, 0),
            (8, 'allow', 'official-representative', 'object:can_use_active_log', NULL, NULL, 1, 0),
            (9, 'allow', 'guest', 'object:can_see_users', NULL, NULL, 1, 0),
            (10, 'allow', 'user', 'object:can_see_users', NULL, NULL, 1, 0),
            (11, 'allow', 'official-representative', 'object:can_see_users', NULL, NULL, 1, 0),
            (12, 'allow', 'administrator', 'object:can_see_users', NULL, NULL, 1, 0);
        ";
        $db->exec($sql);
        
        $this->_addSettings();

        $tempDir = ROOT_PATH . '/public/files/temp';
        if (!file_exists($tempDir)) mkdir($tempDir, 0777, true);
        
        Singular_Core::_("settings")->changeSetting('system_version', $this->getVersion());
    }

    public function hasNewVersion()
    {

    }

    public function getNewVersion()
    {
        if (null == self::$_newVersion) {
            $this->hasNewVersion();
        }

        return self::$_newVersion;
    }
    
    /**
     * Set module information to be available in code and to display on Modules
     * list at backend.
     *
     * @return void
     */
    protected function _info ()
    {
        $this->getContainer()->setName('System')
              ->setDescription('System files and SingularCore powered by Zend Framework')
              ->setVersion('2.3.9')
              ->addDependency('resources', '2.2')
              ->setAuthor('USWebStyle, Inc')
              ->setEmail('support@uswebstyle.com')
              ->setUrl('http://www.uswebstyle.com')
              ->setReleaseDate('2013-10-23');
    }

    /**
     * Define module settings pages to be accessible from backend.
     *
     * @return void
     */
    protected function _settings()
    {
        $front = Zend_Controller_Front::getInstance();
        $this->settingUri = $front->getBaseUrl() . '/admin/settings';
    }
    
    /**
     * Define module settings.
     *
     * @return void
     */
    private function _addSettings()
    {
        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'analytics_header_script',
            'value'       => '',
            'type'        => 'textarea',
            'title'       => 'Header Script (Code)',
            'description' => 'Paste external tracking code that should go before closing &#8249;/head&#8250;',
            'module'      => 'system',
            'section'     => 'analytics'
        ));
        Singular_Core::_("settings")->addSettings(array(
            'key'         => 'analytics_body_script',
            'value'       => '',
            'type'        => 'textarea',
            'title'       => 'Body Script (Code)',
            'description' => 'Paste external tracking code that should go before closing &#8249;/body&#8250;',
            'module'      => 'system',
            'section'     => 'analytics'
        ));
    }
}
