<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Admin-interface controller
 *
 * @uses       System_Controller_Abstract_Admin
 * @category   System
 * @package    System_Controller
 */
class AdminInterfaceController extends System_Controller_Abstract_Admin
{
    /**
     * Index action
     *
     * @return void
     */
    public function indexAction()
    {
        $form = new System_Form_Interface($this->settings->system->interface);
        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            if ($form->isValid($post)) {
                $data = $form->getValues();
                foreach($data as $key => $value){
                    if ($value != '') {
                        if ($key == 'site_default_theme') {
                            Singular_Core::_('settings')->changeSetting('site_default_admin_theme', $value);
                        }
                        Singular_Core::_('settings')->changeSetting($key, $value);
                    }
                }
                $this->view->notify()->success('Interface settings are changed!');
                $this->_helper->redirector->gotoRoute(array(), 'admin_interface');
            }
        }
        $this->view->form = $form;
    }

    /**
     * Widget action
     *
     * Generates graphical widget
     *
     * @return void
     */
    public function widgetAction ()
    {
        if ($this->getRequest()->isPost()) {
            $this->_processForm();
        }

        $this->_doCommonOperations();

        // Widget stack priority object
        $widgetStack = new SplPriorityQueue();

        $form = new System_Form_WidgetFeedback($this->settings->system->widget);
        $this->view->form = $form;
        $widgetStack->insert(
            array(
                 'alias'   => 'feedback',
                 'title'   => $this->view->t('Feedback Tab'),
                 'content' => $this->fetchTemplate('feedback_widget.phtml')
            ), 100
        );

        $form = new System_Form_WidgetSearch($this->settings->system->widget);
        $this->view->form = $form;
        $widgetStack->insert(
            array(
                 'alias'   => 'search',
                 'title'   => $this->view->t('Search Tab'),
                 'content' => $this->fetchTemplate('search_widget.phtml')
            ), 99
        );

        //$this->getView()->additionalModulePath('freepackage');
        $widgetStack->insert(
            array(
                'alias'   => 'chat',
                'title'   => $this->view->t('Chat'),
                'content' => $this->fetchTemplate('_freepackage_widget_chat.phtml')
            ), 98
        );
        $widgetStack->insert(
            array(
                'alias'   => 'knowledge_base',
                'title'   => $this->view->t('Knowledge base'),
                'content' => $this->fetchTemplate('_freepackage_widget_kb.phtml')
            ), 97
        );

        // Dispatch "widgetStack" Event
        Singular_Event::dispatch('widgetStack', $widgetStack);

        $this->view->widgets = $widgetStack;
    }

    protected function _processForm()
    {
        $post = $this->getRequest()->getPost();

        $widgetName = $this->getRequest()->getParam('widget_name');
        if (!$widgetName) return;

        $formClass = 'System_Form_Widget' . ucfirst($widgetName);
        $form = new $formClass($this->settings->system->widget);

        if ($form->isValid($post)) {
            $data = $form->getValues();
            $widgeTextOld = $this->settings->system->widget->widget_text->value;

            foreach ($data as $key => $value) {
                if ($key == 'widget_text' && $widgeTextOld != $value) {
                    $height = 30;
                    $factor = (ord($value[1]) > 127) ? 10 : 16;
                    $textWidth = $factor * strlen($value);
                    $width = $textWidth + 80;
                    $fontSize = 22;
                    $imgHandle = @imagecreatetruecolor($width, $height);

                    if ($imgHandle) {
                        $backColor = imagecolorallocate($imgHandle, 204, 204, 204);
                        imagefilltoborder($imgHandle, 0, 0, $backColor, $backColor);
                        imagecolortransparent($imgHandle, $backColor);
                        $txtColor = imagecolorallocate($imgHandle, 255, 255, 255);
                        $font = ROOT_PATH . DS . 'public' . DS . 'files' . DS . 'logo' . DS . 'trebucbd.ttf';
                        $positionCenter = ($width / 2 - $textWidth / 2);
                        imagettftext($imgHandle, $fontSize, 0, $positionCenter, 27, $txtColor, $font, $value);
                        $rotate = imagerotate($imgHandle, 90, 0);
                        $filename = ROOT_PATH . DS . 'public' . DS . 'files' . DS . 'logo' . DS . 'widget-text.png';
                        imagepng($rotate, $filename);
                        imagedestroy($imgHandle);
                        imagedestroy($rotate);
                    }
                }
                if (Singular_Core::_('Settings')->hasSetting($key)) {
                    Singular_Core::_('Settings')->changeSetting($key, $value);
                }
            }

            $this->view->notify()->success('Widget settings are updated!');
            $this->_redirect($this->_helper->url->url(array(), 'admin_widget'));
        }
    }

    protected function _doCommonOperations()
    {
        $widgetTextFile = ROOT_PATH . DS . 'public' . DS . 'files' . DS . 'logo' . DS . 'widget-text.png';
        $widgetTextUrl = $this->view->baseUrl('/public/files/logo/widget-text.png');

        if (!is_file($widgetTextFile)) {
            $widgetTextFile = ROOT_PATH . DS . 'public' . DS . 'files' . DS . 'logo' . DS . 'widget-text-default.png';
            $widgetTextUrl = $this->view->baseUrl('/public/files/logo/widget-text-default.png');
        }

        $this->view->widgetTextUrl = $widgetTextUrl;
        $info = getimagesize($widgetTextFile);
        $this->view->widgetMarginTop = $info[1] / 4;
        Singular_AppClient::setParam("widgetMarginTop", $this->view->widgetMarginTop);
        Singular_AppClient::setParam("widgetTitle",     $this->settings->system->general->site_title->value);
        Singular_AppClient::setParam("widgetTextUrl",   $widgetTextUrl);
        Singular_AppClient::setParam("defaultTheme",    Singular_Core::_('settings')->getSetting('site_default_theme'));
    }
}
