<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Admin-languages controller
 *
 * @uses       System_Controller_Abstract_Admin
 * @category   System
 * @package    System_Controller
 */
class AdminLanguagesController extends System_Controller_Abstract_Admin
{
    /**
     * Languages dbTable model
     *
     * @var System_Model_DbTable_Languages
     */
    public $languagesDb;

    /**
     * Fetched languages
     *
     * @var Zend_Db_Table_Rowset_Abstract
     */
    public $languages;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();

        $this->languagesDb = Singular_Loader::get('System_Model_DbTable_Languages');
        $this->view->languages = $this->languages = $this->languagesDb->getLanguages();

        $modulesDb = Singular_Loader::get('System_Model_DbTable_Modules');
        $this->modules = $this->view->modules = $modulesDb->findInstalledModules(true);
        $code = Singular_Core::_('settings')->getSetting('site_default_language');
        $this->view->defaultLanguage = $this->defaultLanguage = $this->languages->findByCode($code);

        $this->form = new System_Form_Language();
        $this->view->form = $this->form;
    }

    /**
     * Index action
     *
     * @return void
     */
    public function indexAction ()
    {
    }

    /**
     * Add action
     *
     * Adds new language
     *
     * @return void
     */
    public function addAction ()
    {
        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            if ($this->form->isValid($post)) {
                $language = $this->languagesDb->createRow();
                $values = $this->form->getValues();
                $values['active'] = 1;
                $language->setFromArray($values);
                $language->save();
                $this->view->notify()->success('Language is created');
                $this->_redirect();
            }
        }
    }

    /**
     * Edit action
     *
     * Updates specified language
     *
     * @return void
     */
    public function editAction()
    {
        $code = $this->_getParam('code');
        $language = $this->languagesDb->findByCode($code);
        if (!($language instanceof System_Model_Language)) {
            return $this->_redirect(null, array(), "Language not found", "error");
        }
        $form = new System_Form_Language();
        $validator = $form->code->getValidator('Db_NoRecordExists');
        $validator->setExclude("code<>'$language->code'");
        $form->setDefaults($language->toArray());
        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            if ($form->isValid($post)) {
                $values = $form->getValues();
                $language->save($values);
                return $this->_redirect(null, array(), "Language is updated", "success");
            }
        }
        $this->view->form = $form;
        if ($this->isAjax()) {
            $content = $this->fetchTemplate("system_admin-languages_edit.phtml");
            $this->autoresponder->setOutput($content);
        }
    }

    /**
     * Delete action
     *
     * Deletes specified language
     *
     * @return void
     */
    public function deleteAction()
    {
        $code = $this->_getParam('code');
        $language = $this->languagesDb->findByCode($code);
        if ($language instanceof System_Model_Language) {
            if ($this->defaultLanguage->id != $language->id && !$language->is_system) {
                $language->delete();
                return $this->_redirect(null, array(), "Language is deleted", "success");
            } else {
                return $this->_redirect(null, array(), "Language can't be deleted!", "error");
            }
        } else {
            return $this->_redirect(null, array(), "Error", "error");
        }
        $this->_redirect();
    }

    /**
     * Change action
     *
     * Change language by modifying "site_default_language" setting
     *
     * @return void
     */
    public function changeAction()
    {
        if ($this->getRequest()->isPost()) {
            $code = $this->_getParam('language');
            $language = $this->languagesDb->findByCode($code);
            if ($language instanceof System_Model_Language) {
                Singular_Core::_('settings')->changeSetting('site_default_language', $code);
                Singular_Translate::clearCache();
                Singular_Runtime::extract('cachemanager')
                        ->getCache('default')
                        ->clean(Zend_Cache::CLEANING_MODE_ALL);
                $this->view->notify()->success('Default language is changed');
                $this->_helper->redirector->gotoRoute(array(
                    'lang' => $code
                ), 'admin_languages');
            }
        }
        $this->view->notify()->error('Error');
        $this->_redirect();
    }


    /**
     * Add-dictionary action
     *
     * Adds new language dictionary for specified module
     *
     * @return void
     */
    public function addDictionaryAction()
    {
        $this->view->layout()->setLayout('translation');

        $langCode = $this->getRequest()->getParam('code');
        $moduleName = $this->getRequest()->getParam('module-name');        
        $module = $this->modules->findByName($moduleName);
        
        if (!$module->canAddTranslation($langCode)) {
            $this->view->notify()->error("Dictionary can't be added!");
            $this->_redirect();
        }
        if ($this->getRequest()->isPost()) {
            $translation = $this->getRequest()->getParam('translation');
            $module->addTranslation($translation, $langCode);
            Singular_Runtime::extract('cachemanager')
                    ->getCache('default')
                    ->clean(Zend_Cache::CLEANING_MODE_ALL);
            $this->view->notify()->success("Dictionary is added");
            Singular_Translate::clearCache();
            $this->_redirect();
        }

        $this->view->langKey = $module->getLangKey();
        $this->view->translation = $module->getTranslation($langCode);
    }

    /**
     * Edit-dictionary action
     *
     * Updates language dictionary for specified module
     *
     * @return void
     */
    public function editDictionaryAction()
    {
        $this->view->layout()->setLayout('translation');
        $langCode = $this->_getParam('code');
        $moduleName = $this->_getParam('module-name');
        $module = $this->modules->findByName($moduleName);
        
        if (!$module->canEditTranslation($langCode)) {
            $this->view->notify()->error("Dictionary can't be updated!");
            $this->_redirect();
        }
        if ($this->getRequest()->isPost()) {
            $translation = $this->getRequest()->getParam('translation');
            $module->addTranslation($translation, $langCode);
            Singular_Translate::clearCache();
            Singular_Runtime::extract('cachemanager')
                    ->getCache('default')
                    ->clean(Zend_Cache::CLEANING_MODE_ALL);
            $this->view->notify()->success("Dictionary is updated");
            $this->_redirect();
        }

        $this->view->langKey = $module->getLangKey();
        $this->view->translation = $module->getTranslation($langCode);
    }

    /**
     * Export action
     *
     * Adds new language dictionary for specified module
     *
     * @return void
     */
    public function exportAction()
    {
        $langCode        = $this->_getParam('code');
        $currentLanguage = $this->languages->findByCode($langCode);
        if (!($currentLanguage instanceof System_Model_Language)) {
            $this->view->notify()->error('Language not found');
        }
        $this->_helper->layout()->disableLayout();
        $this->view->langCode        = $langCode;
        $this->view->currentLanguage = $currentLanguage;
        $this->getResponse()->setHeader("Content-Type", "application/xml; charset=utf-8");
        $filename = $currentLanguage->name . "(" . $currentLanguage->code .".UTF-8)";
        $this->getResponse()->setHeader("Content-Disposition", "attachment; filename=\"$filename\"");
    }

    /**
     * Inline redirect helper
     *
     * Redirects to given url and displays specified message
     *
     * @see    parent::redirect()
     * @param  null|string $url       Redirect url ("index" if null)
     * @param  array $options         Options
     * @param  null|string $message   Message that will be shown after redirect
     * @param  string $messageStatus  Message status (success/error)
     * @return void
     */
    protected function  _redirect($url = null, array $options = array(), $message = null, $messageStatus = 'success')
    {
        if (empty($url)) {
            $url = $this->_helper->url->url(array(), 'admin_languages');
        }
        parent::_redirect($url, $options, $message, $messageStatus);
    }
}
