<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Admin-modules controller
 *
 * @uses       System_Controller_Abstract_Admin
 * @category   System
 * @package    System_Controller
 */
class AdminModulesController extends System_Controller_Abstract_Admin
{
    /**
     * @var System_Model_DbTable_Modules
     */
    public $modulesDb = null;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();
        $this->modulesDb = new System_Model_DbTable_Modules();
    }

    /**
     * Index action.
     *
     * @return void
     */
    public function indexAction ()
    {
        $this->view->installedModules = Singular_Module::getInstance()
                ->getInstalledModules();
        $this->view->notInstalledModules = Singular_Module::getInstance()
                ->getNotInstalledModules();
    }

    /**
     * Version action.
     *
     * Returns latest available system module version if current version less than latest.
     * Otherwise returns null.
     * This action accepts on AJAX request!
     */
    public function versionAction()
    {
        if (!$this->isAjax()) die();
        $version = Singular_Module::getInstance()->getModule('system')->getNewVersion();
        $this->autoresponder->setParam('version', $version);
    }
    
    /**
     * Activation action.
     */
    public function activationAction()
    {
        if (!$this->isAjax()) {
            throw new Zend_Exception("Only XmlHttpRequest request");
        }
        $moduleName = $this->_getParam('name');

        $module = $this->modulesDb->findByName($moduleName);
        if ($module instanceof System_Model_Module) {
            $module->active = (int) $this->_getParam('value');

            if ($module->active) {
                Singular_Event::dispatch('moduleActivateBefore', $module);
            } else {
                Singular_Event::dispatch('moduleDeactivateBefore', $module);
            }

            $module->save();

            if ($module->active) {
                Singular_Event::dispatch('moduleActivate', $module);
            } else {
                Singular_Event::dispatch('moduleDeactivate', $module);
            }
            /* clean cache */
            Singular_Core::_('Cache')->cleanAll();

            if ($module->active) {
                $this->autoresponder->setSuccess("Module is successfully activated!");
            } else {
                $this->autoresponder->setSuccess("Module is successfully deactivated!");
            }
        } else {
            $this->autoresponder->setError("Module not found");
        }
    }
    
    /**
     * Install action
     *
     * Installs module by executing ******_Module::install()
     *
     * @return void
     */
    public function installAction()
    {
        $moduleName = $this->_getParam('module-name');
        $module     = $this->modulesDb->findByName($moduleName);

        if (!($module instanceof System_Model_Module)) {
            $module = $this->modulesDb->createRow();
            $module->setFromArray(array(
                'name'      => $moduleName,
                'active'    => 1,
                'is_system' => 0
            ));
            $moduleClass        = $module->getModuleClass();
            $module->load_order = $moduleClass->loadOrder;

            $handler = new stdClass;
            $handler->moduleRow   = $module;
            $handler->moduleClass = $moduleClass;
            Singular_Event::dispatch('moduleInstallBefore', $handler);

            if ($module->save()) {
                $moduleClass->install();
                Singular_Event::dispatch('moduleInstall', $handler);
                /* clean cache */
                Singular_Core::_('Cache')->cleanAll();

            } else {
                $this->view->notify()->error("Db Error!");
            }
            $this->view->notify()->success("Module is installed successfully!");
        } else {
            $this->view->notify()->error("Module is already installed");
        }
        $this->_redirect();
    }

    /**
     * UnInstall action
     *
     * Uninstalls module by executing ******_Module::uninstall()
     *
     * @return void
     */
    public function unInstallAction()
    {
        $moduleName = $this->_getParam('module-name');
        $module = $this->modulesDb->findByName($moduleName);
        if ($module instanceof System_Model_Module) {
            Singular_Event::dispatch('moduleUninstallBefore', $module);
            $module->getModuleClass()->unInstall();
            $module->delete();
            Singular_Event::dispatch('moduleUninstall', $module);
            /* clean cache */
            Singular_Core::_('Cache')->cleanAll();
            
            $this->view->notify()->success("Module is uninstalled!");
        } else {
            $this->view->notify()->error("Module not found");
        }
        $this->_redirect();
    }

    /**
     * Inline redirect helper
     *
     * Redirects to given url and displays specified message
     *
     * @see    parent::redirect()
     * @param  null|string $url       Redirect url ("index" if null)
     * @param  array $options         Options
     * @param  null|string $message   Message that will be shown after redirect
     * @param  string $messageStatus  Message status (success/error)
     * @return void
     */
    protected function  _redirect($url = null, array $options = array(), $message = null, $messageStatus = 'success')
    {
        if (empty($url)) {
            parent::_redirect($this->view->url(array(), 'admin_modules'));
        } else {
            parent::_redirect($url);
        }
    }
}
