<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Admin-settings controller
 *
 * @uses       System_Controller_Abstract_Admin
 * @category   System
 * @package    System_Controller
 */
class AdminSettingsController extends System_Controller_Abstract_Admin
{
    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();
    }

    /**
     * General action
     *
     * @return void
     */
    public function generalAction()
    {
        $form = new System_Form_GeneralSettings($this->settings->system->general);
        if($this->getRequest()->isPost()){
            $post = $this->getRequest()->getPost();
            if($form->isValid($post)){
                $data = $form->getValues();
                if (empty($data['site_logo'])) {
                    unset($data['site_logo']);
                }
                foreach($data as $key => $value){
                    // remove old logo
                    if ($key == 'site_logo') {
                        $filename = ROOT_PATH . '/public/files/logo/' . $this->settings->system->general->site_logo->value;
                        @unlink($filename);
                    }
                    Singular_Core::_('settings')->changeSetting($key, $value);
                }
                Singular_Runtime::extract("cachemanager")
                    ->getCache('default')
                    ->clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG, array('breadcrumbs'));
                $this->view->notify()->success('General settings are changed!');
                $this->_helper->redirector->gotoRoute(array(), 'admin_settings');
            }
        }
        $this->view->form = $form;
    }

    /**
     * Delete-logo action
     *
     * Deletes logo
     *
     * @return void
     */
    public function deleteLogoAction()
    {
        $filename = ROOT_PATH . '/public/files/logo/' . $this->settings->system->general->site_logo->value;
        @unlink($filename);
        Singular_Core::_('settings')->changeSetting('site_logo', '');
        $this->_helper->redirector->gotoRoute(array(), 'admin_settings');
    }

    /**
     * Scheduler action
     *
     * @return void
     */
    public function schedulerAction()
    {
        $this->view->code = Singular_Core::_("Settings")->getSetting('protect_cron');

        $frequently = new Zend_Date();
        $frequently->setTimestamp(Singular_Core::_("settings")->getSetting("last_frequently_activity"));
        $this->view->frequently = $frequently;

        $hourly = new Zend_Date();
        $hourly->setTimestamp(Singular_Core::_("settings")->getSetting("last_hourly_activity"));
        $this->view->hourly = $hourly;

        $daily = new Zend_Date();
        $daily->setTimestamp(Singular_Core::_("settings")->getSetting("last_daily_activity"));
        $this->view->daily = $daily;
    }
}
