<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Admin-trash controller
 *
 * @uses       System_Controller_Abstract_Admin
 * @category   System
 * @package    System_Controller
 */
class AdminTrashController extends System_Controller_Abstract_Admin
{
    /**
     * Index action
     *
     * @return void
     */
    public function indexAction ()
    {
        /* getting request parameters */
        $type      = $this->_getParam('type', "response");
        $page      = $this->_getParam('page', 1);
        $action    = $this->_getParam('trash-action');
        $objectIds = $this->_getParam("objects");
        switch ($type) {
            case "response":
                $objectsDb = Singular_Loader::get('System_Model_DbTable_Responses');
                if ($action == "recovery") {
                    $success = "Response(s) are restored!";
                } elseif($action == "delete") {
                    $success = "Response(s) are deleted!";
                }
                break;
            case "comment":
                $objectsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
                if ($action == "recovery") {
                    $success = "Comment(s) are restored!";
                } elseif($action == "delete") {
                    $success = "Comment(s) are deleted!";
                }
                break;
        }
        if ($this->getRequest()->isPost()) {
            if (!empty($objectIds)) {
                $objects = $objectsDb->findByIds($objectIds);
                if ($objects->valid()) {
                    foreach ($objects as $object) {
                        switch ($action) {
                            case "recovery" :
                                $object->recovery();
                                break;
                            case "delete" :
                                $object->delete();
                                break;
                        }
                    }
                    $this->view->notify()->success($success);
                }
            }
        }
        $this->view->paginator = $objectsDb->getPaginator(null, $page);
        $this->view->type = $type;
        if ($this->isAjax()) {
            $content = $this->fetchTemplate("system_admin-trash_index.phtml");
            $this->autoresponder->setOutput($content);
        }
    }

    /**
     * Clear action
     *
     * Clears recycled objects from trash
     *
     * @return void
     */
    public function clearAction()
    {
        $objectsDb = Singular_Loader::get('System_Model_DbTable_Responses');
        $objects = $objectsDb->findAll();
        foreach($objects as $object) {
            $object->delete();
        }
        $objectsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
        $objects = $objectsDb->findAll();
        foreach($objects as $object) {
            $object->delete();
        }
        $this->view->notify()->success("Trash is successfully cleaned!");
        $this->_helper->redirector->gotoRoute(array(), 'admin_trash');
    }
}
