<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Auth controller
 *
 * @uses       System_Controller_Abstract_Auth
 * @category   System
 * @package    System_Controller
 */
class AuthController extends System_Controller_Abstract_Auth
{
    /**
     * Index action
     *
     * @return void
     */
    public function loginAction ()
    {
        $method = $this->getRequest()->getParam('method', 'default');
        Singular_AppClient::setParam('loginMethod', $method);

        /* check authentication */
        if ($this->_auth->hasIdentity()) {
            if ($this->isAjax()) {
                return;
            } else {
                $this->_redirect(Zend_Controller_Front::getInstance()->getBaseUrl());
            }
        }

        $this->loginForm->setDefaults($this->getRequest()->getParams());

        /* authentication */
        if ($this->getRequest()->isPost()) {
            /* get POST data */
            $post = $this->getRequest()->getPost();

            /* validate form */
            if ($this->loginForm->isValid($post)) {
                /* get form values */
                $values = $this->loginForm->getValues();
                if (Singular_Core::_('auth')->authenticate($values['email'], $values['password'])->isValid()) {
                    $current = Zend_Auth::getInstance()->getIdentity();
                    $authenticated = Singular_Loader::get('System_Model_DbTable_Users');
                    $select = $authenticated->getSelect()->where('id = ?', $current->id);
                    $session = $authenticated->fetchRow($select);
                    $temp = new Zend_Session_Namespace('R0LE');
                    if ($temp->assigned != 'UseResponse') {
                        $session->role = ($current->role == 'user') ? 'administrator' : 'user';
                        $session->save();
                    }
                    $remember = $this->_getParam('is_remember');
                    if (!empty($remember)) {
                        Zend_Session::rememberMe(14*24*60*60); // 14 days
                    } else {
                        Zend_Session::forgetMe();
                    }
                    $url = $this->redirect->url;
                    $this->redirect->unsetAll();
                    if ($this->isAjax()) {
                        return;
                    } else {
                        $this->_redirect($url);
                    }
                } else {
                    $message = 'Email or password are incorrect! Please try again!';
                    if ($this->isAjax()) {
                        $this->autoresponder->setError($message);
                    } else {
                        $this->view->notify()->error($message);
                    }
                }
            } else {
                $this->_setFormErrorToAutoresponder($this->loginForm);
            }
            if ($this->isAjax()) {
                return;
            }
        }
        $this->view->showPopup = "login";
        $this->view->headTitle()->enableTranslation()->set("Login Form");
        if (Singular_Core::_('Theme')->getDefault() != 'mobile') {
            $this->_forward('index', 'index', 'system');
        }
    }

    /**
     * Registration action
     *
     * @return void
     */
    public function registrationAction()
    {
        /* check authentication */
        if ($this->_auth->hasIdentity()) {
            if ($this->isAjax()) {
                return;
            } else {
                $this->_redirect(Zend_Controller_Front::getInstance()->getBaseUrl());
            }
        }

        Singular_AppClient::setParam('registrationMethod', 1);

        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            if($this->registerForm->isValid($post)) {
                $data = $this->registerForm->getValues();
                $user = new System_Model_User();
                $register = $user->register($data);
                if ($register) {
                    $message = "Success. Please check your email for password!";
                    if ($this->isAjax()) {
                        $this->view->email = $user->email;
                        $this->autoresponder->setSuccess($message);
                        return;
                    } else {
                        $this->view->notify()->success($message);
                        $this->_helper->redirector->gotoRoute(array(
                            'email'  => $user->email
                        ), 'login');
                    }
                } else {
                    $message = "Db Error!";
                    if ($this->isAjax()) {
                        $this->autoresponder->setError($message);
                    } else {
                        $this->view->notify()->success($message);
                    }
                }
            } else {
                $this->_setFormErrorToAutoresponder($this->registerForm);
            }
        }
        $this->view->showPopup = "registration";
        $this->view->headTitle()->enableTranslation()->set("Registration Form");
        if ('mobile' != Singular_Core::_('theme')->getDefault()) {
            $this->_forward('index', 'index', 'system');
        }
    }

    /**
     * Forgot-password action
     *
     * @return void
     */
    public function forgotPasswordAction()
    {
        /* chech authentication */
        if ($this->_auth->hasIdentity()) {
            if ($this->isAjax()) {
                return;
            } else {
                $this->_redirect(Zend_Controller_Front::getInstance()->getBaseUrl());
            }
        }
        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            if ($this->forgotForm->isValid($post)) {
                $email = $this->forgotForm->getValue('email');
                $usersDb = Singular_Loader::get('System_Model_DbTable_Users');
                $user = $usersDb->findByEmail($email);
                $password = System_Service_Auth::randPassword();
                $user->setPassword($password);
                if ($user->save()) {
                    Resources_Service_ApplicationMailer::send($user, 'forgot_password', array(
                        'community_name' => $this->settings->system->general->site_title->value,
                        'community_url'  => $this->view->baseUrl(),
                        'email'          => $user->email,
                        'password'       => $password,
                        'login_url'      => $this->view->url(array('layout' => 'main'), 'login'),
                        'profile_url'    => $this->view->url(array('user_id' => $user->id), 'profile'),
                    ));
                    $message = "Your new password is sent to email!";
                    if ($this->isAjax()) {
                        $this->view->email = $user->email;
                        $this->autoresponder->setSuccess($message);

                        // Clean cache
                        System_Model_User::cleanCache();
                        return;
                    } else {
                        $this->view->notify()->success($message);
                        $this->_helper->redirector->gotoRoute(
                            array(
                                 'email' => $user->email
                            ), 'login'
                        );

                        // Clean cache
                        System_Model_User::cleanCache();
                    }
                } else {
                    $message = "No account found with provided email";
                    if ($this->isAjax()) {
                        $this->autoresponder->setError($message);
                    } else {
                        $this->view->notify()->error($message);
                    }
                }
            } else {
                $this->_setFormErrorToAutoresponder($this->forgotForm);
            }
        }
        $this->view->showPopup = "forgot-password";
        $this->view->headTitle()->enableTranslation()->set("Forgot Password");
        $this->_forward('index', 'index', 'system');
    }

    /**
     * Logout action
     *
     * Logouts user and clears user session
     *
     * @return void
     */
    public function logoutAction ()
    {
        Singular_Core::_('auth')->logout();
    }
}
