<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Cron controller
 *
 * @uses       System_Controller_Abstract_Admin
 * @category   System
 * @package    System_Controller
 */
class CronController extends System_Controller_Abstract_Action
{
    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function  init()
    {
        parent::init();
        
        $code = Singular_Core::_("Settings")->getSetting('protect_cron');
        $requestCode = $this->_getParam('code');
        
        if ($code !== $requestCode) {
            throw new Singular_Acl_Exception("Access denied");
        }
        
        $this->_helper->layout->disableLayout();
    }

    /**
     * Daily action
     *
     * Executes daily cron events
     *
     * @return void
     */
    public function dailyAction ()
    {
        echo 'Daily event started on ' . strftime('%b %e. %Y, %I:%M %p') . ".<br/>";
        Singular_Event::dispatch("cronDaily");
        Singular_Core::_('settings')->changeSetting('last_daily_activity', time());
        echo 'Daily event finished on ' . strftime('%b %e. %Y, %I:%M %p') . ".<br/>";
        
        exit;
    }

    /**
     * Houly action
     *
     * Executes hourly cron events
     *
     * @return void
     */
    public function hourlyAction ()
    {
        echo 'Hourly event started on ' . strftime('%b %e. %Y, %I:%M %p') . ".<br/>";
        Singular_Event::dispatch("cronHourly");
        Singular_Core::_('settings')->changeSetting('last_hourly_activity', time());
        echo 'Hourly event finished on ' . strftime('%b %e. %Y, %I:%M %p') . ".<br/>";

        exit;
    }

    /**
     * Frequently action
     *
     * Executes frequently cron events
     *
     * @return void
     */
    public function frequentlyAction ()
    {
        echo 'Frequently event started on ' . strftime('%b %e. %Y, %I:%M %p') . ".<br/>";
        Singular_Event::dispatch("cronFrequently");
        Singular_Core::_('settings')->changeSetting('last_frequently_activity', time());
        echo 'Frequently event finished on ' . strftime('%b %e. %Y, %I:%M %p') . ".<br/>";

        exit;
    }
}
