<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @subpackage System_Controller_Plugin
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * ApiErrorHandler plugin
 *
 * @uses       Zend_Controller_Plugin_Abstract
 * @category   System
 * @package    System_Controller
 * @subpackage System_Controller_Plugin
 */
class System_Controller_Plugin_ApiHandler
    extends Zend_Controller_Plugin_Abstract
{
    /**
     * Default data format
     * 
     * @var string json|xml 
     */
    protected $_format = 'json';
    
    /**
     * Available formats
     * 
     * @var array 
     */
    protected $_availableFormats = array(
        'json' => 'application/json; charset=utf-8',
        'xml'  => 'application/xml; charset=utf-8',
    );
    
    /**
     * Post pre dispatch hook
     *
     * @param Zend_Controller_Request_Abstract $request
     */
    public function preDispatch (Zend_Controller_Request_Abstract $request)
    {
        // Register api error handler
        $front = Zend_Controller_Front::getInstance();
        $front->setParam('noErrorHandler', true);

        // Force routing
        $request = $front->getRequest();
        $front->getRouter()->addRoute('api', new Zend_Controller_Router_Route(
            'api/:format/:module/:action/*',
            array(
                'module' => 'system',
                'controller' => 'api',
                'action' => 'index',
                'format' => 'json',
            )
        ))->route($request);
        
        // Getting api key and user id from request
        $apiKey = $request->getParam('api_key', 'empty');

        // Setup authenticate adapter
        $auth = Zend_Auth::getInstance();
        $auth->setStorage(new Zend_Auth_Storage_NonPersistent());
        Singular_Core_Auth::setupAdapter(null, 'api_key', 'api_key');
        $adapter = Singular_Runtime::extract('auth_adapter');
        $adapter->setIdentity($apiKey)
                ->setCredential($apiKey)
                ->getDbSelect()->where('is_active = ?', 1, Zend_Db::INT_TYPE);

        // Authenticate user by api key
        $result = $auth->authenticate($adapter);
        if ($result->isValid()) {
            $auth->getStorage()->write($adapter->getResultRowObject());
            Singular_Runtime::extract('acl')
                    ->setCurrentRole($adapter->getResultRowObject()->role);
        }
    }
    
    /**
     * Post dispatch hook -- check for exceptions and dispatch error handler if
     * necessary
     *
     * @param Zend_Controller_Request_Abstract $request
     */
    public function postDispatch (Zend_Controller_Request_Abstract $request)
    {
        // Getting content format
        $format = $request->getParam('format');
        if (array_key_exists($format, $this->_availableFormats)) {
            $this->_format = (string) $format;
        }

        // Set content type
        $response = Zend_Controller_Front::getInstance()->getResponse();
        $response->setHeader(
            'Content-Type', $this->_availableFormats[$this->_format]
        );
        
        // Handle!
        $exceptions = $response->getException();
        if (count($exceptions)) {
            $exception = $exceptions[0];
            $exceptionType = get_class($exception);
            switch ($exceptionType) {
                case 'Zend_Controller_Action_Exception':
                case 'Zend_Controller_Dispatcher_Exception':
                    $code = System_Controller_Abstract_Api::ERROR_NOT_FOUND;
                    $content = array('Api module or method is not found.');
                    break;
                case 'System_Controller_Exception_Abstract':
                    $code = $exception->getCode();
                    $content = $request->getParam('response');
                    break;
                default:
                    $code = System_Controller_Abstract_Api::ERROR_OPERATION_FAILED;
                    $content = APPLICATION_ENV == 'production'
                             ? array('Internal api error.')
                             : array('message' => $exception->getMessage());
                    break;
            }
            $response->setHttpResponseCode($code);
            $response->setBody($this->_format($content));
        } else {
            $response->setBody($this->_format($request->getParam('response')));
        }
        $request->setDispatched(true);
    }
    
    /**
     * Convert data to current format
     * 
     * @param  array $data
     *
     * @return string
     */
    protected function _format ($data)
    {
        return call_user_func(
            array('System_Service_Api', $this->_format), $data
        );
    }
}
