<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Response controller
 *
 * @uses       System_Controller_Abstract_Action
 * @category   System
 * @package    System_Controller
 */
class ResponseController extends System_Controller_Abstract_Action
{
    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see parent::init()
     */
    public function init()
    {
        parent::init();

        /* responses */
        $this->responses = new System_Model_DbTable_Responses();
        $responseSlug = $this->_getParam('slug');
        if (isset($responseSlug)) {
            $this->activeResponse = $this->responses->findBySlug($responseSlug);
            if ($this->activeResponse === false) {
                throw new Singular_Acl_Exception($this->view->t('Access Denied!  Insufficient permissions to view this object!'), 404);
            }
            if(!($this->activeResponse instanceof System_Model_Response)){
                throw new Zend_Exception($this->view->t('Incorrect request! No response found!'), 404);
            }
        }
        $responseId = $this->_getParam('response_id');
        if (isset($responseId)) {
            $this->activeResponse = $this->responses->findById($responseId);
            if ($this->activeResponse === false) {
                throw new Zend_Exception($this->view->t('Access Denied!  Insufficient permissions to view this object!'), 404);
            }
            if(!($this->activeResponse instanceof System_Model_Response)){
                throw new Zend_Exception($this->view->t('Incorrect request! No response found!'), 404);
            }
        }
        if (!isset($this->activeResponse)) {
            $this->activeResponse = new System_Model_Response();
        }
        $this->view->activeResponse = $this->activeResponse;
        Singular_Runtime::store('activeResponse', $this->activeResponse);

        $type = $this->_getParam('type');
        foreach ($this->types as $t) {
            if ($type == $t->slug) {
                $this->view->defaultType = $this->defaultType = $t;
            }
        }

        $this->form = $this->view->form = new System_Form_Response;
        if (!$this->activeResponse->isNew()) {
            $this->form->setDefaults($this->activeResponse->toArray());
        }
        $this->view->layout()->setLayout('response');

        Singular_AppClient::setParam('theme', Singular_Core::_('Theme')->getDefault());
    }

    /**
     * Show action
     *
     * @return void
     */
    public function showAction()
    {
        if ($this->activeResponse->hasAnswer()) {
            $answer = $this->activeResponse->getAnswer();
            $formAnswer = new System_Form_AnswerEdit();
            if ($answer instanceof System_Model_ResponseAnswer) {
                $formAnswer->setDefaults($answer->toArray());
            }
        } else {
            $formAnswer = new System_Form_Answer();
        }
        $formAnswer->setDefault('response_id', $this->activeResponse->id);
        $this->view->formAnswer = $formAnswer;

        $data = new stdClass();
        $data->sections = array();
        Singular_Event::dispatch('responseShow', $data);
        $this->view->sections = $data->sections;
        
        $data = new stdClass();
        $data->sections = array();
        Singular_Event::dispatch('responseAnswerShow', $data);
        $this->view->answerSections = $data->sections;
        
        $this->view->shareEnabled = true;
        
        if ($this->_getParam("add-vote", 0) < 0) {
            $this->view->notify()->success("If you disagree, we require adding comment!");
        }
        
        $cookie = $this->getRequest()->getCookie('views', '{}');
        $views = Zend_Json::decode(stripslashes($cookie));
        if (!isset($views[$this->activeResponse->id])) {
            $this->activeResponse->views++;
            $views[$this->activeResponse->id] = 1;
            $value = Zend_Json::encode($views);
            setcookie('views', $value, time()+60*60*24*365, '/');
        }
    }

    /**
     * Add action
     *
     * Adds new user reponse
     *
     * @return void
     */
    public function addAction()
    {
        /* check acl*/
        if (Singular_Core::_('theme')->getDefault() == 'mobile') {
            if (!Singular_Core::_('settings')->getSetting('anonymous_responses')) {
                if ($this->loggedUser->isGuest()) {
                    $this->_redirect( $this->view->url(array(), 'login') . '?redirect=' . $this->view->url() );
                    return;
                }
            }
        }
        /* set layout */
        $this->_helper->layout()->setLayout('response-add');
        
        $text = $this->_getParam('text');
        if (!empty($text)) {
            $this->form->setDefaults(array(
                'title' => $this->_getParam('text'),
            ));
        }
        /* redirect */
        $redirect = new Zend_Session_Namespace('Redirect');
        if ($redirect->url == $this->view->url()) {
            $redirect->unsetAll();
        }
        
        if ($this->getRequest()->isPost()) {
            if (!Singular_Core::_('settings')->getSetting('anonymous_responses')) {
                if ($this->loggedUser->isGuest()) {
                    $this->_redirect( $this->view->url(array(), 'login') . '?redirect=' . $this->view->url() );
                    return;
                }
            }

            $params = (object)array(
                'result'          => true,
                'redirectUrl'     => '',
                'redirectMessage' => '',
            );
            Singular_Event::dispatch('canAddResponse', $params);
            if ($params->result === false) {
                return $this->_redirect($params->redirectUrl, array(), $params->redirectMessage, 'error');
            }

            $post = $this->getRequest()->getPost();
            if ($this->form->isValid($post)) {
                $data = $this->form->getValues();
                /* save */
                if ($this->activeResponse->save($data)) {
                    
                    $this->view->notify()->success($this->view->t('Your :responseType is created!', array('responseType' => $this->view->t(strtolower($this->activeResponse->getType()->title)))));
                    
                    $data = new stdClass();
                    $data->response = $this->activeResponse;
                    $data->redirect = null;
                    Singular_Event::dispatch('responseAdded', $data);
                    
                    $this->_redirect($data->redirect);
                } else {
                    $this->view->notify()->error('Db Error');
                }
                
                $this->_redirect($this->view->url(array(), 'response_add'));
            }
        }
    }

    /**
     * Reload Captcha
     */
    public function reloadCaptchaAction()
    {
        if ($this->isAjax() && $this->getRequest()->isPost()) {
            $form = new System_Form_Response();
            $captcha = $form->getElement('captcha')->getCaptcha();
            $captchaData = array();
            $captchaData['id']  = $captcha->generate();
            $captchaData['src'] = $captcha->getImgUrl() .
                                  $captcha->getId() .
                                  $captcha->getSuffix();
            $this->view->captcha = $captchaData;
        } else {
            throw new Zend_Exception('Bad Request!');
        }
    }
    
    /**
     * Subscribe action
     *
     * Subscribes user to response following
     *
     * @return void
     */
    public function subscribeAction()
    {
        $subscribe = $this->_getParam("subscribe");

        if ($this->loggedUser->isGuest()) {
            $thisUrl = $this->_helper->url->url();
            $this->_helper->redirector->gotoRoute(array('redirect' => urlencode($thisUrl)), 'login');
        }

        if ($subscribe) {
            if (!$this->activeResponse->isSubscribed($this->loggedUser->id)) {
                $this->activeResponse->subscribe($this->loggedUser->id);
                $message = "You are subscribed to this response!";
            } else {
                $message = "You are already subscribed to this response!";
            }
            $this->view->isSubscribe = true;
        } else {
            $this->activeResponse->unSubscribe($this->loggedUser->id);
            $message = "You stopped following this response!";
            $this->view->isSubscribe = false;
        }
        if ($this->isAjax()) {
            $this->autoresponder->setSuccess($message);
            $output = $this->fetchTemplate('system_response_subscribe.phtml');
            $this->autoresponder->setOutput($output);
        } else {
            $this->view->notify()->success($message);
            $this->_redirect($this->activeResponse->getUrl());
        }
    }

    /**
     * Edit action
     *
     * Updates response data
     *
     * @return void
     */
    public function editAction()
    {
        /* check permission */
        if (!$this->activeResponse->canEdit()) {
            if ($this->loggedUser->isGuest()) {
                $this->_redirect($this->view->url(array(), 'login'));
            } else {
                throw new Singular_Acl_Exception($this->view->t('Access denied'));
            }
        }
        $this->_helper->layout()->setLayout('response-add');
        $this->form->setDefaults($this->activeResponse->toArray());
        $this->form->submit->setLabel('Save Changes');
        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();

            if ($this->form->isValid($post)) {
                $data = $this->form->getValues();
                //save
                if ($this->activeResponse->save($data)) {
                    Singular_Core::_('Cache')->getCache('default')->clean(Zend_Cache::CLEANING_MODE_ALL);
                    $this->view->notify()->success($this->view->t(':responseType has been updated!', array('responseType' => $this->view->t(ucfirst($this->activeResponse->getType()->title)))));
                    $this->_redirect();
                } else {
                    $this->view->notify()->error('Db Error');
                }
                $this->_redirect($this->view->url(array(), 'response_add'));
            }
        }
    }

    /**
     * Recovery action
     *
     * Recoveries deleted response
     *
     * @return void
     */
    public function recoveryAction()
    {
        if (!$this->activeResponse->canDelete()) {
             throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        if ($this->activeResponse->recovery()) {
            $this->_redirect($this->activeResponse->getUrl(),
                    array(), 'Response is restored!', 'success');
        } else {
            $this->_redirect($this->activeResponse->getUrl(),
                    array(), 'Db Error!', 'error');
        }
    }

    /**
     * Delete action
     *
     * Deletes specified response
     *
     * @return void
     */
    public function deleteAction()
    {
        if (!$this->activeResponse->canDelete()) {
             throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        $url = $this->_helper->url->url(array(), 'admin_trash');
        if ($this->activeResponse->delete()) {
            $this->_redirect($url, array(), 'Response is deleted!', 'success');
        } else {
            $this->_redirect($url, array(), 'Db Error!', 'error');
        }
    }

    /**
     * SetOrder action
     *
     * set order to response
     *
     * @return void
     */
    public function setOrderAction()
    {
        if (!$this->activeResponse->canDelete()) {
             throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        $order = (int) $this->_getParam('order', 1);
        $this->activeResponse->order = $order;
        if ($this->activeResponse->save()) {
            $this->_redirect(null, array(), 'Response Order Priority is Changed!',  'success');
        } else {
            $this->_redirect(null, array(), 'Db Error',  'error');
        }
    }
    
    /**
     * Trash action
     *
     * Moves specified response into trash
     *
     * @throws Singular_Acl_Exception
     */
    public function trashAction()
    {
        if (!$this->activeResponse->canDelete()) {
             throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        if ($this->activeResponse->trash()) {
            $type = $this->activeResponse->getType();
            $url = $this->_helper->url->url(array('objects' => $type->objects), 'responses_type');
            $this->_redirect($url, array(), 'Response is moved to trash', 'success');
        } else {
            $this->_redirect($this->activeResponse->getUrl(),
                    array(), 'Db Error!', 'error');
        }
    }

    /**
     * Votes action
     *
     * Displays response votes history
     *
     * @throws Singular_Acl_Exception
     */
    public function votesAction()
    {
        if (!$this->loggedUser->canManageResponses()) {
            throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        $slug      = $this->_getParam('slug');
        $responses = Singular_Loader::get('System_Model_DbTable_ResponsesVotes');
        $response  = $responses->findBySlug($slug);
        if (!$response) {
            throw new Singular_Acl_Exception($this->view->t(
                    'Access Denied! Insufficient permissions to view this object!'
                ),
                404
            );
        }
        if (!($response instanceof System_Model_ResponseVotes)) {
            throw new Singular_Acl_Exception(
                $this->view->t('Incorrect request! No response found!'), 404
            );
        }

        Singular_Runtime::store('activeResponse', $response);

        $useNegativeVotes = Singular_Core::_('settings')->getSetting('use_negative_votes');

        $this->view->assign(
            array(
                 'positiveVotes' => $response->getPositiveVotedUsers(),
                 'negativeVotes' => $useNegativeVotes ? $response->getNegativeVotedUsers() : array(),
            )
        );
    }

    /**
     * Vote-add action
     *
     * Adds new vote to specified response
     */
    public function voteAddAction()
    {
        $this->view->responseId = $this->activeResponse->id;
        /* check permission */
        if ($this->loggedUser->isGuest()
                && !$this->settings->system->general->anonymous_votes->value) {
            $url = $this->_helper->url->url(array(), "login");
            return $this->_redirect($url, array(), "Authentication is required to perfom the action!", 'error');
        }

        $params = (object)array(
            'result'           => true,
            'object'           => $this->activeResponse,
            'redirectUrl'      => '',
            'redirectMessage'  => '',
        );
        Singular_Event::dispatch('canVote', $params);
        if ($params->result === false) {
            return $this->_redirect($params->redirectUrl, array(), $params->redirectMessage, 'error');
        }

        /* unset session */
        $redirect = new Zend_Session_Namespace('Redirect');
        if ($redirect->url == $this->view->url()) {
            $redirect->unsetAll();
        }

        $cookie = $this->getRequest()->getCookie('votes', '{}');
        try {
            $cookie = Zend_Json::decode($cookie);
        } catch (Zend_Json_Exception $e) {
            $cookie = array();
        }

        $value = $this->getRequest()->getParam('value');
        $this->view->value = $value;

        if ($this->activeResponse->isVoted($this->loggedUser->id)) {

            if ($this->settings->system->responses->use_negative_votes->value) {
                if ($value < 0 && $this->settings->system->responses->force_to_comment_negative_vote->value && !$this->activeResponse->lock_comments) {
                    return $this->_redirect(null, array(), "Comment is required for negative vote!", 'error');
                }
                if ($this->loggedUser->isGuest()) {
                    /* save cookies */
                    $vote = $this->activeResponse->getGuestVote($value*(-1));
                    if ($vote && $value > 0 && $cookie['response'][$this->activeResponse->id] < 0) {
                        $cookie['response'][$this->activeResponse->id] = 1;
                        $vote->value = 1;
                    } elseif ($vote && $value < 0 && $cookie['response'][$this->activeResponse->id] > 0) {
                        $cookie['response'][$this->activeResponse->id] = -1;
                        $vote->value = -1;
                    } else {
                        return $this->_redirect(null, array(), "You have already voted for this response!", 'error');
                    }

                    $vote->save();
                    $this->view->voteSum = $this->activeResponse->getVotesSum();
                    setcookie("votes", Zend_Json_Encoder::encode($cookie), null, "/");
                    return $this->_redirect(null, array(), "Your vote is accepted!", 'success');

                } else {
                    $vote = $this->activeResponse->getUserVote($this->loggedUser->id);
                    if ($value > 0 && $vote->value < 0) {
                        $vote->value = 1;
                    } elseif ($value < 0 && $vote->value > 0) {
                        $vote->value = -1;
                    } else {
                        return $this->_redirect(null, array(), "You have already voted for this response!", 'error');
                    }
                    $vote->save();
                    $this->view->voteSum = $this->activeResponse->getVotesSum();
                    return $this->_redirect(null, array(), "Your vote is accepted!", 'success');
                }
            } else {
                return $this->_redirect(null, array(), "You have already voted for this response!", 'error');
            }
        } else {
            if ($value < 0 && $this->settings->system->responses->force_to_comment_negative_vote->value) {
                return $this->_redirect($this->activeResponse->getUrl() . "#comment-add", null, "Comment is required for negative vote!", 'error');
            }
            $vote = new Resources_Model_Vote();
            $vote->object = $this->activeResponse->getObject();
            $vote->object_id = $this->activeResponse->id;
            $vote->value = $value;
            if ($vote->save()) {
                if ($this->loggedUser->isGuest()) {
                    //save cookies
                    $cookie['response'][$this->activeResponse->id] = $vote->value;
                    setcookie("votes", Zend_Json_Encoder::encode($cookie), null, "/");
                } else {
                    if (!$this->activeResponse->isSubscribed($this->loggedUser->id)) {
                        $this->activeResponse->subscribe($this->loggedUser->id);
                    }
                }
                $this->view->voteSum = $this->activeResponse->getVotesSum();
                return $this->_redirect(null, array(), "Your vote is accepted!", 'success');
            } else {
                return $this->_redirect(null, array(), "Db error!", 'error');
            }
        }
    }

    /**
     * Answer-add action
     *
     * Adds new answer to specified response
     */
    public function answerAddAction()
    {
        if (!$this->activeResponse->canAddOfficialAnswer()) {
            throw new Singular_Acl_Exception($this->view->t("You can't add official answer"),1);
        }
        
        if ($this->activeResponse->hasAnswer()) {
            $this->view->notify()->error('Answer already added!');
            return $this->_redirect();
        }

        $form = new System_Form_Answer();
        if ($this->getRequest()->isPost()) {
            if ($form->isValid($this->getRequest()->getPost())) {
                $data = $form->getValues();

                $statusReport = new stdClass();
                $statusReport->isChanged = false;
                $statusReport->newStatus = null;
                if (isset($data['response_status']) && (bool)$data['response_status']) {
                    if (false !== $this->activeResponse->switchStatus($data['response_status'])) {
                        $statusReport->isChanged = true;
                        $statusReport->newStatus = true;
                    }
                }
                Singular_Runtime::store('statusReport', $statusReport);

                unset($data['id']);
                $answer = new System_Model_ResponseAnswer();
                $answer->save($data);
                $params = (object) array(
                    'answer'   => $answer,
                    'response' => $this->activeResponse,
                );
                Singular_Event::dispatch('answerAdded', $params);
                $this->view->notify()->success('Answer is added');
            } else {
                $form->renderFlashErrors();
            }
        }
        $this->_redirect();
    }

    /**
     * Answer-edit action
     *
     * Updates answer data
     */
    public function answerEditAction()
    {
        if (!$this->activeResponse->canAddOfficialAnswer()) {
            throw new Singular_Acl_Exception($this->view->t("You can't add official answer"), 1);
        }

        $form = new System_Form_AnswerEdit();
        if ($this->getRequest()->isPost()) {
            if($form->isValid($this->getRequest()->getPost())) {
                 $data = $form->getValues();
                 $answer = Singular_Loader::get('System_Model_DbTable_ResponsesAnswers')
                            ->findById($data['id']);
                 if ($answer instanceof System_Model_ResponseAnswer) {
                     $answer->save($data);
                     if (isset($data['notify']) && $data['notify'] == 1) {
                         $params = (object) array(
                            'answer'   => $answer,
                            'response' => $this->activeResponse,
                        );
                        Singular_Event::dispatch('answerAdded', $params);
                     }
                     $this->view->notify()->success('Official Answer is updated!');
                 }
            } else {
                $form->renderFlashErrors();
            }
        }
        $this->_redirect();
    }

    /**
     * Answer-delete action
     *
     * Deletes Response Official answer.
     *
     * @throws Singular_Acl_Exception
     */
    public function answerDeleteAction()
    {
        if (!$this->activeResponse->canAddOfficialAnswer()) {
            throw new Singular_Acl_Exception($this->view->t("You can't delete official answer"), 1);
        }

        if ($this->activeResponse instanceof System_Model_Response) {
            $this->activeResponse->removeAnswer();
            Singular_Event::dispatch('answerDeleted', $this->activeResponse);
            $this->view->notify()->success('Answer is deleted');
        }
        $this->_redirect();
    }
    
    /**
     * Lock-comments action
     *
     * Disables commenting feature for specified response
     *
     * @return void
     */
    public function lockCommentsAction()
    {
        if ($this->activeResponse->canLockComments()) {
            $this->activeResponse->lock_comments = (int) !$this->activeResponse->lock_comments;
            if ($this->activeResponse->save()) {
                if ($this->activeResponse->lock_comments) {
                    $this->view->notify()->success("Comments are locked!");
                } else {
                    $this->view->notify()->success("Comments are unlocked!");
                }
            }
        }
        $this->_redirect();
    }

    /**
     * Change-status action
     *
     * Changes status for specified response
     *
     * @return void
     */
    public function changeStatusAction()
    {
        if ($this->getRequest()->isPost()) {
            if ($this->activeResponse->canChangeStatus()) {
                $validator = new Zend_Validate_Db_RecordExists(array(
                    'table' => Singular_Loader::get('System_Model_DbTable_ResponsesStatuses')->getTableName(),
                    'field' => 'slug'
                ));
                $validator->getSelect()->where('type = ?', $this->activeResponse->getType()->title);
                $status = $this->_getParam('status');
                if ($validator->isValid($status)) {
                    $this->activeResponse->switchStatus($status);
                    $this->autoresponder->setSuccess($this->view->t(':responseType status has been updated!', array('responseType' => $this->view->t(ucfirst($this->activeResponse->getType()->title)))));
                }
            }
        }
    }

    /**
     * Rss action
     *
     * Provides RSS subscription
     *
     * @return void
     */
    public function rssAction()
    {
        $this->_helper->layout()->disableLayout();
        $rssArray = array(
            'title'       => $this->view->escape($this->activeResponse->title),
            'link'        => $this->activeResponse->getUrl(),
            'lastUpdate'  => $this->activeResponse->getUpdatedAt()->getTimestamp(),
            'published'   => $this->activeResponse->getCreatedAt()->getTimestamp(),
            'charset'     => 'UTF-8',
            'description' => str_replace("&nbsp;"," ", strip_tags($this->view->bbcode($this->activeResponse->content))),
        );
        if ($this->activeResponse->getAnswer()) {
            $rssArray['entries'][] = array(
                'title'       => $this->view->t('Official Answer'),
                'link'        => $this->activeResponse->getUrl(),
                'lastUpdate'  => $this->activeResponse->getAnswer()->getUpdatedAt()->getTimestamp(),
                'published'   => $this->activeResponse->getAnswer()->getCreatedAt()->getTimestamp(),
                'charset'     => 'UTF-8',
                'description' => $this->activeResponse->getAnswer()
            );
        }
        $comments = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')
                        ->findAll(array(
                            'response_id' => $this->activeResponse->id
                        ));
        if ($comments->valid()) {
            foreach($comments as $comment) {
                $rssArray['entries'][] = array(
                    'title'       => str_replace("&nbsp;"," ", strip_tags($this->view->bbcode($comment->content))),
                    'link'        => $comment->getUrl(),
                    'lastUpdate'  => $comment->getUpdatedAt()->getTimestamp(),
                    'published'   => $comment->getCreatedAt()->getTimestamp(),
                    'charset'     => 'UTF-8',
                    'description' => $comment->getAuthor(),
                );
            }
        }
        $rss = Zend_Feed::importArray($rssArray, 'rss');
        $rss->docs = null;
        $rss->send();
    }

    /**
     * Inline redirect helper
     *
     * Redirects to given url and displays specified message
     *
     * @see    parent::redirect()
     * @param  null|string $url       Redirect url ("index" if null)
     * @param  array $options         Options
     * @param  null|string $message   Message that will be shown after redirect
     * @param  string $messageStatus  Message status (success/error)
     * @return void
     */
    protected function  _redirect($url = null, array $options = array(), $message = null, $messageStatus = 'success')
    {
        if ($url == null) {
            parent::_redirect($this->activeResponse->getUrl(), $options, $message, $messageStatus);
        } else {
            parent::_redirect($url, $options, $message, $messageStatus);
        }
    }
}
