<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Responses controller
 *
 * @uses       System_Controller_Abstract_Action
 * @category   System
 * @package    System_Controller
 * @property   System_Model_DbTable_Responses $responsesDb
 */
class ResponsesController extends System_Controller_Abstract_Action
{
    /**
     * class of the Zend_Db_Table_Row object.
     *
     * @var System_Model_ResponseStatus
     */
    public $activeStatus = null;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();

        /* responses */
        $this->responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
        if ($this->isAjax()) {
            $this->_helper->layout()->disableLayout();
        } else {
            $this->_helper->layout()->setLayout('all-topics');
        }

        $objects = $this->view->type = $this->_getParam('objects', 'responses');
        $this->activeType = Singular_Runtime::extract('types')->findByObjects($objects);
        $this->view->activeType = $this->activeType;

        $status = $this->_getParam('status');
        if (!empty($status)) {
            $this->activeStatus       = $this->statuses->findBySlug($status);
            $this->view->activeStatus = $this->activeStatus;
        }
    }

    /**
     * Index action
     *
     * @return void
     */
    public function indexAction ()
    {
        $this->_helper->viewRenderer->setNeverRender();

        $params = $this->_getAllParams();

        $cacheId = $this->loggedUser->role . "_paginator_" . md5(var_export($params, true));
        $cacheId = preg_replace("/[^a-zA-Z0-9_]/", "", $cacheId);
        $cache = Singular_Runtime::extract('cachemanager')->getCache('default');

        $activeStatus = $this->_getParam('status', Singular_Core::_("settings")->getSetting("top_responses"));
        $tags = null;

        if ( ($content = $cache->load($cacheId)) == false ) {
            $page = $this->_getParam('page', 1);

            $where = array(
                'status' => $activeStatus,
                'date'   => $this->_getParam('date'),
                'text'   => $this->_getParam('text')
            );

            if ($this->activeType instanceof System_Model_ResponseType) {
                $where['type'] = $this->activeType->slug;
            }

            $this->view->paginator       = $this->responsesDb->getPaginator($where, $page);
            $this->view->cacheTags       = new StdClass();
            $this->view->cacheTags->tags = array();
            $content = $this->fetchTemplate('system_responses_index.phtml');
            $tags = $this->view->cacheTags->tags;
            $tags[] = 'responses';
            $cache->save($content, $cacheId, $tags);
        }
        Resources_Service_Votes::setUserVotes($cacheId, $tags);

        if ($this->isAjax()) {
            $this->autoresponder->setOutput($content);
        } else {
            $this->_helper->layout()->content = $content;
        }

        // set view vars
        if ($this->activeType instanceof System_Model_ResponseType) {
            if ($this->activeStatus instanceof System_Model_ResponseStatus) {
                $this->view->headTitle($this->view->t($this->activeStatus->title), 'PREPEND');
            }
            $this->view->title = $this->view->t('All :responsesType', array('responsesType' => $this->view->t(ucfirst($this->activeType->objects))));
            $this->view->headTitle($this->view->title, 'PREPEND');
        } else {
            $this->view->title = $this->view->t('All Responses');
        }

        $this->view->filter = $this->view->partial('_responses_filter.phtml', array(
            'activeType'   => $this->activeType,
            'activeStatus' => $activeStatus
        ));
    }

    /**
     * Search Action
     *
     * Retrieves responses using search criteria
     *
     * @return void
     */
    public function searchAction ()
    {
        $page = $this->_getParam('page', 1);
        $activeStatus = $this->view->activeStatus = $this->_getParam('status', null);
        if ($this->searchForm->isValid($this->_getAllParams())) {
            $where = $this->searchForm->getValues();
        }
        $where['status'] = $activeStatus;
        $where['date']   = $this->_getParam('date');
        $where['type']   = $this->_getParam('type');

        $this->view->paginator = $this->responsesDb->getPaginator($where, $page);
        $this->view->text = $where['text'];
        if ($this->isAjax()) {
            $content = $this->fetchTemplate('system_responses_search.phtml');
            $this->autoresponder->setOutput($content);
        }
    }

    /**
     * Quick Search Action
     *
     * Retrieves responses using serach criteria
     *
     * @return void
     */
    public function quickSearchAction()
    {
        $this->_helper->layout()->disableLayout(true);
        $where = array(
            'text' => $this->_getParam('text')
        );
        $this->view->text = $this->_getParam('text');
        $this->view->type = $this->_getParam('type');
        $this->view->paginator = $this->responsesDb->getPaginator($where, 1, null, 5);

        if ($this->isAjax()) {
            $isWidget = $this->_getParam('widget', 0);
            if ($isWidget) {
                $session = new Zend_Session_Namespace('curr_widget');
                $this->view->widgetType = (isset($session->type)) ? $session->type : 'search';
                $content = $this->fetchTemplate('system_responses_quick-search-widget.phtml');
            } else {
                $content = $this->fetchTemplate('system_responses_quick-search.phtml');
            }
            $this->autoresponder->setOutput($content);
        }
    }

    /**
     * Rss action
     *
     * Provides RSS subscription
     *
     * @return void
     */
    public function rssAction ()
    {
        $this->_helper->layout()->disableLayout();
        $responses = $this->responsesDb->findAll(array(
            'status' => 'new'
        ));
        $rssArray = array(
            'title'       => $this->settings->system->general->site_title->value,
            'link'        => Zend_Controller_Front::getInstance()->getBaseUrl(),
            'description' => $this->settings->system->general->site_description->value,
            'charset'     => 'UTF-8',
            'generator'   => 'UseResponse'
        );

        if ($responses->valid()) {
            foreach($responses as $response) {
                $rssArray['entries'][] = array(
                    'title'       => $this->view->escape($response->title),
                    'link'        => $response->getUrl(),
                    'lastUpdate'  => $response->getUpdatedAt()->getTimestamp(),
                    'published'   => $response->getCreatedAt()->getTimestamp(),
                    'charset'     => 'UTF-8',
                    'description' => str_replace("&nbsp;"," ", strip_tags($this->view->bbcode($response->content))),
                );
            }
        }

        $rss = Zend_Feed::importArray($rssArray, 'rss');
        $rss->docs = null;
        $rss->send();
    }
}
