<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * User controller
 *
 * @uses       System_Controller_Abstract_Action
 * @category   System
 * @package    System_Controller
 */
class UserController extends System_Controller_Abstract_Action
{
    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();

        $userId = $this->_getParam('user_id');
        if ($userId) {
            $user = Singular_Loader::get('System_Model_DbTable_Users')->findById($userId);
            if (!($user instanceof System_Model_User)) {
                return $this->_redirect($this->view->url(array(), 'profile_settings_me'));
            }
            $this->activeUser = $this->view->activeUser = $user;
        } else {
            if ($this->loggedUser->isGuest()) {
                return $this->_redirect(
                    $this->_helper->url->url(array('layout' => 'main', 'redirect' => 'profile'), 'login')
                );
            }
            $this->activeUser = $this->view->activeUser = $this->loggedUser;
        }
        Singular_Runtime::store("activeUser", $this->activeUser);
        Singular_AppClient::setParam('activeUserId', $this->activeUser->id);
        System_Service_Environment::getInstance()->setNamespace(System_Service_Environment::PROFILE);

        /* layout */
        $this->_helper->layout()->setLayout('profile');
    }

    /**
     * Index action
     *
     * @return void
     */
    public function indexAction ()
    {
        /** @var $responsesDb System_Model_DbTable_Responses */
        $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
        $page = $this->_getParam('page', 1);
        
        $type = $this->_getParam('type', 'all');
        if ($this->loggedUser->canManagePeople()) {
            $userFilter = $this->_getParam('user_filter', 'all');
        } else {
            $userFilter = $this->_getParam('user_filter', 'all');
        }
        $where = array(
            'type'        => $type,
            'user_filter' => array(
                'type'    => $userFilter,
                'user_id' => $this->activeUser->id
            )
        );
        /** @var $paginator Zend_Paginator */
        $paginator = $responsesDb->getPaginator($where, $page);
        $this->view->userFilter = $userFilter;
        $this->view->typeFilter = $type;
        $this->view->paginator  = $paginator;
        if ($this->isAjax()) {
            $content = $this->fetchTemplate('system_user_index.phtml');
            $this->autoresponder->setOutput($content);
        }
    }

    /**
     * Profile action
     *
     * Displays user's profile
     *
     * @return void
     * @throws Singular_Acl_Exception
     */
    public function profileAction()
    {
        if ($this->loggedUser->id != $this->activeUser->id) {
            if (!$this->loggedUser->canManagePeople($this->activeUser)) {
                throw new Singular_Acl_Exception($this->view->t("You can't manage people"), 1);
            }
            $form = new System_Form_AdminUser();
        } else {
            $form = new System_Form_User();
        }
        $data           = $this->activeUser->toArray();
        $data['config'] = $this->activeUser->getConfigValues();
        $form->setDefaults($data);
        $this->view->form = $form;
        
        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            if ($post['email'] == $this->activeUser->email) {
                $form->email->removeValidator('Zend_Validate_Db_NoRecordExists');
            }
            if ($form->isValid($post)) {
                $data = $form->getValues();
                if (empty($data['photo'])) {
                    unset($data['photo']);
                }

                if (!empty($data['old_password']) && !empty($data['password'])) {
                    if (Singular_Core::_('Auth')->makePassword($data['old_password'])
                        == $this->activeUser->password
                    ) {
                        $data['password'] = Singular_Core::_('Auth')->makePassword($data['password']);
                    } else {
                        $form->old_password->setErrors(
                            array(
                                 'Current Password is incorrect!'
                            )
                        );

                        return;
                    }
                } else {
                    unset($data['password']);
                }

                if ($this->activeUser->save($data)) {
                    $this->view->notify()->success('Profile is updated!');
                } else {
                    $this->view->notify()->error('Db Error');
                }

                // Clean cache
                System_Model_User::cleanCache();

                $this->_helper->redirector->gotoRouteAndExit(
                    array('user_id' => $this->activeUser->id), 'profile_settings'
                );
            }
        }
        Singular_AppClient::setParam('logged_user_id', $this->loggedUser->id);
    }
    
    /**
     * Reset api key
     * 
     * @return void
     * @throws Singular_Acl_Exception
     */
    public function resetApiKeyAction ()
    {
        // Logged user can manage peaple?
        if (!$this->loggedUser->isAdmin()) {
            throw new Singular_Acl_Exception($this->view->t("You can't manage people"), 1);
        }

        // Reset api key for active user
        $this->activeUser->api_key = System_Service_Api::generate();
        $this->activeUser->save();
        
        // On success show flash message and redirecto to profile page
        $this->view->notify()->success('New API Key is Generated!');
        $this->_helper->redirector->gotoRoute(array('user_id' => $this->activeUser->id), 'profile_settings');
    }

    /**
     * Change-password action
     *
     * Displays user's profile
     *
     * @return void
     * @throws Singular_Acl_Exception
     */
    public function changePasswordAction()
    {
        if ($this->loggedUser->id != $this->activeUser->id) {
            if (!$this->loggedUser->canManagePeople($this->activeUser)) {
                throw new Singular_Acl_Exception($this->view->t("You can't manage people"), 1);
            }
        }

        /* if ajax */
        if ($this->isAjax()) {
            $post = $this->getRequest()->getPost();
            if (!empty($post['old_password'])) {
                if (Singular_Core::_('Auth')->makePassword($post['old_password']) == $this->activeUser->password) {
                    if (!empty($post['password']) && !empty($post['password_again'])) {
                        if ($post['password'] == $post['password_again']) {
                            $this->activeUser->password = Singular_Core::_('Auth')->makePassword($post['password']);
                            $this->activeUser->save();
                            $this->autoresponder->setSuccess("Password is successfully changed!");
                        } else {
                            $this->autoresponder->setError("Passwords mismatch!");
                        }
                    } else {
                        $this->autoresponder->setError("New Password is required!");
                    }
                } else {
                    $this->autoresponder->setError("Current Password is incorrect!");
                }
            } else {
                $this->autoresponder->setError("Current Password is required!");
            }
        } else {
            $this->view->notify()->error("Bad request! Please try again later!");
            $this->_helper->redirector->gotoRoute(array('user_id' => $this->activeUser->id), 'profile');
        }
    }

    /**
     * Access action
     *
     * Blocks/unblocks user
     *
     * @return void
     */
    public function accessAction ()
    {        
        if (!$this->loggedUser->canManagePeople($this->activeUser)) {
            throw new Singular_Acl_Exception($this->view->t("You can't manage people"), 1);
        }
        $status = 'ban' == $this->_getParam('access') ? 0 : 1;
        $this->activeUser->switchStatus($status);
        $this->view->userId = $this->activeUser->id;
        $url = $this->_helper->url->url(array(
                'user_id' => $this->activeUser->id
            ), 'profile');

        if ($this->activeUser->is_active) {
            $message = $this->view->t('Access for this user is recovered!');
        } else {
            $message = $this->view->t(
                ':userName is banned to enter system!',
                array('userName' => $this->activeUser->getName())
            );
        }
        $this->_redirect($url, array(), $message, 'success');
    }

    /**
     * Del-photo action
     *
     * Deletes user's photo
     *
     * @return void
     */
    public function delPhotoAction()
    {
        if ($this->loggedUser->id != $this->activeUser->id) {
            if (!$this->loggedUser->canManagePeople($this->activeUser)) {
                throw new Singular_Acl_Exception($this->view->t("You can't manage people"), 1);
            }
        }
        $this->loggedUser->deletePhoto();
        $this->_redirect($this->view->url(array(), 'profile_settings_me'));
    }
    
    /**
     * Delete action
     *
     * Delete user
     *
     * @return void
     */
    public function deleteAction ()
    {
        if ($this->loggedUser->id != $this->activeUser->id) {
            if (!$this->loggedUser->canManagePeople($this->activeUser)) {
                throw new Singular_Acl_Exception($this->view->t("You can't manage people"), 1);
            }
            $reassign = 'user' == $this->_request->getParam('method');
            $this->activeUser->delete($reassign);
            $this->view->notify()->success('User is deleted!');
            $this->_helper->redirector->gotoRouteAndExit(array(), 'users');
        }
    }
}
