<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Users controller
 *
 * @property   System_Model_DbTable_Users $usersDb
 * @uses       System_Controller_Abstract_Action
 * @category   System
 * @package    System_Controller
 */
class UsersController extends System_Controller_Abstract_Action
{
    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();
        $this->usersDb = Singular_Loader::get('System_Model_DbTable_Users');
    }

    /**
     * Index action
     */
    public function indexAction()
    {
        if (!$this->loggedUser->canSeePeople()) {
            throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        $this->view->sort = $this->_getParam('sort');
        $this->view->role = $this->_getParam('role');
        $this->view->usersDb = $this->usersDb;
    }

    /**
     * Paginator action
     */
    public function paginatorAction()
    {
        if (!$this->loggedUser->canManagePeople()) {
            throw new Singular_Acl_Exception($this->view->t('Access denied'));
        }
        $page = $this->_getParam('page');
        $role = $this->view->role = $this->_getParam('role');
        $sort = $this->view->sort = $this->_getParam('sort');
        $search = $this->view->search = $this->_getParam('search');
        $where = array(
            'role'   => $role,
            'sort'   => $sort,
            'search' => ($search == $this->view->t('Search People')) ? '' : $search
        );
        $this->view->paginator = $this->usersDb->getPaginator($where, $page);
        if ($this->isAjax()) {
            $content = $this->fetchTemplate('system_users_paginator.phtml');
            $this->autoresponder->setOutput($content);
        }
    }

    /**
     * Participants action
     *
     * @return void
     */
    public function participantsAction()
    {
        $slug = $this->_getParam('slug');
        $responsesDb = Singular_Loader::get("System_Model_DbTable_Responses");
        $response = $responsesDb->findBySlug($slug);
        if (!($response instanceof System_Model_Response)) {
            throw new Zend_Exception($this->view->t('Incorrect request! No response found!'), 404);
        }
        $this->view->users = $response->getParticipants();
    }

    /**
     * Subscribers action
     *
     * @return void
     */
    public function subscribersAction()
    {
        $slug = $this->_getParam('slug');
        $responsesDb = Singular_Loader::get("System_Model_DbTable_Responses");
        $response = $responsesDb->findBySlug($slug);
        if (!($response instanceof System_Model_Response)) {
            throw new Zend_Exception($this->view->t('Incorrect request! No response found!'), 404);
        }
        $subscribersDb = Singular_Loader::get('System_Model_DbTable_ResponsesSubscribers');
        $this->view->users = $subscribersDb->findUsersByResponse($response);
    }
}
