<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Controller
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Widget controller
 *
 * @uses       System_Controller_Abstract_Action
 * @category   System
 * @package    System_Controller
 */
class WidgetController extends System_Controller_Abstract_Action
{
    protected $_widgetType;

    protected $_allowedTypes = array(
        'feedback',
        'search'
    );

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();
        if($this->getRequest()->isGet()){
            $session = new Zend_Session_Namespace('curr_widget');
            $this->_widgetType = $this->getRequest()->getParam('type');
            if ($this->_widgetType) {
                $session->type = $this->_widgetType;
            }
            $this->view->widgetType = $this->_widgetType;
            if (!isset($session->type) || !$session->type) {
                $session->type = $this->_widgetType;
            } else {
                if (in_array($session->type, $this->_allowedTypes)) {
                    $this->_widgetType = $session->type;
                } else {
                    unset($session->type);
                }
            }
            if (!$this->_widgetType || !in_array($this->_widgetType, $this->_allowedTypes)) {
                die('Invalid widget type!');
            }

            if ($this->_widgetType == 'feedback') {
                $this->_helper->layout()->setLayout('widget-adv');
                $this->activeResponse = new System_Model_Response();
                $this->view->activeResponse = $this->activeResponse;
                Singular_Runtime::store('activeResponse', $this->activeResponse);
                $this->responseAddForm = new System_Form_ResponseWidget();
                $this->view->responseAddForm = $this->responseAddForm;
                $this->view->showPopup = 'login';
            } else {
                $this->_helper->layout()->setLayout('widget');
            }
        }
        $this->view->responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');

    }

    /**
     * Index action
     *
     * @return void
     */
    public function indexAction()
    {

        if ($this->getRequest()->isPost()) {
            $post = $this->getRequest()->getPost();
            $this->activeResponse = new System_Model_Response();
            $this->view->activeResponse = $this->activeResponse;
            Singular_Runtime::store('activeResponse', $this->activeResponse);
            $this->responseAddForm = new System_Form_ResponseWidget();
            $this->view->responseAddForm = $this->responseAddForm;
            $this->view->showPopup = 'login';
            if ($this->responseAddForm->isValid($post)) {
                $data = $this->responseAddForm->getValues();
                /* save */
                if ($this->activeResponse->save($data)) {

                    $data = new stdClass();
                    $data->response = $this->activeResponse;
                    $data->redirect = $this->activeResponse->getUrl();
                    Singular_Event::dispatch('responseAdded', $data);
                    $this->view->notify()->success(
                        $this->view->t(
                            'Your :responseType is created!',
                            array('responseType' => $this->view->t(strtolower($this->activeResponse->getType()->title)))
                        ), 1
                    );
                    exit ('<script type="text/javascript">parent.postMessage("redirect:url(" + "'.$data->redirect.'" + ")", "*")</script>');
                } else {
                    $this->view->notify()->error('Db Error');
                }
            }
        }else{
            $this->view->defaultTab = $this->settings->system->widget->widget_default_tab->value;
            Singular_AppClient::setParam('widget_default_tab', $this->view->defaultTab);
            if ($this->_widgetType == 'feedback'
                || in_array(
                    $this->_widgetType, Singular_Runtime::extract('types')->findValuesBySlug()
                )
            ) {
                $this->template('system_widget_index-advanced.phtml', null, true);
            }
        }
    }
}
