<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Filter
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * System_Filter_FulltextSearch
 *
 * @uses       Zend_Filter_Interface
 * @category   System
 * @package    System_Filter
 */
class System_Filter_FulltextSearch implements Zend_Filter_Interface
{
    /**
     * Filtered array
     * 
     * @var array
     */
    private $_filtered = array();

    /**
     * Defined by Zend_Filter_Interface
     *
     * @param  string $text
     * @see    parent::filter()
     *
     * @return mixed|string
     */
    public function filter ($text)
    {
        $words = mb_split(' +', $text);
        $uniqueWords = array_unique($words);

        foreach ($uniqueWords as $word) {
            $this->_sanitize($word);
            $this->_makeVariations($word);
        }

        return $this->getFiltered(true);
    }

    /**
     * Adds filtered word
     *
     * @param string $word
     */
    public function addFiltered ($word)
    {
        $this->_filtered[] = $word;
    }

    /**
     * Returns filtered words
     *
     * @param bool $implode
     *
     * @return array|string
     */
    public function getFiltered ($implode = false)
    {
        $filtered = array_unique($this->_filtered);
        return ($implode) ? implode(' ', $filtered) : $filtered;
    }

    /**
     * Sanitizes word
     *
     * @param $word
     */
    private function _sanitize ($word)
    {
        $replaces = array(
            '(' => '',
            ')' => ''
        );

        $sanitized = strtr($word, $replaces);
        $sanitized = trim($sanitized);

        $this->addFiltered($sanitized);
    }

    /**
     * Makes word variants if brackets are detected
     *
     * @param $word
     */
    private function _makeVariations ($word)
    {
        preg_match('/(?P<name>\w+)\((?P<variant>.*)\)/', $word, $matches);
        if (isset($matches['name']) && isset($matches['variant'])) {
            $this->addFiltered($matches['name']);
            $this->addFiltered($matches['name'] . $matches['variant']);
        }
    }
}