<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Filter
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * System_Filter_Rename
 *
 * @uses       Zend_Filter_File_Rename
 * @category   System
 * @package    System_Filter
 */
class System_Filter_Rename extends Zend_Filter_File_Rename
{
    /**
     * Internal method to resolve the requested source
     * and return all other related parameters
     *
     * @param  string $file Filename to get the informations for
     * @return array
     */
    protected function _getFileName($file)
    {
        $rename = parent::_getFileName($file);
        foreach ($_FILES as $file) {
            if ($file["tmp_name"] == $rename["source"]) {
                $name = $file["name"];
            }
        }
        if (file_exists($rename["target"])) {
            $name = basename($rename["target"]);
        }

        if ($name) {
            $target = pathinfo($rename["target"]);
            $rename["target"] = $target["dirname"] . DIRECTORY_SEPARATOR . $name;
            if (file_exists($rename["target"])) {
                $pathInfo = pathinfo($rename["target"]);
                $k = 1;
                while (true) {
                    $rename["target"] = $pathInfo["dirname"] . DIRECTORY_SEPARATOR
                                . $pathInfo["filename"] . "_" . $k
                                . "." . $pathInfo["extension"];
                    if (!file_exists($rename["target"])) {
                        break;
                    }
                    $k++;
                }
            }
        }
        return $rename;
    }

    /**
     * Defined by Zend_Filter_Interface
     *
     * Renames the file $value to the new name set before
     * Returns the file $value, removing all but digit characters
     *
     * @param  string $value Full path of file to change
     * @throws Zend_Filter_Exception
     * @return string The new filename which has been set, or false when there were errors
     */
    public function filter($value, $rename = true)
    {
        $file   = $this->getNewName($value, true);

        if (is_string($file)) {
            return $file;
        }

        if ($rename) {
            $result = rename($file['source'], $file['target']);
            if ($result === true) {
                return $file['target'];
            }
        } else {
            return $file['target'];
        }

        require_once 'Zend/Filter/Exception.php';
        throw new Zend_Filter_Exception(sprintf("File '%s' could not be renamed. An error occured while processing the file.", $value));
    }
}