<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Filter
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * System_Filter_ResizeImage
 *
 * @uses       Zend_Filter_Interface
 * @category   System
 * @package    System_Filter
 */
class System_Filter_ResizeImage implements Zend_Filter_Interface
{
   private $_targetDir;
   private $_newSize;
   private $_scaleBy = false;
   private $_saveOriginal = true;

   /**
     * Constructor for filter
     *
     * @param array $options Options to set
     */
   public function __construct($options)
   {
       if(!extension_loaded('gd')) {
            throw new Zend_Filter_Exception('GD not Installed - GD extension is not installed on your system.');
       }
       if ($options instanceof Zend_Config) {
            $options = $options->toArray();
        } elseif (is_string($options)) {
            $options = array('targetDir' => $options);
        } elseif (!is_array($options)) {
            throw new Zend_Filter_Exception('Invalid options argument provided to filter');
        }

        if (isset($options['targetDir']) && $options['targetDir']) {
            if (file_exists($options['targetDir'])) {
                if (is_writable($options['targetDir'])) {
                    $this->_targetDir = $options['targetDir'];
                } else {
                    throw new Zend_Filter_Exception('Directory is not writable. Invalid target directory argument provided to filter');
                }
            } else {
                throw new Zend_Filter_Exception('Directory doesn\'t exists .Invalid target directory argument provided to filter');
            }
        } else {
            throw new Zend_Filter_Exception('Invalid target directory argument provided to filter');
        }

        if (isset($options['rename'])) {
            $this->_rename = (bool) $options['rename'];
        } else {
            $this->_rename = true;
        }

       if (isset($options['scaleBy'])) {
           $this->_scaleBy = $options['scaleBy'];
       }

        if (isset($options['saveOriginal'])) {
            $this->_saveOriginal = (bool) $options['saveOriginal'];
        }

        if (isset($options['newSize']) && $options['newSize']) {
            if (is_string($options['newSize'])) {
                $options['newSize'] = array($options['newSize']);
            }
            foreach($options['newSize'] as $path=>$newSize) {
                $arraySize = explode('*', $newSize);
                if (count($arraySize) == 2 && $arraySize[0] && $arraySize[1]) {
                    if (empty($path)) {
                        $path = "/";
                    } else {
                        $path = "/" . (string)$path;
                    }
                    $this->_newSize[] = array(
                        'path'   => $path,
                        'width'  => (int) $arraySize[0],
                        'height' => (int) $arraySize[1],
                    );
                } else {
                    throw new Zend_Filter_Exception('Invalid image size argument provided to filter. It must be string "int*int".');
                }
            }
            if (!$this->_saveOriginal) {
                
            }
        } else {
            throw new Zend_Filter_Exception('Invalid image size argument provided to filter');
        }
   }


   /**
     * Resize image and copy new file to need directory with equal filename
     *
     * @param string $value full filename
     * @return string
     */
   public function filter($value)
   {
       if (empty($value)) {
           return "";
       }

       $pathInfo = pathinfo($value);
       if ($pathInfo["dirname"] == ".") {
           $pathInfo["dirname"] = FILES_TEMP_PATH;
           $valueNew = FILES_TEMP_PATH . DS . $value;
       } else {
           $valueNew = $value;
       }

       $info = getimagesize($valueNew);
        
        if ($info) {
            switch($info[2]) {
                case IMAGETYPE_JPEG:
                    $fullImage = imagecreatefromjpeg($valueNew);
                    break;
                case IMAGETYPE_GIF:
                    $fullImage = imagecreatefromgif($valueNew);
                    break;
                case IMAGETYPE_PNG:
                    $fullImage = imagecreatefrompng($valueNew);
                    break;
            }
        }
        $x = $info[0];
        $y = $info[1];

        foreach ($this->_newSize as $size) {
            if ($this->_scaleBy) {
                $ratio       = $x / $y;
                $strictScale = true;
                if ('width' == $this->_scaleBy) {
                    // TODO: make scale algo for $this->_scaleBy = 'width'
                    unset($strictScale);
                } elseif ('height' == $this->_scaleBy) {
                    if ($ratio == 1) {
                        $size["width"] = $x * $ratio;
                    } elseif ($ratio < 1) {
                        $size["width"] = $size["height"] * $ratio;
                    } else {
                        $size["width"] = $size["height"] * $ratio;
                    }
                } else {
                    unset($strictScale);
                }
            }
            $scale  = min($size["width"] / $x, $size["height"] / $y);

            if ($scale < 1 || isset($strictScale)) {
                $nx  = floor($scale * $x);
                $ny = floor($scale * $y);
                
                $tnImage = imagecreatetruecolor($size["width"], $size["height"]);
                  
                if ($info[2] == IMAGETYPE_PNG
                     && ord(file_get_contents($valueNew, null, null, 25, 1)) == 6) {
                    $whiteColor = imagecolorallocatealpha($tnImage, 255, 255, 255, 127);
                    imagefill($tnImage, 0, 0, $whiteColor);
                    imagealphablending($tnImage, false);
                    imagesavealpha($tnImage, true);
                } else {
                    $whiteColor = imagecolorallocate($tnImage, 255, 255, 255);
                    imagefill($tnImage, 0, 0, $whiteColor);
                }
                $dst_x = ($size["width"]-$nx)/2;
                $dst_y = ($size["height"]-$ny)/2;

                imagecopyresampled($tnImage, $fullImage, $dst_x, $dst_y, 0, 0, $nx, $ny, $x, $y);

            } else {
                $tnImage = imagecreatetruecolor($size["width"], $size["height"]);
                if ($info[2] == IMAGETYPE_PNG
                     && ord(file_get_contents($valueNew, null, null, 25, 1)) == 6) {

                    $whiteColor = imagecolorallocatealpha($tnImage, 255, 255, 255, 127);
                    imagefill($tnImage, 0, 0, $whiteColor);
                
                    imagealphablending($tnImage, false);
                    imagesavealpha($tnImage, true);
                } else {
                    $whiteColor = imagecolorallocate($tnImage, 255, 255, 255);
                    imagefill($tnImage, 0, 0, $whiteColor);
                }
                $dst_x = ($size["width"]-$x)/2;
                $dst_y = ($size["height"]-$y)/2;
                imagecopy($tnImage, $fullImage, $dst_x, $dst_y, 0, 0, $x, $y);
            }
            
            $fileDir = $this->_targetDir . $size["path"];

            if (!file_exists($fileDir)) {
                mkdir($fileDir);
                chmod($fileDir, 0777);
            }
            
            $filename = $fileDir . DIRECTORY_SEPARATOR . $pathInfo["basename"];

            if ($this->_rename && file_exists($filename)) {
                $filter = new System_Filter_Rename(array(
                    'target'    => $fileDir,
                    'overwrite' => false,
                ));
                $filename = $filter->filter($filename, false);
                $value = basename($filename);
            }

            if ($info) {
                switch($info[2]) {
                    case IMAGETYPE_JPEG:
                        imagejpeg($tnImage, $filename, 80);
                        break;
                    case IMAGETYPE_GIF:
                        imagegif($tnImage, $filename);
                        break;
                    case IMAGETYPE_PNG:
                        imagepng($tnImage, $filename);
                        break;
                }
            }
            imagedestroy($tnImage);
            
        }
        imagedestroy($fullImage);

        if (!$this->_saveOriginal) {
            unlink($valueNew);
        }
        return $value;
   }
}
