<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Analytics filter form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 */
class System_Form_AnalyticsFilter extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();
        
        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            array('ViewScript', array('viewScript' => '_analytics/statistics-filter.phtml')),
            'Form'
        ));
        
        $groupByElement = new Zend_Form_Element_Select(
            'group_by', array(
                'label'      => $this->getView()->t('Group by'),
                'order'      => 10,
                'decorators' => array(
                    'ViewHelper',
                    array('ViewScript', array('viewScript' => 'forms/elements/select.phtml'))
                ),
                'multioptions' => $this->_getGroupingOptions(),
                'value' => $this->getDefault('group_by'),
            )
        );
        
        $reportTypeElement = new Zend_Form_Element_Select(
            'report_type', array(
                'label'      => $this->getView()->t('Report'),
                'order'      => 5,
                'decorators' => array(
                    'ViewHelper',
                    array('ViewScript', array('viewScript' => 'forms/elements/select.phtml'))
                ),
                'multioptions' => $this->getReports($groupByElement->getValue()),
                'value' => $this->getDefault('report_type'),
            )
        );
        
        $rangeDateElement = new Zend_Form_Element_Text(
            'date_range', array(
                'id'         => 'date_range',
                'class'      => 'date-pick',
                'label'      => $this->getView()->t('Date Range'),
                'order'      => 15,
                'decorators' => array(
                    array('ViewHelper'),
                    array('ViewScript', array(
                        'viewScript' => '/forms/elements/analytics-input-date.phtml'
                    ))
                ),
                'value' => $this->getDefault('date_range'),
        ));
        
        $reportPrecisionElement = new System_Form_Element_MultiButton(
            'precision',array(
                'label'      => '',
                'class'      => 'multibutton',
                'order'      => 20,
                'decorators' => array(
                    'ViewHelper',
                    array('ViewScript', array('viewScript' => 'forms/elements/multi-button.phtml'))
                ),
                'multioptions' => $this->_getPrecisionOptions(),
                'value' => $this->getDefault('precision'),
            )
        );
        
        $this->addDisplayGroup(array($reportTypeElement, $groupByElement, $rangeDateElement, $reportPrecisionElement),'top');
    }
    
    /**
     * Return report by selected type
     * 
     * @param type $type
     * @return array
     */
    public function getReports($type)
    {
        $reports = array();
        foreach (Singular_Runtime::extract('analytics_report_config')->reports as $report) {
            switch (true)
            {
                case ($type == 'none' || $type == 'weekday') :
                    $reports[] = array('key' => $report['id'], 'value' => $report['title']);
                    break;
                case ($type == 'type') :
                    if(isset($report['options']['response_types_assoc']) && $report['options']['response_types_assoc'] == true) {
                        $reports[] = array('key' => $report['id'], 'value' => $report['title']);
                    } 
                    break;
            }
        }
        
        return $reports;
    }
    
    /**
     * Return option for "Group by" dropdown.
     * 
     * @return array
     */
    protected function _getGroupingOptions ()
    {
        return $options = array(
            array(
                'key' => 'none',
                'value' => $this->getView()->t('No grouping'),
                'default' => true,
            ),
            array(
                'key' => 'weekday',
                'value' => $this->getView()->t('Weekdays'),
            ),
            array(
                'key' => 'type',
                'value' => $this->getView()->t('Types'),
            ),
        );
    }
    
    /**
     * Return names for "Precision" buttons.
     * 
     * @return array
     */
    protected function _getPrecisionOptions ()
    {
        return $options = array(
            array(
                'key' => 'hour',
                'value' => $this->getView()->t('Hours'),
            ),
            array(
                'key' => 'day',
                'value' => $this->getView()->t('Day'),
                'default' => true,
            ),
            array(
                'key' => 'week',
                'value' => $this->getView()->t('Week'),
            ),
            array(
                'key' => 'month',
                'value' => $this->getView()->t('Month'),
            ),
        );
    }
    
    /**
     * Return default value for any form element.
     * 
     * @param string $type Type of diagram dataset
     * @return string
     */
    public function getDefault($type)
    {
        switch (true) {
            case ($type == 'report_type') :
                return 'system_responses';
            case ($type == 'group_by' || $type == 'precision') :
                if ($type == 'group_by') {
                    $options = $this->_getGroupingOptions();
                } else {
                    $options = $this->_getPrecisionOptions();
                }                
                foreach ($options as $option)
                {
                    if (isset($option['default']) && $option['default'] == true) {
                        return $option['key'];
                    }
                }
                return false;
            case ($type == 'date_range') :
                return '';
        }  
    }
}
