<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Answer form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 *
 * @property Zend_Form_Element_Submit    $submit
 * @property System_Form_Element_Content $answer
 * @property Zend_Form_Element_Hidden    $id
 * @property Zend_Form_Element_Hidden    $response_id
 */
class System_Form_Answer extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();

        $this->setMethod(self::METHOD_POST);
        $this->setAttribs(array(
            'id'     => 'form-official-representative',
            'action' => $this->getView()->url(array(), 'response_add_answer')
        ));

        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $this->setAttrib('data-ajax', "false");
        }
        
        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            array('ViewScript', array('viewScript' => 'forms/answer.phtml')),
            'Form',
        ));

        $this->addElement('hidden', 'id', array(
            'required'   => false,
        ));

        $this->addElement('hidden', 'response_id', array(
            'required'   => true,
        ));

        $validator =  new System_Lib_Validate_StringLengthBbcode(array(
            'min' => 3,
            'max' => null,
        ));
        $validator->setMessages(array(
            Zend_Validate_StringLength::TOO_SHORT => 'Answer should be at least 3 characters!'
        ));
        
        $description = new System_Form_Element_Content('answer', array(
            'required'     => false,
            'withoutLabel' => true,
            'allowEmpty'   => false,
            'filters'      => array('StringTrim'),
            'validators'   => array($validator),
        ));
        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $description->setAttrib('data-role', 'none');
        }
        $this->addElement($description);

        $defaultGroupElements = array();

        $submit = new Zend_Form_Element_Submit('submit', array(
            'ignore'   => true,
            'label'    => 'Submit',
            'attribs'  => array(
                'class' => 'button-type1 floatLeft'
            ),
            'order'    => 3,
            'decorators' => array(
                array('ViewHelper'),
                array('ViewScript', array('viewScript' => 'forms/elements/button.phtml'))
            ),
        ));
        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $submit->setAttrib('data-role', 'none');
        }

        $defaultGroupElements[] = $submit;

        /** @var System_Model_Response $response */
        $response = Singular_Runtime::extract('activeResponse');
        if ($response->isSupportStatuses()) {
            if ($response instanceof System_Model_Response && $response->canChangeStatus()) {
                $responseStatus = new System_Form_Element_ResponseStatus('response_status',
                    array(
                         'order' => 6,
                    )
                );
                $defaultGroupElements[] = $responseStatus;
            }
        }

        $this->addDisplayGroup($defaultGroupElements, 'default');
    }
}
