<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @subpackage System_Form_Element
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Directories form element
 *
 * @uses       Zend_Form_Element_Select
 * @category   System
 * @package    System_Form
 * @subpackage System_Form_Element
 */
class System_Form_Element_Directories extends Zend_Form_Element_Select
{
    /**
     * @var string
     */
    private $_path;

    /**
     * @var string
     */
    private $_mask;

    /**
     * Constructor
     *
     * $spec may be:
     * - string: name of element
     * - array: options with which to configure element
     * - Zend_Config: Zend_Config with options for configuring element
     *
     * @see    parent::__construct()
     * @param  string|array|Zend_Config $spec
     * @param  array|Zend_Config $options
     * @throws Zend_Form_Exception if no element name after initialization
     */
    public function __construct ($spec, $options)
    {
        $this->_path = $options['settings']['path'];
        $this->_mask = $options['settings']['mask'];
        parent::__construct($spec, $options);
    }

    /**
     * Initialize object; used by extending classes
     *
     * @return void
     */
    public function init ()
    {
        $handle = opendir($this->_path);
        $multioptions = array();
        
        while (false !== ($file = readdir($handle))) {
            if (!preg_match('|^\..*$|',$file)) {
                if (isset($this->_mask)) {
                    if (preg_match($this->_mask,$file)) {
                        $multioptions[$file] = $file;
                    }
                } else {
                    $multioptions[$file] = $file;
                }
            }
        }

        $this->addMultiOptions($multioptions);
        $this->setRegisterInArrayValidator(false);
    }
}
