<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @subpackage System_Form_Element
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * MultiCheckbox form element
 *
 * @uses       Zend_Form_Element_MultiCheckbox
 * @category   System
 * @package    System_Form
 * @subpackage System_Form_Element
 */
class System_Form_Element_MultiCheckbox extends Zend_Form_Element_MultiCheckbox
{
    private $_val;
    private $_opt;
    private $_filter;
    private $_multiOptions;

    /**
     * Constructor
     *
     * $spec may be:
     * - string: name of element
     * - array: options with which to configure element
     * - Zend_Config: Zend_Config with options for configuring element
     *
     * @see    parent::__construct()
     * @param  string|array|Zend_Config $cons
     * @param  array|Zend_Config $options
     * @throws Zend_Form_Exception if no element name after initialization
     */
    public function __construct ($cons, $options = null)
    {
        if ($cons['settings']['model'] instanceof Zend_Db_Table_Rowset) {
            $this->_multiOptions = $cons['settings']['model'];
        } else {
            $opt = new $cons['settings']['model'];
            $where = null;
            if (isset($cons['settings']['where'])) {
                $where = $cons['settings']['where'];
            }
            $this->_multiOptions = $opt->fetchAll($where);
        }
        
        $this->_val = $cons['settings']['val'];
        $this->_opt = $cons['settings']['opt'];
        if (isset($cons['settings']['filter'])) {
            $this->_filter = $cons['settings']['filter'];
        }
         
        parent::__construct($cons['name'], $cons['options']);
    }

    /**
     * Initialize object; used by extending classes
     *
     * @return void
     */
    public function init(){

        $_opt = $this->_opt;
        $_val = $this->_val;

        if($this->_multiOptions->valid()){
            foreach($this->_multiOptions as $row){

                $opt = $row->$_opt;
                $val = $row->$_val;
                if(isset($this->_filter)){
                    if(preg_match($this->_filter,$val)){
                        $multioptions[$val] = $opt;
                    }
                }else{
                    $multioptions[$val] = $opt;
                }

            }
            $this->setDisableTranslator(true);
            $this->addMultiOptions($multioptions);
            $this->setDisableTranslator(false);
        }
        $this->setRegisterInArrayValidator(false);
    }
}
