<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @subpackage System_Form_Element
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * ResponseStatus form element
 *
 * @uses       Zend_Form_Element_Select
 * @category   System
 * @package    System_Form
 * @subpackage System_Form_Element
 */
class System_Form_Element_ResponseStatus extends Zend_Form_Element_Select
{
    /**
     * Constructor
     *
     * $spec may be:
     * - string: name of element
     * - array: options with which to configure element
     * - Zend_Config: Zend_Config with options for configuring element
     *
     * @see    parent::__construct()
     * @param  string|array|Zend_Config $cons
     * @param  array|Zend_Config $options
     * @throws Zend_Form_Exception if no element name after initialization
     */
    public function __construct ($cons, $options = null)
    {
        parent::__construct($cons, $options);
    }

    /**
     * Initialize object; used by extending classes
     *
     * @return void
     */
    public function init()
    {
        parent::init();

        /** @var System_Model_Response $response */
        $response = Singular_Runtime::extract('activeResponse');
        if (!$response instanceof System_Model_Response) {
            return;
        }

        $statuses      = $response->getStatuses();
        $statusesPairs = array();
        if ($statuses instanceof System_Model_Rowset) {
            $statusesPairs[0] = '-- ' . self::getView()->t('Update Status') . ' --';
            /**
             * BUG: Unable to walk status as Rowset. Another status change select is disappeared.
             * Convert Rowset into array instead before cycle rolling.
             */
            $statuses = $statuses->toArray();
            foreach ($statuses as $status) {
                $statusesPairs[$status['slug']] = $status['title'];
            }
        }
        if (!empty($statusesPairs)) {
            $this->setMultiOptions($statusesPairs);
        }

        $this->setDecorators(
            array(
                 array('ViewScript',
                       array('viewScript' => 'forms/elements/response-statuses-select.phtml')),
                 'ViewHelper'
            )
        );
    }

    /**
     * Returns default values
     *
     * @return array
     */
    public function getDefaults()
    {
        return $this->_defaults;
    }
}
