<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * GeneralSettings form
 *
 * @uses       System_Form_Settings
 * @category   System
 * @package    System_Form
 */
class System_Form_GeneralSettings extends System_Form_Settings
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();
        
        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            new Zend_Form_Decorator_ViewScript(array(
                    'viewScript' => 'forms/general-settings.phtml'
                )),
            'Form'
        ));
        
        /* slogan */
        $element = $this->getElement('site_slogan');        
        if ($element instanceof Zend_Form_Element) {
            $element->setAllowEmpty(true);
        }
        
        /* description */
        $element = $this->getElement('site_description');
        if ($element instanceof Zend_Form_Element) {
            $element->setAllowEmpty(true);
            $element->setAttribs(array(
                'id' => 'description',
                'rows' => '20',
                'cols' => '10'
            ));
        }

        /* logo element */
        $element = $this->getElement('site_logo');
        if ($element instanceof Zend_Form_Element) {
            $maxFileSize = System_Service_Files::getMaxUploadSize();
            $uploadPath = ROOT_PATH . '/public/files/logo/';
            $element->deleteUrl = $this->getView()->url(array(), 'adimn_delete_logo');
            $element->setRequired(false);
            $element->setDescription($this->getView()->t("The image must be a GIF, JPEG or PNG that is smaller than :size.", array('size' => $maxFileSize)));
            $element->setDestination($uploadPath);
            $element->setValidators(array(
                array('Extension', false, array('gif','png','jpeg','jpg',
                    'messages' => array(
                        Zend_Validate_File_Extension::FALSE_EXTENSION => 'You are trying to use incorrect file type!'
                    )
                )),
            ));
            $element->setFilters(array(
                new System_Filter_ResizeImage(array(
                    'targetDir' => $uploadPath,
                    'newSize'   => '248*66',
                    'scaleBy' => 'height'
                ))
            ));
        }
        
        if ($this->anonymous_responses instanceof Zend_Form_Element_Radio) {
            $this->anonymous_responses->advanceOptions = array(
                0 => $this->anonymous_captcha
            );
        }
    }
}
