<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Language form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 */
class System_Form_Language extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();

        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            new Zend_Form_Decorator_ViewScript(array(
                    'viewScript' => 'forms/system.phtml'
                )),
            'Form'
        ));

        $this->setEnctype(self::ENCTYPE_MULTIPART);

        $this->addElement('text', 'name', array(
            'label'      => 'Name',
            'allowEmpty' => false,
            'filters'    => array('StringTrim'),
            'validators' => array(
                array('validator' => 'NotEmpty', 'options' => array(
                    'messages' => array(
                        Zend_Validate_NotEmpty::IS_EMPTY => 'Name is required!'
                    )
                )),
                array('validator' => 'StringLength', 'options' => array(2, 64))
             ),
            'decorators' => array(
                array('ViewScript', array('viewScript' => 'forms/elements/text-input.phtml')),
                'ViewHelper'
            )
        ));
        
        $locale = Singular_Loader::get('Zend_Locale')->getLocaleList();
        foreach ($locale as $key=>$val) {
            if (strlen($key) == 2) {
                $languages[$key] = $key;
            }
        }
        
        $this->addElement('select', 'code', array(
            'label'              => $this->getView()->t('Code'),
            'allowEmpty'         => false,
            'disableTranslator'  => true,
            'filters'            => array('StringTrim'),
            'validators'         => array(
                array('validator' => 'Db_NoRecordExists', 'options' => array(
                    'table' => Singular_Loader::get('System_Model_DbTable_Languages')->getTableName(),
                    'field' => 'code',
                    'messages' => array(
                        Zend_Validate_Db_NoRecordExists::ERROR_RECORD_FOUND => 'Locale already in use'
                    )
                )),
                array('validator' => 'InArray', 'options' => array(
                    'haystack' => $languages,
                    'messages' => array(
                        Zend_Validate_InArray::NOT_IN_ARRAY => 'Invalid locale'
                    )
                ))
             ),
            'multiOptions'    => $languages,
            'decorators' => array(
                array('ViewScript', array('viewScript' => 'forms/elements/select.phtml')),
                'ViewHelper'
            )
        ));

        $this->addElement('submit', 'submit', array(
            'ignore'   => true,
            'label'    => 'Submit',
            'attribs'    => array(
                'class' => 'button-type2'
            ),
            'decorators' => array(
                array('ViewScript', array('viewScript' => 'forms/elements/button.phtml')),
                'ViewHelper'
            )
        ));
    }
}
