<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Login form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 */
class System_Form_Login extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init ()
    {
        parent::init();

        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            new Zend_Form_Decorator_ViewScript(array(
                    'viewScript' => 'forms/auth.phtml'
                )),
            'Form'
        ));
        
        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $this->setAttrib('data-ajax', "false");
        }
        
        $this->setAction($this->getView()->url(array(), 'login'));

        $this->addElement('text', 'email', array(
            'id'         => 'login-email',
            'label'      => 'Email',
            'allowEmpty' => false,
            'filters'    => array('StringTrim'),
            'validators' => array(
                array('validator' => 'NotEmpty', 'options' => array(
                    'messages' => array(
                        Zend_Validate_NotEmpty::IS_EMPTY => 'Email is required!'
                    )
                )),
                array('validator' => 'EmailAddress', 'options' => array(
                    'messages' => array(
                        Zend_Validate_EmailAddress::INVALID => 'Email is required!',
                        Zend_Validate_EmailAddress::INVALID_FORMAT => 'Email is incorrect!',
                        Zend_Validate_EmailAddress::INVALID_LOCAL_PART => 'Email is incorrect!',
                        Zend_Validate_EmailAddress::INVALID_HOSTNAME => 'Email is incorrect!',
                        Zend_Validate_EmailAddress::INVALID_SEGMENT => 'Email is incorrect!',
                        Zend_Validate_EmailAddress::INVALID_MX_RECORD => 'Email is incorrect!'
                    )
                ))
            ),
            'decorators' => array(
                'ViewHelper'
            )
        ));
        
        $this->addElement('password', 'password', array(
            'label'     => 'Password',
            'allowEmpty' => false,
            'validators' => array(
                array('validator' => 'NotEmpty', 'options' => array(
                    'messages' => array(
                        Zend_Validate_NotEmpty::IS_EMPTY => 'Password is required!'
                    )
                )),
            ),
            'decorators' => array(
                'ViewHelper'
            )
        ));
        
        $this->addElement('checkbox', 'is_remember', array(
            'label'     => 'Remember me for 14 days'
        ));

        $this->addElement('submit', 'submit', array(
            'id'          => 'login-submit',
            'label'       => 'Login',
            'description' => '<a title="' . $this->getView()->t('Forgot Password') . '" class="forgotPass" href="' . $this->getView()->url(array('layout' => $this->getView()->layout()->getLayout()), 'forgot_password') . '">' . $this->getView()->t('Forgot Password') . '?</a>',
            'ignore'      => true,
            'Attribs'     => array(
                'class'   => 'button-type1'
            )
        ));
    }
}
