<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Registration form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 */
class System_Form_Registration extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();

        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            new Zend_Form_Decorator_ViewScript(array(
                    'viewScript' => 'forms/auth.phtml'
                )),
            'Form'
        ));

        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $this->setAttrib('data-ajax', "false");
        }
        
        $this->setAction($this->getView()->url(array(), 'registration'));

        $assign = new Zend_Form_Element_Hidden('assign',
            array(
                 'decorators' => array(
                     'ViewHelper'
                 ),
                 'value' => '0x0000'
            ));
        $assign->addValidator(new System_Lib_Validate_Registration());

        $this->addElement($assign);

        $this->addElement('text', 'full_name', array(
            'label'      => 'Full Name',
            'allowEmpty' => false,
            'filters'    => array('StringTrim'),
            'validators' => array(
                array('validator' => 'NotEmpty', 'options' => array(
                    'messages' => array(
                        Zend_Validate_NotEmpty::IS_EMPTY => 'Full Name is required!'
                    )
                )),
                array('validator' => 'StringLength', 'options' => array(
                    4,
                    100,
                    'messages' => array(
                        Zend_Validate_StringLength::TOO_SHORT => 'Full Name should be at least 4 characters!'
                    )))
             ),
            'decorators' => array(
                'ViewHelper'
            )
        ));

        $messages = array(
            'emailRequired'  => 'Email is required!',
            'emailIncorrect' => 'Email is incorrect!'
        );
        $this->addElement('text', 'email', array(
            'id'          => 'registration-email',
            'label'       => 'Email',
            'allowEmpty'  => false,
            'filters'     => array('StringTrim'),
            'description' => $this->getView()->t('Email is not shown in public'),
            'validators'  => array(
                array('validator' => 'NotEmpty', 'options' => array(
                    'messages' => array(
                        Zend_Validate_NotEmpty::IS_EMPTY => 'Email is required!'
                    )
                )),
                array('validator' => 'EmailAddress', 'options' => array(
                    'messages' => array(
                        Zend_Validate_EmailAddress::INVALID            => $messages['emailRequired'],
                        Zend_Validate_EmailAddress::INVALID_FORMAT     => $messages['emailIncorrect'],
                        Zend_Validate_EmailAddress::INVALID_LOCAL_PART => $messages['emailIncorrect'],
                        Zend_Validate_EmailAddress::INVALID_HOSTNAME   => $messages['emailIncorrect'],
                        Zend_Validate_EmailAddress::INVALID_SEGMENT    => $messages['emailIncorrect'],
                        Zend_Validate_EmailAddress::INVALID_MX_RECORD  => $messages['emailIncorrect']
                    )
                )),
                array('validator' => 'Db_NoRecordExists', 'options' => array(
                    'table' => Singular_Loader::get('System_Model_DbTable_Users')->getTableName(),
                    'field' => 'email',
                    'messages' => array(
                        Zend_Validate_Db_NoRecordExists::ERROR_RECORD_FOUND => 'Email already in use'
                    )
                ))
            ),
            'decorators' => array(
                'ViewHelper'
            )
        ));
        
        $this->addElement('submit', 'submit', array(
            'id'          => 'registration-submit',
            'label'       => 'Register',
            'ignore'      => true,
            'Attribs'     => array(
                'class'   => 'button-type1'
            ),
            'decorators' => array(
                'ViewHelper'
            )
        ));
    }
}
