<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Response form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 */
class System_Form_Response extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();

        $loggedUser = Singular_Runtime::extract('loggedUser');
        $anonymousResponses = Singular_Core::_('settings')->getSetting('anonymous_responses');
        
        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            array('ViewScript', array('viewScript' => 'forms/response-add.phtml')),
            'Form',
        ));

        $this->addAttribs(array(
            'id' => 'response-add'
        ));
        
        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $this->setAttrib('data-ajax', "false");
        }
        
        $title = new Zend_Form_Element_Text('title', array(
            'label'      => 'Title',
            'allowEmpty' => false,
            'filters'    => array('StringTrim'),
            'validators' => array(),
            'decorators' => array(
                array('ViewHelper'),
                array('ViewScript', array('viewScript' => 'forms/elements/title.phtml'))
            ),
        ));
        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $title->setAttrib("data-role", "none");
        }

        $currentRoute = Zend_Controller_Front::getInstance()
                        ->getRouter()->getCurrentRouteName();
        
        if ($currentRoute == 'response_add'
                && !($loggedUser->isAllowed('can_access_admin')
                        || $loggedUser->isAllowed('representative')
                        || $loggedUser->isAllowed('manage_responses'))
                && !($loggedUser->isGuest() && $anonymousResponses) ) {
            
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $select = $responsesDb->select();
            $select->where('created_by_id=?', $loggedUser->id)
                   ->where('created_at>?', date("Y-m-d H:i:s", time() - 60)); // -1 min
            $validator = new Zend_Validate_Db_NoRecordExists($select);
            $validator->setMessages(array(
                Zend_Validate_Db_NoRecordExists::ERROR_RECORD_FOUND => 'You can add object once a minute. Please wait...'
            ));
            $title->addValidator($validator);
        }
        
        $validator =  new System_Lib_Validate_StringLengthBbcode(array(
            'min' => 3,
            'max' => null,
        ));
        $validator->setMessages(array(
            Zend_Validate_StringLength::TOO_SHORT => 'Description should be at least 3 characters!'
        ));
        
        $description = new System_Form_Element_Content('content', array(
            'label'      => 'Description',
            'description'=> $this->getView()->t('Describe your :responseType in details to get quick response', array('responseType' => '<span class="changed-text">' . strtolower($this->getView()->defaultType->title) . '</span>')),
            'allowEmpty' => false,
            'filters'    => array('StringTrim'),
            'validators' => array(
                $validator
            ),
            'attribs'    => array(
                'id'     => 'ta_text'
            )
        ));
        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $description->setLabel('');
            $description->setDescription('');
            $description->setDecorators(array(
                array('ViewHelper'),
                array('ViewScript', array('viewScript' => 'forms/elements/mobile-textarea.phtml'))
            ));
            $description->setAttrib('id', 'content');
            $description->setAttrib("data-role", "none");
        }
        $this->addDisplayGroup(array($title, $description), 'content');

        $params = array(
            'required'   => true,
            'filters'    => array('StringTrim'),
            'validators' => array(
                array('validator' => 'NotEmpty', 'options' => array(
                    'messages' => array(
                        Zend_Validate_NotEmpty::IS_EMPTY => 'Type is required!'
                    )
                ))
            ),
            'decorators' => array(
                array('ViewHelper'),
                array('ViewScript', array('viewScript' => 'forms/elements/types.phtml'))
            ),
            'value'      => $this->getView()->defaultType->slug,
        );
        
        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $params['attribs'] = array(
                'data-mini' => true
            );
            $params['multiOptions'][''] = $this->getView()->t('Select Type');
            $params['value'] = '';
            $types = new Zend_Form_Element_Select('type', $params);
        } else {
            $types = new Zend_Form_Element_Radio('type', $params);
        }
        
        $this->addDisplayGroup(array($types), 'types');

        $tags = new Zend_Form_Element_Text('tags', array(
            'label'       => 'Add Tags...',
            'description' => 'Separated with commas (rock-star, music, cold water)',
            'allowEmpty'  => true,
            'isArray'     => true,
            'filters'     => array('StringTrim', 'StripTags'),
            'decorators'  => array(
                array('ViewHelper'),
                array('ViewScript', array('viewScript' => 'forms/elements/input-tags.phtml'))
            ),
        ));
        if (Singular_Core::_('Theme')->getDefault() != 'mobile') {
            $this->addDisplayGroup(array($tags), 'add-tags-block');
        }
        $otherGroup = array();

        if ($loggedUser->isGuest() && $anonymousResponses) {
            $anonymousGroup = array();
            if (Singular_Core::_('settings')->getSetting('anonymous_captcha')) {
                $captcha = new System_Lib_Captcha_Image(array(
                    'timeout'  => 300,
                    'width'    => 80,
                    'height'   => 26,
                    'fontSize' => 14,
                    'divClass' => 'captcha-img filter-active',
                    'font'     => ROOT_PATH . DS . 'public' . DS . 'files' . DS . 'logo' . DS . 'trebucbd.ttf',
                    'imgDir'  => ROOT_PATH . '/public/files/',
                    'imgUrl'  => $this->getView()->baseUrl('/public/files/'),
                ));
                $captchaElement = new Zend_Form_Element_Captcha('captcha', array(
                    'label'       => 'Enter Captcha',
                    'description' => 'Are you human?',
                    'order'       => 97,
                    'attribs'     => array(
                        'data-role' => 'none'
                    ),
                    'captcha'     => $captcha,
                    'decorators' => array(
                        array('Captcha_Word'),
                        array(array('div1' => 'htmlTag'), array('tag' => 'div', 'class' => 'textInput error')),
                        array('Captcha', array('alt' => '', 'placement' => 'PREPEND')),
                        array(array('div' => 'htmlTag'), array('tag' => 'div', 'class' => 'captcha')),
                    )
                ));
                array_push($anonymousGroup, $captchaElement);
            }

            $messages = array(
                'emailRequired'  => 'Email is required!',
                'emailIncorrect' => 'Email is incorrect!'
            );
            $email = new Zend_Form_Element_Text('notify_email', array(
                'label'       => 'Email',
                'description' => 'Provide email if you want to follow response',
                'order'       => 95,
                'validators'  => array(
                    array('validator' => 'NotEmpty', 'options' => array(
                        'messages' => array(
                            Zend_Validate_NotEmpty::IS_EMPTY => 'Email is required!'
                        )
                    )),
                    array('validator' => 'EmailAddress', 'options' => array(
                        'messages' => array(
                            Zend_Validate_EmailAddress::INVALID            => $messages['emailRequired'],
                            Zend_Validate_EmailAddress::INVALID_FORMAT     => $messages['emailIncorrect'],
                            Zend_Validate_EmailAddress::INVALID_LOCAL_PART => $messages['emailIncorrect'],
                            Zend_Validate_EmailAddress::INVALID_HOSTNAME   => $messages['emailIncorrect'],
                            Zend_Validate_EmailAddress::INVALID_SEGMENT    => $messages['emailIncorrect'],
                            Zend_Validate_EmailAddress::INVALID_MX_RECORD  => $messages['emailIncorrect']
                        )
                    )),
                    array('validator' => 'Db_NoRecordExists', 'options' => array(
                        'table'    => Singular_Loader::get('System_Model_DbTable_Users')->getTableName(),
                        'field'    => 'email',
                        'exclude'  => 'logged_at IS NOT NULL',
                        'messages' => array(
                            Zend_Validate_Db_NoRecordExists::ERROR_RECORD_FOUND => 'Email already in use'
                        )
                    ))
                ),
                'decorators' => array(
                    array('ViewHelper'),
                    array(array('textInput' => 'htmlTag'), array('tag' => 'div', 'class' => 'textInput')),
                    array('Description', array('tag' => 'p', 'class' => '', 'placement' => 'PREPEND')),
                    array('Label', array('placement' => 'PREPEND')),
                    array(array('div' => 'htmlTag'), array('tag' => 'div', 'class' => 'floatLeft notify-email-container')),
                )
            ));
            if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
                $email->setAttrib('data-role', 'none');
            }
            array_push($anonymousGroup, $email);

            $hidden = new Zend_Form_Element_Hidden('as_guest', array(
                'value'      => '0',
                'decorators' => array(
                    array('ViewHelper')
                )
            ));
            array_push($anonymousGroup, $hidden);

            $this->addDisplayGroup($anonymousGroup, 'anonymous');

            if (Singular_Core::_('Theme')->getDefault() != 'mobile') {
                $link = new Zend_Form_Element_Button('submit_guest_link', array(
                    'ignore'   => true,
                    'label'    => 'Anonymously',
                    'order'    => 100,
                    'attribs'  => array(
                        'class' => 'floatLeft'
                    ),
                    'decorators' => array(
                        array('ViewHelper'),
                        array('ViewScript', array('viewScript' => '/forms/elements/link.phtml')),
                    ),
                ));
                if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
                    $link->setAttrib('class', 'btn-anonym');
                    $link->getDecorator('viewScript')
                         ->setOption('viewScript', '/forms/elements/link-anon.phtml');
                }
                array_push($otherGroup, $link);
            }
        }

        $submit = new Zend_Form_Element_Submit('submit', array(
            'ignore'   => true,
            'label'    => 'Post Response',
            'order'    => 99,
            'attribs'  => array(
                'class' => 'button-type2  floatLeft'
            ),
            'decorators' => array(
                array('ViewHelper'),
                array('ViewScript', array('viewScript' => '/forms/elements/button.phtml')),
            ),
        ));
        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $submit->setAttrib('class', 'btn-login light');
            $submit->getDecorator('viewScript')
                   ->setOption('viewScript', '/forms/elements/button2.phtml');
        }
        array_push($otherGroup, $submit);
        
        $this->addDisplayGroup($otherGroup, 'other');
    }
    
    /**
     * Validate the form
     *
     * @param  array $data
     * @return boolean
     */
    public function isValid($data)
    {
        if (isset($data['notify_email']) && !empty($data['notify_email'])) {
            $email = $this->getElement('notify_email');
            if ($email instanceof Zend_Form_Element) {
                $email->setAllowEmpty(false);
            }
        }
        
        if ( !($this instanceof System_Form_ResponseWidget) ) {
            $loggedUser = Singular_Runtime::extract('loggedUser');
            if ($loggedUser->isGuest() && isset($data['as_guest']) && $data['as_guest']) {
                $link = $this->getElement('submit_guest_link');
                if ($link) {
                    $link->setAttrib('class', 'floatLeft back');
                    $link->setLabel('Back');
                }
                // is_submitted flag for mobile theme
                if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
                    $submitted = new Zend_Form_Element_Hidden('is_submitted', array(
                        'value'      => '1',
                        'decorators' => array(
                            array('ViewHelper')
                        )
                    ));
                    $this->getDisplayGroup('other')->addElement($submitted);
                }
            }
        }

        return parent::isValid($data);
    }
}
