<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * ResponseStatus form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 */
class System_Form_ResponseStatus extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see parent::init()
     */
    public function init()
    {
        parent::init();

        $this->setAction($this->getView()->url(array('operation' => 'add'), 'admin_status_manage'));

        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            new Zend_Form_Decorator_ViewScript(array(
                    'viewScript' => 'forms/response-status.phtml'
                )),
            'Form'
        ));

        // Element "type"
        $type = new Zend_Form_Element_Select('type',
            array(
                 'label'      => 'Type',
                 'required'   => true,
                 'attribs'    => array(
                     'class' => 'width100p'
                 ),
                 'filters'    => array('StringTrim'),
                 'validators' => array(
                     new Zend_Validate_NotEmpty()
                 ),
                 'decorators' => array(
                     array('ViewScript', array('viewScript' => 'forms/elements/select.phtml')),
                     'ViewHelper'
                 )
            ));

        $types = Singular_Runtime::extract('types');
        if (!$types instanceof System_Model_Rowset) {
            throw new Singular_Exception('Invalid types object');
        }
        foreach ($types as $_type) {
            if ($_type->has_votes) {
                $type->addMultiOption($_type->slug, $_type->title);
            }
        }
        $this->addElement($type);

        // Element "color"
        $color = new Zend_Form_Element_Text('color',
            array(
                 'label'      => 'Background Color',
                 'required'   => true,
                 'attribs'    => array(
                     'class' => 'color'
                 ),
                 'filters'    => array('StringTrim'),
                 'validators' => array(
                     new Zend_Validate_NotEmpty()
                 ),
                 'decorators' => array(
                     array('ViewScript', array('viewScript' => 'forms/elements/color-input.phtml')),
                     'ViewHelper'
                 )
            ));

        $color->setValue('40BF00');
        $this->getView()->headScript()->appendFile($this->getView()->javascriptSrc('jscolor/jscolor.js', null, false));
        $this->addElement($color);

        // Element "is_default"
        $isDefault = new Zend_Form_Element_Radio('is_default',
            array(
                 'label'      => 'Default',
                 'required'   => true,
                 'attribs'    => array(
                     'class' => 'width125'
                 ),
                 'validators' => array(
                     new Zend_Validate_NotEmpty(),
                     new Zend_Validate_Digits()
                 ),
                 'decorators' => array(
                     array('ViewScript', array('viewScript' => 'forms/elements/multi-radio.phtml')),
                     'ViewHelper'
                 )
            ));
        $isDefault->addMultiOptions(
            array(
                 '1' => 'Yes',
                 '0' => 'No'
            )
        );
        $this->addElement($isDefault);

        // Element "pay_attention"
        $payAttention = new Zend_Form_Element_Radio('pay_attention',
            array(
                 'label'      => 'Pay attention',
                 'required'   => true,
                 'attribs'    => array(
                     'class' => 'width125'
                 ),
                 'validators' => array(
                     new Zend_Validate_NotEmpty(),
                     new Zend_Validate_Digits()
                 ),
                 'decorators' => array(
                     array('ViewScript', array('viewScript' => 'forms/elements/multi-radio.phtml')),
                     'ViewHelper'
                 )
            ));
        $payAttention->addMultiOptions(
            array(
                 '1' => 'Yes',
                 '0' => 'No'
            )
        );
        $this->addElement($payAttention);

        // Element "title"
        $title = new Zend_Form_Element_Text('title',
            array(
                 'label'      => 'Title',
                 'required'   => true,
                 'attribs'    => array(),
                 'filters' => array('StringTrim'),
                 'validators' => array(
                     new Zend_Validate_NotEmpty()
                 ),
                 'decorators' => array(
                     array('ViewScript', array('viewScript' => 'forms/elements/text-input.phtml')),
                     'ViewHelper'
                 )
            ));
        $title->getValidator('Zend_Validate_NotEmpty')->setMessages(
            array(
                 Zend_Validate_NotEmpty::IS_EMPTY => 'Title is required and can\'t be empty',
            )
        );
        $this->addElement($title);

        // Element "text_color"
        $textColor = new Zend_Form_Element_Text('text_color',
            array(
                 'label'      => 'Text Color',
                 'required'   => true,
                 'attribs'    => array(
                     'class' => 'color'
                 ),
                 'filters'    => array('StringTrim'),
                 'validators' => array(
                     new Zend_Validate_NotEmpty()
                 ),
                 'decorators' => array(
                     array('ViewScript', array('viewScript' => 'forms/elements/color-input.phtml')),
                     'ViewHelper'
                 )
            ));

        $textColor->setValue('FFFFFF');
        $this->getView()->headScript()->appendFile($this->getView()->javascriptSrc('jscolor/jscolor.js', null, false));
        $this->addElement($textColor);

        // Element "slug"
        $slug = new Zend_Form_Element_Text('slug',
            array(
                 'label'      => 'Slug',
                 'required'   => true,
                 'attribs'    => array(),
                 'filters'    => array('StringTrim'),
                 'validators' => array(
                     new Zend_Validate_NotEmpty(),
                     new Zend_Validate_Db_NoRecordExists(
                         array(
                              'table' => Singular_Loader::get('System_Model_DbTable_ResponsesStatuses')->getTableName(),
                              'field' => 'slug',
                         ))
                 ),
                 'decorators' => array(
                     array('ViewScript', array('viewScript' => 'forms/elements/text-input.phtml')),
                     'ViewHelper'
                 )
            ));
        $slug->getValidator('Zend_Validate_NotEmpty')->setMessages(
            array(
                 Zend_Validate_NotEmpty::IS_EMPTY => 'Slug is required and can\'t be empty',
            )
        );
        $slug->getValidator('Zend_Validate_Db_NoRecordExists')->setMessages(
            array(
                 Zend_Validate_Db_NoRecordExists::ERROR_RECORD_FOUND => 'Slug already exists! Please change it!',
            )
        );
        $this->addElement($slug);

        $this->addElement('submit', 'submit', array(
            'ignore'   => true,
            'order'    => 10,
            'label'    => 'Submit',
            'attribs'  => array(
                'class' => 'button-type2 inline'
            ),
            'decorators' => array(
                array('ViewScript', array('viewScript' => '/forms/elements/button.phtml')),
                'ViewHelper'
            )
        ));
    }
}
