<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Response Widget form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 */
class System_Form_ResponseWidget extends System_Form_Response
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();
        
        $this->setDecorators(array(
            array('ViewScript', array('viewScript' => 'forms/response-add-widget.phtml')),
            'Form',
        ));
        
        $element = $this->getElement('title');
        if ($element instanceof Zend_Form_Element) {
            $element->setValue($element->getLabel());
            $element->setLabel('');
            $filter = new Zend_Filter_Callback(array(
                'callback' => 'System_Form_ResponseWidget::filter',
                'options'  => array(
                    'value' => $this->getView()->t('Title')
                )
            ));
            $element->addFilter($filter);
            $element->setDescription('');
            $element->getDecorator('viewScript')
                    ->setOption('viewScript', 'forms/elements/search.phtml');
        }
        
        $element = $this->getElement('content');
        if ($element instanceof Zend_Form_Element) {
            $element->setValue($element->getLabel());
            $element->setLabel('');
            $element->setDescription('');
            $filter = new Zend_Filter_Callback(array(
                'callback' => 'System_Form_ResponseWidget::filter',
                'options'  => array(
                    'value' => $this->getView()->t('Description')
                )
            ));
            $element->addFilter($filter);
            $element->rows = 5;
            $element->removeDecorator('viewScript');
            $element->addDecorator('htmlTag', array('tag' => 'div', 'class' => 'contentInput'));
            unset($element->allowedCodes);
        }

        $element = $this->getElement('type');
        if ($element instanceof Zend_Form_Element) {
            $element->setValue('');
        }
        
        $this->removeElement('tags');
    }
    
    /**
     * Filter default values
     *
     * @param string $value
     * @param string $filter
     * @return string
     */
    static public function filter($value, $filter)
    {
        if (Zend_Controller_Front::getInstance()->getRequest()->isPost()) {
            if (str_replace($filter, '', $value) == '') {
                return '';
            }
        }
        return $value;
    }
    
    /**
     * Renders form
     *
     * @param  Zend_View_Interface $view
     * @return string
     */
    public function render(Zend_View_Interface $view = null)
    {
        $element = $this->getElement('content');
        if ($element instanceof Zend_Form_Element
            && $element->hasErrors()) {
            $decorator = $element->getDecorator('htmlTag');
            $decorator->setOption('class', 'contentInput error');
        }
        $element = $this->getElement('submit_guest_link');
        if ($element instanceof Zend_Form_Element_Button
            && $this->isErrors()) {
            $element->setLabel('Back');
            $element->setAttrib('class', 'floatLeft back');
        }
        return parent::render($view);
    }
}
