<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * ResponsesSettings form
 *
 * @uses       System_Form_Settings
 * @category   System
 * @package    System_Form
 * @property   Zend_Form_Element_Radio          $homepage_content
 * @property   Zend_Form_Element_Select         $homepage_responses_display_variant
 * @property   Zend_Form_Element_Select         $homepage_responses_items_num
 * @property   Zend_Form_Element_Select         $show_comments
 * @property   Zend_Form_Element_Select         $top_responses
 * @property   System_Lib_Form_Element_Sortable $response_types_order
 */
class System_Form_ResponsesSettings extends System_Form_Settings
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();

        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            new Zend_Form_Decorator_ViewScript(array(
                    'viewScript' => 'forms/responses-settings.phtml'
                )),
            'Form'
        ));

        if ($this->homepage_responses_display_variant instanceof Zend_Form_Element_Select) {
            $this->homepage_responses_display_variant->setMultiOptions(
                array(
                     'divided_by_type'     => $this->getView()->t('Divided by Type'),
                     'divided_by_category' => $this->getView()->t('Divided by Category'),
                     'mixed'               => $this->getView()->t('Mixed'),
                )
            );
            $this->homepage_responses_display_variant->setAttrib('class', 'width240');
        }

        if ($this->homepage_responses_items_num instanceof Zend_Form_Element_Select) {
            $numOptions = range(1, 20);
            $numOptions = array_combine($numOptions, $numOptions);
            $this->homepage_responses_items_num->setMultiOptions($numOptions);
        }
        
        if ($this->show_comments instanceof Zend_Form_Element_Select) {
            $this->show_comments->setMultiOptions(array(
                1 => 'Ascending',
                0 => 'Descending'
            ));
        }
        
        if ($this->top_responses instanceof Zend_Form_Element_Select) {
            $this->top_responses->setMultiOptions(array(
                'popular' => 'Popular',
                'new'     => 'New',
                'updated' => 'Updated',
            ));
        }

        $responseTypesOrder = new System_Lib_Form_Element_Sortable('response_types_order',
            array(
                 'label'       => $this->getView()->t('Order of Response Types'),
                 'description' => $this->getView()->t('Drag & Drop the types to get required order system wide'),
                 'allowEmpty'  => false,
                 'filters'     => array('StringTrim'),
            )
        );
        $responseTypesOrder->setDecorators(
            array(
                 array('ViewScript', array('viewScript' => 'forms/elements/sortable.phtml')),
                 'ViewHelper'
            )
        );

        $types = Singular_Loader::get('System_Model_DbTable_ResponsesTypes')->findActive();
        foreach ($types as $type) {
            if (!(bool)$type->getModule()->active) continue;
            $responseTypesOrder->addSortable($type->slug,
                array(
                     'title' => $this->getView()->t(
                         $this->getView()->responseTypeTitle($type)
                     ),
                     'icon'  => $type->getImage()
                )
            );
        }

        $this->addElement($responseTypesOrder);
    }
}
