<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Role form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 */
class System_Form_Role extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();

        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            new Zend_Form_Decorator_ViewScript(array(
                    'viewScript' => 'forms/system.phtml'
                )),
            'Form'
        ));

        $this->addElement('text', 'name', array(
            'label'      => 'New Role',
            'order'      => 0,
            'allowEmpty' => false,
            'filters'    => array('StringTrim'),
            'validators' => array(
                array('validator' => 'NotEmpty', 'options' => array(
                    'messages' => array(
                        Zend_Validate_NotEmpty::IS_EMPTY => 'Role name is required!'
                    )
                )),
                array('validator' => 'StringLength', 'options' => array(3, 255))
             ),
            'decorators' => array(
                array('ViewScript', array('viewScript' => '/forms/elements/text-input.phtml')),
                'ViewHelper'
            )
        ));

        $resources = new System_Form_Element_MultiCheckbox(array(
            'name'    => 'resource[]',
            'order'   => 1,
            'options' => array(
                'label' => 'Resources:',
                'required' => false,
                'decorators' => array(
                    array('ViewScript', array('viewScript' => '/forms/elements/multi-checkbox.phtml')),
                    'ViewHelper'
                )
            ),
            'settings' => array(
                'model' => 'System_Model_DbTable_Resources',
                'val' => 'resource',
                'opt' => 'name'
            ),
        ));
        $this->addElement($resources);

        $this->addElement('submit', 'submit', array(
            'ignore'   => true,
            'order'    => 10,
            'label'    => 'Submit',
            'attribs'  => array(
                'class' => 'button-type2 inline'
            ),
            'decorators' => array(
                array('ViewScript', array('viewScript' => '/forms/elements/button.phtml')),
                'ViewHelper'
            )
        ));
    }
}
