<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Settings form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 */
class System_Form_Settings extends Singular_Form
{
    const YES_NO        = 'yes_no';
    const TEXT          = 'text';
    const EMAIL         = 'email';
    const PHONE         = 'phone';
    const NUMBER        = 'number';
    const THEMES        = 'themes';
    const ADMIN_THEMES  = 'admin_themes';
    const SELECT        = 'select';
    const IMAGE         = 'image';
    const TEXTAREA      = 'textarea';
    const PASSWORD      = 'password';
    const RADIO         = 'radio';
    const COLOR         = 'color';
    const CHECKBOX      = 'checkbox';
    const SORTABLE      = 'sortable';

    /**
     * Settings array
     *
     * @var array
     */
    protected $settings = null;

    /**
     * Constructor
     *
     * Registers form view helper as decorator
     *
     * @see   parent::__construct()
     * @param Zend_Config $settings
     * @param mixed $defaults
     * @param mixed $options
     */
    public function  __construct($settings, $defaults = null, $options = null) {

        if($settings instanceof Zend_Config){
            $this->settings = $settings->toArray();
        }

        parent::__construct($defaults, $options);
    }

    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();

        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            new Zend_Form_Decorator_ViewScript(array(
                    'viewScript' => 'forms/system.phtml'
                )),
            'Form'
        ));

        if (is_array($this->settings)) {
            foreach ($this->settings as $setting) {
                $options = array(
                    'label'       => $setting['title'],
                    'description' => $this->getView()->t($setting['description']),
                    'allowEmpty'  => false,
                    'filters'     => array('StringTrim'),
                    'validators'  => array(
                        array('validator' => 'NotEmpty', 'options' => array(
                            'messages' => array(
                                Zend_Validate_NotEmpty::IS_EMPTY => '"' . $setting['title'] . '" is required!'
                            )
                        ))
                    )
                );
                switch ($setting['type']){
                    case (self::ADMIN_THEMES) :
                        $options['settings'] = array(
                            'path' => ROOT_PATH . '/public/themes/',
                            'mask' => '/^(?:admin)/u',
                        );
                        $element = new System_Form_Element_Directories($setting['key'], $options);
                        break;
                    case (self::THEMES) :
                        $options['settings'] = array(
                            'path' => ROOT_PATH . '/public/themes/',
                            'mask' => '/^(?!mobile)/u',
                        );
                        $options['disableTranslator'] = true;
                        $options['label'] = $this->getView()->t($setting['title']);
                        $element = new System_Form_Element_Directories($setting['key'], $options);
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/default.phtml')),
                            'ViewHelper'
                        ));
                        break;
                    case (self::YES_NO) :
                        $element = new Zend_Form_Element_Radio($setting['key'], $options);
                        $element->addMultiOptions(array(
                            '1' => 'Yes',
                            '0' => 'No'
                        ));
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/multi-radio.phtml')),
                            'ViewHelper'
                        ));
                        break;
                    case (self::EMAIL) :
                        $element = new Zend_Form_Element_Text($setting['key'], $options);
                        $element->addValidator('EmailAddress');
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/text-input.phtml')),
                            'ViewHelper'
                        ));
                        break;
                    case (self::SELECT) :
                        $element = new Zend_Form_Element_Select($setting['key'], $options);
                        $element->setAttrib('class', 'width125');
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/select.phtml')),
                            'ViewHelper'
                        ));
                        break;
                    case (self::IMAGE) :
                        $this->setEnctype(self::ENCTYPE_MULTIPART);
                        $element = new System_Lib_Form_Element_File($setting['key'], $options);
                        $element->setAttribs(array(
                            'class' => 'file-upload'
                        ));
                                            
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/image.phtml')),
                            'File'
                        ));
                        break;
                    case (self::TEXTAREA) :
                        $element = new Zend_Form_Element_Textarea($setting['key'], $options);
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/textarea.phtml')),
                            'ViewHelper'
                        ));
                        break;
                    case (self::PASSWORD) :
                        $element = new Zend_Form_Element_Password($setting['key'], $options);
                        $element->setRenderPassword(true);
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/text-input.phtml')),
                            'ViewHelper'
                        ));
                        break;
                    case (self::RADIO) :
                        $element = new Zend_Form_Element_Radio($setting['key'], $options);
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/multi-radio.phtml')),
                            'ViewHelper'
                        ));
                        break;
                    case (self::COLOR) :
                        $this->getView()->headScript()->appendFile($this->getView()->javascriptSrc('jscolor/jscolor.js', null, false));
                        $element = new Zend_Form_Element_Text($setting['key'], $options);
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/color-input.phtml')),
                            'ViewHelper'
                        ));
                        $element->setAttrib("class", "color");
                        break;
                    case (self::CHECKBOX) :
                        $element = new Zend_Form_Element_Checkbox($setting['key'], $options);
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/checkbox.phtml')),
                            'ViewHelper'
                        ));
                        break;
                    case (self::SORTABLE) :
                        $element = new System_Lib_Form_Element_Sortable($setting['key'], $options);
                        $element->setDecorators(
                            array(
                                 array('ViewScript', array('viewScript' => 'forms/elements/sortable.phtml')),
                                 'ViewHelper'
                            )
                        );
                        break;
                    default :
                        $element = new Zend_Form_Element_Text($setting['key'], $options);
                        $element->setDecorators(array(
                            array('ViewScript', array('viewScript' => 'forms/elements/text-input.phtml')),
                            'ViewHelper'
                        ));
                        break;
                }
                $element->setValue($setting['value']);
                $this->addElement($element);
            }

            $this->addElement('submit', 'submit', array(
                'ignore'     => true,
                'label'      => 'Submit',
                'attribs'    => array('class' => 'button-type2'),
                'decorators' => array(
                    array('ViewScript', array('viewScript' => 'forms/elements/button.phtml')),
                    'ViewHelper'
                )
            ));
        }
    }
}
