<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Form
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * User form
 *
 * @uses       Singular_Form
 * @category   System
 * @package    System_Form
 */
class System_Form_User extends Singular_Form
{
    /**
     * Initialize form (used by extending classes)
     *
     * @see    parent::init()
     * @return void
     */
    public function init()
    {
        parent::init();

        $this->setDecorators(array(
            new System_Form_Decorator_FlashErrors(),
            new Zend_Form_Decorator_ViewScript(array(
                    'viewScript' => 'forms/profile.phtml'
                )),
            'Form'
        ));

        if (Singular_Core::_('theme')->getDefault() == 'mobile') {
            $this->setAttrib('data-ajax', 'false');
        }
        
        $oldPassword = new Zend_Form_Element_Password('old_password', array(
            'label'       => 'Change Password',
            'description' => 'Enter your current password',
            'validators'  => array(
                array('StringLength', false, array(5, 20))
            ),
            'decorators' => array(
                'ViewHelper',
                array('ViewScript', array('viewScript' => 'forms/elements/password-input.phtml'))
            )
        ));

        $password = new Zend_Form_Element_Password('password', array(
            'withoutLabel' => true,
            'description'  => 'New Password',
            'validators'   => array(
                array('identical', false, array(
                    'token' => 'password_again',
                    'messages' => array(
                        Zend_Validate_Identical::NOT_SAME => ''
                    )
                )),
                array('Alnum'),
                array('StringLength', false, array(3, 20))
            ),
            'decorators' => array(
                'HtmlTag',
                'ViewHelper',
                array('ViewScript', array('viewScript' => 'forms/elements/password-input.phtml'))
            )
        ));

        $passwordAgain = new Zend_Form_Element_Password('password_again', array(
            'withoutLabel' => true,
            'description'  => 'Password again',
            'validators'   => array(
                array('identical', false, array(
                    'token'    => 'password',
                    'messages' => array(
                        Zend_Validate_Identical::NOT_SAME => 'Passwords mismatch!'
                    )
                )),
                array('Alnum'),
                array('StringLength', false, array(3, 20))
            ),
            'decorators' => array(
                'ViewHelper',
                array('ViewScript', array('viewScript' => 'forms/elements/password-input.phtml'))
            )
        ));

        $this->addDisplayGroup(array($oldPassword, $password, $passwordAgain), 'change-password');

        $fullName = new Zend_Form_Element_Text('full_name', array(
            'id'          => 'full_name',
            'label'       => 'Full Name',
            'description' => 'Enter your nickname or full name for it to be displayed in the system',
            'required'    => true,
            'filters'     => array('StringTrim'),
            'validators'  => array(
                array('validator' => 'StringLength', 'options' => array(2, 100))
             ),
            'decorators'  => array(
                'ViewHelper',
                array('ViewScript', array('viewScript' => 'forms/elements/text-input.phtml'))
            )
        ));
        
        if (Singular_Core::_('theme')->getDefault() == 'mobile') {
            $fullName->setAttrib('data-role', 'none');
            $fullName->setDescription('');
        }

        $table = Singular_Loader::get('System_Model_DbTable_Users')->getTableName();
        $email = new Zend_Form_Element_Text('email', array(
            'id'          => 'email',
            'label'       => 'Email',
            'description' => 'Email is used to login and send notifications. Email is not shown in public',
            'required'    => true,
            'filters'     => array('StringTrim'),
            'validators'  => array(
                'EmailAddress',
                new Zend_Validate_Db_NoRecordExists(array(
                    'table' => $table,
                    'field' => 'email',
                    'exclude' => array(
                        'field' => 'email',
                        'value' => $this->getView()->activeUser->email
                    )
                ))
             ),
            'decorators'  => array(
                'ViewHelper',
                array('ViewScript', array('viewScript' => 'forms/elements/text-input.phtml'))
            )
        ));

        if (Singular_Core::_('theme')->getDefault() == 'mobile') {
            $email->setAttrib('data-role', 'none');
            $email->setDescription('');
        }
        
        $this->addDisplayGroup(array($fullName, $email), 'main');

        if (Singular_Core::_('theme')->getDefault() != 'mobile') {
            $maxFileSize = System_Service_Files::getMaxUploadSize();
            $this->setEnctype(self::ENCTYPE_MULTIPART);
            $avatar = new System_Lib_Form_Element_File('photo', array(
                    'id'          => 'photo',
                    'description' => $this->getView()->t("The image must be a GIF, JPEG or PNG that is smaller than :size.", array('size' => $maxFileSize)),
                    'imageSrc'    => $this->getView()->activeUser->getAvatarSrc(),
                    'deleteUrl'   => ($this->getView()->activeUser->photo) ? $this->getView()->url(array(), 'del_photo') : "",
                    'attribs'     => array(
                        'class' => 'file-upload'
                    ),
                    'filters'     => array(
                        new System_Filter_ResizeImage(array(
                            'targetDir'    => AVATAR_PATH,
                            'newSize'      => '50*50',
                            'saveOriginal' => false
                        )),
                        'HtmlEntities'
                    ),
                    'destination' => AVATAR_PATH,
                    'validators' => array(
                        array('Extension', false, array('gif','png','jpeg','jpg',
                            'messages' => array(
                                Zend_Validate_File_Extension::FALSE_EXTENSION => 'You are trying to use incorrect file type!'
                            )
                        )),
                    ),
                    'decorators' => array(
                        'ViewHelper',
                        array('File'), array('ViewScript', array('viewScript' => 'forms/elements/image2.phtml'))
                    )
                ));
            $this->addElement($avatar);

            $gravatar = new Zend_Form_Element_Text('gravatar_email', array(
                'id'          => 'gravatar_email',
                'description' => 'Enter email to retrieve your gravatar logo',
                'required'    => false,
                'filters'     => array('StringTrim', 'HtmlEntities', 'StripTags'),
                'validators'  => array(
                    'EmailAddress'
                 ),
                'decorators'  => array(
                    'ViewHelper',
                    array('ViewScript', array('viewScript' => 'forms/elements/gravatar.phtml'))
                )
            ));
            $this->addElement($gravatar);
        }

        $settingsDb = Singular_Loader::get('System_Model_DbTable_Settings');
        
        if (Singular_Core::_('theme')->getDefault() != 'mobile') {
            $setting = $settingsDb->getSetting(System_Model_DbTable_UsersConfigs::PHOTO_TYPE);

            $imageType = new Zend_Form_Element_Radio(System_Model_DbTable_UsersConfigs::PHOTO_TYPE, array(
                'attribs' => array(
                    'labelClass' => 'label'
                ),
                'MultiOptions' => array(
                    'avatar' => 'Avatar',
                    'gravatar' => 'Use Gravatar'
                 ),
                'advanceOptions' => array(
                    'avatar' => $avatar,
                    'gravatar' => $gravatar
                ),
                'decorators'  => array(
                    'ViewHelper',
                    array('ViewScript', array('viewScript' => 'forms/elements/multi-radio2.phtml'))
                ),
                'filters' => array('HtmlEntities', 'StripTags'),
                'BelongsTo'  => 'config'
            ));
            $imageType->setBelongsTo('config');
            $imageType->setValue($setting['value']);
            $this->addElement($imageType);
        }

        $settingTime = Singular_Core::_('Settings')->getSetting(System_Model_DbTable_UsersConfigs::SEND_NOTIFICATIONS_ONCE_TIME);
        $time = new System_Form_Element_Time(System_Model_DbTable_UsersConfigs::SEND_NOTIFICATIONS_ONCE_TIME, array(
            'label' => 'Time',
            'value'      => $settingTime,
            'attribs' => array(
                'class' => 'timeInput'
            ),
            'decorators' => array(
                'ViewHelper',
                array('ViewScript', array('viewScript' => 'forms/elements/time-input.phtml'))
            ),
            'BelongsTo'  => 'config'
        ));
        $this->addElement($time);
        $setting = $settingsDb->getSetting(System_Model_DbTable_UsersConfigs::SEND_NOTIFICATIONS);
        
        $params = array(
            'label'        => 'Send Notifications',
            'description'  => $this->getView()->t('All Notifications could be sent once a day or immediately'),
            'required'     => true,
            'value'        => $setting['value'],
            'filters'      => array('StringTrim', 'HtmlEntities', 'StripTags'),
            'multiOptions' => array(
                System_Model_UserConfig::IMMEDIATELY => 'Immediately',
                System_Model_UserConfig::ONCE => 'Once a day'
            ),
            'advanceOptions' => array(
                System_Model_UserConfig::ONCE => $time,
            ),
            'decorators'   => array(
                'ViewHelper',
                array('ViewScript', array('viewScript' => 'forms/elements/multi-radio.phtml'))
            ),
            'BelongsTo'  => 'config'
        );
        if (Singular_Core::_('theme')->getDefault() == 'mobile') {
            $params['attribs'] = array(
                'data-role' => 'slider'
            );
            $params['multiOptions'][System_Model_UserConfig::IMMEDIATELY] = 'On';
            $params['multiOptions'][System_Model_UserConfig::ONCE] = 'Off';
            $element = new Zend_Form_Element_Select(System_Model_DbTable_UsersConfigs::SEND_NOTIFICATIONS, $params);
            $element->getDecorator('viewScript')
                    ->setOption('viewScript', 'forms/elements/yes_no_adv.phtml');
        } else {
            $element = new Zend_Form_Element_Radio(System_Model_DbTable_UsersConfigs::SEND_NOTIFICATIONS, $params);
        }
        $other[0] = $element;
        
        $settings = $settingsDb->findByAll(array(
            'section' => 'user',
            'module'  => 'system',
            'type'    => 'checkbox'
        ));
        if ($settings->valid()) {
            foreach($settings as $setting) {
                $params = array(
                    'label'      => $setting->title,
                    'required'   => false,
                    'value'      => $setting->value,
                    'decorators' => array(
                        'ViewHelper',
                        array('ViewScript', array('viewScript' => 'forms/elements/checkbox.phtml'))
                    ),
                    'BelongsTo'  => 'config'
                );
                if (Singular_Core::_('theme')->getDefault() == 'mobile') {
                    $params['multiOptions'] = array(
                        '0' => 'On',
                        '1' => 'Off'
                    );
                    $params['attribs'] = array(
                        'data-role' => 'slider'
                    );
                    $element = new Zend_Form_Element_Select($setting->key, $params);
                    $element->getDecorator('viewScript')
                            ->setOption('viewScript', 'forms/elements/yes_no.phtml');
                } else {
                    $element = new Zend_Form_Element_Checkbox($setting->key, $params);
                }
                $notifications[] = $element;
            }
        }
        $this->addDisplayGroup($notifications, 'notifications');

        $this->addElement('submit','submit', array(
            'ignore'   => true,
            'label'    => 'Submit',
            'attribs'  => array(
                'class' => 'button-type2'
            ),
            'decorators' => array(
                'ViewHelper',
                array('ViewScript', array('viewScript' => 'forms/elements/button.phtml'))
            )
        ));

        if (Singular_Core::_('theme')->getDefault() == 'mobile') {
            $this->submit->setAttrib('class', 'btn-login light');
            $this->submit->getDecorator('ViewScript')
                         ->setOption('viewScript', 'forms/elements/button2.phtml');
        }
        
        $this->addDisplayGroup($other, 'other');

    }
}
