<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Handler
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * System handler
 *
 * @uses       Singular_Event_Listener_Abstract
 * @category   System
 * @package    System_Handler
 */
class System_Handler_Events extends Singular_Event_Listener_Abstract
{
    const APP_NAME = 'UseResponse';
    /**
     * Binds aggregates on specified events.
     *
     * @return void
     */
    public static function bindListeners ()
    {
        self::attachAggregate('onMainPage', array(__CLASS__, 'onMainPage'));
        self::attachAggregate('routeStartup', array(__CLASS__, 'onRouteStartup'), self::PRIORITY_NORMAL);
        self::attachAggregate('postDispatchOnce', array(__CLASS__, 'onPostDispatchOnce'));
        self::attachAggregate('postSelect', array(__CLASS__, 'onPostSelect'));
        self::attachAggregate('afterSwitchUserStatus', array(__CLASS__, 'onAfterSwitchUserStatus'));
        self::attachAggregate('afterRegistration', array(__CLASS__, 'onAfterRegistration'));
        self::attachAggregate('afterSwitchResponseStatus', array(__CLASS__, 'onAfterSwitchResponseStatus'));
        self::attachAggregate('responseAdded', array(__CLASS__, 'onResponseAdded'));
        self::attachAggregate('answerAdded', array(__CLASS__, 'onAnswerAdded'));
        self::attachAggregate('preDispatch', array(__CLASS__, 'onPreDispatch'));
        self::attachAggregate('dispatchLoopShutdown', array(__CLASS__, 'onDispatchLoopShutdown'));
        self::attachAggregate('afterAnalyticsReportsInit', array(__CLASS__, 'afterAnalyticsReportsInit'));
    }

    /**
     * Subscribes to "mainPage" event.
     *
     * @param  stdClass $sections
     *
     * @return void
     */
    public static function onMainPage($sections)
    {
        if (
            1 != Singular_Core::_("Settings")->getSetting('homepage_content_responses')
            || 'mixed' != Singular_Core::_("Settings")
                ->getSetting("homepage_responses_display_variant")
        ) {
            return;
        }

        $cacheId = Singular_Runtime::extract('loggedUser')->role . "homepage_mixed";
        $cacheId = preg_replace("/[^a-zA-Z0-9_]/", "", $cacheId);
        $tags    = null;
        $cache   = Singular_Runtime::extract('cachemanager')->getCache('default');

        if (($output = $cache->load($cacheId)) == false) {

            $responses = Singular_Loader::get('System_Model_DbTable_Responses')->findTop(
                System_Model_DbTable_Responses::RESPONSE_TYPE_ANY,
                Singular_Core::_("settings")->getSetting("homepage_responses_items_num")
            );

            self::getView()->cacheTags       = new StdClass();
            self::getView()->cacheTags->tags = array();
            self::getView()->paginator       = $responses;
            $output                          = self::getView()->render('_homepage-responses.phtml');

            $tags   = self::getView()->cacheTags->tags;
            $tags[] = 'responses';
            $cache->save($output, $cacheId, $tags);
        }

        $sections->sections[10] = $output;
        Resources_Service_Votes::setUserVotes($cacheId, $tags);
    }

    public static function onRouteStartup (Zend_Controller_Request_Abstract $request)
    {
        if (Zend_Auth::getInstance()->hasIdentity()) {
            $users      = Singular_Loader::get('System_Model_DbTable_Users');
            $userId     = Zend_Auth::getInstance()->getIdentity()->id;
            $loggedUser = $users->findActiveById($userId);
            if (!$loggedUser instanceof System_Model_User) {
                Singular_Core::_('auth')->logout();
                $loggedUser            = new System_Model_User();
                $loggedUser->role      = 'guest';
                $loggedUser->full_name = 'Guest';
                $loggedUser->email     = 'guest@localhost';
            }
        } else {
            $loggedUser       = new System_Model_User();
            $loggedUser->role = 'guest';
        }
        Singular_Runtime::store('loggedUser', $loggedUser);

        if (!Singular_Runtime::inStorage('types')) {
            $types = Singular_Loader::get('System_Model_DbTable_ResponsesTypes')->cache_findActive();
            if ($types instanceof System_Model_Rowset) {
                Singular_Runtime::store('types', $types);
            }
        }

        if (!Singular_Runtime::inStorage('statuses')) {
            $statuses = Singular_Loader::get('System_Model_DbTable_ResponsesStatuses')->cache_findAll();
            self::getView()->statuses = $statuses;
            Singular_Runtime::store('statuses', $statuses);
        }

        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            Singular_Core::_('Cache')->disableAll();
        }
    }

    /**
     * Subscribes to "postSelect" event.
     *
     * @param  Zend_Db_Select $select
     * @return void
     */
    public static function onPostSelect($select)
    {
        $from = $select->getPart(Zend_Db_Select::FROM);
        if (!empty($from)) {
            $tables = array(
                Singular_Loader::get('System_Model_DbTable_Responses')
                    ->getTableName(),
                Singular_Loader::get('Resources_Model_DbTable_ResponsesComments')
                    ->getTableName(),
            );
            foreach ($from as $tableName=>$data) {
                if (in_array($data['tableName'], $tables)) {
                    if (System_Service_Environment::getInstance()->isAdmin()) {
                        $select->where("{$tableName}.deleted_by_id <> 0");
                    } else {
                        $select->where("{$tableName}.deleted_by_id=?", 0);
                    }
                }
            }
        }
    }
    
    /**
     * Subscribes to "preDispatch" event.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public static function onPreDispatch ($request)
    {
        // Set App-client User data
        Singular_AppClient::setParam('userIsLoggedIn', Singular_Core::_('Auth')->hasIdentity());
        Singular_AppClient::setParam('loggedUser',     Singular_Core::_('Auth')->getPublicLoggedUserData());

        if (Singular_Core::_('settings')->getSetting('mobile_interface')) {
            if (Singular_Core::_('Theme')->getDefault() != 'mobile') {
                require_once "Mobile_Detect.php";
                $detect = new Mobile_Detect();
                if ($detect->isMobile()) {
                    $theme = Zend_Controller_Front::getInstance()->getRequest()->getCookie("checkTheme", 0);
                    if (empty($theme)) {
                        $theme = 1;
                        Singular_AppClient::setParam('useMobile', 1);
                    }
                    setcookie("checkTheme", $theme, null, "/");
                }
            } else {
                Singular_Core::_('Cache')->disableAll();
            }
        }

        if (Singular_Core::_('Theme')->getDefault() == 'mobile') {
            $isMobileTheme = true;
        } else {
            $isMobileTheme = false;
        }
        Singular_AppClient::setParam('mobileTheme', $isMobileTheme);

        // system permissions
        $nsp           = new Zend_Session_Namespace('R0LE');
        $nsp->assigned = self::APP_NAME;
    }
    /**
     * Subscribes to "postDispatchOnce" event.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public static function onPostDispatchOnce ($request)
    {
        $loggedUser = Singular_Runtime::extract("loggedUser");
        if (!($loggedUser instanceof System_Model_User)) {
            return;
        }

        $route   = Zend_Controller_Front::getInstance()->getRouter()->getCurrentRouteName();
        $cache   = Singular_Runtime::extract('cachemanager')->getCache('default');
        $cacheId = "breadcrumbs_" . $loggedUser->role;
        $cacheId = preg_replace("/[^a-zA-Z0-9_]/", "", $cacheId);
        if ( ($navigation = $cache->load($cacheId)) === false ) {
            
            $pages['main'] = array(
                'id'    => 'main-page',
                'label' => Singular_Core::_('Settings')->getSetting('site_title'),
                'route' => 'default',
                'pages' => array(
                    array(
                        'id'    => 'overview',
                        'label' => self::getView()->t('Overview'),
                        'route' => 'default'
                    ),
                    array(
                        'id'         => 'search',
                        'label'      => self::getView()->t('Search'),
                        'hide'       => true,
                        'module'     => 'system',
                        'controller' => 'responses',
                        'action'     => 'search'
                    ),
                    array(
                        'id'        => 'responses',
                        'menu_type' => (Singular_Core::_('theme')->getDefault() == 'mobile') ? 'top-menu' : 'top-bar',
                        'label'     => self::getView()->t('Responses'),
                        'order'     => 50,
                        'badge'     => null,
                        'route'     => 'responses',
                        'pages'     => array(
                            array(
                                'id'     => 'responses-all',
                                'label'  => self::getView()->t('All Responses'),
                                'route'  => 'responses',
                                'params' => array(
                                    'objects' => null
                                )
                            ),
                            array(
                                'id'     => 'response-add',
                                'label'  => self::getView()->t('New Response'),
                                'order'  => 10,
                                'route'  => 'response_add',
                                'params' => array(
                                    'type' => $request->getParam("type")
                                )
                            )
                        )
                    )
                )
            );

            if (Zend_Auth::getInstance()->hasIdentity()) {
                if ($loggedUser instanceof System_Model_User && $loggedUser->isAdmin()) {

                    $pages['main']['pages']['admin'] = array(
                        'id'        => 'admin',
                        'menu_type' => 'members-area',
                        'badge'     => null,
                        'label'     => self::getView()->t('Administration'),
                        'order'     => -99,
                        'route'     => 'admin'
                    );

                    /*$pages['main']['pages']['admin'] = array(
                        'id'         => 'admin',
                        'menu_type'  => 'top-menu',
                        'label'      => self::getView()->t('Administration'),
                        'route'      => 'admin'
                    );*/

                    $pages['main']['pages']['admin']['pages'] = array(
                        'overview'       => array(
                            'id'    => 'admin-overview',
                            'label' => self::getView()->t('Overview'),
                            'route' => 'admin'
                        ),
                        'admin-settings' => array(
                            'id'         => 'admin-settings',
                            'menu_type'  => 'admin-sidebar',
                            'image'      => self::getView()->imageSrc('/icons/16/settings.png', null, true),
                            'label'      => self::getView()->t('Settings'),
                            'order'      => 10,
                            'route'      => 'admin_settings'
                        ),
                        'admin-scheduler' => array(
                            'id'         => 'admin-scheduler',
                            'class'      => 'gray',
                            'menu_type'  => 'admin-sidebar',
                            'label'      => self::getView()->t('Scheduled Jobs'),
                            'order'      => 11,
                            'route'      => 'admin_scheduler'
                        ),
                        'admin-responses' => array(
                            'id'         => 'admin-responses',
                            'class'      => 'gray',
                            'menu_type'  => 'admin-sidebar',
                            'label'      => self::getView()->t('Responses'),
                            'order'      => 12,
                            'route'      => 'admin_responses'
                        ),
                        'statuses-management' => array(
                            'id'         => 'admin-statuses',
                            'class'      => 'gray',
                            'menu_type'  => 'admin-sidebar',
                            'label'      => self::getView()->t('Statuses'),
                            'order'      => 13,
                            'route'      => 'admin_statuses'
                        ),
                        'admin-languages' => array(
                            'id'         => 'admin-languages',
                            'menu_type'  => 'admin-sidebar',
                            'class'      => 'gray',
                            'label'      => self::getView()->t('Languages'),
                            'order'      => 14,
                            'route'      => 'admin_languages',
                            'pages'      => array(
                                array(
                                    'id'         => 'admin-languages-add',
                                    'label'      => self::getView()->t('Add'),
                                    //'hide'       => true,
                                    'module'     => 'system',
                                    'controller' => 'admin-languages',
                                    'action'     => 'add',
                                    'route'      => 'admin_languages_add'
                                ),
                                array(
                                    'id'         => 'admin-languages-edit',
                                    'label'      => self::getView()->t('Edit'),
                                    'hide'       => true,
                                    'module'     => 'system',
                                    'controller' => 'admin-languages',
                                    'action'     => 'edit',
                                ),
                                array(
                                    'id'         => 'admin-languages-dictionary-add',
                                    'label'      => self::getView()->t('Add Dictionary'),
                                    'hide'       => true,
                                    'module'     => 'system',
                                    'controller' => 'admin-languages',
                                    'action'     => 'add-dictionary',
                                ),
                                array(
                                    'id'         => 'admin-languages-dictionary-edit',
                                    'label'      => self::getView()->t('Edit Dictionary'),
                                    'hide'       => true,
                                    'module'     => 'system',
                                    'controller' => 'admin-languages',
                                    'action'     => 'edit-dictionary',
                                )
                            )
                        ),                        
                        'admin-interface' => array(
                            'id'         => 'admin-interface',
                            'menu_type'  => 'admin-sidebar',
                            'image'      => self::getView()->imageSrc('/icons/16/interface.png', null, true),
                            'label'      => self::getView()->t('Interface'),
                            'order'      => 40,
                            'route'      => 'admin_interface'
                        ),
                        'admin-interface-widget' => array(
                            'id'         => 'admin-interface-widget',
                            'menu_type'  => 'admin-sidebar',
                            'class'      => 'gray',
                            'label'      => self::getView()->t('Widgets'),
                            'order'      => 41,
                            'route'      => 'admin_widget'
                        ),
                        'admin-analytics' => self::_getAnalitycsNavConfig(),
                        'admin-mailing' => array(
                            'id'         => 'admin-mailing',
                            'menu_type'  => 'admin-sidebar',
                            'image'      => self::getView()->imageSrc('/icons/16/mailing.png', null, true),
                            'label'      => self::getView()->t('Mailing'),
                            'order'      => 60,
                            'route'      => 'admin_mailing'
                        ),
                        'admin-mailing-templates' => array(
                            'id'         => 'admin-mailing-templates',
                            'menu_type'  => 'admin-sidebar',
                            'class'      => 'gray',
                            'label'      => self::getView()->t('Templates'),
                            'order'      => 61,
                            'route'      => 'admin_templates',
                            'pages'      => array(
                                array(
                                    'id'         => 'admin-mailing-templates-edit',
                                    'label'      => self::getView()->t('Edit'),
                                    'hide'       => true,
                                    'module'     => 'resources',
                                    'controller' => 'admin-templates',
                                    'action'     => 'edit',
                                )
                            )
                        ),
                        'admin-modules' => array(
                            'id'         => 'admin-modules',
                            'menu_type'  => 'admin-sidebar',
                            'image'      => self::getView()->imageSrc('/icons/16/modules.png', null, true),
                            'label'      => self::getView()->t('Modules'),
                            'order'      => 90,
                            'route'      => 'admin_modules'
                        ),
                        'admin-trash' => array(
                            'id'         => 'admin-trash',
                            'menu_type'  => 'admin-sidebar',
                            'image'      => self::getView()->imageSrc('/icons/16/trash.png', null, false),
                            'label'      => self::getView()->t('Trash'),
                            'order'      => 100,
                            'route'      => 'admin_trash'
                        )
                    );
                    if (key_exists('role', self::getView())
                            && self::getView()->role instanceof System_Model_Role) {
                        $pages['main']['pages']['admin']['pages'][3]['pages'][] = array(
                            'id'         => 'admin-roles-edit',
                            'label'      => self::getView()->role->name,
                            'hide'       => true,
                            'module'     => 'system',
                            'controller' => 'admin-roles',
                            'action'     => 'edit',
                        );
                    }
                }

                if (Singular_Core::_('theme')->getDefault() != 'mobile') {
                    if ($loggedUser instanceof System_Model_User && $loggedUser->canUseActivityLog()) {

                        $pages['main']['pages']['activity_log'] = array(
                            'id'         => 'activity_log',
                            'menu_type'  => 'members-area',
                            'badge'      => null,
                            'label'      => self::getView()->t('Activity Log'),
                            'order'      => 120,
                            'module'     => 'system',
                            'controller' => 'activity',
                            'action'     => 'index',
                            'route'      => 'activity_log'
                        );
                    }
                }

                $hideProfileSettings = !in_array($route, array('profile', 'users_paginator'));

                $pages['main']['pages']['profile'] = array(
                    'id'         => 'profile',
                    'label'      => self::getView()->t('Profile'),
                    'order'      => 121,
                    'route'      => 'profile_me',
                    'pages'      => array(
                        array(
                            'id'        => 'profile-activity',
                            'menu_type' => 'members-area',
                            'label'     => self::getView()->t('My Activity'),
                            'badge'     => null,
                            'order'     => 122,
                            'visible'   => $hideProfileSettings,
                            'route'     => 'profile_me'
                        ),
                        array(
                            'id'         => 'profile-settings',
                            'class'      => 'settings',
                            'menu_type'  => 'members-area',
                            'order'      => 123,
                            'label'      => self::getView()->t('Profile Settings'),
                            'badge'      => null,
                            'module'     => 'system',
                            'controller' => 'user',
                            'action'     => 'profile',
                            'route'      => 'profile_settings_me'
                        )
                    )
                );

            } else {
                $pages['main']['pages'][] = array(
                    'id'         => 'auth-login',
                    'label'      => self::getView()->t('Login'),
                    'class'      => 'login-popup',
                    'liClass'    => 'itTab',
                    'topMenu'    => true,
                    'icoClass'   => 'ilogin',
                    'hide'       => true,
                    'menu_type'  => 'login-register',
                    'route'      => 'login',
                    'aRel'        => 'form-login',
                    'params'     => array(
                        'redirect' => 'reset'
                    )
                );
                $pages['main']['pages'][] = array(
                    'id'         => 'auth-register',
                    'label'      => self::getView()->t('Register'),
                    'class'      => 'register-popup',
                    'liClass'    => 'itTab',
                    'topMenu'    => true,
                    'icoClass'   => 'iregister',
                    'hide'       => true,
                    'menu_type'  => 'login-register',
                    'route'      => 'registration',
                    'aRel'        => 'form-registration',
                    'params'     => array(
                        'redirect' => 'reset'
                    )
                );
                $pages['main']['pages'][] = array(
                    'id'         => 'auth-forgot',
                    'label'      => self::getView()->t('Forgot Password'),
                    'hide'       => true,
                    'route'      => 'forgot_password',
                    'aRel'        => 'form-forgotpassword',
                );
            }

            $hidePeopleNav = !$loggedUser->canSeePeople();

            $pages['main']['pages']['people'] = array(
                'id'            => 'people',
                'menu_type'     => ($hidePeopleNav) ? 'hidden' : 'top-bar',
                'label'         => self::getView()->t('People'),
                'hide_in_bread' => $hidePeopleNav,
                'hide'          => $hidePeopleNav,
                'order'         => 60,
                'badge'         => null,
                'module'        => 'system',
                'controller'    => 'users',
                'action'        => 'index',
                'route'         => 'users'
            );

            $pages['main']['pages']['people_paginator'] = array(
                'id'         => 'people_paginator',
                'hide'       => true,
                'label'      => self::getView()->t('People'),
                'module'     => 'system',
                'controller' => 'users',
                'action'     => 'paginator'
            );

            $navigation = new Singular_Navigation();
            $navigation->addPages($pages);        
            Singular_Event::dispatch('buildNavigation', $navigation);
            
            $cache->save($navigation, $cacheId, array("breadcrumbs"));
        }
        
        if ($loggedUser instanceof System_Model_User) {
            $activeUser = Singular_Runtime::extract("activeUser");
            if ($activeUser instanceof System_Model_User) {
                if ($loggedUser->id != $activeUser->id) {
                    $item = array(
                        'id'         => 'profile-' . $activeUser->id,
                        'label'      => $activeUser->getName(),
                        'route'      => 'profile',
                        'params'      => array(
                            'user_id' => $activeUser->id,
                        ),
                    );
                    if ($loggedUser->canManagePeople($activeUser)) {
                        $item['pages'][] = array(
                            'id'         => 'profile-settings',
                            'class'      => 'settings',
                            'order'      => 10,
                            'menu_type'  => 'profile-nav',
                            'label'      => self::getView()->t('Profile Settings'),
                            'route'      => 'profile_settings',
                            'params'     => array(
                                'user_id' => $activeUser->id
                            )
                        );
                    }
                    $page = $navigation->findOneBy('id', 'people');
                    $page->addPage($item);

                    if ($loggedUser->canManagePeople($activeUser)) {
                        if ($activeUser->is_active) {
                            $label  = 'Ban User';
                            $access = 'ban';
                            $class  = "banUser";
                        } else {
                            $access = 'recovery';
                            $class  = "recoveryUser iaccept";
                            $label  = 'Recover Access';
                        }

                        $items[] = array(
                            'id'         => 'profile-ban',
                            'menu_type'  => 'profile-nav',
                            'class'      => $class,
                            'order'      => 20,
                            'label'      => self::getView()->t($label),
                            'params'     => array(
                                'user_id' => $activeUser->id,
                                'access' => $access
                            ),
                            'route'      => 'user_ban'
                        );

                        $items[] = array(
                            'id'         => 'user-delete',
                            'menu_type'  => 'profile-nav',
                            'class'      => 'delete',
                            'order'      => 30,
                            'label'      => self::getView()->t('Delete'),
                            'url'        => '#',
                            'params'     => array(
                                'user_id' => $activeUser->id,
                            ),
                            'route'      => 'user_delete'
                        );
                        $page = $navigation->findOneBy('id', 'profile');
                        if($page){
                            $page->addPages($items);
                        }
                    }
                }
            }
        }
        
        $currentRouteName = Zend_Controller_Front::getInstance()
                            ->getRouter()->getCurrentRouteName();
        if ($currentRouteName == "users_participants") {
            $page = $navigation->findOneBy('id', 'people');
            $page->addPage(array(
                'id'         => 'people_participants',
                'label'      => self::getView()->t('Participants'),
                'route'      => 'users_participants',
                'params'     => array(
                    'slug' => $request->getParam('slug')
                )                
            ));
        }
        if ($currentRouteName == "users_subscribers") {
            $page = $navigation->findOneBy('id', 'people');
            $page->addPage(array(
                'id'         => 'people_subscribers',
                'label'      => self::getView()->t('Subscribers'),
                'route'      => 'users_subscribers',
                'params'     => array(
                    'slug' => $request->getParam('slug')
                )
            ));
        }

        if ('response_votes_history' == $route) {
            $activeResponse = Singular_Runtime::extract("activeResponse");
            $page           = $navigation->findBy('id', 'responses');
            $page->addPage(
                array(
                     'id'     => 'response-' . $activeResponse->id,
                     'label'  => self::getView()->truncate($activeResponse->title, '30', '...'),
                     'route'  => 'responses_show',
                     'params' => array(
                         'slug' => $activeResponse->slug
                     ),
                     'pages'  => array(
                         array(
                             'id'     => 'response_votes_history',
                             'label'  => self::getView()->t('Vote History'),
                             'route'  => 'response_votes_history',
                             'params' => array('slug' => $activeResponse->slug),
                         )),
                )
            );
        }

        Singular_Event::dispatch('buildNavigationAfter', $navigation);

        self::getView()->navigation($navigation);
    }
    
    /**
    * Returns analytics admin menu form item
    *
    * @return array
    **/
    private static function _getAnalitycsNavConfig()
    {
        return array(
            'id'         => 'admin-analytics',
            'menu_type'  => 'admin-sidebar',
            'image'      => self::getView()->imageSrc('/icons/16/analytics.png', null, true),
            'label'      => self::getView()->t('Analytics'),
            'order'      => 50,
            'route'      => 'admin_analytics'
        );
    }        

    /**
     * Subscribes to "afterSwitchUserStatus" event.
     *
     * @param  System_Model_User $user
     * @return void
     */
    public static function onAfterSwitchUserStatus ($user)
    {
        if ($user->is_active) {
            $tpl = 'recovered_access';
            $status = self::getView()->t('active');
            $message = self::getView()->t('Access for this user is recovered!');
        } else {
            $tpl = 'restricted_access';
            $status = self::getView()->t('banned');
            $message = self::getView()->t(
                ':userName is banned to enter system!',
                array('userName' => $user->getName())
            );
        }
        $community = self::getView()->settings->system->general->site_title->value;
        Resources_Service_ApplicationMailer::send($user, $tpl, array(
            'community_name' => $community,
            'community_url'  => self::getView()->baseUrl(),
            'user_status'    => $status
        ));
    }
    
    /**
     * Subscribes to "afterRegistration" event.
     *
     * @param  System_Model_User $user
     * @return void
     */
    public static function onAfterRegistration($user)
    {
        $community = self::getView()->settings->system->general->site_title->value;
        $login     = self::getView()->url(array('layout' => 'main'), 'login');
        $profile   = self::getView()->url(array('user_id' => $user->id), 'profile');
        Resources_Service_ApplicationMailer::send(
            $user, 'registration',
            array(
                 'community_name' => $community,
                 'community_url'  => self::getView()->baseUrl(),
                 'email'          => $user->email,
                 'password'       => $user->getCleanPassword(),
                 'login_url'      => $login,
                 'profile_url'    => $profile,
            )
        );
    }
    
    /**
     * Subscribes to "afterSwitchResponseStatus" event.
     *
     * @param  System_Model_Response $response
     * @return void
     */
    public static function onAfterSwitchResponseStatus ($response)
    {
        $route = Zend_Controller_Front::getInstance()->getRouter()->getCurrentRouteName();
        if ('comments_comment_add' == $route || 'response_add_answer' == $route) {
            return;
        }
        // Getting response users
        $users = Singular_Loader::get('System_Model_DbTable_ResponsesSubscribers' )
            ->findUsersByResponse($response);

        // Sending emails
        $community = self::getView()->settings->system->general->site_title->value;
        Resources_Service_ApplicationMailer::send($users, 'change_response_status', array(
            'community_name'  => $community,
            'community_url'   => self::getView()->baseUrl(),
            'response_name'   => $response->title,
            'response_type'   => self::getView()->t($response->type),
            'response_url'    => $response->getUrl(),
            'response_body'   => self::getView()->bbcode($response->content),
            'response_status' => $response->getStatus(),
            'profile_url'     => self::getView()->url(array(), 'profile_settings_me')
        ));
    }
    
    /**
     * Subscribes to "responseAdded" event.
     *
     * @param  stdClass $data
     * @return void
     */
    public static function onResponseAdded ($data)
    {
        // Active response
        $response = $data->response;

        // Getting admins and officials representative
        $users = Singular_Loader::get('System_Model_DbTable_Users')
            ->findByPrivileges(
                array(
                     'object:can_access_admin',
                     'object:representative'
                )
            );
        
        // Getting logged user
        $loggedUser = Singular_Runtime::extract('loggedUser');

        if ($loggedUser->isGuest()) {
            $usersDb       = Singular_Loader::get('System_Model_DbTable_Users');
            $user          = $usersDb->findById($response->created_by_id);
            $createdByName = $user->getName();
            $createdByUrl  = self::getView()->url(array('user_id' => $user->id), 'profile');
        } else {
            $createdByName = $loggedUser->getName();
            $createdByUrl  = self::getView()->url(array('user_id' => $loggedUser->id), 'profile');
        }
        
        // Sending emails
        $community = self::getView()->settings->system->general->site_title->value;
        Resources_Service_ApplicationMailer::send($users, 'new_response', array(
            'community_name'  => $community,
            'community_url'   => self::getView()->baseUrl(),
            'response_name'   => $response->title,
            'response_type'   => self::getView()->t($response->type),
            'response_url'    => $response->getUrl(),
            'response_body'   => self::getView()->bbcode($response->content),
            'response_status' => $response->getStatus(),
            'created_by_url'  => $createdByUrl,
            'created_by_name' => $createdByName,
            'profile_url'     => self::getView()->url(array(), 'profile_settings_me')
        ));
    }
    
    /**
     * Subscribes to "answerAdded" event.
     *
     * @param  stdClass $params
     * @return void
     */
    public static function onAnswerAdded ($params)
    {
        // Getting objects
        $response = $params->response;
        $answer   = $params->answer;

        // Getting logged user
        $loggedUser = Singular_Runtime::extract('loggedUser');
        
        // Getting subscribers and admins
        $users = Singular_Loader::get(
            'System_Model_DbTable_ResponsesSubscribers'
        )->findUsersByResponse($response);
        $admins = Singular_Loader::get('System_Model_DbTable_Users')
            ->findByPrivileges(array(
                'object:can_access_admin',
                'object:representative'
            ));
        $users->addItems($admins);

        $responseStatus = '';
        if (Singular_Runtime::inStorage('statusReport')) {
            $statusReport = Singular_Runtime::extract('statusReport');
            if (isset($statusReport->isChanged) && $statusReport->isChanged) {
                $responseStatus = '[' . $response->getStatus() . '] ';
            }
        }

        // Sending emails
        $community = self::getView()->settings->system->general->site_title->value;
        Resources_Service_ApplicationMailer::send($users, 'official_answer', array(
            'community_name'  => $community,
            'community_url'   => self::getView()->baseUrl(),
            'response_name'   => $response->title,
            'response_type'   => self::getView()->t($response->type),
            'response_url'    => $response->getUrl(),
            'response_body'   => self::getView()->bbcode($response->content),
            'response_status' => $responseStatus,
            'created_by_url'  => self::getView()->url(array('user_id' => $loggedUser->id), 'profile'),
            'created_by_name' => $loggedUser->getName(),
            'official_answer' => self::getView()->bbcode($answer->answer),
            'profile_url'     => self::getView()->url(array(), 'profile_settings_me')
        ));
    }
    
    /**
     * Subscribes to "afterAnalyticsReportsInit" event.
     *
     * @param  stdClass $reportsConf
     * @return void
     */
    public static function afterAnalyticsReportsInit ($reportsConf) 
    {
        $reportsConf->reports[] = array(
            'id' => 'system_users',
            'title' => self::getView()->t('Users'),
            'model' => 'System_Model_DbTable_Users',
            'counter' => 'count',
            'order' => '10',
            'options' => array(
                'row_type' => 'default',
            )
        );
        
        $reportsConf->reports[] = array(
            'id' => 'system_responses',
            'title' => self::getView()->t('Responses'),
            'model' => 'System_Model_DbTable_Responses',
            'counter' => 'countByType',
            'order' => '20',
            'options' => array(
                'row_type' => 'default',
                'response_types_assoc' => true
            )
        );
        
        $reportsConf->reports[] = array(
            'id' => 'system_views',
            'title' => self::getView()->t('Views'),
            'model' => 'System_Model_DbTable_Responses',
            'counter' => 'sumViews',
            'order' => '110',
            'options' => array(
                'row_type' => 'total',
            )
        ); 
    }
    
    /**
     * Called before Zend_Controller_Front exits its dispatch loop.
     *
     * @return void
     */
    public static function onDispatchLoopShutdown()
    {
        /** @var $view Singular_View */
        $front = Zend_Controller_Front::getInstance();
        if ($front->getRequest()->isXmlHttpRequest()) {
            return;
        }
        self::getView()->additionalModulePath('system');
        
        $bodyContent = Singular_Core::_('settings')->getSetting('analytics_body_script');
        $front->getResponse()->setBody(
            str_replace('</body>', PHP_EOL . $bodyContent . "\n</body>", $front->getResponse()->getBody())
        );
        
        $headerContent = Singular_Core::_('settings')->getSetting('analytics_header_script');
        $front->getResponse()->setBody(
            str_replace('</head>', PHP_EOL . $headerContent . "\n</head>",
        $front->getResponse()->getBody())
        );
    }
}
