<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * AnalyticsStatisticsTable view helper
 *
 * @uses       Zend_View_Helper_Abstract
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 */
class System_View_Helper_AnalyticsStatisticsTable extends Zend_View_Helper_Abstract 
{
    /**
     * Table rows
     * 
     * @var array
     */
    protected $_rows = array();
    
    /**
     * Path to template
     * 
     * @var string 
     */
    protected $_template;
    
    /**
     * Module context ( for render template)
     * 
     * @var string 
     */
    protected $_module;

    /**
     * Initialization of table
     * 
     * @return void 
     */
    public function init ()
    {
        $this->_template = '_analytics'.DS.'statistics-table.phtml';
        $this->setModule('system')->_initRows();        
    }        
    
    /**
     * Rows initialization
     * 
     * @return \System_View_Helper_AnalyticsStatisticsTable
     */
    protected function _initRows ()
    {
        $rowTemplate = '_analytics'.DS.'statistics-table-row.phtml';
        
        $rowsConfig = Singular_Runtime::extract('analytics_report_config')->reports;
        $idPrefix = 'statistics_table';
        
        foreach ($rowsConfig as $rowConfig) {
            $rowType  = 'System_Lib_Analytics_View_Table_Row_'.ucfirst($rowConfig['options']['row_type']);
            $rowConfig['id'] = $idPrefix.'_'.$rowConfig['id']; 
            $this->addRow(new $rowType (
                $rowTemplate,
                $this->_module,
                $rowConfig
            ));  
        }
        return $this;    
    }        

    /**
     * Render table
     * 
     * @return string
     */
    public function analyticsStatisticsTable ()
    {
        $this->init();
        $this->_orderRows();
        $options = array (
            'rows' => $this->_rows
        );
        return $this->view->partial($this->_template, $this->_module, $options);
    }

    /**
     * Set module name
     * 
     * @param type $name
     * @return \System_View_Helper_AnalyticsStatisticsTable
     */
    public function setModule ($name)
    {
        $this->_module = $name;
        return $this;
    }
    
    /**
     * Add row to table
     * 
     * @param System_Lib_Analytics_View_Table_Row_Abstract $row
     * @return \System_View_Helper_AnalyticsStatisticsTable
     */
    public function addRow(System_Lib_Analytics_View_Table_Row_Abstract $row)
    {
        array_push($this->_rows, $row);
        return $this;
    }
    
    /**
     * Sort rows in a table
     * 
     * @return \System_View_Helper_AnalyticsStatisticsTable
     */
    protected function _orderRows ()
    {
        $rows = array() ;
        foreach ($this->_rows as $row) {
            if(isset($row->order) && (int)$row->order > 0 ) {
                $rows[(int)$row->order] = $row;
            } else {
                $rows[0] = $row;
            }
        }
        ksort($rows);
        $this->_rows = $rows;
        return $this;
    }               
}