<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Bbcode view helper
 *
 * @uses       Zend_View_Helper_Abstract
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 */
class System_View_Helper_Bbcode extends Zend_View_Helper_Abstract
{
    /**
     * Reforms input text following bbcode format convention
     *
     * @param  string $text
     * @return string
     */
    public function bbcode ($text, $br2p = true, $object = null)
    {
        if (empty($text)) {
            return '';
        }
        if (Singular_Runtime::inStorage('bbcode')) {
            $bbcode = Singular_Runtime::extract('bbcode');
        } else {
            $bbcode = new System_Lib_Markup_Renderer_Html();
            Singular_Runtime::store('bbcode', $bbcode);
        }
        Singular_Runtime::store('renderObject', $object);
        $text = $bbcode->render(trim($text));
        
        $text = '<p>' . preg_replace("/<br(\s+)?\/?>/i", ' </p>' . "\n" . '<p>', $text) . '</p>';
        $text = preg_replace('/<p>\s?&nbsp;<\/p>/', '', $text);
        $text = str_replace("\n",'',$text);
        $text = str_replace("\r",'',$text);

        $linkCallback = function ($matches) {
            $matches[0] = trim($matches[0]);
            $matches[0] = str_replace('&nbsp;', '', $matches[0]);

            return '<a href="' . $matches[0] . '" rel="nofollow" target="_blank">' . $matches[0] . '</a>';
        };

        $linkCallback2 = function ($matches) use ($linkCallback) {
            $matches[0] = 'http://' . $matches[0];
            return $linkCallback($matches);
        };

        // Find and replace all http/https/ftp/ftps links that are not part of an existing html anchor
        $text = preg_replace_callback(
            '~\b(?<!href|src|value|codebase|original="|\'|">|\'>|")(?:ht|f)tps?://[^<\s]+(?:/|\b)~i', $linkCallback, $text
        );

        // Find and replace all naked www.links.com (without http://)
        $text = preg_replace_callback(
            '~\b(?<!://|">)www(?:\.[a-z0-9][-a-z0-9]*+)+\.[a-z]{2,6}[^<\s]*\b~i', $linkCallback2,
            $text
        );

        return $text;
    }
}