<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Photo view helper
 *
 * @uses       Zend_View_Helper_Abstract
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 */
class System_View_Helper_Photo extends Zend_View_Helper_Abstract
{
    /**
     * Renders user photo
     *
     * @param  Zend_Db_Table_Row|array $user
     * @param  string|null $class
     * @return string
     */
    public function photo($user, $class = null, $width = null)
    {
        if (!is_string($class)) {
            $class = 'author-avatar';
        }
        if (!empty($width)) {
            $width = " style=\"width:{$width}px;\"";
        }

        if ($user instanceof System_Model_User && !$user->isGuest()) {
            $configs = $user->getConfigValues();
            if (isset($configs['photo_type'])
                    && $configs['photo_type'] == System_Model_UserConfig::GRAVATAR) {
                $img = (string) $this->view->gravatar($user->gravatar_email, array('imgSize' => 50), array('class' => $class, 'alt' => $this->view->t('photo')));
            } else {
                $img = '<img class="' . $class . '"' . $width . ' src="' . $user->getAvatarSrc() . '" alt="' .
                    $this->view->t
                ('photo') . '" />';
            }
        } else {
            $img = '<img class="' . $class . '"' . $width . ' src="' . $this->view->baseUrl(AVATAR_URI .  '/avatar-default.png') . '" alt="' . $this->view->t('photo') . '" />';
        }
        
        $params = (object) array(
            'image' => $img,
            'user'  => $user,
            'class' => $class,
        );

        Singular_Event::dispatch('renderAvatar', $params);

        return $params->image;
    }
}
