<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * RelativeDateTime view helper
 *
 * @uses       Zend_View_Helper_Abstract
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 */
class System_View_Helper_RelativeDateTime extends Zend_View_Helper_Abstract
{
    const YEAR    = 'YEAR';
    const MONTH   = 'MONTH';
    const WEEK    = 'WEEK';
    const DAY     = 'DAY';
    const HOUR    = 'HOUR';
    const MINUTE  = 'MINUTE';
    const SECOND  = 'SECOND';
    const YEARS   = 'YEARS';
    const MONTHS  = 'MONTHS';
    const WEEKS   = 'WEEKS';
    const DAYS    = 'DAYS';
    const HOURS   = 'HOURS';
    const MINUTES = 'MINUTES';
    const SECONDS = 'SECONDS';

    /**
     * Unit template
     *
     * @var array
     */
    protected $_unitTemplates = array(
        self::YEAR    => '%value% year',
        self::MONTH   => '%value% month',  self::WEEK    => '%value% week',
        self::DAY     => '%value% day',    self::HOUR    => '%value% hour',
        self::MINUTE  => '%value% minute', self::SECOND  => '%value% second',
        self::YEARS   => '%value% years',  self::MONTHS  => '%value% months',
        self::WEEKS   => '%value% weeks',  self::DAYS    => '%value% days',
        self::HOURS   => '%value% hours',  self::MINUTES => '%value% minutes',
        self::SECONDS => '%value% seconds'
    );

    /**
     * Class constructor
     *
     * @param mixed $config
     */
    public function __construct ($config = null)
    {}

    /**
     * Relatively formats date
     *
     * @param  Zend_Date|null $date
     * @return string
     */
    public function relativeDateTime (Zend_Date $date = null)
    {
        if (null === $date) {
            return $this;
        }
        $todayDate = new Zend_Date();
        $diff = $todayDate->sub($date);
        $mt = new Zend_Measure_Time($diff);
        $units = $mt->getConversionList();
        
        $chunks = array(Zend_Measure_Time::YEAR, Zend_Measure_Time::MONTH,
        Zend_Measure_Time::WEEK, Zend_Measure_Time::DAY, Zend_Measure_Time::HOUR,
        Zend_Measure_Time::MINUTE, Zend_Measure_Time::SECOND);
        for ($i = 0, $count = count($chunks); $i < $count; ++ $i) {
            $seconds = $units[$chunks[$i]][0];
            $unitKey = $chunks[$i];
            
            if (0.0 !==
             ($result = floor($diff->get(Zend_Date::TIMESTAMP) / $seconds))) {
                break;
            }
        }
        $translateHelper = new Zend_View_Helper_Translate();
        if ($result === (float) 1) {
            $formatedString = $translateHelper->translate(
            $this->getUnitTemplate($unitKey));
        } else {
            $formatedString = $translateHelper->translate(
            $this->getUnitTemplate($unitKey . 'S'));
        }
        $formatedString = str_replace('%value%', (string) $result,
        $formatedString);

        return '<span class="time-help" title="' . $date->toString() . '">' . $formatedString . '</span>';
    }

    /**
     * Sets unit template
     *
     * @param  string $template
     * @param  string $unitKey
     * @return System_View_Helper_RelativeDateTime
     * @throws Singular_Exception
     */
    public function setUnitTemplate ($template, $unitKey)
    {
        if (! isset($this->_unitTemplates[$unitKey])) {
            require_once 'Zend/Helper/Exception.php';
            throw new Singular_Exception(
            "No unit template exists for key '$unitKey'");
        }
        $this->_unitTemplates[$unitKey] = $template;

        return $this;
    }

    /**
     * Sets multiple unit templates
     *
     * @param  array $templates
     * @return System_View_Helper_RelativeDateTime
     */
    public function setUnitTemplates (array $templates)
    {
        foreach ($templates as $key => $template) {
            $this->setUnitTemplate($template, $key);
        }
        return $this;
    }

    /**
     * Returns current unit templates
     *
     * @return array
     */
    public function getUnitTemplates ()
    {
        return $this->_unitTemplates;
    }

    /**
     * Returns unit template by given key
     *
     * @param  string $unitKey
     * @return string
     * @throws Singular_Exception
     */
    public function getUnitTemplate ($unitKey)
    {
        if (!isset($this->_unitTemplates[$unitKey])) {
            throw new Singular_Exception("No unit template exists for key '$unitKey'");
        }
        
        return $this->_unitTemplates[$unitKey];
    }
}
