<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * ResponseActions view helper
 *
 * @uses       Zend_View_Helper_Abstract
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 */
class System_View_Helper_ResponseActions extends Zend_View_Helper_Abstract
{
    /**
     * Renders response actions
     *
     * @return string
     */
    public function responseActions ()
    {
        $activeResponse = Singular_Runtime::extract("activeResponse");
        if ($activeResponse instanceof System_Model_Response) {
            $pages = array();
            if ($activeResponse->canEdit()) {
                $pages[] = array(
                    'id'        => 'response_edit',
                    'label'     => 'Edit',
                    'route'     => 'response_edit',
                    'params'    => array(
                        'slug'    => $activeResponse->slug
                    )
                );
            }
            if ($activeResponse->canDelete()) {
                if ($activeResponse->deleted_by_id) {
                    $pages[] = array(
                        'id'        => 'response_recovery',
                        'label'     => 'Restore from Trash',
                        'route'     => 'response_recovery',
                        'params'    => array(
                            'slug'    => $activeResponse->slug
                        )
                    );
                } else {
                    $pages[] = array(
                        'id'        => 'response_trash',
                        'label'     => 'Move to Trash',
                        'route'     => 'response_trash',
                        'params'    => array(
                            'slug'    => $activeResponse->slug
                        )
                    );
                }
            }
            if ($activeResponse->canLockComments()) {
                $page = array(
                    'id'        => 'response_id',
                    'route'     => 'response_lock_comments',
                    'params'    => array(
                        'response_id'    => $activeResponse->id
                    )
                );
                if ($activeResponse->lock_comments) {
                    $page['label'] = 'Unlock Comments';
                } else {
                    $page['label'] = 'Lock Comments';
                }
                $pages[] = $page;
            }
        }
        $actions = new Singular_Navigation($pages);
        Singular_Event::dispatch("responseActions", $actions);

        return $actions;
    }
}
