<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * RightMenu view helper
 *
 * @uses       Zend_View_Helper_Abstract
 * @category   System
 * @package    System_View
 * @subpackage System_View_Helper
 */
class System_View_Helper_RightMenu extends Zend_View_Helper_Abstract
{
    /**
     * Renders right menu
     *
     * @return string
     */
    public function rightMenu()
    {
        $cache      = Singular_Runtime::extract('cachemanager')->getCache('default');
        /** @var System_Model_User $loggedUser */
        $loggedUser = Singular_Runtime::extract('loggedUser');
        $cacheId    = "rightMenu_" . $loggedUser->role;
        $cacheId    = preg_replace("/[^a-zA-Z0-9_]/", "", $cacheId);
        if ( ($output = $cache->load($cacheId)) === false ) {

            $commentsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
            $usersDb    = Singular_Loader::get('System_Model_DbTable_Users');

            $pages = array(
                array(
                    'id'    => 'statistics',
                    'label' => $this->view->t('Statistics'),
                    'theme' => 1,
                    'class' => 'categories-block',
                    'pages' => array(
                        array(
                            'id'    => 'comments',
                            'order' => 49,
                            'count' => $commentsDb->count(),
                            'icon'  => $this->view->imageSrc('/icons/16/comments.png', null, false),
                            'label' => 'Comments',
                            'noUrl' => true,
                        )
                    )
                )
            );

            if ($loggedUser->canSeePeople()) {
                $pages[0]['pages'][] = array(
                    'id'    => 'users',
                    'order' => 50,
                    'count' => $usersDb->count(),
                    'icon'  => $this->view->imageSrc('/icons/16/users.png', null, false),
                    'label' => 'Users',
                    'route' => 'users',
                );
            }

            $menu              = new Zend_Navigation($pages);
            $params            = new stdClass();
            $params->menu      = $menu;
            $params->cacheKeys = array('users', 'comments', 'responses');
            Singular_Event::dispatch('rightMenu', $params);
            $this->view->menu = $menu;
            $output           = $this->view->render('_right-menu.phtml');

            $cache->save($output, $cacheId, $params->cacheKeys);
        }
        return $output;
    }
}
