<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @copyright  USWebStyle Inc., 2011-2013 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Analytics report absract class
 * 
 * Splits time period on a parts (with the required precission) and returns a report of quantity for each part.
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analytics
 */
abstract class System_Lib_Analytics_Model_Report_Abstract 
{
    /**
     * Counter model name
     * 
     * @var string 
     */
    public $model = null;
    
    /**
     * Counter method name
     * 
     * @var string
     */
    public $counter = null;
    
    /**
     * Constructor
     * $spec may be:
     * - array: options with which to configure model
     *
     * @param  array $options 
     * @param type $options
     * @throws Exception
     */
    public function __construct ($options) 
    {
        //check dbTableName
        if ( !isset($options['model']) || empty($options['model'])) {
            throw new Exception('"model" value must be defined.');
        }
        
        //check counter
        if( !isset($options['counter']) || empty($options['counter']) ) {
            throw new Exception('"counter" value must be defined.');
        }
        
        //set options
        $this->_addOptionsRecursively($options);
    }
    
    /**
     * Return report
     */
    abstract public function getReport ();

    /**
     * add recursively options in to object this class 
     * 
     * @param array $options
     */
    protected function _addOptionsRecursively(array $options)
    {
        foreach ($options as $optionName => $option) {
            if ($optionName == 'options' && is_array($option)) {
                $this->_addOptionsRecursively($option);
                continue;
            }
            $this->_addOption($optionName,$option);    
        }
    }
    
    /**
     * Return object of counter model
     * 
     * @return \System_Model_DbTable_Abstract
     * @throws Exception
     */
    protected function _getModel ()
    {
        $model = Singular_Loader::get($this->model); 
        if ($model instanceof System_Model_DbTable_Abstract)
        {
            return $model;
        }    
        throw new Exception('Unexpected model object.');
    }     
    
    /**
     * 
     * @param string $name
     * @param mixed $value
     * @return \System_Lib_Analytics_Model_Report_Abstract
     */
    protected function _addOption ($name, $value)
    {
        $name = preg_replace('/_(.?)/e',"strtoupper('$1')", $name);
        $this->$name = $value;
        return $this;
    }
    
    /**
     * Return quantity for given query
     * 
     * @param string $query
     * @return string
     */
    protected function _getQuantity($query)
    {
        $model = $this->_getModel();
        return call_user_func_array( array($model, $this->counter), $query);
    }
}

