<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Analytics time report abstract class
 *
 * @uses       System_Lib_Analytics_Model_Report_Abstract
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analytics
 */
abstract class System_Lib_Analytics_Model_Report_Time_Abstract extends System_Lib_Analytics_Model_Report_Abstract  
{
    /*
     * End date of the date range
     * 
     * @var Zend_Date
     */
    protected $_dateTo;
    /*
     * Start date of the date range
     * 
     * @var Zend_Date
     */
    protected $_dateFrom;
    
    /**
     *
     * @var string
     */
    protected $_dateFormat = 'Y-m-d H:i:s';
    
    /**
     * Date type (database field name)
     * 
     * @var type 
     */
    public $actionType = 'created_at';
    
    /**
     * initialization data
     * 
     * @var array
     */
    protected $_data = array();
    
    /**
     * Retrieve StartDate of the date range
     * 
     * @return Zend_Date 
     */
    public function getDateFrom () 
    {
        return clone $this->_dateFrom;
    }
    
    /**
     * Return date format
     * 
     * @return string
     */
    public function getDateFormat() 
    {
        return $this->_dateFormat;
    }

    /**
     * Retrieve StartDate of the date range
     * 
     * @return Zend_Date
     */
    public function getDateTo () 
    {
        return clone $this->_dateTo;
    }
    
    /**
     * Constructor
     *
     * $spec may be:
     * - array: options with which to configure model
     *
     * @see parent::__construct
     * @param  array $options
     */
    public function __construct ($options) 
    {
        if (isset($options['dateRangeData']) && !empty($options['dateRangeData'])) {
            $this->_data = $options['dateRangeData'];
        } else {
            throw new Exception('Config data is empty.');
        }
        unset($options['dateRangeData']);
        
        $this->_prepareDates();
        parent::__construct($options);
    }
    
    /**
     * Preapare start and end dates of the date range
     * 
     * @return System_Lib_Analytics_Model_Report_Time_Abstract 
     */
    protected function _prepareDates ()
    {
        switch (true) {
            case (isset($this->_data['dateFrom'] ) && !empty($this->_data['dateFrom']) && isset($this->_data['dateTo'] ) && !empty($this->_data['dateTo']) ):
                $this->_dateFrom = $this->_createDate($this->_data['dateFrom']);
                $this->_dateTo   = $this->_createDate($this->_data['dateTo'])->addDay(1)->subSecond(1);
                break;
            case (isset($this->_data['dateFrom'] ) && !empty($this->_data['dateFrom']) && (!isset($this->_data['dateTo']) || empty($this->_data['dateTo'])) ):
                if ($this->_createDate($this->_data['dateFrom'])->get() == $this->_createDate()->get()) {
                    $this->_dateFrom = $this->_createDate($this->_data['dateFrom']);
                    $this->_dateTo   = $this->_createNowDate();
                }
                $this->_dateFrom = $this->_createDate($this->_data['dateFrom']);
                $this->_dateTo   = $this->_createDate($this->_data['dateFrom'])->addHour(23)->addMinute(59)->addSecond(59);
                break;
            default :
                $this->_dateFrom = $this->_createDate()->subMonth(1);
                $this->_dateTo   = $this->_createNowDate()->setMinute(0)->setSecond(0);
        }
        return $this;
    }
    
    /**
     * Retrive initialized Date 
     * 
     * @param Zend_Date|string $date
     * @return Zend_Date
     */
    protected function _createDate ($date = null)
    {
        $dateObj = $this->_createNowDate();
        $dateObj->setHour(0)->setMinute(0)->setSecond(0);
        if(!empty($date))
        {
            $dateObj->set($date, $this->_dateFormat);
        }
        return $dateObj;
    }
    
    /**
     * Create now Date
     * 
     * @return Zend_Date
     */
    protected function _createNowDate ()
    {
        $dateObj = new Zend_Date();
        $dateObj->setOptions(array('format_type' => 'php'));
        return $dateObj;
    }
    
    /**
     * Return Db Query to retrieve the Report Data (quantity)
     * 
     * @param Zend_Date $fromDate
     * @param Zend_Date $toDate
     * @return string
     */
    protected function _getRangeQueryPart(Zend_Date $fromDate, Zend_Date $toDate)
    {
        $model = $this->_getModel()->getAdapter();
        return $this->actionType.'  >='.$model->quote($fromDate->get($this->_dateFormat)).' AND '.$this->actionType.' <'.$model->quote($toDate->get($this->_dateFormat));
    }
}

