<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2013. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analytics
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * class System_Lib_Analitics_View_Table_Row 
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analitics
 */
abstract class System_Lib_Analytics_View_Abstract
{
    /**
     * View object instance
     * 
     * @static
     * @var Zend_View
     */
    public static $view;
    
    /**
     * Class constructor
     * 
     * @return void
     */
    public function __construct() {
        if (!self::$view instanceof Zend_View) {
            self::$view = Singular_Runtime::extract('view');
        }
    }

    /**
     * Renders a template or picture.
     *
     * @return string
     */
    abstract public function render();
    
    /** 
     * 
     * @return string
     */
    public function __toString()
    {
        try {
            $return = $this->render();
            return $return;
        } catch (Exception $e) {
            $message = "Exception caught by template renderer: " . $e->getMessage()
                     . "\nStack Trace:\n" . $e->getTraceAsString();
            trigger_error($message, E_USER_WARNING);
            return '';
        }
    }
    
    /**
     * Get current locale of system
     * 
     * @return type
     */
    protected function _getLocale ()
    {
        return Singular_Core::_('settings')->getSetting('site_default_language');
    }        
}
