<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Analytics PChart Round Diagram class
 * 
 * Render PChart Diagramm for given  report
 *
 * @uses       System_Lib_Analytics_View_Abstract
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analytics
 */
class System_Lib_Analytics_View_Diagram_Round extends System_Lib_Analytics_View_Abstract 
{
    /**
     * Chart object
     * 
     * @var System_Lib_Pchart_Chart 
     */
    protected $_chart;
    
    /**
     * Diagram initialization data. 
     * 
     * @var System_Lib_Pchart_Data 
     */
    protected $_data;

    /**
     *
     * 
     * @var string 
     */
    public $fontPath;
    
    /**
     * Diagram aread dimensions
     * @var int 
     */
    public $areaDimensionX;
    public $areaDimensionY;
    
    /**
     * Legend labels
     * 
     * @var type 
     */
    protected $_legendData = array();
    
    /**
     *  Constructor
     */
    public function __construct() 
    {  
        parent::__construct();
        $this->fontPath = ROOT_PATH . DS . 'public' . DS . 'files' . DS . 'logo' . DS . 'trebucbd.ttf';
        
        $this->areaDimensionX = 747;
        $this->areaDimensionY = 310;
        $this->_chart = new System_Lib_Pchart_Chart($this->areaDimensionX, $this->areaDimensionY);  
        $this->_data = new System_Lib_Pchart_Data();
    }
    
    /**
     * Add legend data
     * 
     * @param array $data
     * @return \System_Lib_Analytics_View_Diagram_Round
     */
    public function addLegendDataSet(array $data)
    {
        $this->_data->SetSerieName('Legend','legend');
        $this->_data->AddPoint($data,'legend');
        return $this;
    }        

    /**
     * Legend data initialization
     * 
     * @return \System_Lib_Analytics_View_Diagram_Round
     */
    public function _initLegendData()
    {
        $this->_data->SetSerieName('Legend','legend');
        $this->_data->AddPoint( $this->_legendData,'legend');
        return $this;
    }
    
    /**
     * Add pie on to canvas
     * 
     * @return \System_Lib_Analytics_View_Diagram_Round
     */
    protected function _drawCircle()
    {
        $centerX = 310;
        $centerY = 155;
        $radius = 137;

        // This will draw a shadow under the pie chart  
        if (!$this->_isEmptyDataSet()) {
            $this->_chart->drawFilledCircle($centerX + 2,$centerY +2 ,$radius,200,200,200);
        }
        
        // Draw the pie chart  
        $this->_chart->setFontProperties($this->fontPath,10);
        $this->_chart->drawBasicPieGraph($this->_data->getData(),$this->_data->GetDataDescription(),$centerX,$centerY,$radius,PIE_PERCENTAGE,255,255,218);  
        return $this;
    }
    
    /**
     * Add legen on to canvas
     * 
     * @return \System_Lib_Analytics_View_Diagram_Round
     */
    protected function _drawLegend()
    {
        $positionX = 600;
        $positionY = 75;
        $this->_chart->drawPieLegend($positionX,$positionY,$this->_data->GetData(),$this->_data->GetDataDescription(),250,250,250);
        return $this;
    }        

    /**
     * Render diagram
     * 
     * @return string
     */
    public function render () 
    {
        if ($this->_isEmptyDataSet()) {
            return self::$view->partial( '_analytics'.DS.'statistics-diagram-empty.phtml' ,'system' , null);
        }
        $this->_data->AddAllSeries();  
        $this->_data->SetAbsciseLabelSerie('legend');

        $this->_drawCircle();
        $this->_drawLegend();
    
        ob_start();
            $this->_chart->Stroke();
            $result = ob_get_contents();
        ob_end_clean();
        $output = self::$view->partial( '_analytics'.DS.'statistics-image-diagram.phtml' ,'system' , array ('image'=> $this->_imageToBase64($result)) ); 
        return $output;
    }
    
    /**
     * Check empty data set or no
     * 
     * @return boolean
     */
    protected function _isEmptyDataSet () 
    {
        $summ = null;
        foreach ($this->_data->GetData() as $data) {
            $summ += $data['dataSet'];
        }
        if (empty($summ)) {
            return true;
        }
        return false;
    }
    
    /**
     * Add report values for render on diaram
     * 
     * @param array $dataSet
     * @return \System_Lib_Analytics_View_Diagram_Round
     */
    public function addDataSet (array $dataSet)
    {
        $this->_data->SetSerieName('Data','dataSet');
        $this->_data->AddPoint($dataSet,'dataSet');
        return $this;
    }
    
    /**
     * Present image as data url
     * 
     * @param type $sImg
     * @return string
     */
    protected function _imageToBase64($sImg)
    {
        $result = base64_encode($sImg);
        return 'data:image/png;base64,'.$result;
    }
    
}


