<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Analytics Svg Diagram class
 * 
 * Render Svg Diagramm for given  report
 *
 * @uses       System_Lib_Analytics_View_Abstract
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analytics
 */
class System_Lib_Analytics_View_Diagram_Svg_DateRange extends System_Lib_Analytics_View_Abstract
{
    /**
     * The maximum number of points on the x-axis
     * 
     * @var int 
     */
    protected  $_maxXPointsQnty = 168;

    /**
     * Report for presentation on diagramm
     * 
     * @var System_Lib_Analytics_Model_Report_Time_Abstract 
     */
    protected  $_report;
    
    /**
     * Report data
     * 
     * @var type 
     */
    protected  $_data;
    
    /**
     * Data for render template
     * 
     * @var type 
     */
    protected  $_templateData = array();

    /**
     *  Class constructor
     * 
     * @param array $config
     * @return void
     */
    public function __construct($config) 
    {
        parent::__construct($config);
        $this->_data = new System_Lib_Pchart_Data();
        $reportConfig = $config['reportConfig'];
        $reportConfig['dateRangeData']['maxValueQuantity'] = $this->_maxXPointsQnty;
        $this->_report = new System_Lib_Analytics_Model_Report_Time_DateRange($reportConfig);
        
        $this->addDataSet($this->_report->getDataSet());
    }      

    /**
     * Add report data for diagram
     * 
     * @param array $dataSet
     * @return \System_Lib_Analytics_View_Diagram_Svg_DateRange
     */
    public function addDataSet (array $dataSet)
    {
        $this->_data->SetSerieName('Data','dataSet');
        $this->_data->AddPoint($dataSet,'dataSet');
        return $this;
    }
    
    /**
     * Render diagramm
     * 
     * @return string
     */
    public function render () 
    {
        $this->_prepareScaleLabels();
        $this->_initValueSet();
        $this->_initScaleOptions();
        $this->_initBar();
        $output = self::$view->partial( '_analytics'.DS.'statistics-svg-diagram.phtml' ,'system' , $this->_templateData );
        return $output;
    }
    
    /**
     * Prepare Diagram Bar properties
     * 
     * @return \System_Lib_Analytics_View_Diagram_Svg_DateRange
     */
    public function _initBar () 
    {
        $barSpaceWidth = 2;
        $minBarWidth = 2.1;
        
        $barQty = count( $this->_report->getDataSet() ) + 1;
        $barSpacesQty = $barQty + 1;
        
        $axisWidth = ($this->_maxXPointsQnty + 1) * $minBarWidth + ($this->_maxXPointsQnty + 2) * $barSpaceWidth; 
        
        $this->_templateData['barWidth'] = (int) floor( ($axisWidth - $barSpacesQty)/$barQty );
        return $this;
    }

    /**
     * Prepare Diagram Scale options
     * 
     * @return \System_Lib_Analytics_View_Diagram_Svg_DateRange
     */
    protected function _initScaleOptions() 
    {
        $this->_initTickLabelFrequency();
        
        $maxYValue = round ((float)$this->_getMaxDataValue(),-1,PHP_ROUND_HALF_UP );
        if($maxYValue < 10 ) {
            $maxYValue = 10 ;
        }
        $this->_templateData['minY'] = 0;
        $this->_templateData['maxY'] = $maxYValue;
        return $this;
    }
    
    /**
     * Prepare Diagram Scale Labels
     * 
     * @return \System_Lib_Analytics_View_Diagram_Svg_DateRange
     */
    protected function _prepareScaleLabels ()
    {
        $precision = $this->_report->getPrecision();
        $this->_data->AddPoint($this->_report->getDateRanges(),'dateRange');
        $frequency = $this->_getLabelFrequencyProperties();
        
        switch ($precision['type']) {
            case 'hour' :
                $params = $this->_getHourLabelParams();
                if ($frequency['type'] == 'single') {
                    $format = $params['single']['format'];
                    break;
                }
                if ($frequency['type'] == 'range') {
                    $format = $params['range']['format'];
                    break;
                }
            case 'day'  :
                $params = $this->_getDayLabelParams();
                if ($frequency['type'] == 'single') {
                    $format = $params['single']['format'];
                    break;
                }
                if ($frequency['type'] == 'range') {
                    $format = $params['range']['format'];
                    break;
                }
            case 'week' :
                $params = $this->_getDayLabelParams();
                if ($frequency['type'] == 'single') {
                    $format = $params['single']['format'];
                    break;
                }
                if ($frequency['type'] == 'range') {
                    $format = $params['range']['format'];
                    break;
                }
            case 'month':
                $params = $this->_getMonthLabelParams();
                if ($frequency['type'] == 'single') {
                    $format = $params['single']['format'];
                    break;
                }
                if ($frequency['type'] == 'range') {
                    $format = $params['range']['format'];
                    break;
                }
        }
               
        $i = $frequency['frequency'];
        $locale = $this->_getLocale();
        foreach ($this->_data->Data as $key=>$data) {
            if ( $i == $frequency['frequency'] ) {
                $i = 0;
                if ($frequency['type'] == 'single') {
                    $this->_data->Data[$key]['Name'] = (is_string($format))? 
                        $data['dateRange']['dateFrom']->get($format, $locale)
                        :$format->setDate($data['dateRange']['dateFrom'])->render();
                }
                if ($frequency['type'] == 'range') {
                    $this->_data->Data[$key]['Name'] = $data['dateRange']['dateFrom']->get($format, $locale).' - '.$data['dateRange']['dateTo']->get($format, $locale);
                }
            } else {
                $this->_data->Data[$key]['Name'] = '';
            }
            $i++;
        }
        $this->_initTickLabels();
        return $this;
    }
    
    /**
     * Prepare Diagram Bar ordinates
     * 
     * @return \System_Lib_Analytics_View_Diagram_Svg_DateRange
     */
    protected function _initValueSet()
    {
        $dataSet = array();
        array_push($dataSet, array(0,'','',''));
        foreach ($this->_data->GetData() as $key => $reportPart) {
            $dateFrom  = $reportPart['dateRange']['dateFrom']->get('j M H:i',$this->_getLocale());
            $dateTo    = $reportPart['dateRange']['dateTo']->get('j M H:i',$this->_getLocale());
            $dataSet[] = array($key+1,$reportPart['dataSet'],$dateFrom,$dateTo); 
        }
        array_push($dataSet, array($key+2,'','',''));
        
        $this->_templateData['dataSet'] = json_encode($dataSet);
        
        array_shift($dataSet);
        array_pop($dataSet);
        $this->_templateData['highlightSet'] = json_encode($dataSet);
        
        return $this;
    }
    
    /**
     * Prepare Tick Labels for Diagram
     * 
     * @return \System_Lib_Analytics_View_Diagram_Svg_DateRange
     */
    protected function _initTickLabels ()
    {
        $ticks = array();
        array_push($ticks, array(0,''));
        foreach ($this->_data->GetData() as $key => $unit ) {
            if ($unit['Name'] !='') {
                $ticks[] = array($key+1, $unit['Name']);
            }
        }
        array_push($ticks, array($key+2,''));
        
        reset($ticks);
        $firstTick = current($ticks); 
        $lastTick  = end($ticks);
        reset($ticks);
        $this->_templateData['disableTicks'] = json_encode( array($firstTick[0], $lastTick[0]) );
        
        $this->_templateData['ticks'] = json_encode($ticks);
        return $this;        
    }
    
    /**
     * Init Axis tick Label frequency
     * 
     * @return \System_Lib_Analytics_View_Diagram_Svg_DateRange
     */
    protected function _initTickLabelFrequency ()
    {
        $tickFrequencyProperties = $this->_getLabelFrequencyProperties();
        $this->_templateData['frequency'] = $tickFrequencyProperties['frequency'];
        return $this;
    }        

    /**
     * Retrieve max. Diagram Bar quantity 
     * 
     * @return \System_Lib_Analytics_View_Diagram_Svg_DateRange
     */
    protected function _getMaxDataValue ()
    {
        $data = array();
        foreach ($this->_data->getData() as $value) {
            $data[] = $value['dataSet'];
        }
        return max($data);
    }
    
    /**
     * Return Tick Label frequency params.
     * 
     * @return array
     */
    protected function _getLabelFrequencyProperties ()
    {
        $precision = $this->_report->getPrecision(); 
        $singleFrequency = $this->_getLabelFrequencyByType('single');
        $rangeFrequency =  $this->_getLabelFrequencyByType('range');
        $diff = $this->_getDateDiff();
        
        //range frequency
        $rangeFrequency = 0;
        if ($rangeFrequency == 1) {
            switch (true) {
                case ($precision['type'] == 'hour' && $diff['year'] == 0 && $diff['day'] >= 2 && $diff['day'] <= 6 ):
                    $type = 'range';
                    break;
                case ($precision['type'] == 'day' && $diff['year'] == 0 && $diff['day'] > 31 && $diff['day'] <= 125):
                    $type = 'range';
                    break;
                case ($precision['type'] == 'week' && $diff['year'] == 0 && $diff['day'] <= 140):
                    $type = 'range';
                    break;
                case ($precision['type'] == 'month' && $diff['year'] >= 2):
                    $type = 'range';
                    break;
                default :
                    $type = 'single';
            }
        } else {
            $type = 'single';
        }
        
        if ($type == 'range') {
            return array('type' => 'range', 'frequency' => $rangeFrequency); 
        } else {
            return array('type' => 'single', 'frequency' => $singleFrequency);
        }
    }
    
    /**
     * Return Axis Scale properties depending on the type of precision
     * 
     * @param type $type
     * @return array
     */
    protected function _getScalePropertiesByType ($type) 
    {
        switch ($type) {
            case 'hour' :
                $params = $this->_getHourLabelParams();
                break;
            case 'day'  :
                $params = $this->_getDayLabelParams();
                break;
            case 'week' :
                $params = $this->_getDayLabelParams();
                return array('single' => $params['range']['maxFrequency'], 'range' => $params['range']['maxFrequency']);
            case 'month':
                $params = $this->_getMonthLabelParams();
        }
        return array('single' => $params['single']['maxFrequency'], 'range' => $params['range']['maxFrequency']);
    }
    
    /**
     * Return Format Label params for 'Hour' type precission
     * 
     * @return array
     */
    protected function _getHourLabelParams ()
    {
        $diff = $this->_getDateDiff();
        if ( $diff['year'] == 0 && $diff['day'] <= 1 ) {
            return array(
                'single' => array(
                    'format' => 'G',
                    'maxFrequency' => '25',
                ),
                'range'  => array(
                    'format' => 'G',
                    'maxFrequency' => '7',
                )
            );
        } else {
            $singleFormat = new System_Lib_Analytics_View_Diagram_Svg_TickLabel_DateSingle();
            return array(
                'single' => array(
                    'format' => $singleFormat->setFormat('top','j M')->setFormat('bottom','G:i'),
                    'maxFrequency' => '15',
                ),
                'range'  => array(
                    'format' => 'j G:i',
                    'maxFrequency' => '9',
                )
            );
        }
    }
    
    /**
     * Return Format Label params for 'Day' type precission
     * 
     * @return array
     */
    protected function _getDayLabelParams ()
    {
        $diff = $this->_getDateDiff();
        if ( $diff['year'] == 0 && $diff['day'] <= 1 ) {
            return array(
                'single' => array(
                    'format' => 'Y M j',
                    'maxFrequency' => '10',
                ),
                'range'  => array(
                    'format' => 'Y M j',
                    'maxFrequency' => '10',
                )
            );
        }
        if ( $diff['year'] == 0 && $diff['day'] <= 25 ) {
            return array(
                'single' => array(
                    'format' => 'j',
                    'maxFrequency' => '18',
                ),
                'range'  => array(
                    'format' => 'j M',
                    'maxFrequency' => '15',
                )
            );
        } 
        if ( $diff['year'] == 0 && $diff['day'] > 25 && $diff['day'] < 60) {
            return array(
                'single' => array(
                    'format' => 'j M',
                    'maxFrequency' => '14',
                ),
                'range'  => array(
                    'format' => 'j M',
                    'maxFrequency' => '15',
                )
            );
        } else {
            return array(
                'single' => array(
                    'format' => 'j M',
                    'maxFrequency' => '20',
                ),
                'range'  => array(
                    'format' => 'j M',
                    'maxFrequency' => '15',
                )
            );
        }
    }
    
    /**
     * Return Format Label params for 'Month' type precission
     * 
     * @return return
     */
    protected function _getMonthLabelParams ()
    {
        $diff = $this->_getDateDiff();
        
        if ( $diff['year'] == 0 && $diff['day'] < 28 ) {
            return array(
                'single' => array(
                    'format' => 'M',
                    'maxFrequency' => '21',
                ),
                'range'  => array(
                    'format' => 'Y Mj',
                    'maxFrequency' => '17',
                )
            );
        }    
        
        if ( $diff['year'] == 0 && $diff['day'] > 28 && $diff['day'] < 365) {
            return array(
                'single' => array(
                    'format' => 'M',
                    'maxFrequency' => '21',
                ),
                'range'  => array(
                    'format' => 'M',
                    'maxFrequency' => '17',
                )
            );
        } else {
            return array(
                'single' => array(
                    'format' => 'M Y',
                    'maxFrequency' => '18',
                ),
                'range'  => array(
                    'format' => 'M Y',
                    'maxFrequency' => '10',
                )
            );
        }
    }
    
    /**
     * Return difference between the start and end date
     * 
     * @return array
     */
    protected function _getDateDiff ()
    {
        return $this->_report->getDateDiff();
    }
    
    /**
     * Return label frequency by precision type
     * 
     * @param type $type
     * @return type
     */
    protected function _getLabelFrequencyByType ($type)
    {
        $precision = $this->_report->getPrecision(); 
        $valueQuantity = count($this->_data->Data);
        $scalePorerties = $this->_getScalePropertiesByType($precision['type']);
        $frequency = (float)($valueQuantity/$scalePorerties[$type]);
        return ($frequency < 1)? 1 : ceil($frequency);
    }
    
}
