<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Analytics Tick Label formatter class
 * 
 * Represent given date in required format
 *
 * @uses       System_Lib_Analytics_View_Abstract
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analytics
 */
class System_Lib_Analytics_View_Diagram_Svg_TickLabel_DateSingle extends System_Lib_Analytics_View_Abstract  
{
    /**
     *  Data for render template
     * 
     * @var array 
     */
    protected $_data = array();
    
    /**
     * Path to template
     * 
     * @var type 
     */
    protected $_template;

    /**
     * Constructor
     * 
     */
    public function __construct() {
        parent::__construct();
        $this->_template = '_analytics'.DS.'statistics-svg-diagram-tick-label.phtml'; 
    }
    
    /**
     * Set template path
     * 
     * @param string $path
     */
    public function setTemplate ($path)
    {
        $this->_template = $path;
    }        

    /**
     * Render template
     * 
     * @return string
     */
    public function render() 
    {
         $output = self::$view->partial( $this->_template ,'system' , $this->_getTemplateData());
         return $output;
    }
    
    /**
     * Set date format for given position
     * 
     * @param type $stringPosition Available date string position
     * @param type $format Date format for given position
     * @return \System_Lib_Analytics_View_Diagram_Svg_TickLabel_DateSingle
     */
    public function setFormat($stringPosition, $format)
    {
        $this->_data['format'][$stringPosition] = $format;
        return $this;
    }
    
    /**
     * Set date for render
     * 
     * @param Zend_Date $date
     * @return \System_Lib_Analytics_View_Diagram_Svg_TickLabel_DateSingle
     */
    public function setDate (Zend_Date $date)
    {
        $this->_data['date'] = $date;
        return $this;
    }
    
    /**
     * Get initialized template data
     * 
     * @return type
     */
    protected function _getTemplateData ()
    {
        return array(
            'top'    => $this->_data['date']->get($this->_data['format']['top']),
            'bottom' => $this->_data['date']->get($this->_data['format']['bottom'])
        );
    }
}
