<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2013. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analitics_View_Table_Row
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Class for render AnalyticsStatisticsTable row.  
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analitics_View_Table_Row
 */
abstract class System_Lib_Analytics_View_Table_Row_Abstract extends System_Lib_Analytics_View_Abstract  
{
    /**
     *
     * @var string 
     */
    protected $_dateFormat = 'Y-m-d H:i:s';
    
    /**
     * Path to template
     * 
     * @var string 
     */
    protected $_template;
    
    /**
     * Module context
     * 
     * @var type 
     */
    protected $_module;
    
    /**
     * Counter model name
     * 
     * @var string 
     */
    public $model = null;
    
    /**
     * Count method name
     * 
     * @var string
     */
    public $counter = null;
    
    /**
     * Date type (database field name)
     * 
     * @var type 
     */
    public $actionType = 'created_at';

    /**
     * Condstructor
     * 
     * options:
     *  -statisticsDbTable
     *  -counter
     *  -template
     *  -id
     *  -title
     *  
     * @param mixed $config
     */
    public function __construct ($template, $module, $options = null)
    {
        parent::__construct();
        $this->_template = $template;
        $this->_module = $module;
        
        //check dbTableName
        if ( !isset($options['model']) || empty($options['model'])) {
            throw new Exception('"model" value must be defined.');
        }
        
        //check counter
        if( !isset($options['counter']) || empty($options['counter']) ) {
            throw new Exception('"counter" value must be defined.');
        }
        
        //set options
        $this->_addOptionsRecursively($options);
    }
    
    /**
     * add recursively options in to object this class 
     * 
     * @param array $options
     */
    protected function _addOptionsRecursively(array $options)
    {
        foreach ($options as $optionName => $option) {
            if ($optionName == 'options' && is_array($option)) {
                $this->_addOptionsRecursively($option);
                continue;
            }
            $this->_addOption($optionName,$option);    
        }
    }        
    
    /**
     * 
     * @param string $name
     * @param mixed $value
     * @return \System_Lib_Analytics_Model_Report_Abstract
     */
    protected function _addOption ($name, $value)
    {
        $name = preg_replace('/_(.?)/e',"strtoupper('$1')", $name);
        $this->$name = $value;
        return $this;
    }        

    /**
     * Retrieve template initialization data
     */
    abstract protected function _getTemplateData();

    /**
     * Renders a Analitics Statistics Table Row template.
     *
     * @return string|Zend_View_Helper_Partial
     */
    public function render ()
    {
        $templateData = $this->_getTemplateData();
        $htmlOutput = self::$view->partial( $this->_template , $this->_module , $templateData );
        return $htmlOutput;
    }
    
    /**
     * Get now Date
     * 
     * @return \Zend_Date
     */
    protected function _getDate ()
    {
        $dateObj = new Zend_Date();
        $dateObj->setOptions(array('format_type' => 'php'));
        $dateObj->setHour(0)->setMinute(0)->setSecond(0);
        return $dateObj;
    }
    
    /**
     * Retrieve counter model object
     * 
     * @return \System_Model_DbTable_Abstract
     * @throws Exception
     */
    protected function _getModel ()
    {
        $model = Singular_Loader::get($this->model); 
        if ($model instanceof System_Model_DbTable_Abstract)
        {
            return $model;
        }    
        throw new Exception('Unexpected model object.');
    }
    
    /**
     * Get db table fild (date type)
     * 
     * @return string
     */
    protected function _getActionType()
    {
        return $this->_actionType;
    }
    
    /**
     * Retrieve Begin This Week Date db query
     * 
     * @return string
     */
    protected function _getBeginThisWeekDate ()
    {
        $dayThisWeek = (int) $this->_getDate()->get('N');
        $delta = $dayThisWeek - 1;
        if (!empty($delta)) {
            $beginThisWeekDate = $this->_getDate()->subWeekday($delta);
        } else {
            $beginThisWeekDate = $this->_getDate();
        }
        return $beginThisWeekDate;
    }
    
    /**
     * Retrieve Begin This Month db query
     * 
     * @return string
     */
    protected function _getBeginThisMonthDate () 
    {
        $dayThisMonth = (int) $this->_getDate()->get('j');
        $delta = $dayThisMonth - 1; 
        if (!empty($delta)) {
            return $this->_getDate()->subDay($delta);
        }
        return $this->_getDate();
    }
    
    /**
     * Return quantity (report)
     * 
     * @param string $query
     * @return string 
     */
    protected function _getQuantity($query)
    {
        $aQuery = $this->_prepareQuery($query); 
        $model = $this->_getModel();
        return call_user_func_array( array($model, $this->counter), $aQuery);
    }
    
    /**
     * Prepare db query
     * 
     * @param string $query
     * @return string
     */
    protected function _prepareQuery($query)
    {
        if (isset($this->responseType)&& !empty($this->responseType)) {
            return array($query, $this->responseType);
        }
        return array($query);
    }        
}
