<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2013. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analytics
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * class System_Lib_Analitics_View_Table_Row 
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Analitics
 */
class System_Lib_Analytics_View_Table_Row_Default extends System_Lib_Analytics_View_Table_Row_Abstract  
{
    
    /**
     * Return data for render template
     * 
     * @return type
     */
    protected function _getTemplateData()
    {
        $filter = new System_Filter_ZeroToDash();
        return array(
            'id'        => $this->id,
            'title'     => $this->title,
            'icon'      => (isset($this->icon))? $this->icon : '',
            'today'     => $filter->filter( $this->_getQuantity($this->_getToday())     ),
            'yesterday' => $filter->filter( $this->_getQuantity($this->_getYesterday()) ),
            'thisWeek'  => $filter->filter( $this->_getQuantity($this->_getThisWeek())  ),
            'lastWeek'  => $filter->filter( $this->_getQuantity($this->_getLastWeek())  ),
            'thisMonth' => $filter->filter( $this->_getQuantity($this->_getThisMonth()) ),
            'lastMonth' => $filter->filter( $this->_getQuantity($this->_getLastMonth()) ),
            'total'     => $filter->filter( $this->_getQuantity($this->_getTotal())     ),
        );
    }
    
    /**
     *
     * Return "Today" query for counter.
     *
     * @return string
     */
    protected function _getToday ()
    {
        $model = $this->_getModel()->getAdapter();
        $nowDate = $this->_getDate()->get($this->_dateFormat);
        return $this->actionType.' = '.$model->quote($nowDate);
    }
    
    /**
     *
     * Return "Yesterday" query for counter.
     *
     * @return string
     */
    protected function _getYesterday ()
    {
        $model = $this->_getModel()->getAdapter();
        $yesterdayDate = $this->_getDate()->subDay(1)->get($this->_dateFormat);
        return $this->actionType.' = ' . $model->quote($yesterdayDate);
    }
    
    /**
     *
     * Return "This Week" query for counter.
     *
     * @return string
     */
    protected function _getThisWeek ()
    {
        $model = $this->_getModel()->getAdapter();
        return $this->actionType.' >='.$model->quote($this->_getBeginThisWeekDate()->get($this->_dateFormat));
    }
    
    /**
     *
     * Return "Last Week" query for counter.
     *
     * @return string
     */
    protected function _getLastWeek ()
    {
        $model = $this->_getModel()->getAdapter();
        $beginThisWeekDate = $this->_getBeginThisWeekDate()->get($this->_dateFormat);
        $beginLastWeekDate = $this->_getBeginThisWeekDate()->subWeek(1)->get($this->_dateFormat);
        return $this->actionType.'  >='.$model->quote($beginLastWeekDate).' AND '.$this->actionType.' <'.$model->quote($beginThisWeekDate);
    }
    
    /**
     *
     * Return "This Month" query for counter.
     *
     * @return string
     */
    protected function _getThisMonth ()
    {
        $model = $this->_getModel()->getAdapter();
        $beginThisMonthDate = $this->_getBeginThisMonthDate()->get($this->_dateFormat);
        return $this->actionType.' >='.$model->quote($beginThisMonthDate);
    }
    
    /**
     *
     * Return "Last Month" query for counter.
     *
     * @return string
     */
    protected function _getLastMonth ()
    {
        $model = $this->_getModel()->getAdapter();
        $beginThisMonthDate = $this->_getBeginThisMonthDate()->get($this->_dateFormat);
        $beginLastMonthDate = $this->_getBeginThisMonthDate()->subMonth(1)->get($this->_dateFormat);
        return $this->actionType.' >='.$model->quote($beginLastMonthDate).' AND '.$this->actionType.' <'.$model->quote($beginThisMonthDate);
    }
    
    /**
     *
     * Return "Total" query for counter.
     *
     * @return string
     */
    protected function _getTotal ()
    {
        return '';
    }
}
