<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Form_Element
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * System_Lib_Form_Element_File
 *
 * @category   System
 * @package    System_Lib
 * @subpackage System_Lib_Form
 * @copyright  Copyright (c) 2005-2011 Zend Technologies USA Inc. (http://www.zend.com)
 * @license    http://framework.zend.com/license/new-bsd     New BSD License
 * @version    $Id: File.php 23871 2011-04-23 22:40:16Z ramon $
 */
class System_Lib_Form_Element_File extends Zend_Form_Element_File
{
    /**
     * Skip File Decorator flag.
     *
     * @var bool
     */
    private $_skipFileDecorator = false;

    /**
     * Sets skip File Decorator flag to prevent
     * double-rendering of File element.
     *
     * @param bool $flag
     *
     * @return System_Lib_Form_Element_File
     */
    public function setSkipFileDecorator($flag)
    {
        $this->_skipFileDecorator = (bool) $flag;
        return $this;
    }

    /**
     * Returns skip File Decorator flag
     *
     * @return bool
     */
    public function getSkipFileDecorator()
    {
        return $this->_skipFileDecorator;
    }

    /**
     * Validate upload
     *
     * @param  string $value   File, can be optional, give null to validate all files
     * @param  mixed  $context
     *
     * @return bool
     */
    public function isValid($value, $context = null)
    {
        $userAgent = $_SERVER['HTTP_USER_AGENT'];
        if (strpos($userAgent, 'iPhone') || strpos($userAgent, 'iPod') || strpos($userAgent, 'iPad')) {
            return true;
        }
        return parent::isValid($value, $context);
    }

    /**
     * Processes the file, returns null or the filename only
     * For the complete path, use getFileName
     *
     * @return null|string
     */
    public function getValue()
    {
        $userAgent = $_SERVER['HTTP_USER_AGENT'];
        if (strpos($userAgent, 'iPhone') || strpos($userAgent, 'iPod') || strpos($userAgent, 'iPad')) {
            return null;
        }
        return parent::getValue();
    }

    /**
     * Render form element
     * Checks for decorator interface to prevent errors
     *
     * @param  Zend_View_Interface $view
     *
     * @return string
     * @throws Zend_Form_Element_Exception
     */
    public function render(Zend_View_Interface $view = null)
    {
        $marker = false;
        foreach ($this->getDecorators() as $decorator) {
            if ($decorator instanceof Zend_Form_Decorator_Marker_File_Interface) {
                $marker = true;
            }
        }

        if (!$marker) {
            throw new Zend_Form_Element_Exception('No file decorator found... unable to render file element');
        }

        if ($this->_isPartialRendering) {
            return '';
        }

        if (null !== $view) {
            $this->setView($view);
        }

        $content = '';
        foreach ($this->getDecorators() as $name => $decorator) {
            if ($this->getSkipFileDecorator() && 'Zend_Form_Decorator_File' == $name) continue;
            $decorator->setElement($this);
            $content = $decorator->render($content);
        }
        return $content;
    }
}
