<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * logs table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_Logs extends System_Model_DbTable_Abstract
{
    const ADDED                 = 'added';
    const UPDATED               = 'updated';
    const CREATED               = 'created new';
    const COMMENTED             = 'commented';
    const CHANGE_STATUS         = 'changed status';
    const ANSWER_ADDED          = 'added off. answer in';
    const MOVED_TO_TRASH        = 'moved to trash';
    const RECOVERY              = 'restored';
    const LIMIT                 = 10;

    /**
     * The table name
     *
     * @var string
     */
    protected $_name            = 'activity_logs';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary         = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass        = 'System_Model_Log';

    /**
     * Inserts a new log row.
     *
     * @see    parent::insert()
     * @param  array  $data  Column-value pairs.
     * @return mixed         The primary key of the row inserted.
     */
    public function insert (array $data)
    {
        if (!isset($data['created_at'])) {
            $data = $this->_setDate($data, 'created_at');
        }
        if (!isset($data['created_by_id'])) {
            $data = $this->_setActor($data, 'created_by_id');
        }
        $loggedUser = Singular_Runtime::extract('loggedUser');
        if (!isset($data['created_by_name'])) {
            $data['created_by_name'] = $loggedUser->getName();
        }
        if (!isset($data['created_by_email'])) {
            $data['created_by_email'] = $loggedUser->email;
        }
        
        $comment = $data['comment'];
        $comment = trim($comment);
        $comment = preg_replace("/\[reply\].*\[\/reply\]/", "", $comment);
        $view = Zend_Layout::getMvcInstance()->getView();
        $comment = $view->bbcode($comment);
        $comment = strip_tags($comment);
        $fullLength = mb_strlen($comment, 'utf-8');
        if ($fullLength > 200) {
            $comment = trim(mb_substr($comment, 0, 200, 'utf-8')) . "...";
        }
        $comment = str_replace("\n", " ", $comment);
        $data['comment'] = $comment;

        return parent::insert($data);
    }

    /**
     * Returns all activity by object
     *
     * @param  Zend_Db_Table_Row $object
     * @return Zend_Db_Table_Rowset
     */
    public function findByObject($object)
    {
        $select = $this->getSelect();
        $select->where('object_id=?', $object->id)
               ->order('created_at ASC');
        
        $query = "";
        $types = Singular_Runtime::extract('types');
        foreach ($types as $type) {
            $query .= $this->getAdapter()->quoteInto('object=? OR ', $type->slug);
        }
        $types->rewind();
        $query = substr($query, 0, strlen($query)-4);
        $select->where($query);
        
        return $this->fetchAll($select);
    }
    
    /**
     * Return fetched paginator rows
     *
     * @param  array $where
     * @param  int $page
     * @param  int|null $limit
     * @return mixed
     */
    public function getPaginator ($where, $page, $limit = null)
    {
        $tableName = $this->getTableName();
        $select = $this->getSelect();
        if ($where['date'] == 'today') {
            $select->where("$tableName.created_at > ?", date("Y-m-d H:i:s", strtotime("-1 day")));
        } elseif ($where['date'] == 'yesterday') {
            $select->where("$tableName.created_at > ?", date("Y-m-d H:i:s", strtotime("-2 day")));
            $select->where("$tableName.created_at < ?", date("Y-m-d H:i:s", strtotime("-1 day")));
        }elseif ($where['date'] == 'this_week') {
            $select->where("$tableName.created_at > ?", date("Y-m-d H:i:s", strtotime("last Monday")));
        }elseif ($where['date'] == 'last_week') {
            $select->where("$tableName.created_at > ?", date("Y-m-d H:i:s", strtotime("-7 day last Monday")));
            $select->where("$tableName.created_at < ?", date("Y-m-d H:i:s", strtotime("last Monday")));
        }elseif ($where['date'] == 'this_month') {
            $select->where("$tableName.created_at > ?", date("Y-m-d H:i:s", mktime("00", "00", "00", date('m'), "01", date('Y'))));
        }elseif ($where['date'] == 'last_month') {
            $select->where("$tableName.created_at > ?", date("Y-m-d H:i:s", mktime("00", "00", "00", date('m', strtotime("last month")), "01", date('Y'))));
            $select->where("$tableName.created_at < ?", date("Y-m-d H:i:s", mktime("00", "00", "00", date('m'), "01", date('Y'))));
        }
        if (empty($limit)) {
            $limit = self::LIMIT;
        }
        $select->order("$tableName.created_at DESC");
        return $this->getPaginatorRows($select, null, null, $page, $limit);
    }

    /**
     * Returns last n activity data
     *
     * @param  int $limit
     * @return Zend_Db_Table_Rowset|null
     */
    public function findLastActivity ($limit = 5)
    {
        $select = $this->getSelect();
        $select->limit($limit)
               ->order('created_at DESC');
        return $this->fetchAll($select);
    }
}
