<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * modules table model
 *
 * @uses       Singular_Core_System_Model_DbTable_Modules
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_Modules extends Singular_Core_System_Model_DbTable_Modules
{
    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'System_Model_Module';

    /**
     * Rowset class
     *
     * @var string
     */
    protected $_rowsetClass = 'System_Model_Rowset';

    /**
     * Returns all installed modules
     *
     * @param  bool $active
     * @return System_Model_Rowset
     */
    public function findInstalledModules ($active = true)
    {
        $select = $this->select();
        $select->order('id');

        if (true === (bool) $active) {
            $select->where('active = ?', 1, Zend_Db::INT_TYPE);
        }

        $dbRows = $this->_fetch($select);

        /* get default modules */
        $config = Singular_Runtime::extract("config");
        $defModules = $config->resources->autoloader->zend->modules->toArray();

        $rows = array();
        foreach ($defModules as $module) {
            $rows[] = array(
                'id' => 0,
                'name' => $module,
                'active' => 1,
                'is_system' => 1
            );
        }
        $rows = array_merge_recursive($rows, $dbRows);
        
        $data  = array(
            'table'    => $this,
            'data'     => $rows,
            'readOnly' => $select->isReadOnly(),
            'rowClass' => $this->getRowClass(),
            'stored'   => true
        );

        $rowsetClass = $this->getRowsetClass();
        if (!class_exists($rowsetClass)) {
            require_once 'Zend/Loader.php';
            Zend_Loader::loadClass($rowsetClass);
        }

        return new $rowsetClass($data);
    }

    /**
     * Returns module by given name
     *
     * @param  string $name
     * @return Zend_Db_Table_Row_Abstract|null
     */
    public function findByName($name)
    {
        $select = $this->select();
        $select->where('name=?', $name);
        return $this->fetchRow($select);
    }
}
