<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * privileges table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_Privileges extends System_Model_DbTable_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'privileges';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'System_Model_Privilege';

    /**
     * Returns all privileges
     * If $activeOnly = true returns only active privileges
     *
     * @param  bool $activeOnly
     * @return mixed
     */
    public function getAllPrivileges ($activeOnly = false)
    {
        $select = $this->select()->order('id ASC');
        if (true === $activeOnly) {
            $select->where('is_active = ?', 1, Zend_Db::INT_TYPE);
        }
        $result = $this->fetchAll($select);

        return (null !== $result) ? $result->toArray() : $result;
    }

    /**
     * Returns privilege by given role and resource
     *
     * @param  string $role
     * @param  string $resource
     * @return System_Model_Privilege
     */
    public function findByRoleAndResource($role, $resource)
    {
        $select = $this->getSelect();
        $select->where("roles=?", $role);
        $select->where("resources=?", $resource);

        return $this->fetchRow($select);
    }

    /**
     * Returns all privileges by given user role
     *
     * @param  string $role
     * @return Zend_Db_Table_Rowset
     */
    public function findByRole($role)
    {
        $where = $this->getAdapter()->quoteInto('roles=?', $role);
        return $this->fetchAll($where);
    }

    /**
     * Returns (id => name) privileges list bu given role
     *
     * @param  string $role
     * @return array
     */
    public function findResourcesNameByRole($role)
    {
        $select = $this->getAdapter()->select();
        $select->from(array('p' => $this->getTableName()), array('p.id','p.resources'));
        $where = $this->getAdapter()->quoteInto('roles=?', $role);
        $select->where($where);
        $where = $this->getAdapter()->quoteInto('type=?', System_Model_Privilege::ALLOW);
        $select->where($where);

        return $this->getAdapter()->fetchPairs($select);
    }
}
