<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * resources table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @uses       Singular_Db_Table_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_Resources extends System_Model_DbTable_Abstract
{
    const ADMIN_ACCESS           = 'object:can_access_admin';
    const OFFICIAL_REPRESENTATIVE = 'object:representative';
    const MANAGE_PEOPLE           = 'object:manage_people';
    const MANAGE_RESPONSES        = 'object:manage_responses';
    const USE_ACTIVITY_LOG        = 'object:can_use_active_log';
    
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'resources';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'System_Model_Resource';

    /**
     * Returns resource row by it's name
     *
     * @param  string $resource
     * @return System_Model_Resource
     */
    public function findByResource ($resource)
    {
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.resource=?", $resource);
        return $this->fetchRow($select);
    }

    /**
     * Returns multiple resources by theirs name
     *
     * @param  array $resources
     * @return Zend_Db_Table_Rowset
     */
    public function findByResources ($resources)
    {
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.resource IN (?)", $resources);
        return $this->fetchAll($select);
    }

    /**
     * Returns all resources (ordered)
     *
     * @return Zend_Db_Table_Rowset
     */
    public function findAll ()
    {
        $select = $this->getSelect();
        $select->order('id ASC');
        return $this->fetchAll($select);
    }

}
