<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * response_answers table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_ResponsesAnswers extends System_Model_DbTable_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'response_answers';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'System_Model_ResponseAnswer';

    /**
     * Reference map
     *
     * @var array
     */
    protected $_referenceMap = array(
        'Response' => array(
            'columns'           => array('response_id'),
            'refTableClass'     => 'System_Model_DbTable_Responses',
            'refColumns'        => array('id'),
            'onDelete'          => self::CASCADE,
        ),
        'CreatedBy' => array(
            'columns'           => array('created_by_id'),
            'refTableClass'     => 'System_Model_DbTable_Users',
            'refColumns'        => array('id'),
            'onDelete'          => self::CASCADE,
        )
    );

    /**
     * Inserts a new answer row.
     *
     * @see    parent::insert()
     * @param  array  $data  Column-value pairs.
     * @return mixed         The primary key of the row inserted.
     */
    public function insert (array $data)
    {
        $data = $this->_setDate($data, 'created_at');
        $data = $this->_setActor($data, 'created_by_id');

        return parent::insert($data);
    }

    /**
     * Updates existing answers rows.
     *
     * @param  array        $data  Column-value pairs.
     * @param  array|string $where An SQL WHERE clause, or an array of SQL WHERE clauses.
     * @return int          The number of rows updated.
     */
    public function update (array $data, $where) {

        $data = $this->_setDate($data, 'updated_at');
        $data = $this->_setActor($data, 'updated_by_id');

        parent::update($data , $where);
    }

    /**
     * Returns answer by id
     *
     * @param  int $id
     * @return System_Model_ResponseAnswer
     */
    public function findById($id)
    {
        $select = $this->getSelect();
        $select->where("id=?", $id);
        return $this->fetchRow($select);
    }

    /**
     * Returns answer by response_id
     * @param  int $responseId
     * @return mixed
     */
    public function findByResponseId($responseId)
    {
        $select = $this->getSelect();
        $select->where("response_id=?", $responseId);
        return $this->fetchRow($select);
    }

    /**
     * Returns select object
     *
     * @see    parent::getSelect()
     * @return Zend_Db_Select
     */
    public function getSelect()
    {
        $select = parent::getSelect();
        $select->where("deleted_by_id=0");
        return $select;
    }
}
