<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * response_statuses table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_ResponsesStatuses extends System_Model_DbTable_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'response_statuses';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'slug';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'System_Model_ResponseStatus';

    /**
     * Dependent Tables
     *
     * @var array
     */
    protected $_dependentTables = array(
        'System_Model_DbTable_Responses'
    );

    /**
     * Reference map
     *
     * @var array
     */
    protected $_referenceMap = array(
        'Type' => array(
            'columns' => array('type'),
            'refTableClass' => 'System_Model_DbTable_ResponsesTypes',
            'refColumns' => array('slug'),
            'onDelete' => self::CASCADE
        )
    );

    /**
     * Returns all response statuses
     *
     * @param  bool $typeOrder
     *
     * @return Zend_Db_Table_Rowset|null
     */
    public function findAll($typeOrder = false)
    {
        $select = $this->getSelect();
        if ($typeOrder) {
            $select->order("{$this->getTableName()}.type ASC");
        }
        $select->order("{$this->getTableName()}.order ASC");
        return $this->fetchAll($select);
    }

    /**
     * Returns response status by slug
     *
     * @param  string $slug
     * @return System_Model_ResponseStatus
     */
    public function findBySlug($slug)
    {
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.slug=?", $slug);
        return $this->fetchRow($select);
    }

    /**
     * Returns default response status by type.
     *
     * @param  string $type
     *
     * @return System_Model_ResponseStatus
     */
    public function findDefaultByType($type)
    {
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.type=?", $type);
        $select->where("{$this->getTableName()}.is_default=?", 1);
        return $this->fetchRow($select);
    }

    /**
     * Returns max response status order by type.
     *
     * @param  string $type
     *
     * @return mixed
     */
    public function findMaxOrderByType($type)
    {
        $select = $this->getSelect();
        $select->setIntegrityCheck(false);
        $select->from($this, array('max_order' => new Zend_Db_Expr("MAX({$this->getTableName()}.order)")));
        $select->where("{$this->getTableName()}.type=?", $type);
        $select->group("{$this->getTableName()}.order");
        $select->order("{$this->getTableName()}.order DESC");
        $select->limit(1);
        return $this->fetchRow($select);
    }

    /**
     * Returns all response statuses by type.
     *
     * @param  string $type
     *
     * @return Zend_Db_Table_Rowset|null
     */
    public function findByType($type)
    {
        $select = $this->getSelect();
        $select->where("type=?", $type);
        $select->order("{$this->getTableName()}.order ASC");
        return $this->fetchAll($select);
    }

    /**
     * Returns Zend_Db_Select.
     *
     * @return Zend_Db_Select
     */    
    public function getSelect()
    {
        $select = parent::getSelect();
        $modulesDbName = Singular_Loader::get('System_Model_DbTable_Modules')
                            ->getTableName();
        $responseTypesDbName = Singular_Loader::get('System_Model_DbTable_ResponsesTypes')
                            ->getTableName();
        $select->joinInner(array("t" => $responseTypesDbName),
                "{$this->getTableName()}.type=t.slug",
                array())
               ->joinInner(array("m" => $modulesDbName),
                "t.module_id=m.id",
                array())
               ->where("m.active=1");
        return $select;
    }
}
