<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * response_types table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_ResponsesTypes extends System_Model_DbTable_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'response_types';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'slug';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'System_Model_ResponseType';

    /**
     * Dependent Tables
     *
     * @var array
     */
    protected $_dependentTables = array(
        'System_Model_DbTable_ResponsesStatuses'
    );

    /**
     * Returns active responses types
     *
     * @return Zend_Db_Table_Rowset
     */
    public function findActive()
    {
        $select = $this->getSelect();
        $select->where("{$this->getTableName()}.is_active=1");
        $select->order("{$this->getTableName()}.order ASC");
        return $this->fetchAll($select);
    }

    /**
     * Returns responses types by slug
     *
     * @param  string $slug
     * @return System_Model_ResponseType
     */
    public function findBySlug($slug)
    {
        $select = $this->getSelect();
        $select->where("slug=?", $slug);
        return $this->fetchRow($select);
    }

    /**
     * Returns response type by object
     *
     * @param  string $objects
     * @return System_Model_ResponseType
     */
    public function findByObjects($objects)
    {
        $select = $this->getSelect();
        $select->where("objects=?", $objects);
        return $this->fetchRow($select);
    }
    
    /**
     * Returns Zend_Db_Select
     *
     * @return Zend_Db_Select
     */    
    public function getSelect()
    {
        $select = parent::getSelect();
        $modulesDbName = Singular_Loader::get('System_Model_DbTable_Modules')
                            ->getTableName();
        $select->joinInner(array("m" => $modulesDbName),
                "{$this->getTableName()}.module_id=m.id",
                array())
               ->where("m.active=1");
        return $select;
    }
}
