<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * roles table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_Roles extends System_Model_DbTable_Abstract
{
    const OFFICIAL_REPRESENTATIVE = 'official-representative';
    const ADMINISTRATOR           = 'administrator';
    const GUEST                   = 'guest';

    /**
     * The table name
     *
     * @var string
     */
    protected $_name            = 'roles';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary         = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass        = 'System_Model_Role';

    /**
     * Allows pre-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @return void
     */
    protected function _insert()
    {
        /* Generate slug from name */
        $title = strtolower($this->name);
        $title = preg_replace("/(?:[\'|\"|\.|\,|\!|\?|\&|\`|\:])/", "", $title);
        $this->role = preg_replace("/(?:[^a-z])/", "-", $title);
    }

    /**
     * Returns all roles
     *
     * @return Zend_Db_Table_Rowset
     */
    public function findAll()
    {
        $select = $this->getSelect();
        return $this->fetchAll($select);
    }

    /**
     * Returns all roles
     * when $activeOnly = true returns only active roles
     *
     * @param  bool $activeOnly
     * @return array
     */
    public function getAllRoles ($activeOnly = false)
    {
        $select = $this->select()->order('id ASC');
        if (true === $activeOnly) {
            $select->where('is_active = ?', 1, Zend_Db::INT_TYPE);
        }
        $result = $this->fetchAll($select);
        
        return (null !== $result) ? $result->toArray() : $result;
    }

    /**
     * Returns role by it's name
     *
     * @param  string $role
     * @return System_Model_Role
     */
    public function findByRole($role)
    {
        $where = $this->getAdapter()->quoteInto('role=?', $role);
        return $this->fetchRow($where);
    }

    /**
     * Returns roles
     *
     * @param  string|array $resources
     * @return Zend_Db_Table_Rowset
     */
    public function findWithoutResources($resources)
    {
        $select = $this->getSelect();
        $roles = $this->findRolesSlugsByResourcesDeny($resources);
        $select->where("{$this->getTableName()}.role IN (?)", $roles);
        
        return $this->fetchAll($select);
    }
    
    /**
     * Returns roles slugs
     *
     * @param  string|array $resources
     * @return array
     */
    public function findRolesSlugsByResourcesDeny($resources)
    {
        if (is_string($resources)) {
            $resources = array($resources);
        }
        $acl = Singular_Runtime::extract('acl');
        $roles = $acl->getRoles();
        $allowedRoles = array();
        foreach ($roles as $role) {
            $add = true;
            foreach ($resources as $resource) {
                if ($acl->isAllowed($role, $resource)) {
                    $add = false;
                }
            }
            if ($add) {
                $allowedRoles[] = $role;
            }
        }
        return $allowedRoles;
    }
}
