<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * user_configs table model
 *
 * @uses       System_Model_DbTable_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 */
class System_Model_DbTable_UsersConfigs extends System_Model_DbTable_Abstract
{
    /**
     * Constants
     */
    const SEND_NOTIFICATIONS = 'send_notifications';
    const PHOTO_TYPE = 'photo_type';
    const SEND_NOTIFICATIONS_ONCE_TIME = 'send_notifications_once_time';

    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'user_configs';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';

    /**
     * Row class
     *
     * @var string
     */
    protected $_rowClass = 'System_Model_UserConfig';

    /**
     * Reference map
     *
     * @var array
     */
    protected $_referenceMap = array(
        'User' => array(
            'columns' => array('user_id'),
            'refTableClass' => 'Users_Model_DbTable_Users',
            'refColumns' => array('id'),
            'onDelete' => self::CASCADE
        )
    );

    /**
     * Returns whole user-config by user_id
     *
     * @param  int $userId
     * @return array
     */
    public function findValuesByUserId($userId)
    {
        $select = $this->getAdapter()->select();
        $select->from($this->getTableName(), array('config_key', 'value'))
            ->where('user_id=?', $userId);

        return $this->getAdapter()->fetchPairs($select);
    }

    /**
     * Returns one entry by key and iser_id
     *
     * @param  string $key
     * @param  int $userId
     * @return System_Model_UserConfig
     */
    public function findByKeyAndUserId($key, $userId)
    {
        $select = $this->select()
            ->where('config_key=?', $key)
            ->where('user_id=?', $userId);

        return $this->fetchRow($select);
    }
}
