<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * languages row class
 *
 * @uses       System_Model_Row
 * @category   System
 * @package    System_Model
 */
class System_Model_Language extends System_Model_Row
{
    /**
     * Table class
     *
     * @var string
     */
    protected $_tableClass = 'System_Model_DbTable_Languages';

    /**
     * Allows pre-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_insert()
     * @return void
     */
    public function _insert()
    {
        parent::_insert();
        $pathName = APPLICATION_PATH . DS . 'localization' . DS . $this->code;
        mkdir($pathName);
        chmod($pathName, 0777);
    }

    /**
     * Allows post-update logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_postUpdate()
     * @return void
     */
    public function _postUpdate()
    {
        parent::_postUpdate();
        
        if ($this->_cleanData['code'] != $this->code) {
            $oldName = APPLICATION_PATH . DS . 'localization' . DS . $this->_cleanData['code'];
            $newName = APPLICATION_PATH . DS . 'localization' . DS . $this->code;
            rename($oldName, $newName);
            
            $templatesLang = Singular_Loader::get('Resources_Model_DbTable_TemplatesLang');
            $where = $templatesLang->getAdapter()->quoteInto('lang=?', $this->_cleanData['code']);
            $templatesLang->update(array(
                'lang' => $this->code
            ), $where);
        }
        $defaultCode = Singular_Core::_('settings')->getSetting('site_default_language');
        if ($defaultCode == $this->_cleanData['code']
                && $defaultCode != $this->code) {
            Singular_Core::_('settings')->changeSetting('site_default_language', $this->code);
        }
        Singular_Translate::clearCache();
        Singular_Runtime::extract('cachemanager')
                ->getCache('default')
                ->clean(Zend_Cache::CLEANING_MODE_ALL);
    }
    
    /**
     * Allows pre-delete logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::delete()
     * @return void
     */
    public function delete()
    {
        $code = $this->code;
        if (parent::delete()) {
            $pathName = APPLICATION_PATH . DS . 'localization' . DS . $code;
            System_Service_Files::removeDir($pathName);
        }
    }
}
