<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * logs row class
 *
 * @uses       System_Model_Row
 * @category   System
 * @package    System_Model
 */
class System_Model_Log extends System_Model_Row
{
    /**
     * Object
     *
     * @var Resources_Model_DbTable_ResponsesComments|System_Model_DbTable_Responses
     */
    protected $_object = null;
    
    /**
     * Table class
     *
     * @var string
     */
    protected $_tableClass = 'System_Model_DbTable_Logs';

    /**
     * Returns ISO-formatted created_at field
     *
     * @return Zend_Date
     */
    public function getCreatedAt()
    {
        return new Zend_Date($this->created_at, Zend_Date::ISO_8601);
    }

    /**
     * Returns log linked object
     *
     * @return Resources_Model_ResponseComment|System_Model_Response
     */
    public function getObject()
    {
        if ($this->_object == null) {
            if ($this->object == Singular_Loader::get('Resources_Model_ResponseComment')->type) {
                $commentsDb = Singular_Loader::get('Resources_Model_DbTable_ResponsesComments');
                $comment = $commentsDb->findById($this->object_id);
                if ($comment instanceof Resources_Model_ResponseComment && $comment->getObject() == $this->object) {
                    $this->_object = $comment;
                }
            } else {
                $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
                $response = $responsesDb->findById($this->object_id);
                if ($response instanceof System_Model_Response && $response->getObject() == $this->object) {
                    $this->_object = $response;
                }
            }
        }
        return $this->_object;
    }

    /**
     * Returns linked log image
     *
     * @return string
     */
    public function getImage()
    {
        $view = Zend_Layout::getMvcInstance()->getView();
        $icon = '';
        switch ($this->action) {
            case System_Model_DbTable_Logs::CREATED :
                $icon = $view->imageSrc('plus.png', null, false);
                break;
            case System_Model_DbTable_Logs::UPDATED :
                $icon = $view->imageSrc('/icons/16/edit.png', null, false);
                break;
            case System_Model_DbTable_Logs::COMMENTED :
                $icon = $view->imageSrc('/icons/16/comments.png', null, false);
                break;
            case System_Model_DbTable_Logs::ANSWER_ADDED :
                $icon = $view->imageSrc('/icons/16/off_answer.png', null, false);
                break;
            case System_Model_DbTable_Logs::CHANGE_STATUS :
                $icon = $view->imageSrc('/icons/16/status.png', null, false);
                break;
            case System_Model_DbTable_Logs::MOVED_TO_TRASH :
                $icon = $view->imageSrc('/icons/16/trash.png', null, false);
                break;
            case System_Model_DbTable_Logs::RECOVERY :
                $icon = $view->imageSrc('/icons/16/move.png', null, false);
                break;
        }
        return $icon;
    }

    /**
     * Cleans cache.
     *
     * @param array $additionalTags
     * @param bool  $onlyTags
     */
    public static function cleanCache($additionalTags = array(), $onlyTags = false)
    {
        $tags = array('log', 'logs', 'activity', 'activity_log', 'activity_logs', Singular_Loader::get(__CLASS__)
            ->getTable()->getTableName());
        if (is_array($additionalTags) && count($additionalTags)) {
            if ($onlyTags) {
                $tags = $additionalTags;
            } else {
                $tags = $tags + $additionalTags;
            }
        }
        Singular_Core::_('cache')->removeAllByTags($tags);
    }
}
