<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * modules row class
 *
 * @uses       System_Model_Row
 * @category   System
 * @package    System_Model
 */
class System_Model_Module extends System_Model_Row
{
    /**
     * Table class
     *
     * @var string
     */
    protected $_tableClass = 'System_Model_DbTable_Modules';

    /**
     * Module container
     *
     * @var Singular_Module_Container
     */
    protected $_moduleClass = null;

    /**
     * Magic method __toString
     *
     * @return mixed
     */
    public function  __toString() {
        return $this->name;
    }

    /**
     * Returns module name
     *
     * @return string
     */
    public function getName()
    {
        if ($this->_moduleClass === null) {
            $this->_moduleClass = $this->getModuleClass();
            if (isset($this->_moduleClass)) {
                return $this->_moduleClass->getContainer()->getName();
            }
            return '';
        } else {
            return $this->_moduleClass->getContainer()->getName();
        }
    }

    /**
     * Returns module class
     *
     * @return Singular_Module_Abstract
     */
    public function getModuleClass()
    {
        $class = null;
        $filename = MODULES_PATH . DS . $this->name . DS . 'Module.php';
        if (file_exists($filename)) {
            require_once $filename;
            $className = Singular_Stdlib_Module::formatName($this->name) . '_Module';
            if (class_exists($className, false)) {
                $class = new $className($this->name);
            }
        }
        return $class;
    }

    /**
     * Checks if module has translation file
     *
     * @param  string $langCode
     * @return bool
     */
    public function hasTranslation($langCode)
    {
        $filename = APPLICATION_PATH . DS . 'localization' . DS . $langCode . DS . $this->name . '.php';
        if (is_file($filename)) {
            return true;
        }
        return false;
    }

    /**
     * Checks if module translation file is editable
     *
     * @param  string $langCode
     * @return bool
     */
    public function canEditTranslation($langCode)
    {
        if ($this->hasTranslation($langCode)) {
            $filename = APPLICATION_PATH . DS . 'localization' . DS . $langCode . DS . $this->name . '.php';
            if (is_writable($filename)) {
                return true;
            }
        }
        return false;
    }

    /**
     * Checks if module translation file can be added
     *
     * @param  string $langCode
     * @return bool
     */
    public function canAddTranslation($langCode)
    {
        $localizationPath = APPLICATION_PATH . DS . 'localization' . DS . $langCode;
        if (is_writable($localizationPath) && !$this->hasTranslation($langCode)) {
            return true;
        }
        return false;
    }

    /**
     * Adds new module translation
     *
     * @param  array $translation
     * @param  string $langCode
     * @return void
     */
    public function addTranslation($translation, $langCode)
    {
        $content = "<?php \n return array(";
        foreach ($translation as $key=>$value) {
            if (!empty($value)) {
                $value = trim($value);
                if (!empty($value)) {
                    $key = preg_replace("/(\\\\'|\\\\{1,}){1,}+/", '', $key);
                    $value = preg_replace("/(\\\\\'|\\\\{1,}){1,}+/", '', $value);
                    $content .= "'" . str_replace("'", "\'", $key) . "' => '" . str_replace("'", "\'", $value) . "', \n";
                }
            }
        }
        $content .= ");";
        $filename = APPLICATION_PATH . DS . 'localization' . DS . $langCode . DS . $this->name . '.php';

        file_put_contents($filename, $content); @chmod($filename, 0777);
    }

    /**
     * Returns translation by given language
     *
     * @param  string $langCode
     * @return array            array with translations
     */
    public function getTranslation($langCode)
    {
        if ($this->hasTranslation($langCode)) {
            $filename = APPLICATION_PATH . DS . 'localization' . DS . $langCode . DS . $this->name . '.php';
            return include $filename;
        }
        return array();
    }

    /**
     * Returns translations keys for current module
     * 
     * @return array
     */
    public function getLangKey()
    {
        $filename = MODULES_PATH . DS . $this->name . DS . 'lang.php';
        if (is_file($filename)) {
            return include $filename;
        }

        return array();
    }
}
