<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * resources row class
 *
 * @uses       System_Model_Row
 * @category   System
 * @package    System_Model
 */
class System_Model_Resource extends System_Model_Row
{
    /**
     * Table class
     *
     * @var string
     */
    protected $_tableClass = 'System_Model_DbTable_Resources';

    /**
     * Magic method __toString
     *
     * @return mixed
     */
    public function __toString()
    {
        return $this->name;
    }

    /**
     * Allows post-delete logic to be applied to row.
     * Subclasses may override this method.
     *
     * @see    parent::_postDelete()
     * @return void
     */
    public function _postDelete()
    {
        $privilegesDb = Singular_Loader::get("System_Model_DbTable_Privileges");
        $where = $privilegesDb->getAdapter()->quoteInto("resources=?", $this->resource);
        $privilegesDb->delete($where);
        parent::_postDelete();
    }

    /**
     * Cleans cache.
     *
     * @param array $additionalTags
     * @param bool  $onlyTags
     */
    public static function cleanCache($additionalTags = array(), $onlyTags = false)
    {
        $tags = array('resource', 'resources', Singular_Loader::get(__CLASS__)->getTable()->getTableName());
        if (is_array($additionalTags) && count($additionalTags)) {
            if ($onlyTags) {
                $tags = $additionalTags;
            } else {
                $tags = $tags + $additionalTags;
            }
        }
        Singular_Core::_('cache')->removeAllByTags($tags);
    }
}
