<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * response_answers row class
 *
 * @uses       System_Model_Row
 * @category   System
 * @package    System_Model
 */
class System_Model_ResponseAnswer extends System_Model_Row
{
    /**
     * Name of the class of the Zend_Db_Table_Abstract object.
     *
     * @var string
     */
    protected $_tableClass = 'System_Model_DbTable_ResponsesAnswers';

    /**
     * @var string
     */
    public $type = 'answer';
    
    /**
     * Magic method __tostring
     *
     * @return string
     */
    public function  __toString() {
        return $this->answer;
    }

    /**
     * Allows post-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @return void
     */
    protected function _postInsert()
    {
        /* log */
        $log = new System_Model_Log();
        $log->save(array(
            'object'    => $this->getResponse()->getObject(),
            'object_id' => $this->getResponse()->id,
            'action'    => System_Model_DbTable_Logs::ANSWER_ADDED,
            'title'     => $this->getResponse()->title,
            'comment'   => $this->answer
        ));
        
        $activeResponse = Singular_Runtime::extract('activeResponse');
        $cacheTags = array($activeResponse->type . '_' . $activeResponse->id);
        $cache = Singular_Runtime::extract('cachemanager')->getCache('default');
        $cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, $cacheTags);
        if (class_exists('Supercache', false)) {
            SuperCache::getInstance()->getCache()->clean(Zend_Cache::CLEANING_MODE_ALL);
        }
    }

    /**
     * Returns ISO-formatted `updated_at` field
     *
     * @return Zend_Date
     */
    public function getUpdatedAt()
    {
        return new Zend_Date($this->updated_at, Zend_Date::ISO_8601);
    }
    
    /**
     * Returns ISO-formatted `created_at` field
     *
     * @return Zend_Date
     */
    public function getCreatedAt()
    {
        return new Zend_Date($this->created_at, Zend_Date::ISO_8601);
    }

    /**
     * Returns parent response for current response answer
     *
     * @return System_Model_Response
     */
    public function getResponse()
    {
        if(!isset($this->_properties['Response'])){
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $result = $responsesDb->findById($this->response_id);
            if ($result instanceof System_Model_Response) {
                $this->_properties['Response'] = $result;
            }
        }
        return $this->_properties['Response'];
    }

    /**
     * Returns author for current response answer
     *
     * @return System_Model_User
     */
    public function getAuthor()
    {
        if(!isset($this->_properties['CreatedBy'])){
            $usersDb = Singular_Loader::get('System_Model_DbTable_Users');
            $result = $usersDb->findById($this->created_by_id);
            if($result instanceof System_Model_User){
                $this->_properties['CreatedBy'] = $result;
            }
        }        
        return $this->_properties['CreatedBy'];
    }

    /**
     * Returns moderator for current response answer
     *
     * @return System_Model_User
     */
    public function getModerator()
    {
        if(!isset($this->_properties['UpdatedBy']) && isset($this->updated_by_id)){
          $result = $this->findSystem_Model_DbTable_UsersByResponseModerator()->current();
            if($result instanceof System_Model_User){
                $this->_properties['UpdatedBy'] = $result;
            }
        }
        return $this->_properties['UpdatedBy'];
    }

    /**
     * Returns deleter for current response answer
     *
     * @return System_Model_User
     */
    public function getDeleter()
    {
        if(!isset($this->_properties['DeletedBy']) && isset($this->deleted_by_id)){
            $result = $this->findSystem_Model_DbTable_UsersByResponseDeleter()->current();
            if($result instanceof System_Model_User){
                $this->_properties['DeletedBy'] = $result;
            }
        }
        return $this->_properties['DeletedBy'];
    }

    /**
     * Returns url for current response answer
     *
     * @return string
     */
    public function getUrl()
    {
        return $this->getResponse()->getUrl();
    }

    /**
     * Cleans cache.
     *
     * @param array $additionalTags
     * @param bool  $onlyTags
     */
    public static function cleanCache($additionalTags = array(), $onlyTags = false)
    {
        $tags = array('answer', 'answers', 'response_answer', Singular_Loader::get(__CLASS__)->getTable()
            ->getTableName());
        if (is_array($additionalTags) && count($additionalTags)) {
            if ($onlyTags) {
                $tags = $additionalTags;
            } else {
                $tags = $tags + $additionalTags;
            }
        }
        Singular_Core::_('cache')->removeAllByTags($tags);
    }
}
