<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Model
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * response_statuses row class
 *
 * @uses       System_Model_Row
 * @category   System
 * @package    System_Model
 */
class System_Model_ResponseStatus extends System_Model_Row
{
    /**
     * Name of the class of the Zend_Db_Table_Abstract object.
     *
     * @var string
     */
    protected $_tableClass = 'System_Model_DbTable_ResponsesStatuses';

    /**
     * @var bool
     */
    protected $_ignoreIsDefaultCheck = false;

    /**
     * Magic method __tostring
     *
     * @return string
     */
    public function  __toString()
    {
        $translate = Singular_Runtime::extract('Zend_Translate');
        return $translate->translate($this->title);
    }

    /**
     * Allows pre-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @return void
     */
    protected function _insert()
    {
        //$this->slug = str_replace('-', '_', $this->createSlug('slug', $this->slug));
        $this->_preInsertUpdate();
        parent::_insert();
    }

    /**
     * Allows pre-update logic to be applied to row.
     * Subclasses may override this method.
     *
     * @return void
     */
    protected function _update()
    {
        $this->_preInsertUpdate();
        parent::_update();
    }

    /**
     * Common pre-insert/update operations
     */
    private function _preInsertUpdate()
    {
        if (null === $this->is_protected) {
            $this->is_protected = 0;
        }
        if ($this->color && false === strpos($this->color, '#')) {
            $this->color = '#' . $this->color;
        }
        if ($this->text_color && false === strpos($this->text_color, '#')) {
            $this->text_color = '#' . $this->text_color;
        }
        if (1 == $this->is_default && !$this->getIgnoreIsDefaultCheck()) {
            $default = $this->getTable()->findDefaultByType($this->type);
            if ($default instanceof System_Model_ResponseStatus && $this->slug != $default->slug) {
                $default->setIgnoreIsDefaultCheck();
                $default->is_default = 0;
                $default->save();
            }
        }
    }

    /**
     * Allows post-insert logic to be applied to row.
     * Subclasses may override this method.
     *
     * @return void
     */
    protected function _postInsert()
    {

    }

    /**
     * Sets flag to ignore checking "is_default" field
     *
     * @param bool $flag
     */
    public function setIgnoreIsDefaultCheck($flag = true)
    {
        $this->_ignoreIsDefaultCheck = $flag;
    }

    /**
     * Returns true if "is_default" field checking is disabled
     * Needed to prevent duplicate rows with "is_default = 1"
     *
     * @return bool
     */
    public function getIgnoreIsDefaultCheck()
    {
        return $this->_ignoreIsDefaultCheck;
    }


    /**
     * Returns status background color
     *
     * @param bool $withHash
     *
     * @return string
     */
    public function getBackgroundColor($withHash = true)
    {
        return $this->_getColor('color', $withHash);
    }

    /**
     * Returns status text color
     *
     * @param bool $withHash
     *
     * @return string
     */
    public function getTextColor($withHash = true)
    {
        return $this->_getColor('text_color', $withHash);
    }

    /**
     * Common method to get color by filed
     *
     * @param  string $field
     * @param  bool   $withHash
     *
     * @return string
     */
    private function _getColor($field, $withHash = true)
    {
        $color = strtoupper($this->{$field});
        if (!$withHash) {
            $color = ltrim($color, '#');
        }
        return $color;
    }

    /**
     * Returns true if status is protected
     *
     * @return bool
     */
    public function getIsProtected()
    {
        return (bool) $this->is_protected;
    }

    /**
     * Returns true if status is default
     *
     * @return bool
     */
    public function getIsDefault()
    {
        return (bool) $this->is_default;
    }

    /**
     * Saves the properties to the database.
     *
     * This performs an intelligent insert/update, and reloads the
     * properties with fresh data from the table on success.
     *
     * @param  array|null $data
     *
     * @return mixed The primary key value(s), as an associative array if the
     *     key is compound, or a scalar if the key is single-column.
     */
    public function save($data = null)
    {
        $result = parent::save($data);
        if ($result) {
            //$this->savePrivileges((array)$data['resource']);
        }
        return $result;
    }

    /**
     * Cleans cache.
     *
     * @param array $additionalTags
     * @param bool  $onlyTags
     */
    public static function cleanCache($additionalTags = array(), $onlyTags = false)
    {
        $tags = array('status', 'statuses', 'response_status', 'response_statuses', Singular_Loader::get(__CLASS__)
            ->getTable()->getTableName());
        if (is_array($additionalTags) && count($additionalTags)) {
            if ($onlyTags) {
                $tags = $additionalTags;
            } else {
                $tags = $tags + $additionalTags;
            }
        }
        Singular_Core::_('cache')->removeAllByTags($tags);
    }
}
