<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Service
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Api service
 *
 * @category   System
 * @package    System_Service
 */
class System_Service_Environment
{
    /**
     * Self class instance
     *
     * @var System_Service_Environment
     */
    protected static $_instance;

    /**
     * Current namespace
     *
     * @var string
     */
    protected $_namespace;

    /**
     * Areas
     */
    const FRONTEND = 'frontend';
    const ADMIN    = 'admin';
    const PROFILE  = 'profile';

    /**
     * Class constructor
     */
    private function  __construct()
    {
    }

    /**
     * Prevents from object cloning
     */
    private function __clone()
    {
    }

    /**
     * Singleton instance
     *
     * @static
     * @return System_Service_Environment
     */
    public static function getInstance()
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new System_Service_Environment();
        }
        return self::$_instance;
    }

    /**
     * Sets current namespace
     *
     * @param string $namespace
     */
    public function setNamespace($namespace)
    {
        $this->_namespace = $namespace;
    }

    /**
     * Returns current namespace
     *
     * @return string
     */
    public function getNamespace()
    {
        return $this->_namespace;
    }

    /**
     * Returns TRUE if cureent namespace is FRONTEND
     *
     * @return bool
     */
    public function isFrontend()
    {
        if ($this->getNamespace() == self::FRONTEND) {
            return true;
        }
        return false;
    }

    /**
     * Returns TRUE if cureent namespace is ADMIN
     *
     * @return bool
     */
    public function isAdmin()
    {
        if ($this->getNamespace() == self::ADMIN) {
            return true;
        }
        return false;
    }

    /**
     * Returns TRUE if cureent namespace is PROFILE
     *
     * @return bool
     */
    public function isProfile()
    {
        if ($this->getNamespace() == self::PROFILE) {
            return true;
        }
        return false;
    }
}
