<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Service
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Api service
 *
 * @category   System
 * @package    System_Service
 */
class System_Service_Files
{
    /**
     * Returns max filesize that can be transfered via POST method
     *
     * @static
     * @return mixed
     */
    static public function getMaxUploadSize()
    {
        $filter = new System_Filter_ByteString();
        $postMaxSize = $filter->filter(ini_get("post_max_size"));
        $uploadMaxFilesize = $filter->filter(ini_get("upload_max_filesize"));
        $maxSize = array(
            $postMaxSize => ini_get("post_max_size"),
            $uploadMaxFilesize => ini_get("upload_max_filesize"),
        );
        $min = min($postMaxSize, $uploadMaxFilesize);

        return $maxSize[$min];
    }

    /**
     * Recursively removes directory from filesystem
     *
     * @static
     * @param  string $dir
     * @return void
     */
    static public function removeDir($dir, $rmMainDir = true)
    {
        if (is_dir($dir)) {
            $items = new DirectoryIterator($dir);
            foreach ($items as $item) {
                if (!$item->isDot()) {
                    if ($item->isFile()) {
                        unlink($item->getRealPath());
                    } else {
                        self::removeDir($item->getRealPath());
                    }
                }
            }
            if ($rmMainDir) {
                rmdir($dir);
            }
        }
    }

    /**
     * Returns calculated directory depth size
     *
     * @static
     * @param  string $dir
     * @param  bool $skipFilesStartingWithDot
     * @return int
     */
    static public function dirSize($dir, $skipFilesStartingWithDot = true)
    {
        $totalsize = 0;
        if ($dirstream = @opendir($dir)) {
            while (false !== ($filename = readdir($dirstream))) {
                if ($skipFilesStartingWithDot) {
                    if (($filename != '.') && ($filename != '..') && ($filename[0] != '.')) {
                        $path = realpath($dir) . '/' . $filename;
                        if (is_file($path))
                            $totalsize += filesize($path);
                        if (is_dir($path))
                            $totalsize += self::dirSize($path, $skipFilesStartingWithDot);
                    }
                } else {
                    if (($filename != '.') && ($filename != '..')) {
                        $path = realpath($dir) . '/' . $filename;
                        if (is_file($path))
                            $totalsize += filesize($path);
                        if (is_dir($path))
                            $totalsize += self::dirSize($path, $skipFilesStartingWithDot);
                    }
                }
            }
            closedir($dirstream);
        }
        return $totalsize;
    }

    /**
     * Copies directory from one location to another
     *
     * @param  string $sourceDir
     * @param  string $destinationDir
     * @return bool
     */
    static function copyDir($sourceDir, $destinationDir)
    {
        if (!is_dir($sourceDir)) {
            return false;
        } else if (!is_dir($destinationDir)) {
            if (!mkdir($destinationDir, 0777, true)) {
                return false;
            }
        }

        $dh = opendir($sourceDir);
        while ($file = readdir($dh)) {
            if (($file != ".") && ($file != "..")) {
                $fullPath = $sourceDir . "/" . $file;
                $destPath = $destinationDir . "/" . $file;

                if (!is_dir($fullPath)) {
                    $tmpResult = copy($fullPath, $destPath);
                    if (!$tmpResult) {
                        closedir($dh);
                        return false;
                    }
                } else {
                    if (!mkdir($destPath, 0777, true)) {
                        return false;
                    }
                    $tmpResult = self::copyDir($fullPath, $destPath);
                    if (!$tmpResult) {
                        closedir($dh);
                        return false;
                    }
                }
            }
        }
        closedir($dh);
        return true;
    }
}
