<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   System
 * @package    System_Service
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

require_once 'phpmorphy/src/common.php';

/**
 * Api service
 *
 * @category   System
 * @package    System_Service
 */
class System_Service_Morphy
{
    /**
     * phpMorphy object instance
     *
     * @var phpMorphy
     */
    protected $_morphy = null;

    /**
     * Minimum word lenght that will be included in search
     */
    const MIN_LEN = 2;

    /**
     * Class constructor
     */
    public function __construct()
    {
        $lang = Singular_Core::_("settings")->getSetting("site_default_language");

        if (is_file(LIBRARY_PATH . "/phpmorphy/dicts/common_aut." . $lang . '_' . $lang . ".bin")) {
            $dictBundle = new phpMorphy_FilesBundle(LIBRARY_PATH . "/phpmorphy/dicts/", $lang . '_' . $lang);
            $this->_morphy = new phpMorphy(
                $dictBundle, array(
                    'storage'           => PHPMORPHY_STORAGE_FILE,
                    'with_gramtab'      => false,
                    'predict_by_suffix' => true,
                    'predict_by_db'     => true
                )
            );
        }
    }

    /**
     * Returns base form of given word
     *
     * @param  string $text
     * @return string
     */
    public function getBaseForm($text)
    {
        if (is_null($this->_morphy)) {
            return "";
        }
        $words = preg_replace('#\[.*\]#isU', '', $text);
        $words = preg_split('#\s|[,.:;!?"\'()]#', $words, -1, PREG_SPLIT_NO_EMPTY);

        $bulkWords = array();
        foreach ($words as $word) {
            if (strlen($word) > self::MIN_LEN) {
                $bulkWords[] = $this->strtoupper($word);
            }
        }
        $baseForm = $this->_morphy->getBaseForm($bulkWords);
        $fullList = array();
        if (is_array($baseForm) && count($baseForm)) {
            foreach ($baseForm as $k => $v) {
                if (is_array($v)) {
                    foreach ($v as $v1) {
                        if (strlen($v1) > 3) {
                            $fullList[$v1] = 1;
                        }
                    }
                }
            }
        }
        return join(' ', array_keys($fullList));
    }

    /**
     * Returns all known forms of given word
     *
     * @param  string $text
     * @return string
     */
    public function getAllForm($text)
    {
        if (is_null($this->_morphy)) {
            return $text;
        }
        $words = preg_split('#\s|[,.:;!?"\'()]#', $text, -1, PREG_SPLIT_NO_EMPTY);

        $bulkWords = array();
        foreach ($words as $word)
            if (strlen($word) > self::MIN_LEN) {
                $bulkWords[] = $this->strtoupper($word);
            }
        $allFormArray = $this->_morphy->getAllForms($bulkWords);
        $allForm = "";
        if (!empty($allFormArray)) {
            foreach ($allFormArray as $word) {
                if (!empty($word)) {
                    foreach ($word as $w) {
                        $allForm .= $w . " ";
                    }
                }
            }
        }
        return $allForm;
    }

    /**
     * Makes a string uppercase
     *
     * @param  string $text
     * @return string
     */
    public function strtoupper($text)
    {
        $convert_from = array(
            "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u",
            "v", "w", "x", "y", "z", "à", "á", "â", "ã", "ä", "å", "æ", "ç", "è", "é", "ê", "ë", "ì", "í", "î", "ï",
            "ð", "ñ", "ò", "ó", "ô", "õ", "ö", "ø", "ù", "ú", "û", "ü", "ý", "а", "б", "в", "г", "д", "е", "ё", "ж",
            "з", "и", "й", "к", "л", "м", "н", "о", "п", "р", "с", "т", "у", "ф", "х", "ц", "ч", "ш", "щ", "ъ", "ы",
            "ь", "э", "ю", "я"
        );
        $convert_to = array(
            "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U",
            "V", "W", "X", "Y", "Z", "À", "Á", "Â", "Ã", "Ä", "Å", "Æ", "Ç", "È", "É", "Ê", "Ë", "Ì", "Í", "Î", "Ï",
            "Ð", "Ñ", "Ò", "Ó", "Ô", "Õ", "Ö", "Ø", "Ù", "Ú", "Û", "Ü", "Ý", "А", "Б", "В", "Г", "Д", "Е", "Ё", "Ж",
            "З", "И", "Й", "К", "Л", "М", "Н", "О", "П", "Р", "С", "Т", "У", "Ф", "Х", "Ц", "Ч", "Ш", "Щ", "Ъ", "Ы",
            "Ь", "Э", "Ю", "Я"
        );

        return str_replace($convert_from, $convert_to, $text);
    }
}
