<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Thanks
 * @package    Thanks_Module
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Thanks module identity class
 *
 * @uses       Singular_Module_Abstract
 * @category   Thanks
 * @package    Thanks_Module
 */
class Thanks_Module extends Singular_Module_Abstract
{
    /**
     * Priority for module load order, used in module installation process.
     *
     * @ignore
     * @var int
     */
    public $loadOrder = 4;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @return void
     */
    public function init ()
    {
      $this->_info();
    }

    /**
     * Module installation method.
     *
     * @return void
     */
    public function install()
    {
        $modulesDb = Singular_Loader::get('System_Model_DbTable_Modules');
        $module = $modulesDb->findByName('thanks');
        //types
        $type = new System_Model_ResponseType();
        $type->save(array(
            'slug'              => 'thanks',
            'title'             => 'Thanks',
            'objects'           => 'thanks',
            'icon'              => 'icon_medium.png',
            'module_id'         => $module->id,
            'has_comments'      => 1,
            'has_negative_vote' => 0,
            'is_active'         => 1,
            'is_default'        => 0,
            'order'             => 4
        ));
    }

    /**
     * Module uninstallation method.
     *
     * @return void
     */
    public function uninstall()
    {       
        $typeDb = Singular_Loader::get('System_Model_DbTable_ResponsesTypes');
        $typeDb->deleteRowByField('slug', 'thanks');

        $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
        $responses = $responsesDb->fetchAll("type='thanks'");
        foreach($responses as $response) {
            $response->delete();
        }
    }

    /**
     * Set module information to be available in code and to display on Modules
     * list at backend.
     *
     * @return void
     */
    protected function _info()
    {
        $this->getContainer()->setName('Thanks')
              ->setDescription('Users can give praise to you, so all know how customers are satisifed with provided quality')
              ->setVersion('2.1')
              ->addDependency('system', '2.3')
              ->addDependency('resources', '2.1')
              ->setAuthor('USWebStyle, Inc')
              ->setEmail('support@uswebstyle.com')
              ->setUrl('http://www.uswebstyle.com')
              ->setReleaseDate('2013-03-15');
    }
}
