<?php
/**************************************************************
 * Copyright USWebStyle Inc., 2011-2012. All Rights Reserved.
 **************************************************************
 * NOTICE:  This source code is the property of USWebStyle, Inc.
 * In no case shall you rent, lease, lend, redistribute
 * any of below source code to a 3rd party individual or entity.
 *
 * @category   Thanks
 * @package    Thanks_Handler
 * @copyright  USWebStyle Inc., 2011-2012 (http://www.uswebstyle.com)
 * @license    http:/www.useresponse.com/licensing (Commercial)
 */

/**
 * Thanks handler
 *
 * @uses       Singular_Event_Listener_Abstract
 * @category   Thanks
 * @package    Thanks_Handler
 */
class Thanks_Handler_Events extends Singular_Event_Listener_Abstract
{
    /**
     * Binds aggregates on specified events.
     *
     * @return void
     */
    public static function bindListeners ()
    {
        self::attachAggregate('rightMenu',                 array(__CLASS__, 'onRightMenu'));
        self::attachAggregate('onMainPage',                array(__CLASS__, 'onMainPage'));
        self::attachAggregate('postDispatchOnce',          array(__CLASS__, 'onPostDispatchOnce'));
        self::attachAggregate('buildNavigation',           array(__CLASS__, 'onNavigation'));
        self::attachAggregate('insert',                    array(__CLASS__, 'onInsert'));
        self::attachAggregate('formInitAfter',             array(__CLASS__, 'onFormInitAfter'));
        self::attachAggregate('responseShow',              array(__CLASS__, 'onResponseShow'));
        self::attachAggregate('commentActions',            array(__CLASS__, 'onCommentActions'));
        self::attachAggregate('afterAnalyticsReportsInit', array(__CLASS__, 'afterAnalyticsReportsInit'));
    }

    /**
     * Subscribes to "commentActions" event.
     *
     * @param  stdClass $params
     * @return void
     */
    public static function onCommentActions($params)
    {
        $response = Singular_Runtime::extract("activeResponse");
        if ($response instanceof System_Model_Response
            && $response->type == 'thanks'
        ) {
            $translate = Singular_Runtime::extract('Zend_Translate');
            $params->actions['good']['label'] = $translate->translate('Good Comment');
        }
    }

    /**
     * Subscribes to "responseShow" event.
     *
     * @param  stdClass $data
     * @return void
     */
    public static function onResponseShow($data)
    {
        $response = Singular_Runtime::extract("activeResponse");
        if ($response instanceof System_Model_Response
            && $response->type == 'thanks'
        ) {
            self::getView()->additionalModulePath('thanks');
            $data->sections['votes'] = self::getView()->render('thanks_votes.phtml');
        }
    }

    /**
     * Subscribes to "insert" event.
     *
     * @param  System_Model_Response $object
     * @return void
     */
    public static function onInsert($object)
    {
        if ($object instanceof System_Model_Response && $object->type == 'thanks') {
            $object->status = '';
        }
    }

    /**
     * Subscribes to "postDispatchOnce" event.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public static function onPostDispatchOnce($request)
    {
        self::getView()->headLink()->prependStylesheet(self::getView()->stylesheetSrc('frontend.css', 'thanks'));
        self::getView()->headScript()->appendFile(self::getView()->javascriptSrc('main.js', 'thanks'));
        
        $activeResponse = Singular_Runtime::extract('activeResponse');
        
        if ($activeResponse instanceof System_Model_Response
            && $activeResponse->type == 'thanks'
        ) {
            $item =
                array(
                    'id' => 'thanks-' . $activeResponse->id,
                    'label' => self::getView()->truncate($activeResponse->title, '16', '...'),
                    'route' => 'responses_show',
                    'params' => array(
                        'slug' => $activeResponse->slug
                    ),
                    'pages' => array(
                        array(
                            'id' => 'thanks-edit',
                            'label' => 'Edit',
                            'route' => 'response_edit',
                            'params' => array(
                                'slug' => $activeResponse->slug
                            )
                        )
                    )
                );
            $navigation = self::getView()->navigation()->getContainer();
            $page = $navigation->findOneBy('id', 'thanks');
            if ($page instanceof Zend_Navigation_Page) {
                $page->addPage($item);
            }
        }
    }
    
    /**
     * Subscribes to "navigation" event.
     *
     * Builds navigation
     *
     * @param  Singular_Navigation $navigation
     * @return void
     */
    public static function onNavigation($navigation)
    {
        $item = array(
            'id'     => 'thanks',
            'class'  => 'right-menu',
            'label'  => self::getView()->t('Thanks'),
            'order'  => Singular_Runtime::extract("types")->findBySlug('thanks')->order,
            'route'  => 'responses_type',
            'params' => array(
                'objects' => 'thanks'
            ),
            'pages'   => array(
                'all' => array(
                    'id'     => 'thanks-all',
                    'label'  => self::getView()->t('All Thanks'),
                    'route'  => 'responses_type',
                    'params' => array(
                        'objects' => 'thanks',
                        'status'  => 'all'
                    )
                )
            )
        );

        $page = $navigation->findOneBy('id', 'responses');
        if ($page instanceof Zend_Navigation_Page) {
            $page->addPage($item);
        }
    }

    /**
     * Subscribes to "rightMenu" event.
     *
     * @param  stdClass $params
     * @return void
     */
    public static function onRightMenu($params)
    {
        $menu = $params->menu;
        
        if (Singular_Runtime::inStorage('thanksCount')) {
            $thanksCount = Singular_Runtime::extract('thanksCount');
        } else {
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $thanksCount = $responsesDb->count(array('type' => 'thanks'));
        }

        $page = $menu->findById("statistics");
        $page->addPage(
            array(
                 'id'     => 'thanks',
                 'order'  => Singular_Runtime::extract("types")->findBySlug('thanks')->order,
                 'count'  => $thanksCount,
                 'icon'   => self::getView()->imageSrc('icon_small.png', 'thanks'),
                 'label'  => 'Thanks',
                 'route'  => 'responses_type',
                 'params' => array(
                     'objects' => 'thanks'
                 )
            )
        );
        
        $params->cacheKeys[] = 'thanks';
    }

    /**
     * Subscribes to "mainPage" event.
     *
     * @param  stdClass $sections
     * @return void
     */
    public static function onMainPage($sections)
    {
        if (
            1 != Singular_Core::_("Settings")->getSetting('homepage_content_responses')
            || 'divided_by_type' != Singular_Core::_("settings")
                ->getSetting("homepage_responses_display_variant")
        ) {
            return;
        }
        $cache = Singular_Runtime::extract("cachemanager")->getCache("default");
        $loggedUser = Singular_Runtime::extract("loggedUser");
        $cacheId = "homepageThanks" . "_" . $loggedUser->role;
        $cacheId = preg_replace("/[^a-zA-Z0-9_]/", "", $cacheId);
        $tags = null;
        
        if ( ($output = $cache->load($cacheId)) == false ) {
            
            $responsesDb = Singular_Loader::get('System_Model_DbTable_Responses');
            $thanks = $responsesDb->findTop('thanks');

            self::getView()->thanks = $thanks;
            $thanksCount = $responsesDb->count(array('type' => 'thanks'));
            $thanksLeftCount = $thanksCount - count($thanks);
            self::getView()->thanksCount = $thanksLeftCount;
            Singular_Runtime::store('thanksCount', $thanksCount);

            self::getView()->additionalModulePath('thanks');
            self::getView()->cacheTags = new StdClass();
            self::getView()->cacheTags->tags = array();
            $output = self::getView()->render('_homepage-thanks.phtml');
            $tags = self::getView()->cacheTags->tags;
            $tags[] = 'responses';
            $cache->save($output, $cacheId, $tags);
        }
        Resources_Service_Votes::setUserVotes($cacheId, $tags);
        $order = Singular_Runtime::extract("types")->findBySlug('thanks')->order + 10;
        $sections->sections[$order] = $output;
    }

    /**
     * Subscribes to "formInitAfter" event.
     *
     * @param  System_Form_Response|System_Form_SearchAdd|System_Form_WidgetFeedback|System_Form_WidgetSearch $form
     * @return void
     */
    public static function onFormInitAfter($form)
    {
        if ($form instanceof System_Form_Response) {
            if ($form instanceof System_Form_ResponseWidget) {
                $form->type->addMultiOption('thanks', 'Thanks');
            } else {
                $form->type->addMultiOption('thanks', 'Say thanks');
            }
        }
        if ($form instanceof System_Form_SearchAdd) {
            $form->type->addMultiOption('thanks', 'Say thanks');
            if (!$form->submit->getDescription()) {
                $form->submit->setDescription(self::getView()->t("Tell us kind words or what's on your mind"));
            }
        }
        if ($form instanceof System_Form_WidgetFeedback || $form instanceof System_Form_WidgetSearch) {
            $form->widget_default_tab->addMultiOption('thanks', 'Thanks');
        }

        // Don't allow to add statuses to response with type "thanks"
        if ($form instanceof System_Form_ResponseStatus) {
            /** @var Zend_Form_Element_Select $type */
            if ($type = $form->getElement('type')) {
                $type->removeMultiOption('thanks');
            }
        }
    }

    /**
     * Subscribes to "afterAnalyticsReportsInit" event.
     *
     * @param  stdClass $reportsConf
     * @return void
     */
    public static function afterAnalyticsReportsInit ($reportsConf) 
    {
        $reportsConf->reports[] = array(
            'id' => 'thanks_responses',
            'title' => self::getView()->t('Thanks'),
            'model' => 'System_Model_DbTable_Responses',
            'counter' => 'countByType',
            'order' => '40',
            'options' => array(
                'row_type' => 'default',
                'response_type' => 'thanks',
                'icon' => 'url('.self::getView()->imageSrc('icon_small.png', 'thanks', false).')',
            )
        );
    }    
}
