<?php
ob_start();

/* benchmark */
$_ENV['bench_initial'] = microtime(true);

/* set system paths */
define('DS',               DIRECTORY_SEPARATOR);
define('ROOT_PATH',        dirname(__FILE__));
define('CACHE_PATH',       ROOT_PATH . DS . 'cache');
define('APPLICATION_PATH', ROOT_PATH . DS . 'application');
define('LIBRARY_PATH',     ROOT_PATH . DS . 'library');
define('MODULES_PATH',     APPLICATION_PATH . DS . 'modules');
define('FILES_TEMP_PATH',  ROOT_PATH . DS . 'public' . DS . 'files' . DS . 'temp');
define('FILES_TEMP_URI',   '/public/files/temp');
define('AVATAR_PATH',      ROOT_PATH . DS . 'public' . DS . 'files' . DS . 'avatars');
define('AVATAR_URI',       '/public/files/avatars');

$_ENV['TMPDIR'] = ROOT_PATH . DS . 'cache';

/* set include path */
set_include_path(LIBRARY_PATH);

/* load Singular autostart instructions */
require 'SingularAutostart.php';

/* application environment */
define(
    'APPLICATION_ENV', getenv('APPLICATION_ENV')
        ? getenv('APPLICATION_ENV')
        : (getenv('REDIRECT_APPLICATION_ENV')
                ? getenv('REDIRECT_APPLICATION_ENV')
                : 'production')
);

if (isset($disableMvc)) return;

/* SINGULAR */
require_once 'Singular/Bootstrap.php';

/* CONFIGURATION */
$configFile = APPLICATION_PATH . '/configs/application.inc';

/* INSTALLATION */
if (!is_file($configFile)) {
    define('INSTALLATION', true);
    $installConfig = ROOT_PATH . '/installation/templates/application.default.inc';
    require_once     ROOT_PATH . '/installation/index.php';
    $app           = new Singular_Bootstrap(APPLICATION_ENV, $installConfig, false);

    require 'Useresponse.php';

    $app->bootstrap();

    $installation = new Installation($configFile, $installConfig);
    if ('POST' == strtoupper($_SERVER['REQUEST_METHOD'])) echo $installation->execute();
    else echo $installation;

/* APPLICATION */
} else {
    require_once 'Zend/Config.php';
    $config = new Zend_Config(include $configFile, true);
    $baseUrl = $config->resources->frontController->baseUrl;

    preg_match('@^[^:]*://([^/]+)@i', $baseUrl, $matches);
    if ($matches[1] != $_SERVER['HTTP_HOST']) {
        $url = $matches[0] . $_SERVER['REQUEST_URI'];
        header("Location: $url"); die();
    }

    /* SuperCache */
    if (isset($config->supercache_enabled) && $config->supercache_enabled) {
        require 'SuperCache.php';
        SuperCache::getInstance()->setBaseUrl($baseUrl)->run();
    }

    $hotPlug = true;
    if (class_exists('SuperCache', false)) {
        if (SuperCache::getInstance()->isOnAir()) {
            $hotPlug = false;
        }
    }
    if ($hotPlug) require 'hotFiles.php';

    $app = new Singular_Bootstrap(APPLICATION_ENV, $config);

    require 'Useresponse.php';

    $app->bootstrap();
    if (PHP_SAPI !== 'cli') $app->run();
}

/* Activate client cache */
$content = ob_get_contents();
ob_end_clean();
$etag       = md5($content);
$etagHeader = (isset($_SERVER['HTTP_IF_NONE_MATCH']) ? trim($_SERVER['HTTP_IF_NONE_MATCH']) : false);

header("Last-Modified: " . gmdate("D, d M Y H:i:s", time() - 7200) . " GMT");
header("Etag: \"$etag\"");
header('Cache-Control: public');

if ($etagHeader == $etag) {
    header("HTTP/1.1 304 Not Modified");
    exit;
}
echo $content;
