<?php
/**
 * Singular
 *
 * LICENSE
 *
 * The SOFTWARE PRODUCT is protected by copyright treaties,
 * as well as other intellectual property laws and treaties.
 * The SOFTWARE PRODUCT is licensed, not sold.
 * Software.
 * You may install and use one copy of the SOFTWARE PRODUCT
 * for each license purchased or if a site license then you
 * may install any number of copies at the site where the
 * site license is defined.
 * Your rights to use the SOFTWARE PRODUCT are not transferable.
 *
 * @category   Singular
 * @package    Singular_Acl
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */

 /**
 * @category   Singular
 * @package    Singular_Acl
 * @throws     Singular_Acl_Exception
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */
class Singular_Acl extends Zend_Acl
{
    /**
     * Default roles
     */
    const ROLE_GUEST = 'guest';
    const ROLE_USER = 'user';
    const ROLE_ADMINISTRATOR = 'administrator';

    /*
     * Resource types
     */
    const RESOURCE_TYPE_MVC = 'mvc'; 
    const RESOURCE_TYPE_OBJECT = 'object';
    const RESOURCE_TYPE_ROUTE = 'route';

    const TYPE_DIVIDER = ':';

    const RESOURCE_DIVIDER = '.';

    const MULTI_OPTIONS_DIVIDER = ',';

    const CASE_ALL = '*';

    protected static $_instance = null;

    private $_currentRole = self::ROLE_GUEST;

    /**
     * Build ACL
     */
    public function  __construct ()
    {
    }

    public function addRoles ($roles)
    {
        if (is_array($roles)) {
            foreach ($roles as $role) {
                $_role = $role['role'];
                $parent = (null !== $role['parent'] && '' != $role['parent'])
                        ? $role['parent']
                        : null;
                $this->addRole(new Zend_Acl_Role($_role), $parent);
            }
        }
    }

    public function addResources ($resources)
    {
        if (!$this->has(self::RESOURCE_TYPE_MVC)) {
            $this->addResource(new Zend_Acl_Resource(
                                   self::RESOURCE_TYPE_MVC . self::TYPE_DIVIDER));
        }
        if (is_array($resources)) {
            foreach ($resources as $res) {
                switch ($res['type']) {
                    case self::RESOURCE_TYPE_MVC:
                    case self::RESOURCE_TYPE_OBJECT:
                    case self::RESOURCE_TYPE_ROUTE:
                        break;
                }

                $this->addResource(new Zend_Acl_Resource($res['resource']), $res['parent']);
            }
        }
    }

    public function addPrivileges ($privileges)
    {
        if (is_array($privileges)) {
            foreach ($privileges as $privilege) {
                $grantType = $privilege['type'];
                $roles = (null !== $privilege['roles'] && '' != $privilege['roles'])
                        ? $this->_multiOptionsFormat($privilege['roles'])
                        : null;
                $resources = (null !== $privilege['resources'] && '' != $privilege['resources'])
                        ? $this->_multiOptionsFormat($privilege['resources'])
                        : null;
                $_privileges = (null !== $privilege['privileges'] && '' != $privilege['privileges'])
                        ? $this->_multiOptionsFormat($privilege['privileges'])
                        : null;
                
                $this->$grantType($roles, $resources, $_privileges);
            }
        }
    }

    public function setCurrentRole ($role) {
        $this->_currentRole = $role;
    }

    public function getCurrentRole ()
    {
        return $this->_currentRole;
    }

    public function cleanCache ()
    {
        $rolesModel = new Singular_Core_System_Model_DbTable_Roles();
        $rolesModel->cleanCache();
        $resourcesModel = new Singular_Core_System_Model_DbTable_Resources();
        $resourcesModel->cleanCache();
        $privilegesModel = new Singular_Core_System_Model_DbTable_Privileges();
        $privilegesModel->cleanCache();
    }

    private function _multiOptionsFormat ($str)
    {
        $str = str_replace(' ', '', $str);
        return explode(self::MULTI_OPTIONS_DIVIDER, $str);
    }

    /**
     *
     * @return Singular_Acl
     */
    public static function getInstance()
    {
        if (null === self::$_instance) {
            self::$_instance = new self();
        }
        
        return self::$_instance;
    }

    public static function resetInstance()
    {
        self::$_instance = null;
        self::getInstance();
    }
}
