<?php

class Singular_AppClient extends ArrayObject
{
    private static $_storage = null;

    /**
     * Retrieves the default registry instance.
     *
     * @static
     * @return Zend_Registry
     */
    public static function getInstance ()
    {
        if (self::$_storage === null) {
            self::init();
        }

        return self::$_storage;
    }

    /**
     * Initialize the default registry instance.
     *
     * @static
     * @return void
     */
    protected static function init ()
    {
        if (self::$_storage !== null) {
            throw new Zend_Exception('Storage is already initialized');
        }

        self::$_storage = new self();
    }

    /**
     * Setter method, basically same as offsetSet().
     *
     * This method can be called from an object of type Singular_AppClient, or it
     * can be called statically.  In the latter case, it uses the default
     * static instance stored in the class.
     *
     * @static
     * @param string $name The location in the ArrayObject in which to store
     * the value.
     * @param mixed $value The object to store in the ArrayObject.
     * @return void
     */
    public static function setParam ($name, $value)
    {
        if (!is_string($name)) {
            throw new Singular_AppClient_Exception(
                'Invalid argument: $name must be an string'
            );
        }
        $instance = self::getInstance();
        $instance->offsetSet($name, $value);
    }

    /**
     * Getter method, basically same as offsetGet().
     *
     * This method can be called from an object of type Zend_Registry, or it
     * can be called statically.  In the latter case, it uses the default
     * static instance stored in the class.
     *
     * @static
     * @param  string $name - get the value associated with $name
     * @return mixed
     * @throws Zend_Exception if no entry is registerd for $name.
     */
    public static function getParam ($name)
    {
        $instance = self::getInstance();
        
        if (!$instance->offsetExists($name)) {
            throw new Singular_AppClient_Exception(
                "No entry is registered for key '$name'"
            );
        }

        return $instance->offsetGet($name);
    }

    /**
     * Returns TRUE if the $name is a named value in the storage,
     * or FALSE if $name was not found in the storage.
     *
     * @static
     * @param  string $name
     * @return boolean
     */
    public static function hasParam ($name)
    {
        $instance = self::getInstance();
        return $instance->offsetExists($name);
    }

    /**
     * Checks if one or more params has set
     *
     * @static
     * @return bool
     */
    public static function hasParams ()
    {
        $instance = self::getInstance();
        return (bool) count($instance->getArrayCopy());
    }


    /**
     * Returns all params as array
     * 
     * @static
     * @return array
     */
    public static function getAllParams ()
    {
        $instance = self::getInstance();
        return $instance->getArrayCopy();
    }

    /**
     * Clears all params
     *
     * @static
     * @return void
     */
    public static function clearAllParams ()
    {
        $instance = self::getInstance();
        $instance->exchangeArray(array());
    }

    /**
     * Constructs a parent ArrayObject with default
     * ARRAY_AS_PROPS to allow acces as an object
     *
     * @param array $array data array
     * @param integer $flags ArrayObject flags
     */
    public function __construct ($array = array(), $flags = parent::ARRAY_AS_PROPS)
    {
        parent::__construct($array, $flags);
    }

    /**
     * @param  string $index
     * @return mixed
     *
     * Workaround for http://bugs.php.net/bug.php?id=40442 (ZF-960).
     */
    public function offsetExists ($index)
    {
        return array_key_exists($index, $this);
    }
}
