<?php
class Singular_Bench
{
    const MSG_START         = 'Benchmark started';
    const MSG_FINISH        = 'Benchmark finished';
    const MSG_INTERMEDIATE  = 'Intermediate benchmark';
    const TYPE_START        = 1;
    const TYPE_FINISH       = 2;
    const TYPE_INTERMEDIATE = 3;

    private static $_enabled = false;

    private static $_throwExceptiowns = false;

    private static $_useBacktrace = true;

    private static $_roundPrecision = 4;

    private static $_initiaalTime = null;

    private static $_position = 1;

    private static $_stack = array();

    public static function getInitialTime ()
    {
        if (null === self::$_initiaalTime) {
            self::$_initiaalTime = (isset($_ENV['bench_initial'])) ? $_ENV['bench_initial'] : microtime(true);
        }
        return round(self::$_initiaalTime, self::$_roundPrecision);
    }

    public static function setEnabled ($flag = true)
    {
        self::$_enabled = (bool) $flag;
    }

    public static function isEnabled ()
    {
        return self::$_enabled;
    }

    public static function setThrowExceptions ($flag = true)
    {
        self::$_throwExceptiowns = (bool) $flag;
    }

    public static function getThrowExceptions ()
    {
        return self::$_throwExceptiowns;
    }

    public static function setUseBacktrace ($flag = true)
    {
        self::$_useBacktrace = (bool) $flag;
    }

    public static function getUseBacktrace ()
    {
        return self::$_useBacktrace;
    }

    public static function start ($name, $message = null)
    {
        if (!self::isEnabled()) return;
        self::_stack(self::TYPE_START, $name, $message);
    }

    public static function intermediate ($name, $message = null)
    {
        if (!self::isEnabled()) return;
        self::_stack(self::TYPE_INTERMEDIATE, $name, $message);
    }

    public static function finish ($name, $message = null)
    {
        if (!self::isEnabled()) return;
        self::_stack(self::TYPE_FINISH, $name, $message);
    }

    public static function render ($benchName = null, $exit = false)
    {
        $benches = array();
        if (null != $benchName) {
            if (isset(self::$_stack[$benchName])) {
                $benches[$benchName] = self::$_stack[$benchName];
            }
        } else {
            $benches = self::$_stack;
        }

        $out = '<div style="font-size:11px; font-family:Verdana;">';
        foreach ($benches as $name => $params) {
            $out .= '<hr /><h3>Bench <span style="color:green;">&laquo;' . $name . '' . '&raquo;</span> &#8212 ' . ' started <small>(at ' . ($params['start']['time'] - self::getInitialTime()) . ' ms.)</small></h3>';
            $out .= '<strong>Message:</strong> ' . $params['start']['message'] . '<br />';
            if (null !== $params['start']['backtrace']) {
                $out .= self::_renderBacktrace($params['start']['backtrace']);
            }
            if (0 < count($params['intermediate'])) {
                $out .= '<div style="margin-left:50px;">' . self::_renderIntermediate($params['intermediate']) . '</div>';
            }
            if ($params['closed']) {
                $out .= '<h3>Bench <span style="color:green;">&laquo;' . $name . '' . '&raquo;</span> &#8212 ' . ' finished <small>(at ' . ($params['finish']['time'] - self::getInitialTime()) . ' ms.)</small></h3>';
                $out .= '<strong>Message:</strong> ' . $params['finish']['message'] . '<br />';
                if (null !== $params['finish']['backtrace']) {
                    $out .= self::_renderBacktrace($params['finish']['backtrace']);
                    $out .= '<h3><span style="color:green;">&laquo;' . $name . '&raquo; total:</span> ' . $params['total'] . ' ms. </h3>';
                }
            } else {
                $out .= '<h3 style="color:red;">Bench &laquo;' . $name . '&raquo; was not finished!</h3>';
            }
            $out .= '<br /><br />';
         }

        $out .= '</div>';

        if ($exit) die($out);

        return $out;
    }

    protected static function _renderBacktrace ($backtrace)
    {
        $out = '';
        $out .= '<strong>File:</strong> ' . $backtrace['file'] . ' <strong>(line: ' . $backtrace['line'] . ')</strong><br />';
        $out .= '<strong>Code:</strong> ' . $backtrace['class'] . $backtrace['type'] . $backtrace['function'] . '(' . implode(', ', $backtrace['args']) . ')';
        return $out;
    }

    protected static function _renderIntermediate ($intermediate)
    {
        $out = '<h3>Intermediate benches: </h3>';
        foreach ($intermediate as $iIdx => $iParams) {
            $out .= '<h3><span style="color:green;">#' . ($iIdx + 1) . '</span> &#8212 ' . '  <small>' . $iParams['lap'] . ' ms.</small> spent</h3>';
            $out .= '<strong>Message:</strong> ' . $iParams['message'] . '<br />';
            if (null !== $iParams['backtrace']) {
                $out .= self::_renderBacktrace($iParams['backtrace']);
            }
        }
        return $out;
    }

    protected static function _backtrace ()
    {
        if (!self::getUseBacktrace()) return null;
        $backtrace = debug_backtrace(1);
        return $backtrace[2];
    }

    protected static function _stack ($type, $name, $message = null)
    {
        /* current microtime */
        $time = microtime(true);

        if (!isset(self::$_stack[$name])) {
            self::$_stack[$name] = array(
                'start'        => array(),
                'intermediate' => array(),
                'finish'       => array(),
                'total'        => null,
                'closed'       => false
            );
        }

        if (self::$_stack[$name]['closed']) {
            self::_exception("Benchmark '{$name}' has has finished'");
            return;
        }

        switch ($type) {
            case self::TYPE_START:
                self::$_stack[$name]['start'] = array(
                    'time'      => $time,
                    'message'   => (null == $message) ? self::MSG_START : $message,
                    'backtrace' => self::_backtrace()
                );
                break;

            case self::TYPE_FINISH:
                self::$_stack[$name]['closed'] = true;
                self::$_stack[$name]['total']  = round($time, self::$_roundPrecision) - round(self::$_stack[$name]['start']['time'], self::$_roundPrecision);
                self::$_stack[$name]['finish'] = array(
                    'time'      => $time,
                    'message'   => (null == $message) ? self::MSG_FINISH : $message,
                    'backtrace' => self::_backtrace()
                );
                break;

            case self::TYPE_INTERMEDIATE:
                $itrCount = count(self::$_stack[$name]['intermediate']);
                if (0 == $itrCount) {
                    $lap = round($time, self::$_roundPrecision) - round(self::$_stack[$name]['start']['time'], self::$_roundPrecision);
                } else {
                    $lap = round($time, self::$_roundPrecision) - round(self::$_stack[$name]['intermediate'][$itrCount-1]['time'], self::$_roundPrecision);
                }
                self::$_stack[$name]['intermediate'][] = array(
                    'time'      => $time,
                    'message'   => (null == $message) ? self::MSG_INTERMEDIATE : $message,
                    'backtrace' => self::_backtrace(),
                    'lap'       => $lap
                );
                break;
        }
    }

    protected static function _exception ($msg)
    {
        if (!self::getThrowExceptions()) return;
        throw new Exception($msg);
    }
}
