<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller_Action
 * @subpackage Helper
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Helper to access to sidebar
 *
 * @category   Singular
 * @package    Singular_Controller_Action
 * @subpackage Helper
 * @see        Zend_Controller_Action_Helper_Abstract
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Controller_Action_Helper_Sidebar extends Zend_Controller_Action_Helper_Abstract
{
    /**
     * Sidebar instance
     * 
     * @var Singular_Sidebar_Container|null
     */
    protected $_instance = null;

    /**
     * Current sidebar name
     * 
     * @var string|null
     */
    protected $_name = null;

    /**
     * Gets sidebar by given name
     *
     * @param  string|null $name
     * @return Singular_Controller_Action_Helper_Sidebar
     */
    public function direct ($name)
    {
        if (!is_string($name)) {
            throw new Singular_Controller_Action_Exception(
                'Invalid argument: $name must be an string'
            );
        }

        $this->_instance = Singular_Sidebar::getSidebar($name);
        $this->_name     = $name;
        
        return $this;
    }

    /**
     * Return true if given sider is exists
     * otherwise false
     *
     * @return bool
     */
    public function isExists ()
    {
        return Singular_Sidebar::hasSidebar($this->_name);
    }

    /**
     * Remove sidebar
     * 
     * @return Singular_Controller_Action_Helper_Sidebar
     */
    public function remove ()
    {
        Singular_Sidebar::removeSidebar($this->_name);

        return $this;
    }

    /**
     * Magic class method __call for dynamically use methods
     * from Singular_Sidebar_Container class
     *
     * @param  string $name
     * @param  array $arguments
     * @return mixed|null
     */
    public function __call ($name, $arguments = array())
    {
        if (method_exists($this->_instance, $name)) {
            return call_user_func_array(array($this->_instance, $name), $arguments);
        }

        trigger_error("Method \"{$name}\" does not exist and was not trapped in __call()", E_USER_ERROR);
    }

    /**
     * Magic class method __toString
     * 
     * @return string
     */
    public function __toString ()
    {
        return $this->_instance->render();
    }
}
