<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Controller_Plugin_Abstract
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Controller_Plugin_Acl extends Zend_Controller_Plugin_Abstract
{
    /**
     * ACL instance
     * 
     * @var Singular_Acl
     */
    protected $_acl = null;

    /**
     * Access flag
     *
     * @var bool
     */
    private $_accessAllowed = false;
    
    /**
     * Called before Zend_Controller_Front begins evaluating the
     * request against its routes.
     *
     * @param Zend_Controller_Request_Abstract $request
     * @return void
     */
    public function preDispatch (Zend_Controller_Request_Abstract $request)
    {
        // get MVC names
        $module     = $request->getModuleName();
        $controller  = $request->getControllerName();
        $action       = $request->getActionName();
        // get config
        $config = Singular_Runtime::extract('config');
        // get ACL
        $this->_acl = Singular_Runtime::extract('acl');

        $checkResource = array(
            Singular_Acl::RESOURCE_TYPE_MVC . Singular_Acl::TYPE_DIVIDER,
            Singular_Acl::RESOURCE_TYPE_MVC . Singular_Acl::TYPE_DIVIDER . $module,
            Singular_Acl::RESOURCE_TYPE_MVC . Singular_Acl::TYPE_DIVIDER . $module
            . Singular_Acl::RESOURCE_DIVIDER . $controller
        );

        $checkPrivilege = array(
            null,
            $action
        );

        // check access
        $this->_checkAccess($checkResource, $checkPrivilege);
        
        if (!$this->_accessIsAllowed()) {
            $mvcError = $config->resources->acl->error;
            $request->setModuleName($mvcError->moduleName);
            $request->setControllerName($mvcError->controllerName);
            $request->setActionName($mvcError->actionName);
        }
    }

    /**
     * Check access
     *
     * @param  array $resources
     * @param  array $privileges
     * @return void
     */
    protected function _checkAccess($resources, $privileges)
    {
        if (is_array($resources) && is_array($privileges)) {
            foreach ($resources as $resource) {
                foreach ($privileges as $privilege) {
                    if ($this->_acl->has($resource)) {
                        if ($this->_acl->isAllowed(
                            $this->_acl->getCurrentRole(), $resource, $privilege)) {
                            $this->_setAccessAllow();
                        } else {
                            $this->_setAccessDeny();
                        }
                    }
                }
            }
        }
    }

    /**
     * Deny access setter
     *
     * @return void
     */
    protected function _setAccessDeny ()
    {
        $this->_accessAllowed = false;
    }

    /**
     * Allow access setter
     *
     * @return void
     */
    protected function _setAccessAllow ()
    {
        $this->_accessAllowed = true;
    }

    /**
     * Allow access getter
     *
     * @return bool
     */
    protected function _accessIsAllowed ()
    {
        return (bool) $this->_accessAllowed;
    }
}
