<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Controller_Plugin_Abstract
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Controller_Plugin_AppClient extends Zend_Controller_Plugin_Abstract
{
    /**
     * Zend_View instance
     *
     * @var Zend_View
     */
    public $view = null;

    /**
     * Called after an action is dispatched by Zend_Controller_Dispatcher.
     *
     * This callback allows for proxy or filter behavior. By altering the
     * request and resetting its dispatched flag (via
     * {@link Zend_Controller_Request_Abstract::setDispatched() setDispatched(false)}),
     * a new action may be specified for dispatching.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public function postDispatch (Zend_Controller_Request_Abstract $request)
    {
        /* except error handler and AJAX requestes */
        if (true === $request->isXmlHttpRequest() || Singular_Core::_('Environment')->isErrorRequest()) {
            return;
        }

        /* get view instance */
        $this->view = Zend_Layout::getMvcInstance()->getView();
        /* append script */
        $this->view->headScript()->appendScript($this->_assembleScript($request));
    }

    /**
     * Application javascript assembler
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return string
     */
    protected function _assembleScript (Zend_Controller_Request_Abstract $request)
    {
        $themeParam = (Singular_Core::_('Environment')->isAdminRequest())
                ? Singular_Core::_('Theme')->getAdmin()
                : Singular_Core::_('Theme')->getDefault();

        $script = 'if(typeof App == "undefined") { App = {}; }';
        $script .= 'App.moduleName = \'' . $request->getModuleName() . '\';' . PHP_EOL;
        $script .= 'App.controllerName = \'' . $request->getControllerName() . '\';' . PHP_EOL;
        $script .= 'App.actionName = \'' . $request->getActionName() . '\';' . PHP_EOL;
        $script .= 'App.routeName = \'' . Zend_Controller_Front::getInstance()->getRouter()->getCurrentRouteName() . '\';' . PHP_EOL;
        $script .= 'App.config.basePath = \'' . (('' != $this->view->baseUrl()) ? $this->view->baseUrl() : '/') . '\';' . PHP_EOL;
        $script .= 'App.config.theme = \'' . Singular_Core::_('Settings')->getSetting($themeParam) . '\';' . PHP_EOL;
        $script .= 'App.translations = ' . $this->_getTranslations() . ';' . PHP_EOL;
        if (Singular_AppClient::hasParams()) {
            $script .= 'App.addParams(' . $this->_getDefinedVars() . ');' . PHP_EOL;
        }
        $script .= 'App.reinit();' . PHP_EOL;

        return $script;
    }

    /**
     * Gets all user-defined params
     *
     * @uses   Zend_Json
     * @return string
     */
    protected function _getDefinedVars ()
    {
        return Zend_Json::encode(
            Singular_AppClient::getAllParams()
        );
    }

    /**
     * Gets all translate messages
     *
     * @uses   Zend_Json
     * @return string
     */
    protected function _getTranslations ()
    {
        $translations = null;
        if (Singular_Runtime::inStorage('Zend_Translate')) {
            /** @var $translate Singular_Translate */
            $translate    = Singular_Runtime::extract('Zend_Translate');

            if (null !== $translate->getAdapter()->getList()) {
                $translations = $translate->getAdapter()->getMessages();

                if (Singular_Runtime::inStorage('jsLangKeys')) {
                    $jsLangKeys = Singular_Runtime::extract('jsLangKeys');

                    if (is_array($jsLangKeys) && !empty($jsLangKeys)) {
                        $filtered = $this->_filterJsKeys($translations, $jsLangKeys);

                        if (!empty($filtered)) {
                            $translations = $filtered;
                            unset($filtered);
                        }
                    }
                }
            }
        }

        return (null !== $translations) ? Zend_Json::encode($translations) : '{}';
    }

    private function _filterJsKeys ($translations, $jsLangKeys)
    {
        $filtered = array();
        foreach ($jsLangKeys as $jsKey) {
            if (isset($translations[$jsKey])) {
                $filtered[$jsKey] = $translations[$jsKey];
            }
        }

        return $filtered;
    }
}
