<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Controller_Plugin_Abstract
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Controller_Plugin_Debug extends Zend_Controller_Plugin_Abstract
{
    /**
     * Called before Zend_Controller_Front exits its dispatch loop.
     *
     * @param Zend_Controller_Request_Abstract $request
     * @return void
     */
    public function postDispatch (Zend_Controller_Request_Abstract $request)
    {
        // only development environment
        if ('development' == APPLICATION_ENV) {
            // get view instance
            $view = Zend_Layout::getMvcInstance()->getView();
            // get firebug helper
            $firebug = Zend_Controller_Action_HelperBroker::getStaticHelper('Firebug');

            // is exists helper
            if (null !== $firebug) {
                // view variables
                $firebug->direct('View:', Zend_Log::INFO);
                $firebug->direct($this->_cleanData($view->getVars()));
                // request
                $firebug->direct('Request:', Zend_Log::INFO);
                $firebug->direct($this->_cleanData($this->getRequest()->getParams()));
                // registry
                $firebug->direct('Registry:', Zend_Log::INFO);
                $firebug->direct($this->_cleanData(Zend_Registry::getInstance()));
                // session
                $firebug->direct('$_SESSION:', Zend_Log::INFO);
                $firebug->direct($this->_cleanData($_SESSION));
                // $_COOKIE
                $firebug->direct('$_COOKIE:', Zend_Log::INFO);
                $firebug->direct($this->_cleanData($_COOKIE));
                // $_SERVER
                $firebug->direct('$_SERVER:', Zend_Log::INFO);
                $firebug->direct($this->_cleanData($_SERVER));
                // $_FILES
                $firebug->direct('$_FILES:', Zend_Log::INFO);
                $firebug->direct($this->_cleanData($_FILES));
                // request headers
                $firebug->direct('Request headers:', Zend_Log::INFO);
                $firebug->direct(apache_request_headers());
                // response headers
                $firebug->direct('Response headers:', Zend_Log::INFO);
                $firebug->direct(apache_response_headers());
                // debug info
                Singular_Timer::stop('application');
                $firebug->direct('Debug information:', Zend_Log::INFO);
                $firebug->direct(array(
                    'totalTime'   => Singular_Timer::total('application'),
                    'memoryUsage' => memory_get_usage(),
                ));
            }
        }
    }

    /**
     * Clean data
     *
     * @param mixed $values
     * @return array
     */
    protected function _cleanData ($values)
    {
        $data = array();
        
        if (is_array($values)) {
            ksort($values);
        }

        foreach ($values as $key => $value) {
            if (is_numeric($value)) {
                $data[$key] = $value;
            } else if (is_string($value)) {
                $data[$key] = $value;
            } else if (is_array($value)) {
                $data[$key] = self::_cleanData($value);
            } else if (is_object($value)) {
                $data[$key] = get_class($value);
            } else if (is_null($value)) {
                $data[$key] = null;
            }
        }
        
        return $data;
    }
}
