<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Controller_Plugin_Abstract
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
final class Singular_Controller_Plugin_Head extends Zend_Controller_Plugin_Abstract
{
    /**
     * Called before an action is dispatched by Zend_Controller_Dispatcher.
     *
     * This callback allows for proxy or filter behavior.  By altering the
     * request and resetting its dispatched flag (via
     * {@link Zend_Controller_Request_Abstract::setDispatched() setDispatched(false)}),
     * the current action may be skipped.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public function preDispatch (Zend_Controller_Request_Abstract $request)
    {
        // except error handler and AJAX requestes
        if (true === $request->isXmlHttpRequest() || Singular_Core::_('Environment')->isErrorRequest()) {
            return;
        }

        // get settings
        $settings = Zend_Controller_Action_HelperBroker::getStaticHelper('Settings');
        // get config
        $config = Singular_Runtime::extract('config');
        // get view instance
        $view = Zend_Layout::getMvcInstance()->getView();
        // set title
        $view->headTitle($settings->direct('site_title'), 'APPEND');
        // set title separator
        $view->headTitle()->setSeparator(' ' . $settings->direct('site_title_separator') . ' ');
        // set doctype
        $view->doctype('XHTML1_STRICT');
        // set content type
        $view->headMeta()->appendHttpEquiv(
            'content-type',
            'text/html;charset=' . $config->resources->view->encoding
        );
    }
}
