<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Controller_Plugin_Abstract
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
final class Singular_Controller_Plugin_PageDefaults extends Zend_Controller_Plugin_Abstract
{
    /**
     * Zend_View instance
     * 
     * @var Zend_View
     */
    public $view = null;
    
    /**
     * Called before an action is dispatched by Zend_Controller_Dispatcher.
     *
     * This callback allows for proxy or filter behavior.  By altering the
     * request and resetting its dispatched flag (via
     * {@link Zend_Controller_Request_Abstract::setDispatched() setDispatched(false)}),
     * the current action may be skipped.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public function preDispatch (Zend_Controller_Request_Abstract $request)
    {
        // Except error handler and AJAX requestes
        if (true === $request->isXmlHttpRequest() || Singular_Core::_('Environment')->isErrorRequest()) {
            return;
        }

        // get config
        $config = Singular_Runtime::extract('config');
        // get current module && controller name
        $currModuleName = $request->getModuleName();
        // get Module assets directory
        $assetsDir = $config->htmlElement->assetsPathModules;
        // get Base URI
        $baseUrl = $config->resources->frontController->baseUrl;
        $isAdminRequest = Singular_Core::_('Environment')->isAdminRequest();
        
        $cssFile = $isAdminRequest
            ? $config->pageDefaults->admin->stylesheetFile
            : $config->pageDefaults->frontend->stylesheetFile;

        $jsFile = $isAdminRequest
            ? $config->pageDefaults->admin->javascriptFile
            : $config->pageDefaults->frontend->javascriptFile;

        $stylesheetFilePath = "{$assetsDir}/{$currModuleName}/"
                . $config->htmlElement->stylesheetDirName . '/' . $cssFile;
        $javascriptFilePath = "{$assetsDir}/{$currModuleName}/"
                . $config->htmlElement->javascriptDirName . '/' . $jsFile;

        /** @var $this->view Zend_View */
        $this->view = Zend_Layout::getMvcInstance()->getView();

        // check for existence & plug
        if ('system' != $currModuleName && $isAdminRequest) {
            $adminCss = str_replace($currModuleName, 'system', $stylesheetFilePath);
            $this->_insertCss($adminCss, $cssFile, 'system');

            $adminJs = str_replace($currModuleName, 'system', $javascriptFilePath);
            $this->_insertJs($adminJs, $jsFile, 'system');
        }
        
        $this->_insertJs($javascriptFilePath, $jsFile, $currModuleName);
        $this->_insertCss($stylesheetFilePath, $cssFile, $currModuleName);
    }

    /**
     * Append javascript file
     *
     * @param  string $javascriptFilePath
     * @param  string $jsFile
     * @param  string $moduleName
     * @return void
     */
    protected function _insertJs ($javascriptFilePath, $jsFile, $moduleName)
    {
        if (file_exists(ROOT_PATH . DS . $javascriptFilePath)
                && is_readable(ROOT_PATH . DS . $javascriptFilePath)) {
            $this->view->headScript()->appendFile($this->view->javascriptSrc($jsFile, $moduleName));
        }
    }

    /**
     * Append stylesheet file
     * 
     * @param  string $stylesheetFilePath
     * @param  string $cssFile
     * @param  string $moduleName
     * @return void
     */
    protected function _insertCss ($stylesheetFilePath, $cssFile, $moduleName)
    {
        if (file_exists(ROOT_PATH . DS . $stylesheetFilePath)
                && is_readable(ROOT_PATH . DS . $stylesheetFilePath)) {
            $this->view->headLink()->appendStylesheet($this->view->stylesheetSrc($cssFile, $moduleName));
        }
    }
}
