<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Controller_Plugin_Abstract
 * @category   Singular
 * @package    Singular_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Controller_Plugin_Themes extends Zend_Controller_Plugin_Abstract
{
    /**
     * Called before an action is dispatched by Zend_Controller_Dispatcher.
     *
     * This callback allows for proxy or filter behavior. By altering the
     * request and resetting its dispatched flag (via
     * {@link Zend_Controller_Request_Abstract::setDispatched() setDispatched(false)}),
     * the current action may be skipped.
     *
     * @param  Zend_Controller_Request_Abstract $request
     * @return void
     */
    public function dispatchLoopStartup (Zend_Controller_Request_Abstract $request)
    {
        /* get themes config */
        $themes = Singular_Runtime::extract('themes');
        /* get Layout */
        $layout = Zend_Layout::getMvcInstance();
        /* get view */
        $view = $layout->getView();
        /* get frontController instance */
        $frontController = Zend_Controller_Front::getInstance();

        /* get theme name */
        if (Singular_Core::_('Environment')->isAdminRequest()) {
            $themeName = Singular_Core::_('Theme')->getAdmin();
        } else {
            $themeName = Singular_Core::_('Theme')->getDefault();
        }
        
        if (Singular_Runtime::inStorage('site_theme')) {
            $themeName = Singular_Runtime::extract('site_theme');
        }
        
        Singular_Runtime::store('site_theme', $themeName);

        /* module path */
        $modulePath = $frontController->getModuleDirectory();
        $themeModulePath = ROOT_PATH . DS . $themes->path . DS . $themeName;

        // redefine views paths
        $viewRenderer = Zend_Controller_Action_HelperBroker::getStaticHelper('ViewRenderer');
        $viewRenderer->setNoController(true);
        $viewRenderer->setViewScriptPathNoControllerSpec(':action.:suffix');
        $viewRenderer->setViewScriptPathSpec(':module_:controller_:action.:suffix');
        $viewRenderer->setViewBasePathSpec($themeModulePath . DS . 'templates');
        $view->addScriptPath($modulePath . DS . 'templates');

        // set Layout paths
        $layoutPath = $layout->getLayoutPath();
        $layout->setLayoutPath($themeModulePath . DS . 'layouts');
        $view->addScriptPath($layoutPath);
        // add views elements path
        $view->addScriptPath($themeModulePath . $themes->views->common->path);

        // set view helpers and filters paths
        $modules = Singular_Runtime::extract('active_modules');
        foreach ($modules as $module) {
            $moduleHelperPathNamespace = Singular_Stdlib_Module::formatName($module) . '_View_Helper_';
            $moduleDir = $frontController->getModuleDirectory($module);
            // helper path
            $view->addHelperPath($moduleDir . DS . 'helpers', $moduleHelperPathNamespace);
            // filter path
            $view->addFilterPath($moduleDir . DS . 'filters', $moduleHelperPathNamespace);
            // block path
            Singular_View_Helper_Block::addPath($moduleDir);
            // section path
            Singular_View_Helper_Section::addPath($moduleDir);
        }

        // additional paths
        $view->addHelperPath($themeModulePath . DS . 'helpers', 'Theme_View_Helper_');
        $view->addFilterPath($themeModulePath . DS . 'filters' . DS . $module, 'Theme_View_Filter_');
        Singular_View_Helper_Block::addPaths(array($themeModulePath, APPLICATION_PATH));
        Singular_View_Helper_Section::addPaths(array($themeModulePath, APPLICATION_PATH));
    }
}
