<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Config core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_Config
{
    /**
     * @var Zend_Config
     */
    private static $_config = null;

    /**
     * Class constructor
     */
    public function __construct()
    {
        if (null === $this->getConfig()) {
            $this->_initConfig();
        }
    }

    /**
     * Retrieve config key value
     *
     * @throws Singular_Core_Exception If the $name config key is not exists
     * @param  string $name            The user-specified config key.
     * @return mixed
     */
    public function __get ($name)
    {
        if (isset($this->getConfig()->{$name})) {
            return $this->getConfig()->{$name};
        }
        throw new Singular_Core_Exception("Config key \"{$name}\" wasn't found.");
    }

    /**
     * Retrieve config object
     *
     * @return Zend_Config
     */
    public function getConfig ()
    {
        return self::$_config;
    }

    /**
     * Initialize config
     *
     * @throws Singular_Core_Exception
     * @return void
     */
    private function _initConfig ()
    {
        $config = Singular_Runtime::extract('config');

        if (!$config instanceof Zend_Config) {
            throw new Singular_Core_Exception('Config has not loaded yet.');
        }

        self::$_config = $config;
    }
}
