<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Cache core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_Db
{
    /**
     * @var Zend_Db_Adapter_Abstract
     */
    public $adapter = null;

    /**
     * Class constructor
     */
    public function __construct()
    {
        if (null === $this->adapter) {
            $this->adapter = Zend_Db_Table::getDefaultAdapter();
        }
    }

    /**
     * Get db adapter
     *
     * @return Zend_Db_Adapter_Abstract
     */
    public function getAdapter ()
    {
        return $this->adapter;
    }

    /**
     * Executes direct SQL query.
     *
     * @param  string $sql
     *
     * @return mixed
     */
    public function execute ($sql)
    {
        return $this->getAdapter()->exec($sql);
    }

    /**
     * Returns table prefix defined in application.inc.
     * If $tableName not null return table name with prefix.
     *
     * @param null|string $tableName
     *
     * @return string
     */
    public function getTablePrefix ($tableName = null)
    {
        return Singular_Core::_('Config')->getConfig()->resources->db->params->prefix . $tableName;
    }

    /**
     * Returns all available db tables
     *
     * @return array
     */
    public function getTables ()
    {
        return $this->getAdapter()->listTables();
    }

    /**
     * Checks wherever given table exists
     *
     * @param  string $table
     * @return bool
     */
    public function hasTable ($table)
    {
        return in_array($this->getTablePrefix($table), $this->getTables());
    }

    /**
     * Returns true when specified $table has $field
     *
     * @param  string $field
     * @param  string $table
     *
     * @return bool
     */
    public function hasTableField($field, $table)
    {
        if ($this->hasTable($table)) {
            if (array_key_exists(
                $field,
                $this->getAdapter()->describeTable($this->getTablePrefix($table))
            )
            ) {
                return true;
            }
        }
        return false;
    }
}