<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Environment core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_Environment
{
    /**
     * Front controller instance
     *
     * @var Zend_Controller_Front
     */
    protected $_frontController = null;

    /**
     * Class constructor
     */
    public function __construct()
    {
        $this->_initFrontController();
    }

    /**
     * Initialize front controller
     *
     * @return void
     */
    protected function _initFrontController ()
    {
        $this->_frontController = Zend_Controller_Front::getInstance();
    }

    /**
     * Retrieve front controller
     *
     * @return Zend_Controller_Front
     */
    public function getFrontController ()
    {
        return $this->_frontController;
    }

    /**
     * Returns true when the request was maked from frontend controller type
     *
     * @return bool
     */
    public function isFrontendRequest ()
    {
        return (true === $this->isAdminRequest()) ? false : true;
    }

    /**
     * Returns true when the request was maked from admin controller type
     *
     * @return bool
     */
    public function isAdminRequest ()
    {
        $request = $this->getFrontController()->getRequest();

        if (strpos($request->getControllerName(), 'admin') === 0) {
            return true;
        }

        return false;
    }

    /**
     * Returns true when error occured during request
     *
     * @return bool
     */
    public function isErrorRequest ()
    {
        $request = $this->getFrontController()->getRequest();

        try {
            /** @var $config Zend_Config */
            $config = Singular_Runtime::extract('config');
            $errModule = $config->resources->frontController->plugins->errorhandler->options->module;
            $errController = $config->resources->frontController->plugins->errorhandler->options->controller;
        } catch (Exception $e) {
            $errModule = null;
            $errController = null;
        }

        if (null != $errModule && null != $errController) {
            if ($errModule == $request->getModuleName() && $errController == $request->getControllerName()) {
                return true;
            }
        }

        return false;
    }

    /**
     * Returns current request protocol
     *
     * Available protocols: http, https
     *
     * @return string
     */
    public function getRequestProtocol ()
    {
        if (isset($_SERVER['HTTPS'])) {
            if ('on' == strtolower($_SERVER['HTTPS']))
                return 'https';
            if ('1' == $_SERVER['HTTPS'])
                return 'https';
        } elseif (isset($_SERVER['SERVER_PORT']) && ('443' == $_SERVER['SERVER_PORT'])) {
            return 'https';
        }
        return 'http';
    }

    /**
     * Returns true if request was made via "https" protocol
     *
     * @return bool
     */
    public function isHttpsRequest ()
    {
        return ('https' == $this->getRequestProtocol());
    }

    /**
     * Returns true if request was made via "http" protocol
     *
     * @return bool
     */
    public function isHttpRequest ()
    {
        return ('http' == $this->getRequestProtocol());
    }
}
