<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Info core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_Info
{
    const CORE_LICENSE = 'GNU/GPL';
    
    private $_info = array();

    private static $_retrieveZfLatestVersion = false;
    
    /**
     * Class constructor
     */
    public function __construct()
    {
        if (empty($this->_info)) {
            $this->_init();
        }
    }

    /**
     * Enables or disables retrieving ZF latest version
     * while getting environment information
     *
     * @static
     * @param bool $flag
     */
    public static function setRetrieveZfLatestVersion ($flag = true)
    {
        self::$_retrieveZfLatestVersion = (bool) $flag;
    }

    /**
     * Returns flag for retrieving ZF latest version
     *
     * @static
     * @return bool
     */
    public static function getRetrieveZfLatestVersion ()
    {
        return self::$_retrieveZfLatestVersion;
    }

    /**
     * Gets info param by name
     * 
     * @throws Singular_Core_Exception
     * @param  string $param
     * @return mixed
     */
    public function getParam ($param)
    {
        if (!array_key_exists($param, $this->_info)) {
            throw new Singular_Core_Exception(
                "Invalid parameter {$param}"
            );
        }

        return $this->_info[$param];
    }

    /**
     * Gets all info params
     * 
     * @return array
     */
    public function getAllParams ()
    {
        return $this->_info;
    }

    /**
     * Initializes system information
     * 
     * @return void
     */
    protected function _init ()
    {
        $dbAdapterName   = (Singular_Runtime::inStorage('config'))
                               ? @Singular_Runtime::extract('config')->resources->db->adapter
                               : 'unknown';
        $zfLatestVesrion = (self::$_retrieveZfLatestVersion) ? Zend_Version::getLatest() : Zend_Version::VERSION;
        $dbAdapter       = Zend_Db_Table::getDefaultAdapter();



        $this->_info = array(
            'coreVersion'       => Singular_Version::VERSION,
            'coreVersionStatus' => Singular_Version::STATUS,
            'coreCodename'      => Singular_Version::CODENAME,
            'zfVersion'         => Zend_Version::VERSION,
            'zfLatestVersion'   => $zfLatestVesrion,
            'zfIsLatest'        => (Zend_Version::compareVersion($zfLatestVesrion) <= 0) ? true : false,
            'coreLicense'       => self::CORE_LICENSE,
            'phpVersion'        => phpversion(),
            'mysqlVersion'      => (null !== $dbAdapter) ? $dbAdapter->getServerVersion() : 'unknown',
            'dbAdapter'         => (null != $dbAdapterName)
                                    ? strtoupper($dbAdapterName) . ' ' . phpversion($dbAdapterName)
                                    : 'unknown',
            'serverDatetime'  => new Zend_Date()
        );
    }
}
