<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Controller
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @abstract
 * @uses       Singular_Controller_Action
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Controller
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
abstract class Singular_Core_System_Controller_Auth
    extends Singular_Controller_Action
{
    /**
     * Login types
     */
    const LOGIN_TYPE_ADMIN = 1;
    const LOGIN_TYPE_USER = 2;

    /**
     * Authentication object
     *
     * @var Singular_Core_Auth
     */
    protected $_auth = null;

    /**
     * Authentication result
     *
     * @var Zend_Auth_Result
     */
    protected $_result = null;

    /**
     * Authentication error flag
     *
     * @var bool
     */
    protected $_isError = false;

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @return void
     */
    public function init ()
    {
        if ('index' != $this->getRequest()->getActionName()) {
            $this->_auth = Singular_Core::_('Auth');
        }
    }

    /**
     * Display layout with login form
     *
     * @return void
     */
    public function indexAction ()
    {
        // set login layout
        $this->_helper->layout->setLayout('login');
    }

    /**
     * Login action
     *
     * @return void
     */
    public function loginAction ()
    {
        if (!$this->getRequest()->isPost()) {
            return $this->_forward('index');
        }

        // get login params
        $username = $this->_getParam('username', null);
        $password = $this->_getParam('password', null);
        $type =     $this->_getParam('login_type', null);

        if (null !== $username && null !== $password) {
            /** @var $result Zend_Auth_Result */
            $this->_result = $this->_auth->authenticate($username, $password);

            if ($this->_result instanceof Zend_Auth_Result) {
                switch ($this->_result->getCode()) {
                    case Zend_Auth_Result::SUCCESS:
                        $this->_isError = false;
                    break;
                    case Zend_Auth_Result::FAILURE:
                    case Zend_Auth_Result::FAILURE_IDENTITY_NOT_FOUND:
                    case Zend_Auth_Result::FAILURE_IDENTITY_AMBIGUOUS:
                    case Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID:
                    case Zend_Auth_Result::FAILURE_UNCATEGORIZED:
                        $this->_isError = true;
                        break;
                }
            }

            return $this->_loginAnswer($type);
        }

        return $this->_forward('index');
    }

    /**
     * Logout action
     *
     * @return void
     */
    public function logoutAction ()
    {
        $redirectUrl = $this->_getParam('redirect', null);
        $redirectUrl = (null !== $redirectUrl)
                ? rawurldecode($redirectUrl)
                : $this->_baseUrl;

        $this->_auth->clearIdentity();
        $this->_redirect($redirectUrl);
    }

    /**
     * After login process
     *
     * @param string $type
     * @return void
     */
    protected function _loginAnswer ($type)
    {
        $erroMsg = $this->view->t('Invalid username or password');

        if (!$this->_isError) {
            switch ($type) {
                case self::LOGIN_TYPE_ADMIN:
                    $route = 'admin_dashboard';
                    break;
                case self::LOGIN_TYPE_USER:
                default:
                    $route = 'frontend_home';
                    break;
            }

            // assemble after login url
            $url = $this->_helper->url->url(array(), $route);

            $username   = $this->_auth->getIdentity()->username;
            $successMsg = sprintf($this->view->t('Successfully logged in as %s'), $username);
        }

        if ($this->isAjax()) {
            $this->autoresponder->setFormat('json');
            $this->autoresponder->removeMvcParams();

            if ($this->_isError) {
                $this->autoresponder->setError($erroMsg, 901);
            } else {
                $this->autoresponder->setSuccess($successMsg);
                $this->autoresponder->setParam('url', $url);
            }
        } else {
            if ($this->_isError) {
                $this->view->errorMessage = $erroMsg;
                $this->_forward('index');
            } else {
                $this->view->notify()->success($successMsg);
                $this->_redirect($url);
            }
        }
    }
}
