<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Controller
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @abstract
 * @uses       Singular_Controller_Action
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Controller
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
abstract class Singular_Core_System_Controller_Error extends Singular_Controller_Action
{
    /**
     * Default error action
     *
     * @return void
     */
    public function errorAction ()
    {
        // get error handler
        $errorHandler = $this->_getParam('error_handler');

        // clear body from response
        $this->getResponse()->clearBody();

        // switch error type
        switch ($errorHandler->type) {
            // 404 error - controller or action not found
            case Zend_Controller_Plugin_ErrorHandler::EXCEPTION_NO_CONTROLLER:
            case Zend_Controller_Plugin_ErrorHandler::EXCEPTION_NO_ACTION:
                $this->getResponse()->setHttpResponseCode(404);
                $this->view->headTitle('404. Not Found');
                $this->view->errorTitle = $this->_helper->t('404. Not Found');
                $this->view->errorText  = $this->_helper->t('Requested page is not found.');
                break;
            /// other errors
            default:
                switch (get_class($errorHandler->exception)) {
                    case 'Singular_Acl_Exception':
                        $this->getResponse()->setHttpResponseCode(403);
                        $this->view->headTitle('403. Forbidden');
                        $this->view->errorTitle = $this->_helper->t('403. Forbidden');
                        $this->view->errorText  = $this->_helper->t('Access denied.');
                    break;
                    default:
                        $this->getResponse()->setHttpResponseCode(500);
                        $this->view->headTitle('500. Application error');
                        $this->view->errorTitle = $this->_helper->t('500. Application error');
                        $this->view->errorText  = $this->_helper->t('Error occured. Please visit us later.');
                    break;
                }
        }
        
        // assign view vars
        $this->view->exception   = $errorHandler->exception;
        $this->view->request     = $errorHandler->request;
        $this->view->admin_email = $this->_helper->settings('site_admin_email');

        // set error layout
        $this->_helper->layout->setLayout('error');

        // log errors
        $this->_helper->log($errorHandler->exception->getMessage() . PHP_EOL . 
            $errorHandler->exception->getTraceAsString(), Zend_Log::ERR
        );
    }

    public function accessDeniedAction ()
    {
        die('Access denied!');
    }
}
