<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Model_DbTable
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Singular_Db_Table_Abstract
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Model_DbTable
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_System_Model_DbTable_Roles
    extends Singular_Db_Table_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'roles';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'role';
    
    /**
     * Row class
     * 
     * @var string
     */
    protected $_rowClass = 'Singular_Core_System_Model_Role';

    /**
     * Dependent Tables
     *
     * @var array
     */
    protected $_dependentTables = array(
        'System_Model_DbTable_Privileges',
        'Singular_Core_System_Model_DbTable_Users'
    );

    /**
     * Fetches all roles
     *
     * if $active = true - fetches only active roles
     *
     * @param  bool $active
     * @return array|null
     */
    public function getAllRoles ($active = false)
    {
        $select = $this->select()->order('role ASC');
        if (true === $active) {
            $select->where('is_active = ?', 1, Zend_Db::INT_TYPE);
        }
        $result = $this->fetchAll($select);
        
        return (null !== $result) ? $result->toArray() : $result;
    }
    
    /**
     * Find system roles
     * 
     * @param void
     * @return Zend_Db_Table_Rowset
     */
    public function findSystemRoles ()
    {
        $select = $this->select();
        $select->order('role');
        $select->where('is_protected = ?', 1, Zend_Db::INT_TYPE);
        return $this->fetchAll($select);
    }
    
    /**
     * Find custom roles
     * 
     * @param void
     * @return Zend_Db_Table_Rowset
     */
    public function findCustomRoles ()
    {
        $select = $this->select();
        $select->order('role');
        $select->where('is_protected = ?', 0, Zend_Db::INT_TYPE);
        return $this->fetchAll($select);
    }
    
    /**
     * Fetch list for html options
     * 
     * @param void
     * @return Zend_Db_Table_Rowset
     */
    public function findAll ()
    {
        $select = $this->select();
        $select->order('role');
        return $this->fetchAll($select);
    }
}
