<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @abstract
 * @uses       Singular_Db_Table_Abstract
 * @category   System
 * @package    System_Model
 * @subpackage System_Model_DbTable
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_System_Model_DbTable_UserSettings extends Singular_Db_Table_Abstract
{
    /**
     * Classname for row
     *
     * @var string
     */
    protected $_rowClass = 'Singular_Core_System_Model_UserSetting';
    
    /**
     * Primary key
     * 
     * @var array
     */
    protected $_primary = array('user_id', 'name');
    
    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'settings_users';
    
    /**
     * Reference map
     * 
     * @var array 
     */
    protected $_referenceMap = array(
        'Users' => array(
            self::COLUMNS         => array('user_id'),
            self::REF_TABLE_CLASS => 'Singular_Core_System_Model_DbTable_Users',
            self::REF_COLUMNS     => array('id'),
            self::ON_DELETE       => self::CASCADE,
            self::ON_UPDATE       => self::CASCADE,
        ),
    );
    
    /**
     * Fetch specific setting
     * 
     * @param  string $name
     * @param  Singular_Core_System_Model_User $user
     * @return Singular_Core_System_Model_UserSetting
     */
    public function findByUserAndName (Singular_Core_System_Model_User $user, $name)
    {
        $select = $this->select();
        $select->where('user_id = ?', $user->id, Zend_Db::INT_TYPE);
        $select->where('name = ?', $name);
        return $this->fetchRow($select);
    }
    
    /**
     * Fetch settings for user
     * 
     * @param Singular_Core_System_Model_User $user
     * @return array
     */
    public function findByUser (Singular_Core_System_Model_User $user)
    {
        $select = $this->select();
        $select->where('user_id = ?', $user->id, Zend_Db::INT_TYPE);
        $select->from($this->_name, array('name', 'value'));
        
        return $this->getAdapter()->fetchPairs($select);
    }
    
    /**
     * Fetch settings by name
     * 
     * @param string $name
     * @return Zend_Db_Table_Rowset
     */
    public function findByName ($name)
    {
        $select = $this->select();
        $select->where('name = ?', $name);
        
        return $this->fetchAll($select);
    }
    
    /**
     * Remove rows by setting name
     * 
     * @param  string $setting
     * @return int The number of rows deleted.
     */
    public function removeByName ($setting)
    {
        $where = $this->getAdapter()->quoteInto('name = ?', $setting);
        return $this->delete($where);
    }
    
    /**
     * Remove rows by user
     * 
     * @param  Singular_Core_System_Model_User $user
     * @return int The number of rows deleted.
     */
    public function removeByUser (Singular_Core_System_Model_User $user)
    {
        $where = $this->getAdapter()
            ->quoteInto('user_id = ?', $user->id, Zend_Db::INT_TYPE);
        
        return $this->delete($where);
    }
}
