<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Model_DbTable
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Singular_Db_Table_Abstract
 * @throws     Singular_Exception
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Model_DbTable
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_System_Model_DbTable_Users
    extends Singular_Db_Table_Abstract
{
    /**
     * The table name
     *
     * @var string
     */
    protected $_name = 'users';

    /**
     * The primary key column or columns.
     * A compound key should be declared as an array.
     * You may declare a single-column primary key
     * as a string.
     *
     * @var mixed
     */
    protected $_primary = 'id';
    
    /**
     * Row class
     * 
     * @var string
     */
    protected $_rowClass = 'Singular_Core_System_Model_User';

    /**
     * Filtered columns
     *
     * @var array
     */
    protected $_filteredCols = array(
        'email', 'first_name', 'last_name', 'username'
    );

    /**
     * Reference map
     *
     * @var array
     */
    protected $_referenceMap = array(
        'Roles' => array(
            self::COLUMNS         => array('role'),
            self::REF_TABLE_CLASS => 'Singular_Core_System_Model_DbTable_Roles',
            self::REF_COLUMNS     => array('role'),
            self::ON_DELETE       => self::RESTRICT,
            self::ON_UPDATE       => self::RESTRICT,
        ),
    );

    /**
     * Dependent Tables
     *
     * @var array
     */
    protected $_dependentTables = array(
        'Singular_Core_System_Model_DbTable_UserSettings'
    );

    /**
     * Paginate users
     *
     * @param int $pageNumber
     * @param array $filter
     * @return Zend_Paginator
     */
    public function pagination ($pageNumber = 1, $filter = null)
    {
        $select = $this->select();

        if (is_array($filter)) {
            $field = $filter['field'];
            $text = $filter['text'];
            if ($field && $text) {
                if (in_array($field, $this->_filteredCols)) {
                    $select->where($field . ' LIKE ?', '%' . $text . '%');
                }
            }
        }
        ;

        // make paginator
        $paginator = new Zend_Paginator(
            new Zend_Paginator_Adapter_DbTableSelect($select)
        );

        // paginator settings
        $pageRange = Singular_Core::_('Settings')
                ->getSetting('paginator_page_range');
        $paginator->setCurrentPageNumber((int)$pageNumber);
        $paginator->setItemCountPerPage(10);
        $paginator->setPageRange($pageRange);

        // return
        return $paginator;
    }

    /**
     * Find user objects by list of ID-s
     *
     * @param array $ids
     * @return Zend_Db_Table_Rowset|null
     */
    public function findByIds ($ids)
    {
        if (!is_array($ids)) {
            return null;
        }

        $select = $this->select();
        $select->where('id IN (?)', $ids, Zend_Db::INT_TYPE);

        return $this->fetchAll($select);
    }

    /**
     * Retrieve all users
     *
     * if $active = true - fetch only active users
     *
     * @param  bool $active
     * @return array|null
     */
    public function getAllUsers ($active = false)
    {
        $select = $this->select()->order('id ASC');
        if (true === $active) {
            $select->where('is_active = ?', 1, Zend_Db::INT_TYPE);
        }
        $result = $this->fetchAll($select);

        return (null !== $result) ? $result->toArray() : $result;
    }
}
