<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Model_DbTable
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Singular_Db_Table_Abstract
 * @throws     Singular_Exception
 * @category   Singular
 * @package    Singular_Core
 * @subpackage Singular_Core_System_Model_DbTable
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_System_Model_EmailTemplate
    extends Singular_Db_Table_Row
{
    protected $_tableClass = 'Singular_Core_System_Model_DbTable_EmailTemplates';

    public    $lang        = null;

    public    $langRow     = null;

    public function init ()
    {
        parent::init();
        
        $this->lang = Singular_Core::_('settings')
            ->getSetting('site_default_language');
    }

    public function getVariables ()
    {        
        return $this->variables ?
            explode(',', $this->variables) :
            array();
    }

    public function getLangRow ()
    {
        if (null === $this->langRow) {
            
            $templatesLang = Singular_Loader
                ::get('Singular_Core_System_Model_DbTable_EmailTemplatesLang');
            
            $select = $templatesLang->select();
            $select->where('template_id = ?', $this->id, Zend_Db::INT_TYPE);
            $select->where('lang = ?', $this->lang);
            $this->langRow = $templatesLang->fetchRow($select);
            
            if (!($this->langRow instanceof
                Singular_Core_System_Model_EmailTemplateLang
            )) {
                
                $langTable = Singular_Loader
                    ::get('Singular_Core_System_Model_DbTable_EmailTemplatesLang');
                
                $defaultLang = $langTable->findByTemplateIdAndLang($this->id);
                
                $this->langRow = $langTable->createRow(array(
                    'lang'        => $this->lang,
                    'template_id' => $this->id,
                    'subject'     => $defaultLang->subject,
                    'body'        => $defaultLang->body,
                ));
            }
        }
        
        return $this->langRow;
    }

    public function getBody ()
    {
        return $this->getLangRow()->body;
    }

    public function getSubject ()
    {
        return $this->getLangRow()->subject;
    }

    public function setFromArray (array $data)
    {
        $data = $data['email'];
        
        if ($data['subject']) {
            $this->getLangRow()->subject = $data['subject'];
        }
        if ($data['body']) {
            $this->getLangRow()->body = $data['body'];
        }
        parent::setFromArray($data);
    }

    public function save ($data = null)
    {
        if (parent::save($data)) {
            $this->getLangRow()->save();
        }
    }
}
