<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Theme core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_Theme
{
    protected static $_siteDefaultTheme = null;
    protected static $_siteAdminTheme   = null;

    /**
     * Session namespace
     *
     * @var Zend_Session_Namespace
     */
    private $_ns = null;

    private $_enableOverrides = true;

    /**
     * Returns current default theme name
     *
     * @return string
     */
    public function getDefault ()
    {
        if (null !== self::$_siteDefaultTheme) return self::$_siteDefaultTheme;
        return $this->_determine();
    }

    /**
     * Returns current admin theme name
     *
     * @return string
     */
    public function getAdmin ()
    {
        if (null !== self::$_siteAdminTheme) return self::$_siteAdminTheme;
        return $this->_determine(true);
    }

    /**
     * Overrides current default theme
     *
     * @param  string $value
     * @return Singular_Core_Theme
     */
    public function overrideDefault ($value)
    {
        $this->_getNs()->default = trim($value);
        return $this;
    }

    /**
     * Overrides current admin theme
     *
     * @param  string $value
     * @return Singular_Core_Theme
     */
    public function overrideAdmin ($value)
    {
        $this->_getNs()->admin = trim($value);
        return $this;
    }

    /**
     * Clears default theme override.
     *
     * @return Singular_Core_Theme
     */
    public function clearDefault ()
    {
        if (isset($this->_getNs()->default)) unset($this->_getNs()->default);
        return $this;
    }

    /**
     * Clears admin theme override.
     *
     * @return Singular_Core_Theme
     */
    public function clearAdmin ()
    {
        if (isset($this->_getNs()->admin)) unset($this->_getNs()->admin);
        return $this;
    }

    /**
     * Returns true if current theme is mobile.
     *
     * @return bool
     */
    public function isMobile()
    {
        return 'mobile' == $this->getDefault();
    }

    /**
     * Clears all theme overrides.
     *
     * @return Singular_Core_Theme
     */
    public function clearAllOverrides ()
    {
        $this->_getNs()->unsetAll();
        return $this;
    }

    /**
     * Sets $_disableOverrides flag to false.
     *
     * Temporary disables overrides using
     *
     * @return Singular_Core_Theme
     */
    public function disableOverrides ()
    {
        $this->_enableOverrides = false;
        return $this;
    }

    /**
     * Sets $_disableOverrides flag to true
     *
     * Enables overrides using
     *
     * @return Singular_Core_Theme
     */
    public function enableOverrides ()
    {
        $this->_enableOverrides = true;
        return $this;
    }

    /**
     * Return $_enableOverrides flag
     *
     * @return bool
     */
    public function getEnableOverrides ()
    {
        return $this->_enableOverrides;
    }

    /**
     * Returns theme name in various cases
     *
     * @param  bool $forAdmin
     * @return string
     */
    private function _determine ($forAdmin = false)
    {
        $param    = (!$forAdmin) ? 'default' : 'admin';
        $setting  = (!$forAdmin) ? 'site_default_theme' : 'site_default_admin_theme';
        $propName = '_site' . ucfirst($param) . 'Theme';
        if (isset($this->_getNs()->{$param}) && !empty($this->_getNs()->{$param}) && $this->getEnableOverrides()) {
            // disable cache if theme has overriden
            $this->_disableCache();
            $theme = $this->_getNs()->{$param};
        } else {
            $theme = Singular_Core::_('Settings')->getSetting($setting);
        }

        self::${$propName} = $theme;

        return self::${$propName};
    }

    /**
     * Disables cache
     *
     * @return void
     */
    private function _disableCache ()
    {
        Singular_Core::_('Cache')->disableAll();
    }

    /**
     * Returns theme session namespace
     *
     * @return Zend_Session_Namespace
     */
    private function _getNs ()
    {
        if (null === $this->_ns) {
            $this->_ns = new Zend_Session_Namespace('Theme');
        }

        return $this->_ns;
    }
}
