<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Time core resource accessor
 *
 * @category   Singular
 * @package    Singular_Core
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Core_Timer
{
    /**
     * @var ZFDebug_Controller_Plugin_Debug_Plugin_Time
     */
    private static $_zfDebugTime = null;

    /**
     * @var ZFDebug_Controller_Plugin_Debug_Plugin_Memory
     */
    private static $_zfDebugMemory = null;

    /**
     * @var bool
     */
    private static $_pluginLoaded = false;

    /**
     * @var bool
     */
    private static $_checkAttempt = false;

    /**
     * Class constructor
     */
    public function __construct()
    {
        if (!self::$_pluginLoaded && !self::$_checkAttempt) {
            $this->_initTimer();
        }
    }

    /**
     * Measure executing time
     * 
     * @param  string $name
     * @return Singular_Core_Timer
     */
    public function time ($name)
    {
        if (self::$_pluginLoaded) {
            self::$_zfDebugTime->mark($name);
        }
        
        return $this;
    }

    /**
     * Measure allocated memory
     *
     * @param  string $name
     * @return Singular_Core_Timer
     */
    public function memory ($name)
    {
        if (self::$_pluginLoaded) {
            self::$_zfDebugMemory->mark($name);
        }

        return $this;
    }

    /**
     * Initializes timer
     *
     * @return void
     */
    protected function _initTimer ()
    {
        $frontController = Zend_Controller_Front::getInstance();
        
        /** @var $zfDebug ZFDebug_Controller_Plugin_Debug */
        $zfDebug = $frontController->getPlugin('ZFDebug_Controller_Plugin_Debug');

        if (false !== $zfDebug) {
            self::$_zfDebugTime   = $zfDebug->getPlugin('Time');
            self::$_zfDebugMemory = $zfDebug->getPlugin('Memory');
            self::$_pluginLoaded  = true;
            self::$_checkAttempt  = true;
        }
    }
}
