<?php
/**
 * Singular
 *
 * LICENSE
 *
 * The SOFTWARE PRODUCT is protected by copyright treaties,
 * as well as other intellectual property laws and treaties.
 * The SOFTWARE PRODUCT is licensed, not sold.
 * Software.
 * You may install and use one copy of the SOFTWARE PRODUCT
 * for each license purchased or if a site license then you
 * may install any number of copies at the site where the
 * site license is defined.
 * Your rights to use the SOFTWARE PRODUCT are not transferable.
 *
 * @category   Singular
 * @package    Singular_Event
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */

/**
 * @category   Singular
 * @package    Singular_Event
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */
class Singular_Event
{
    /**
     * Dispatched events list
     *
     * @var array
     */
    private static $_dispatched = array();

    /**
     * Events that are been stopped from propagation
     *
     * @var array
     */
    private static $_stopped = array();

    /**
     * Dispatch specific event
     *
     * @static
     * @param string     $event  Event name
     * @param mixed|null $params Argument that will be passed to event aggregate
     */
    public static function dispatch ($event, &$params = null)
    {
        if (self::isPropagationStopped($event)) {
            return;
        }

        if (null !== $aggregates = Singular_Event_Listener_Abstract::getAggregates($event)) {
            foreach ($aggregates as $callback) {
                if (self::isPropagationStopped($event)) {
                    continue;
                }
                if (is_string($callback->class)) {
                    $class = $callback->class; $method = $callback->method;
                    $class::$method($params);
                } elseif (is_object($callback->class)) {
                    $callback->class->{$callback->method}($params);
                }
            }

            self::$_dispatched[] = $event;
        }
    }

    /**
     * Adds event to propagation stop-list
     *
     * @param $event
     */
    public static function stopPropagation($event)
    {
        $event = strtolower($event);
        if (!in_array($event, self::$_stopped)) {
            self::$_stopped[] = $event;
        }
    }

    /**
     * Returns true if event propagation were stopped.
     *
     * @param  string $event
     *
     * @return bool
     */
    public static function isPropagationStopped($event)
    {
        return in_array(strtolower($event), self::$_stopped);
    }

    /**
     * Checks for event dispatched or not
     *
     * @static
     * @param  $event
     * @return bool
     */
    public static function isDispatched ($event)
    {
        return in_array($event, self::$_dispatched);
    }

    /**
     * Returns all already dispatched events
     *
     * @static
     * @return array
     */
    public static function getDispatched ()
    {
        return self::$_dispatched;
    }
}
