<?php
/**
 * Rename file with unique name
 */
class Singular_Filter_File_UniqueName implements Zend_Filter_Interface
{
    /**
     * Filter options
     *
     * @var array
     */
    private $_options;

    /**
     * @throws Zend_Filter_Exception
     * @param array $options
     */
    public function __construct($options)
    {
        if ($options instanceof Zend_Config) {
            $options = $options->toArray();
        } elseif (is_string($options)) {
            $options = array(
                'targetDir' => $options,
                'nameLength' => 10
            );
        } elseif (!is_array($options)) {
            throw new Zend_Filter_Exception('Invalid options argument provided to filter');
        }

        $this->_options = $options;
    }

    /**
     * Filter
     *
     * @param  string $fileSource Full path to the file
     * @return string Path to the new file or Exception if failed
     */
    public function filter($fileSource)
    {
        if (!file_exists($fileSource)) {
            return $fileSource;
        }

        $pathInfo = pathinfo($fileSource);
        $targetDir = isset($this->_options['targetDir']) ? $this->_options['targetDir'] : $pathInfo['dirname'];

        // While new filename wasn't generated
        while (true) {
            $fileName = self::_randStr($this->_options['nameLength']) . '.' . $pathInfo['extension'];
            $fileTarget = $targetDir . DIRECTORY_SEPARATOR . $fileName;
            if (!file_exists($fileTarget)) {
                break;
            }
        }

        $result = rename($fileSource, $fileTarget);

        if ($result === true) {
            return $fileTarget;
        }

        throw new Zend_Filter_Exception(sprintf("File '%s' could not be renamed. An error occured while processing the file.", $fileSource));
    }


    /**
     * Generates random string by given lenght
     *
     * @param integer $length
     * @return string
     */
    private static function _randStr($length = 10)
    {
        $chars = array('a', 'b', 'c', 'd', 'e', 'f',
                       'g', 'h', 'i', 'j', 'k', 'l',
                       'm', 'n', 'o', 'p', 'r', 's',
                       't', 'u', 'v', 'x', 'y', 'z',
                       'A', 'B', 'C', 'D', 'E', 'F',
                       'G', 'H', 'I', 'J', 'K', 'L',
                       'M', 'N', 'O', 'P', 'R', 'S',
                       'T', 'U', 'V', 'X', 'Y', 'Z',
                       '1', '2', '3', '4', '5', '6',
                       '7', '8', '9', '0');

        $name = "";

        for ($i = 0; $i < $length; $i++) {
            $index = rand(0, count($chars) - 1);
            $name .= $chars[$index];
        }

        return $name;
    }
}
