<?php
/**
 * Singular
 *
 * LICENSE
 *
 * The SOFTWARE PRODUCT is protected by copyright treaties,
 * as well as other intellectual property laws and treaties.
 * The SOFTWARE PRODUCT is licensed, not sold.
 * Software.
 * You may install and use one copy of the SOFTWARE PRODUCT
 * for each license purchased or if a site license then you
 * may install any number of copies at the site where the
 * site license is defined.
 * Your rights to use the SOFTWARE PRODUCT are not transferable.
 *
 * @category   Singular
 * @package    Singular_Filter
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */

 /**
 * @category   Singular
 * @package    Singular_Filter
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */
class Singular_Filter_Minify_Styles implements Zend_Filter_Interface
{
    /**
     * Defined by Zend_Filter_Interface
     *
     * Returns a string containing the minified Css. Accepts a string
     * or filename of css to be minified
     *
     * @param  string $value
     * @return string
     */
    public function filter ($value)
    {
        //Was the passed $value a file?
        if(is_file($value)){
            $value = file_get_contents($value);
        }

        //Perform minification on $value
        $value = $this->_minify($value);

        //Return the minified string
        return $value;
    }

    /**
     * Take a string of css and minify it
     *
     * Also takes into account the set options when performing minification
     *
     * @param  string $css
     * @return string
     */
    protected function _minify ($css)
    {
        $css = preg_replace( '#\s+#', ' ', $css );
        $css = preg_replace( '#/\*.*?\*/#s', '', $css );
        $css = str_replace( '; ', ';', $css );
        $css = str_replace( ': ', ':', $css );
        $css = str_replace( ' {', '{', $css );
        $css = str_replace( '{ ', '{', $css );
        $css = str_replace( ', ', ',', $css );
        $css = str_replace( '} ', '}', $css );
        $css = str_replace( ';}', '}', $css );
        $css = trim($css);

        return $css;
    }
}
