<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Filter
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Settings core resource accessor
 *
 * @category   Singular
 * @package    Singular_Filter
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Filter_Slugify implements Zend_Filter_Interface
{
    /**
     * Words separator
     *
     * @var string
     */
    protected $_separator = '-';

   /**
    * Get the separator
    *
    * @param void
    * @return string
    */
    public function getSeparator ()
    {
        return $this->_separator;
    }

   /**
    * Set the separator
    *
    * @param string $separator
    * @return Singular_Filter_Slugify
    */
    public function setSeparator ($separator)
    {
        $this->_separator = $separator;

        return $this;
    }

    /**
     * Constructor
     *
     * @param array $options
     */
    public function __construct ($options = null)
    {
        if ($options instanceof Zend_Config) {
            $options = $options->toArray();
        } else if (!is_array($options)) {
            $options = func_get_args();
            $temp = array();

            if (!empty($options)) {
                $temp['separator'] = array_shift($options);
            }

            $options = $temp;
        }

        if (array_key_exists('separator', $options)) {
            $this->setSeparator($options['separator']);
        }
    }

    /**
     * Process
     * 
     * @param string $string
     * @return string 
     */
    public function filter ($string)
    {
        $string = Zend_Layout::getMvcInstance()->getView()->translite($string);

        // transliterate, silently
        $clean = iconv('UTF-8', 'ASCII//TRANSLIT', $string);
        $clean = preg_replace("/[^a-zA-Z0-9\/_|+ -]/", '', $clean);
        $clean = strtolower(trim($clean, '-'));
        $clean = preg_replace("/[\/_|+ -]+/", $this->getSeparator(), $clean);

        return $clean;
    }
}
