<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Form
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Singular_Form extended class
 *
 * @category   Singular
 * @package    Singular_Form
 * @uses       Zend_Form
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Form extends Zend_Form
{
    const FORM_TABS_WIDGET_ID = 'singular-form-tabs';

    /**
     * Wherever use FormTabs widget or not
     *
     * true  - yes
     * false - no
     *
     * @var bool
     */
    private static $_useFormTabsWidget = true;

    /**
     * FormTabs widget render status
     * 
     * @var bool
     */
    private static $_formTabsIsRendered = false;

    /**
     * Constructor
     *
     * Registers form view helper as decorator
     *
     * @param mixed $options
     */
    public function __construct ($options = null)
    {
        parent::__construct($options);

        // dispatch event after form has initialized
        Singular_Event::dispatch('formInitAfter', $this);
    }

    /**
     * Initialize form (used by extending classes)
     * 
     * @return void
     */
    public function init ()
    {
        // Scaffolding
        $scaffolding = Singular_Inflector::scaffold(get_class($this));

         // Html settings
        $this->setAttribs(array(
            'id'             => $scaffolding,
            'class'          => $scaffolding,
            'accept-charset' => 'utf-8'
        ));

        // Form settings
        $this->setName($scaffolding);
        $this->setAction($this->getView()->url());
        $this->setMethod(Zend_Form::METHOD_POST);
        $this->setEnctype(Zend_Form::ENCTYPE_URLENCODED);

        // Form decorators
        $this->setDisableLoadDefaultDecorators(true);
        $this->addDecorator('FormElements')
                ->addDecorator('Form');

        // Add custom validators, filters and decorators
        $this->addElementPrefixPaths(array(
            array('prefix' => 'Singular_Validate_',
                  'path'   => 'Singular/Validate',
                  'type'   => Zend_Form_Element::VALIDATE,
            ),
            array('prefix' => 'Singular_Filter_',
                  'path'   => 'Singular/Filter',
                  'type'   => Zend_Form_Element::FILTER,
            ),
            array('prefix' => 'Singular_Form_Decorator_',
                  'path'   => 'Singular/Form/Decorator',
                  'type'   => Zend_Form_Element::DECORATOR,
            ),
        ));
    }

    /**
     * Sets flag for using FormTabs widget
     *
     * @static
     * @param  bool $flag
     *              true  - yes
     *              false - no
     * @return void
     */
    public static function setUseFormTabsWidget ($flag)
    {
        self::$_useFormTabsWidget = (bool) $flag;
    }

    /**
     * Gets FormTabs widget using flag
     * 
     * @static
     * @return bool
     */
    public static function getUseFormTabsWidget ()
    {
        return self::$_useFormTabsWidget;
    }

    /**
     * Returns all errors occurred during form processing
     *
     * @return array
     */
    public function getAllErrors ()
    {
        $elements = $this->getErrors();
        if (is_array($elements)) {
            $errorMsgs = array();
            foreach ($elements as $element => $errors) {
                if (is_array($errors)) {
                    foreach ($errors as $error) {
                        $messages = $this->$element->getMessages();
                        $errorMsgs[] = " " . $messages[$error];
                    }
                }
            }
            return $errorMsgs;
        }

        return array();
    }

    /**
     * Renders form
     *
     * @param  Zend_View_Interface $view
     * @return string
     */
    public function render (Zend_View_Interface $view = null)
    {
        if (null !== $view) {
            $this->setView($view);
        }

        $this->rewind();

        // FormTabs widget
        if (!self::$_formTabsIsRendered
            && self::getUseFormTabsWidget()
            && Singular_Sidebar::hasDefault()
            && Singular_Sidebar::getDefault()->isEnabled()
            && count($this->getSubForms()) > 1) {
            $ftWidget = new Singular_Widget_FormTabs();
            $ftWidget->setForms($this->getSubForms());

            // dispatch 'formWidgetFormTabs' event
            Singular_Event::dispatch('formWidgetFormTabs', $ftWidget);

            Singular_Sidebar::getDefault()->addWidget(
                $ftWidget, self::FORM_TABS_WIDGET_ID, null, -1000
            );

            self::$_formTabsIsRendered = true;
        }

        $content = '';
        foreach ($this->getDecorators() as $decorator) {
            $decorator->setElement($this);
            $content = $decorator->render($content);
        }

        $this->_setIsRendered();
        return $content;
    }
}
