<?php
/**
 * Base element for XHTML elements
 *
 * @category   Singular
 * @package    Singular_Form
 * @subpackage Element
 */
abstract class Singular_Form_Element_Xhtml extends Zend_Form_Element
{
    protected $_labelPrefix = null;

    protected $_labelPostfix = null;

    protected $_inputPrefix = null;

    protected $_inputPostfix = null;
    
    protected $_isSetting = false;
    
    protected $_isUserSetting = false;
    
    public function setLabelPrefix($prefix)
    {
        if (!is_string($prefix) && !is_null($prefix)) {
            return;
        }
        $this->_labelPrefix = $prefix;

        return $this;
    }

    public function getLabelPrefix()
    {
        return $this->_labelPrefix;
    }

    public function hasLabelPrefix()
    {
        return (null !== $this->getLabelPrefix()) ? true : false;
    }

    public function setLabelPostfix($postfix)
    {
        if (!is_string($postfix) && !is_null($postfix)) {
            return;
        }
        $this->_labelPostfix = $postfix;

        return $this;
    }

    public function getLabelPostfix()
    {
        return $this->_labelPostfix;
    }

    public function hasLabelPostfix()
    {
        return (null !== $this->getLabelPostfix()) ? true : false;
    }

    public function setInputPrefix($prefix)
    {
        if (!is_string($prefix) && !is_null($prefix)) {
            return;
        }
        $this->_inputPrefix = $prefix;

        return $this;
    }

    public function getInputPrefix()
    {
        return $this->_inputPrefix;
    }

    public function hasInputPrefix()
    {
        return (null !== $this->getInputPrefix()) ? true : false;
    }

    public function setInputPostfix($postfix)
    {
        if (!is_string($postfix) && !is_null($postfix)) {
            return;
        }
        $this->_inputPostfix = $postfix;

        return $this;
    }

    public function getInputPostfix()
    {
        return $this->_inputPostfix;
    }

    public function hasInputPostfix()
    {
        return (null !== $this->getInputPostfix()) ? true : false;
    }
    
    public function setIsSetting ($flag = true)
    {
        if ((bool) $flag) {
            $value = Singular_Core::_('settings')->getSetting($this->getName());
            $this->setValue($value);
            $this->_isSetting = true;
        }
        
        return $this;
    }
    
    public function setIsUserSetting ($user)
    {        
        $value = Singular_Core::_('UserSettings')
            ->getValue($this->getName(), $user);

        $this->setValue($value);
        $this->_isUserSetting = true;
        
        return $this;
    }
    
    public function isSetting ()
    {
        return $this->_isSetting;
    }
    
    public function isUserSetting ()
    {
        return $this->_isUserSetting;
    }

    /**
     * Render form element
     *
     * @param  Zend_View_Interface $view
     * @return string
     */
    public function render (Zend_View_Interface $view = null)
    {
        $this->setAttrib('id', $this->getId());

        if ($this->_isPartialRendering) {
            return '';
        }

        if (null !== $view) {
            $this->setView($view);
        }

        $content = '';
        foreach ($this->getDecorators() as $decorator) {
            $decorator->setElement($this);
            $content = $decorator->render($content);
        }
        return $content;
    }

    protected function _initElementViewScriptDecorator ($name)
    {
        $requestEnv = (Singular_Core::_('Environment')->isAdminRequest()) ? 'admin' : 'frontend';
        $scriptPath = $requestEnv . "/form/elements/{$name}.phtml";
        $paths = $this->getView()->getScriptPaths();

        $scriptIsExists = false;
        foreach ($paths as $path) {
            if (file_exists($path . $scriptPath)) {
                $scriptIsExists = true;
            }
        }
        
        if ($scriptIsExists) {
            $this->addDecorators(
                array(
                     array('ViewScript', array(
                         'viewScript' => $scriptPath,
                         'placement' => false
                     )),
                     array('FormElements')
                )
            );
        }
    }
}