<?php
/**
 * Singular
 *
 * LICENSE
 *
 * The SOFTWARE PRODUCT is protected by copyright treaties,
 * as well as other intellectual property laws and treaties.
 * The SOFTWARE PRODUCT is licensed, not sold.
 * Software.
 * You may install and use one copy of the SOFTWARE PRODUCT
 * for each license purchased or if a site license then you
 * may install any number of copies at the site where the
 * site license is defined.
 * Your rights to use the SOFTWARE PRODUCT are not transferable.
 *
 * @category   Singular
 * @package    Singular_Inflector
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */

 /**
 * @category   Singular
 * @package    Singular_Inflector
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */
final class Singular_Inflector
{
    /**
     * Returns the given lower_case_and_underscored_word as a CamelCased word.
     *
     * @param string $lower_case_and_underscored_word Word to camelize
     * @return string Camelized word. LikeThis.
     * @static
     */
	public static function camelize ($lowerCaseAndUnderscoredWord)
    {
		return str_replace(' ', '', ucwords(str_replace('_', ' ', $lowerCaseAndUnderscoredWord)));
	}

    /**
     * Returns the given camelCasedWord as an underscored_word.
     *
     * @param string $camelCasedWord Camel-cased word to be "underscorized"
     * @return string Underscore-syntaxed version of the $camelCasedWord
     * @static
     */
	public static function underscore ($camelCasedWord)
    {
		return strtolower(preg_replace('/(?<=\\w)([A-Z])/', '_\\1', $camelCasedWord));
	}

    /**
     * Returns the given underscored_word_group as a Human Readable Word Group.
     * (Underscores are replaced by spaces and capitalized following words.)
     *
     * @param string $lower_case_and_underscored_word String to be made more readable
     * @return string Human-readable string
     * @static
     */
	public static function humanize ($lowerCaseAndUnderscoredWord)
    {
		return ucwords(str_replace('_', ' ', $lowerCaseAndUnderscoredWord));
	}

    /**
     * Returns the given 'Zend_Controller_Front' as a 'zend-controller-front'.
     *
     * @param string $classNameWord String to be made more readable
     * @return string Scaffold string
     * @static
     */
    public static function scaffold ($classNameWord)
    {
        return strtolower(str_replace('_', '-', $classNameWord));
    }
}