<?php

class Singular_Loader
{
    private static $_resources = array();

    private static $_options = array(
        'always_persistent' => false
    );

    public static function get ($className, $args = array())
    {
        return self::_load($className, $args, true);
    }

    public static function once ($className, $args = array())
    {
        return self::_load($className, $args, false);
    }

    public static function has ($resource)
    {
        return array_key_exists($resource, self::$_resources);
    }

    private static function _load ($className, $args, $persistent = false)
    {
        if (true === (bool) self::getOption('always_persistent')) {
            $persistent = true;
        }

        if ($persistent && self::has($className)) {
            return self::_extract($className);
        }

        if (!class_exists($className, true)) {
            //if (class_exists(Singular_Loader_Autoloader))
            //Zend_Loader::loadClass($className);
        }

        // case when class::__construct() has arguments
        if (is_array($args) && !empty($args)) {
            // make a reflection object
            $reflectionObj = new ReflectionClass($className);
            $class = $reflectionObj->newInstanceArgs($args);
        } else {
            $class = new $className();
        }

        if ($persistent) {
            self::_store($className, $class);
            $class = self::_extract($className);
        }

        return $class;
    }

    protected static function _store ($name, $var)
    {
        self::$_resources[$name] = $var;
    }

    protected static function _extract ($name)
    {
        return self::$_resources[$name];
    }

    public static function setOption ($option, $value)
    {
        if (array_key_exists($option, self::$_options)) {
            self::$_options[$option] = $value;
        }
    }

    public static function getOption ($option)
    {
        return (array_key_exists($option, self::$_options)) ? self::$_options[$option] : null;
    }
}
