<?php

class Singular_Module_Settings extends Singular_Module_ModuleClassExtender
{
    private $_settings = array();

    /**
     * Class constructor
     *
     * @param Singular_Module_Abstract $moduleClass
     */
    public function __construct (Singular_Module_Abstract $moduleClass)
    {
        $this->_setModuleClass($moduleClass);
    }

    /**
     * Adds new shortcut into section on settings panel
     *
     * @throws Singular_Module_Exception
     * @param string $name
     * @param Singular_Module_Settings_Container $container
     * @return void
     */
    public function addShortcut ($name, Singular_Module_Settings_Container $container)
    {
        if (!$container instanceof Singular_Module_Settings_Container) {
            throw new Singular_Module_Exception('Container must be an instance of Singular_Module_Settings_Container.');
        }
        if (is_numeric($name)) {
            throw new Singular_Module_Exception('Shortcut name must be not numeric.');
        }

        $container->setModuleClass($this->getModuleClass());
        $this->_settings[$name] = $container;
    }

    /**
     * Remove shortcut by given name
     *
     * @param string $name
     * @return void
     */
    public function removeShortcut ($name)
    {
        if ($this->hasShortCut($name)) {
            unset($this->_settings[$name]);
        }
    }

    /**
     * Remove all setting shortcuts for current module
     *
     * @return void
     */
    public function removeAllShortcuts ()
    {
        $this->_settings = array();
    }

    /**
     * Is a particular shortcut has set
     *
     * @param string $name
     * @return bool
     */
    public function hasShortcut ($name)
    {
        return isset($this->_settings[$name]);
    }

    /**
     * Checking for one or more shortcuts are exists
     *
     * @return bool
     */
    public function hasShortcuts ()
    {
        return !empty($this->_settings);
    }

    /**
     * Return shortcut as settings container by name
     * return null if container with given name is not exists
     *
     * @param string $name
     * @return Singular_Module_Settings_Container|null
     */
    public function getShortcut ($name)
    {
        return ($this->hasShortCut($name)) ? $this->_settings[$name] : null;
    }

    /**
     * Return all settings shortcuts for this module
     *
     * @return array
     */
    public function getAllShortcuts ()
    {
        return $this->_settings;
    }

    public function getShortcutNames ()
    {
        return array_keys($this->_settings);
    }
}
