<?php
 
class Singular_Module_Settings_SupplyBlock
{
    /**
     * @var array
     */
    private $_block = array();

    public function __construct()
    {
        $this->_initBlock();
    }

    private function _initBlock ()
    {
        $this->_block = array(
            'title'   => null,
            'text'    => null,
            'info'    => array(),
            'buttons' => array()
        );
    }

    public function __set ($name, $value)
    {
        if (!$this->_hasParam($name)) {
            throw new Singular_Module_Exception("Invalid block param \"{$name}\".");
        }

        $this->_block[$name] = $value;
    }

    public function __get ($name)
    {
        return ($this->_hasParam($name)) ? $this->_block[$name] : null;
    }

    protected function _hasParam ($key)
    {
        return array_key_exists($key, $this->_block);
    }

    public function setTitle ($value)
    {
        $this->title = $value;

        return $this;
    }

    public function getTitle ()
    {
        return $this->title;
    }

    public function setText ($value)
    {
        $this->text = $value;

        return $this;
    }

    public function getText ()
    {
        return $this->text;
    }

    public function setInfo ($value)
    {
        if (!is_array($value)) {
            throw new Singular_Module_Exception("Value must be an array. " . gettype($value) . ' was given.');
        }

        $this->info = $value;

        return $this;
    }

    public function addInfo ($param, $value = null)
    {
        if (is_array($param)) {
            $this->setInfo(array_merge($this->getAllInfo(), $param));
        } else {
            $this->_block['info'][$param] = $value;
        }

        return $this;
    }

    public function hasInfo ()
    {
        $info = $this->info;

        return !empty($info);
    }

    public function clearInfo ()
    {
        $this->info = array();

        return $this;
    }

    public function getAllInfo ()
    {
        return $this->info;
    }

    public function addButton ($name, $value, $mca = array(), $route = null, $params = array())
    {
        if (null === $route && !isset($mca['module'])) {
            throw new Singular_Module_Exception("Module, controller, action or route must be specified for button.");
        }

        if (!is_array($mca)) {
            $mca = array();
        }

        $router = Zend_Controller_Front::getInstance()->getRouter();

        $data = array(
            'name'         => 'settings-button-' . strtolower($name),
            'value'        => $value,
            'module'       => (isset($mca['module'])) ? $mca['module'] : null,
            'controller'   => (isset($mca['controller'])) ? $mca['controller'] : 'index',
            'action'       => (isset($mca['action'])) ? $mca['action'] : 'index',
            'route'        => $route,
            'params'       => $params
        );

        if (null !== $route) {
            $data['href'] = $router->assemble($params, $route, true);
        } else {
            $data['href'] = $router->assemble(array_merge($params, $mca), 'default');
        }

        $this->_block['buttons'][$name] = $data;

        return $this;
    }

    public function hasButtons ()
    {
        $buttons = $this->buttons;

        return !empty($buttons);
    }

    public function removeButton ($name)
    {
        if (isset($this->_block['buttons'][$name])) {
            unset($this->_block['buttons'][$name]);
        }
    }

    public function clearButtons ()
    {
        $this->buttons = array();
    }

    public function getAllButtons ()
    {
        return $this->buttons;
    }
}
