<?php

class Singular_Runtime
{
    const STORE_TYPE_LOCALLY = 1;
    
    const STORE_TYPE_BY_REGISTRY_REFERENCE = 2;
    
    const STORE_TYPE_IN_CACHE = 2;
    
    const DATA_KEY_OBJECT_NAME = 'name';
    
    const DATA_KEY_OBJECT_TYPE = 'type';
    
    const DATA_KEY_STORAGE_TYPE = 'storage_type';
    
    const DATA_KEY_REGISTRY_KEY = 'registry_key';
    
    const DATA_KEY_DATA = 'data';
    
    private static $_storage = array();

    private static $_dataArray = array(
        self::DATA_KEY_OBJECT_NAME     => null
        , self::DATA_KEY_OBJECT_TYPE    => null
        , self::DATA_KEY_STORAGE_TYPE => self::STORE_TYPE_LOCALLY
        , self::DATA_KEY_REGISTRY_KEY  => null
        , self::DATA_KEY_DATA                => null
    );
    
    public static function store ($name, $object, $inRegistry = false)
    {
        if ((is_string($name)) && (trim($name) != '')) {
            self::_storeObject($name, $object, $inRegistry);
        } else {
            throw new Singular_Runtime_Exception("Incorrect object name \"{$name}\"");
        }
    }
    
    protected static function _storeObject ($name, $object, $inRegistry = false)
    {
        $data = self::$_dataArray;
        $data[self::DATA_KEY_OBJECT_NAME] = $name;
        $data[self::DATA_KEY_OBJECT_TYPE] = gettype($object);
        
        if (true === (bool) $inRegistry) {
            Zend_Registry::set($name, $object);
            $data[self::DATA_KEY_STORAGE_TYPE] = self::STORE_TYPE_BY_REGISTRY_REFERENCE;
            $data[self::DATA_KEY_REGISTRY_KEY] = $name;
        } else {
            $data[self::DATA_KEY_STORAGE_TYPE] = self::STORE_TYPE_LOCALLY;
            $data[self::DATA_KEY_DATA] = $object;
        }
        
        self::$_storage[$name] = $data;
    }
    
    public static function extract ($name, $full = false)
    {
        return self::_extractElement($name, $full);
    }

    public static function remove ($name)
    {
        if (self::inStorage($name)) {
            unset(self::$_storage[$name]);
            return true;
        }
        return false;
    }
    
    protected static function _extractElement ($name, $full = false)
    {
        if (self::inStorage($name)) {
            $obj = self::$_storage[$name];
            if (self::STORE_TYPE_BY_REGISTRY_REFERENCE
                == $obj[self::DATA_KEY_STORAGE_TYPE]) {
                $obj[self::DATA_KEY_DATA] = Zend_Registry::get($name);
            }
            return (true === (bool) $full)
                ? $obj
                : $obj[self::DATA_KEY_DATA];
        }

        return null;
    }
    
    public static function inStorage ($name)
    {
        if (array_key_exists($name, self::$_storage)) {
            return true;
        }
        return false;
    }
}
