<?php
/**
 * Extended ArrayIterator class
 *
 * This iterator allows to unset and modify values and keys while iterating
 * over Arrays and Objects.
 *
 * @link http://php.net/manual/en/class.arrayiterator.php
 */
class Singular_Stdlib_ArrayIterator extends ArrayIterator
{
    /**
     * Previous key
     *
     * @var mixed
     */
    private $_previousKey;

    /**
     * Interanl numeric position
     *
     * @var int
     */
    private $_position = 0;

    /**
     * Construct an ArrayIterator
     *
     * @link http://php.net/manual/en/arrayiterator.construct.php
     *
     * @param array $array The array or object to be iterated on.
     * @param int   $flags Flags to control the behaviour of the ArrayObject object.
     *
     * @see  ArrayObject::setFlags()
     */
    public function __construct($array = array(), $flags = 0)
    {
        parent::__construct($array, $flags = 0);
        $this->_previousKey = null;
    }

    /**
     * Move to next entry
     *
     * @link http://php.net/manual/en/arrayiterator.next.php
     *
     * @return void
     */
    public function next()
    {
        $this->_previousKey = $this->key();
        $this->_position++;
        parent::next();
    }

    /**
     * Move to previous entry
     *
     * @return void
     */
    public function prev()
    {
        if (null !== $this->_previousKey) {
            $this->_position--;
            $this->seek($this->_position);
        }
    }

    /**
     * Rewind array back to the start
     *
     * @link http://php.net/manual/en/arrayiterator.rewind.php
     *
     * @return void
     */
    public function rewind()
    {
        parent::rewind();
        $this->_previousKey = null;
        $this->_position = 0;
    }

    /**
     * Returns true if current entry is last
     *
     * @return bool
     */
    public function isLast()
    {
        $this->next();
        $valid = $this->valid();
        if ($valid) {
            $this->prev();
        }

        return !$valid;
    }

    /**
     * Returns true if current entry is first
     *
     * @return bool
     */
    public function isFirst()
    {
        return null === $this->_previousKey;
    }
}