<?php

class Singular_Taxonomy_Cluster_Data
{
    private $_data = array();

    public function __construct($data = array(), $fromXml = false)
    {
        $dataArray = null;
        
        if (is_array($data)) {
            $dataArray = $data;
        } elseif ($data instanceof Zend_Config) {
            $dataArray = $data->toArray();
        } elseif (is_string($data) && true === (bool) $fromXml) {
            $xml = new Zend_Config_Xml($data);
            $dataArray = $xml->toArray();
        } else {
            throw new Singular_Taxonomy_Exception('Invalid data type in ' .  __CLASS__ . '::__construct(). Data must be an instance of Zend_Config, array type or xml string. ' . gettype($data) . ' was given.');
        }

        if (is_array($dataArray)) {
            $this->fromArray($dataArray);
        }
    }

    public function __set ($name, $value)
    {
        $this->set($name, $value);
    }

    public function __get ($name)
    {
        return $this->get($name);
    }

    public function __isset($name)
    {
        return $this->is_set($name);
    }

    public function __unset($name)
    {
        if (isset($this->_data[$name])) {
            unset($this->_data[$name]);
        }
    }

    public function is_set ($name)
    {
        if (isset($this->_data[$name])) {
            return true;
        }

        return false;
    }

    public function get ($name)
    {
        if (!isset($this->_data[$name])) {
            $this->_data[$name] = new self();
        }

        return $this->_data[$name];
    }

    public function set ($name, $value)
    {
        $this->_data[$name] = $value;
    }

    public function isEmpty ()
    {
        if (empty($this->_data)) {
            return true;
        }

        return false;
    }

    public function clear ()
    {
        $this->_data = array();
    }

    public function fromArray ($data, $clear = false)
    {
        if (true === (bool) $clear) {
            $this->clear();
        }
        
        if (is_array($data) && !empty($data)) {
            foreach ($data as $key => $value) {
                if (is_array($value)) {
                    $value = new self($value);
                }

                $currNode = null;
                
                if ($this->is_set($key)) {
                    $currNode = $this->get($key);
                }
                
                $this->set($key, $value);
                
                if ($currNode instanceof self) {
                    $currNode = $currNode->toArray();

                    foreach ($currNode as $currNodeKey => $currNodeVal) {
                        if (!isset($this->$key->$currNodeKey)) {
                            $this->$key->$currNodeKey = $currNodeVal;
                        }
                    }
                }
                
                unset($currNode);
            }
        }
    }

    public function toArray ()
    {
        $arrayData = array();
        foreach ($this->_data as $key => $val) {
            $arrayData[$key] = ($val instanceof self)
                    ? $val->toArray()
                    : $val ;
        }

        return $arrayData;
    }

    public function toXml ()
    {
        $arrayData = $this->toArray();
        $dataConfig = new Zend_Config($arrayData);
        unset($arrayData);
        $writer = new Singular_Taxonomy_Cluster_Writer_Xml(
            array(
                 'config' => $dataConfig,
            )
        );

        return $writer->render();
    }
}
