<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Update
 * @subpackage Driver
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Loader_Autoloader_Resource
 * @category   Singular
 * @package    Singular_Update
 * @subpackage Driver
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
abstract class Singular_Update_Driver
{
    protected $_requestData = array();

    /**
     * Class constructor
     *
     * @param array $options
     */
    public function __construct ($options = array())
    {
        $this->init($options);
    }

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @abstract
     * @param array $options
     * @return void
     */
    abstract public function init ($options = array());

    /**
     * @param  string $key
     * @param  string|array|int $value
     * @return Singular_Update_Driver
     * @throws Singular_Update_Exception
     */
    public function setRequestKey ($key, $value)
    {
        if (!is_string($key)) {
            throw new Singular_Update_Exception(
                'Invalid argument: $key must be an string'
            );
        }

        if (!is_string($value) && !is_array($value) && !is_int($value)) {
            throw new Singular_Update_Exception(
                'Invalid argument: $value must be an string, array or integer'
            );
        }

        $this->_requestData[$key] = $value;

        return $this;
    }

    /**
     * Gets request key by key name
     *
     * @param  string $key
     * @return null
     */
    public function getRequestKey ($key)
    {
        return (array_key_exists($key, $this->_requestData)) ? $this->_requestData[$key] : null;
    }

    /**
     * Gets request collection of keys as array
     *
     * @return array
     */
    public function getRequestData ()
    {
        return $this->_requestData;
    }
}
