<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Update
 * @subpackage Driver
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @uses       Zend_Loader_Autoloader_Resource
 * @category   Singular
 * @package    Singular_Update
 * @subpackage Driver
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Update_Driver_Modules extends Singular_Update_Driver
{
    /**
     * Modules for checking
     *
     * @var array
     */
    private $_modulesToCheck = array();

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @abstract
     * @param array $options
     * @return void
     */
    public function init ($options = array())
    {
        $modules = array();
        $installedModules = Singular_Module::getInstance()->getInstalledModules();

        if (isset($options['modules'])) {
            if (is_array($options['modules']) && !empty($options['modules'])) {
                $modules = $options['modules'];
            } elseif (is_string($options['modules'])) {
                $modules = (array) $options['modules'];
            }
        } else {
            $modules = array_keys($installedModules);
        }

        /** collecting modules */
        foreach ($modules as $module) {
            if (isset($installedModules[$module])) {
                $this->_addModuleToCheck($module, $installedModules[$module]);
            }
        }

        /** add modules to request */
        $_modules = $this->_getAllModulesToCheck();
        if (!empty($_modules)) {
            $this->setRequestKey('modules', $_modules);
        }
    }

    /**
     * Gets all collected modules info
     *
     * @return array
     */
    protected function _getAllModulesToCheck ()
    {
        return $this->_modulesToCheck;
    }

    /**
     * Adds module info to check
     *
     * @param  string $name
     * @param  Singular_Module_Abstract $instance
     * @throws Singular_Update_Exception
     */
    protected function _addModuleToCheck ($name, Singular_Module_Abstract $instance)
    {
        if (!$instance instanceof Singular_Module_Abstract) {
            throw new Singular_Update_Exception(
                'Invalid argument: $instance must be an instance of Singular_Module_Abstract'
            );
        }

        $name = strtolower($name);

        /** module info array */
        $data = array(
            'name'    => $name,
            'version' => $instance->getContainer()->getVersion()
        );

        $this->_modulesToCheck[$name] = $data;
    }
}
