<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    System_Validate
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * @category   Singular
 * @package    System_Validate
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Validate_UniqueRow extends Zend_Validate_Abstract
{
    /**
     * Message template name
     */
    const ROW_EXISTS = 'rowExists';

    /**
     * Message templates
     * 
     * @var array
     */
    protected $_messageTemplates = array(
        self::ROW_EXISTS => "is already in use."
    );
    
    /**
     * Db Table
     * 
     * @var Singular_Db_Table_Abstract
     */
    protected $_dbTable = null;
    
    /**
     * Unique field
     * 
     * @var string
     */
    protected $_field = null;
    
    /**
     * Old value
     * 
     * @var mixed
     */
    protected $_old = null;
    
    /**
     * Override construct
     * 
     * @param Singular_Db_Table_Abstract $dbTable
     * @param string $field 
     * @param mixed $old
     * @return void
     */
    public function __construct (
        Singular_Db_Table_Abstract $dbTable, $field, $old = null)
    {
        $this->_dbTable = $dbTable;
        $this->_field = $field;
        $this->_old = $old;
    }

    /**
     * Valid method
     * 
     * @see Zend_Validate_Interface
     * @param mixed $value
     * @return bool 
     */
    public function isValid ($value)
    {
        $this->_setValue((string) $value);
        
        if ($this->_old === $value) {
            return true;
        }
        
        try {
            
            $row = $this->_dbTable->getRowByField($this->_field, $value);
            
            if (null === $row) {
                return true;
            }
            
            $this->_error(self::ROW_EXISTS);
            return false;
            
        } catch (Exception $e) {
            throw new Exception('No row should exist on signup.');
        }

        return true;
    }
}
