<?php

class Singular_Vendor
{
    /**
     * @var null|array
     */
    private static $_vendors = null;

    /**
     * @static
     * @return void
     */
    protected static function _initVendors ()
    {
        $vendorPath = Singular_Core::_('Config')->directories->vendors;
        $vendorDir = new DirectoryIterator($vendorPath);
        
        /** @var $vendor DirectoryIterator */
        foreach ($vendorDir as $vendor) {
            if ($vendor->isFile()) {
                $name = $vendor->getBasename('.php');
                if (strpos($name, '.') !== false) {
                    continue;
                }
                $hasDir = (file_exists($vendorPath . DS . $name)) ? true : false;
                $options = array(
                    'name' => $name,
                    'plug_filename' => $vendor->getFileInfo()->getFilename(),
                    'plug_file_path' => $vendor->getPathname(),
                    'has_dir' => $hasDir
                );
                self::addVendor($name, $options);
            }
        }
    }

    /**
     * @static
     * @param string $vendor
     * @param array $options
     * @return void
     */
    public static function addVendor ($vendor, $options)
    {
        if (null === self::$_vendors) {
            self::$_vendors = array();
        }
        self::$_vendors[$vendor] = new Singular_Vendor_Container($options);
    }

    /**
     * @static
     * @throws Singular_Vendor_Exception
     * @param string $vendor
     * @return Singular_Vendor_Container
     */
    public static function getVendor ($vendor)
    {
        if (self::isExists($vendor)) {
            return self::$_vendors[$vendor];
        }
        
        $vendorPath = Singular_Core::_('Config')->directories->vendors;
        throw new Singular_Vendor_Exception("Vendor \"{$vendor}\" wasn't found in {$vendorPath}");
    }

    /**
     * @static
     * @param string $vendor
     * @return bool
     */
    public static function isExists ($vendor)
    {
        if (array_key_exists($vendor, self::$_vendors)) {
            return true;
        }
        return false;
    }

    /**
     * @static
     * @throws Singular_Vendor_Exception
     * @param string $vendor
     * @param null|array $config
     * @param bool $setInstance
     * @return object
     */
    public static function load ($vendor, $config = null, $setInstance = false)
    {
        if (null === self::$_vendors) {
            self::_initVendors();
        }
        
        if (self::isExists($vendor)) {
            /** @var $container Singular_Vendor_Container */
            $container = self::$_vendors[$vendor];

            return $container->run($config, $setInstance);
        }
        
        $vendorPath = Singular_Core::_('Config')->directories->vendors;
        throw new Singular_Vendor_Exception("Vendor \"{$vendor}\" wasn't found in {$vendorPath}");
    }
}
