<?php

class Singular_Vendor_Container
{
    private $_instance = null;

    private $_name = null;

    private $_plugFilename = null;

    private $_plugFilePath = null;

    private $_hasDir = false;

    protected $_hasRun = false;

    public function __construct($options)
    {
        foreach ($options as $option => $value) {
            switch ($option) {
                case 'name':
                        $this->setName($value);
                    break;
                case 'plug_filename':
                    $this->setPlugFilename($value);
                    break;
                case 'plug_file_path':
                    $this->setPlugFilePath($value);
                    break;
                case 'has_dir':
                    $this->setHasDir($value);
                    break;
            }
        }
    }

    public function setHasDir ($hasDir)
    {
        $this->_hasDir = $hasDir;
    }

    public function getHasDir ()
    {
        return $this->_hasDir;
    }

    public function setName ($name)
    {
        $this->_name = $name;
    }

    public function getName ()
    {
        return $this->_name;
    }

    public function setPlugFilePath ($plugFilePath)
    {
        $this->_plugFilePath = $plugFilePath;
    }

    public function getPlugFilePath ()
    {
        return $this->_plugFilePath;
    }

    public function setPlugFilename ($plugFilename)
    {
        $this->_plugFilename = $plugFilename;
    }

    public function getPlugFilename ()
    {
        return $this->_plugFilename;
    }

    public function getVendorDirectoryPath ()
    {
        if (true === $this->getHasDir()) {
            $dirPath = str_replace($this->getPlugFilename(),
                                   $this->getName(),
                                   $this->getPlugFilePath());
            return $dirPath;
        }
    }

    public function getVendorDirectoryBasePath ()
    {
        $rootPath = str_replace('\\', '/', ROOT_PATH);
        $vendorPath = str_replace('\\', '/', $this->getVendorDirectoryPath());
        return str_replace($rootPath, '', $vendorPath);
    }

    public function setInstance ($instance)
    {
        if (!is_object($instance)) {
            throw new Singular_Vendor_Exception('Instance must be an object, "' . gettype($instance) . '" was given.');
        }
        $this->_instance = $instance;
    }

    public function getInstance ()
    {
        if (null === $this->_instance) {
            $name = $this->getName();
            throw new Singular_Vendor_Exception("Vendor \"{$name}\" wasn't initialized.");
        }
        return $this->_instance;
    }

    public function run ($config = null, $setInstance = false)
    {
        $class = $this->getName();
        
        if (false === $this->_hasRun) {
            require_once $this->getPlugFilePath();

            if (!class_exists($class)) {
                throw new Singular_Vendor_Exception("Class \"{$class}\" wasn't found in " . $this->getPlugFilePath());
            }
        }
        
        $this->_hasRun = true;

        $instance = new $class($config);

        if (true === (bool) $setInstance) {
            $this->setInstance($instance);
            return $this->getInstance();
        }

        return $instance;
    }
}
