<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_View
 * @subpackage Helper
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * Settings core resource accessor
 *
 * @category   Singular
 * @package    Singular_View
 * @subpackage Helper
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_View_Helper_DateTime extends Zend_View_Helper_Abstract
{
    /**
     * Constants
     */
    const YEAR    = 'YEAR';
    const MONTH   = 'MONTH';
    const WEEK    = 'WEEK';
    const DAY     = 'DAY';
    const HOUR    = 'HOUR';
    const MINUTE  = 'MINUTE';
    const SECOND  = 'SECOND';
    const YEARS   = 'YEARS';
    const MONTHS  = 'MONTHS';
    const WEEKS   = 'WEEKS';
    const DAYS    = 'DAYS';
    const HOURS   = 'HOURS';
    const MINUTES = 'MINUTES';
    const SECONDS = 'SECONDS';
    
    /**
     * Templates
     * 
     * @var array
     */
    protected $_unitTemplates = array(
        self::YEAR    => '%value% year ago',
        self::YEARS   => '%value% years ago',
        self::MONTH   => '%value% month ago',
        self::MONTHS  => '%value% months ago',
        self::WEEK    => '%value% week ago',
        self::WEEKS   => '%value% weeks ago',
        self::DAY     => '%value% day ago',
        self::DAYS    => '%value% days ago',
        self::HOUR    => '%value% hour ago',
        self::HOURS   => '%value% hours ago',
        self::MINUTE  => '%value% min ago',
        self::MINUTES => '%value% mins ago',
        self::SECOND  => '%value% sec ago',
        self::SECONDS => '%value% secs ago',
    );
    
    /**
     * Proxy method
     * 
     * @return Singular_View_Helper_DateTime
     */
    public function dateTime ()
    {
        return $this;
    }
    
    /**
     * Format date time
     * 
     * @param  null|string|Singular_Date $date
     * @param  string $format
     * @param  string $outputFormat
     * @return string
     */
    public function format (
        $date = null, $format = null, $outputFormat = null)
    {
        if (null === $format) {
            $format = Singular_Date::FORMAT_MYSQL_DATETIME;
        }
        if (is_string($date)) {
            if ($this->isNeverDate($date)) {
                return $this->view->t('Never');
            }
            $date = new Singular_Date($date, $format);
        } else if (null === $date) {
            $date = Singular_Date::now();
        }
        
        $user = null;
        if (Singular_Runtime::inStorage('logged_user')) {
            $user = Singular_Runtime::extract('logged_user');
        }
        $timezone = Singular_Core::_('UserSettings')
            ->getValue('site_default_timezone', $user);
        
        $date->setTimezone($timezone);
        
        if (null === $outputFormat) {
            $outputFormat = Singular_Core::_('UserSettings')
                ->getValue('datetime_format', $user);
        }

        return $date->toString($outputFormat);
    }
    
    /**
     * Relative
     *
     * @param  null|string|Singular_Date $date
     * @param  string $format
     * @return string
     */
    public function relative (
        $date = null,
        $format = Singular_Date::FORMAT_MYSQL_DATETIME,
        $outputFormat = null
    ) {
        if (is_string($date)) {
            $date = new Singular_Date($date, $format);
        } else if (null === $date) {
            $date = Singular_Date::now();
        }
        
        $user = null;
        if (Singular_Runtime::inStorage('logged_user')) {
            $user = Singular_Runtime::extract('logged_user');
        }
        $timezone = Singular_Core::_('UserSettings')
            ->getValue('site_default_timezone', $user);
        
        if (null === $outputFormat) {
            $outputFormat = Singular_Core::_('UserSettings')
                ->getValue('datetime_format', $user);
        }

        $now = new Singular_Date();
        
        $date->setTimezone($timezone);
        $now->setTimezone($timezone);
        
        $diff = $now->sub($date);
        $mt = new Zend_Measure_Time($diff);
        $units = $mt->getConversionList();
        
        $chunks = array(
            Zend_Measure_Time::YEAR,
            Zend_Measure_Time::MONTH,
            Zend_Measure_Time::WEEK,
            Zend_Measure_Time::DAY,
            Zend_Measure_Time::HOUR,
            Zend_Measure_Time::MINUTE,
            Zend_Measure_Time::SECOND
        );
        
        for ($i = 0, $count = count($chunks); $i < $count; ++ $i) {
            $seconds = $units[$chunks[$i]][0];
            $unitKey = $chunks[$i];
            if (0.0 !==
             ($result = floor($diff->get(Zend_Date::TIMESTAMP) / $seconds))) {
                break;
            }
        }
        
        $translateHelper = new Zend_View_Helper_Translate();
        if ($result === (float) 1) {
            $formatedString = $translateHelper->translate(
            $this->getUnitTemplate($unitKey));
        } else {
            $formatedString = $translateHelper->translate(
            $this->getUnitTemplate($unitKey . 'S'));
        }
        
        $formatedString = str_replace('%value%', (string) $result,
        $formatedString);
        
        return "<span class=\"ago\" title=\"{$date->toString($outputFormat)}\">{$formatedString}</span>" . PHP_EOL;
    }
    
    /**
     * Set unit template
     * 
     * @param string $template
     * @param string $unitKey
     * @return void
     */
    public function setUnitTemplate ($template, $unitKey)
    {
        if (! isset($this->_unitTemplates[$unitKey])) {
            throw new Zend_Validate_Exception(
            "No unit template exists for key '$unitKey'");
        }
        $this->_unitTemplates[$unitKey] = $template;
    }
    
    /**
     * Set unit templates
     * 
     * @param  array $templates
     * @return void
     */
    public function setUnitTemplates (array $templates)
    {
        foreach ($templates as $key => $template) {
            $this->setUnitTemplate($template, $key);
        }
    }
    
    /**
     * Get all templates
     * 
     * @return array
     */
    public function getUnitTemplates ()
    {
        return $this->_unitTemplates;
    }
    
    /**
     * Get unit template
     * 
     * @param  string $unitKey
     * @return string
     */
    public function getUnitTemplate ($unitKey)
    {
        if (!isset($this->_unitTemplates[$unitKey])) {
            throw new Singular_Exception(
                "No unit template exists for key '$unitKey'"
            );
        }
        
        return $this->_unitTemplates[$unitKey];
    }

    /**
     * Checks wherever date is 0000-00-00 00:00:00
     * 
     * @param  string $date
     * @return bool
     */
    public function isNeverDate ($date)
    {
        if (is_string($date)) {
            $filter = new Zend_Filter_Alnum(false);
            if ('00000000000000' == $filter->filter($date)) {
                return true;
            }
        }

        return false;
    }
}
