<?php
/**
 * Singular
 *
 * LICENSE
 *
 * The SOFTWARE PRODUCT is protected by copyright treaties,
 * as well as other intellectual property laws and treaties.
 * The SOFTWARE PRODUCT is licensed, not sold.
 * Software.
 * You may install and use one copy of the SOFTWARE PRODUCT
 * for each license purchased or if a site license then you
 * may install any number of copies at the site where the
 * site license is defined.
 * Your rights to use the SOFTWARE PRODUCT are not transferable.
 *
 * @category   Singular
 * @package    Singular_View
 * @subpackage Helper
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */

 /**
 * @category   Singular
 * @package    Singular_View
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */
class Singular_View_Helper_HeadLink extends Zend_View_Helper_HeadLink
{
    /**
     * Internal option, combine disabled
     *
     * @var    boolean
     * @static
     */
    protected static $_combineDisabled = true;

    /**
     * Internal option, minify disabled
     *
     * @var    boolean
     * @static
     */
    protected static $_minifyDisabled = true;
    
     /**
     * Internal option, print source path disabled
     *
     * @var    boolean
     * @static
     */
    protected static $_printSourcePathDisabled = true;

    /**
     * Combine inline css flag
     *
     * @var bool
     */
    protected static $_combineInlineCss = true;

    /**
     * Internal option, correction for stylesheet resource path
     *
     * @var    boolean
     * @static
     */
    protected static $_stylesheetResourcePathCorrectionDisabled = true;
    
    /**
     * Internal option, cache direcory
     *
     * @var    string
     * @static
     */
    protected static $_cacheDir = '';

    /**
     * Static list
     *
     * @var array
     */
    protected $_static = array();

    /**
     * Base URL
     *
     * @var string
     */
    protected $_baseUrl = '';
    
    /**
     * Allowed extensions
     *
     * @var array
     */
    protected $_cExtensions = array('css');

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        $this->_baseUrl = Zend_Controller_Front::getInstance()->getBaseUrl();
    }

    /**
     * Retrieve string representation
     *
     * @param  string|int $indent
     * @return string
     */
    public function toString ($indent = null)
    {
        return !self::$_combineDisabled ? $this->_toString($indent) : parent::toString($indent);
    }

    /**
     * Retrieve string representation
     *
     * @return string
     */
    protected function _toString ($indent = null)
    {
        /// is not set cache directory
        $indent = !$indent ? $this->getWhitespace($indent) : $this->getIndent();

        $items = array();
        $this->getContainer()->ksort();
        foreach ($this as $item) {
            if (!$this->_isValid($item)) {
                continue;
            }
            if (!$this->_isCachable($item)) {
                $items[] = $this->itemToString($item);
            } else {
                $this->_cache($item);
            }
        }

        array_unshift($items, $this->itemToString($this->_getCombinedItem()));
        return $indent . implode($this->_escape($this->getSeparator()) . $indent, $items);
    }

    /**
     * Is the script provided cachable?
     *
     * @param  mixed $value
     * @return bool
     */
    protected function _isCachable ($item)
    {
        if (isset($item->conditionalStylesheet)
            && !empty($item->conditionalStylesheet)
            && is_string($item->conditionalStylesheet))
        {
            return false;
        }

        if (!isset($item->href) || empty($item->href)) {
            return false;
        }

        return true;
    }

    /**
     * Add to cachable list
     */
    protected function _cache ($item)
    {
        $file = ROOT_PATH . str_replace($this->_baseUrl, '', $item->href);
        $href = str_replace($this->_baseUrl, '', $item->href);
        list( , $ext) = explode('.', $href);
        
        if (in_array($ext, $this->_cExtensions)) {
            $this->_static[] = array('file' => $file, 'mtime' => filemtime($file));
        }
    }

    /**
     * Retrieve scripts file path
     *
     * @return stdClass
     */
    protected function _getCombinedItem ()
    {
        $content = '';
        $isMinify = !self::$_minifyDisabled ? '_minify' : '';
        $filename = self::$_cacheDir . '/' . md5(serialize($this->_static)) . $isMinify . '.css';
        
        if (!is_file($filename) || !is_readable($filename)) {
            $contentArray = array();
            foreach ($this->_static as $key=>$file) {
                $contentArray[$key] = '';
                if (is_array($file)) {
                    if (!self::$_printSourcePathDisabled) {
                        $contentArray[$key] .= "\n\n/** File: " . substr($file['file'], strlen(ROOT_PATH) + 1) . " **/\n\n";
                    }
                    
                    $contentArray[$key] .= file_get_contents($file['file']);
                    
                    if (!self::$_stylesheetResourcePathCorrectionDisabled) {
                        $pathFilter = new Singular_Filter_Paths_Styles();
                        $pathFilter->setPath($file['file']);
                        $contentArray[$key] = $pathFilter->filter($contentArray[$key]);
                    }
                } else {
                    $contentArray[$key] .= "\n\n/** Inline! **/\n\n";
                    $contentArray[$key] .= $file;
                }
            }
            $content = implode(PHP_EOL, $contentArray);
            if (!self::$_minifyDisabled) {
                $filter = new Singular_Filter_Minify_Styles();
                $content = $filter->filter($content);
            }

            file_put_contents($filename, $content, LOCK_EX);
        }

        $href = str_replace(ROOT_PATH, $this->_baseUrl, $filename);
        return $this->createDataStylesheet(array('href' => $href));
    }

    /**
     * Disables the combine static
     *
     * @param unknown_type $flag
     */
    public static function disableCombine ($flag = true)
    {
        self::$_combineDisabled = (boolean) $flag;
    }

    /**
     * Disables the minify static
     *
     * @param unknown_type $flag
     */
    public static function disableMinify ($flag = true)
    {
        self::$_minifyDisabled = (bool) $flag;
    }
    
    /**
     * Disables print source path inside combined file
     *
     * @param bool $flag
     */
    public static function disablePrintSourcePath ($flag = true)
    {
        self::$_printSourcePathDisabled = (bool) $flag;
    }

    /**
     * Disables inline css combining
     *
     * @static
     * @param bool $flag
     */
    public static function disableCombineInlineCss ($flag = true)
    {
        self::$_combineInlineCss = (bool) !$flag;
    }
    
    /**
     * Disables stylesheet resource path correction
     *
     * @param bool $flag
     */
    public static function disableStylesheetResourcePathCorrection ($flag = true)
    {
        self::$_stylesheetResourcePathCorrectionDisabled = (bool) $flag;
    }
    
    /**
     * Set cache direcory
     *
     * @param string $cacheDir
     */
    public static function setCacheDir ($cacheDir)
    {
        if (!is_dir($cacheDir) || !is_writable($cacheDir)) {
            throw new Singular_Exception('Direcory "' . $cacheDir . '" does not exist or do not have permission to write.');
        }
        self::$_cacheDir = rtrim((string) $cacheDir, '/');
    }
}
