<?php
/**
 * Singular
 *
 * LICENSE
 *
 * The SOFTWARE PRODUCT is protected by copyright treaties,
 * as well as other intellectual property laws and treaties.
 * The SOFTWARE PRODUCT is licensed, not sold.
 * Software.
 * You may install and use one copy of the SOFTWARE PRODUCT
 * for each license purchased or if a site license then you
 * may install any number of copies at the site where the
 * site license is defined.
 * Your rights to use the SOFTWARE PRODUCT are not transferable.
 *
 * @category   Singular
 * @package    Singular_View
 * @subpackage Helper
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */

 /**
 * @category   Singular
 * @package    Singular_View
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */
class Singular_View_Helper_HeadScript extends Zend_View_Helper_HeadScript
{
    /**
     * Internal option, combine disabled
     *
     * @var    boolean
     * @static
     */
    protected static $_combineDisabled = true;

    /**
     * Internal option, minify disabled
     *
     * @var    boolean
     * @static
     */
    protected static $_minifyDisabled = true;
    
     /**
     * Internal option, print source path disabled
     *
     * @var    boolean
     * @static
     */
    protected static $_printSourcePathDisabled = true;

    /**
     * Combine inline js flag
     *
     * @var bool
     */
    protected static $_combineInlineJs = true;

    /**
     * Internal option, cache direcory
     *
     * @var    string
     * @static
     */
    protected static $_cacheDir = '';

    /**
     * Static list
     *
     * @var array
     */
    protected $_static = array();

    /**
     * Base URL
     *
     * @var string
     */
    protected $_baseUrl = '';
    
    /**
     * Allowed extensions
     *
     * @var array
     */
    protected $_cExtensions = array('js');

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        $this->_baseUrl = Zend_Controller_Front::getInstance()->getBaseUrl();
    }

    /**
     * Retrieve string representation
     *
     * @param  string|int $indent
     * @return string
     */
    public function toString ($indent = null)
    {
        return !self::$_combineDisabled ? $this->_toString($indent) : parent::toString($indent);
    }

    /**
     * Retrieve string representation
     *
     * @return string
     */
    protected function _toString ($indent = null)
    {
        /// is not set cache directory
        $indent = !$indent ? $this->getWhitespace($indent) : $this->getIndent();

        $useCdata = false;
        if ($this->view) {
            $useCdata = $this->view->doctype()->isXhtml() ? true : false;
        } else {
            $useCdata = $this->useCdata ? true : false;
        }

        $escapeStart = ($useCdata) ? '//<![CDATA[' : '//<!--';
        $escapeEnd   = ($useCdata) ? '//]]>'       : '//-->';

        $items = array();

        $this->getContainer()->ksort();
        foreach ($this as $item) {
            if (!$this->_isValid($item)) {
                continue;
            }
            if (!$this->_isCachable($item)) {
                $items[] = $this->itemToString($item, $indent, $escapeStart, $escapeEnd);
            } else {
                $this->_cache($item);
            }
        }

        array_unshift($items, $this->itemToString($this->_getCombinedItem(), $indent, $escapeStart, $escapeEnd));

        if (!self::$_combineInlineJs) {
            $inline = '';
            foreach ($this->_static as $file) {
                if (null != $file['source']) $inline .= $file['source'] . PHP_EOL;
            }
            if ('' != $inline) {
                $items[] = $this->itemToString(
                    $this->createData('text/javascript', array(), $inline),
                    $indent, $escapeStart, $escapeEnd
                );
            }
        }

        return implode($this->getSeparator(), $items);
    }

    /**
     * Is the script provided cachable?
     *
     * @param  mixed $item
     * @return bool
     */
    protected function _isCachable ($item)
    {
        if (isset($item->attributes['conditional'])
            && !empty($item->attributes['conditional'])
            && is_string($item->attributes['conditional']))
        {
            return false;
        }

        if (!empty($item->source) && false === strpos($item->source, '//@non-cache')) {
            return true;
        }

        if (!isset($item->attributes['src']) || empty($item->attributes['src'])) {
            return false;
        }

        return true;
    }

    /**
     * Adds js resource to cachable lists
     *
     * @param  stdClass $item
     * @return void
     */
    protected function _cache (stdClass $item)
    {
        $script = array(
            'src'    => null,
            'source' => null,
            'mtime'  => null
        );

        if (isset($item->attributes['src'])) {
            $file = ROOT_PATH . str_replace($this->_baseUrl, '', $item->attributes['src']);
            $script['mtime'] = filemtime($file);
            $script['src']  = $file;
        } elseif (isset($item->source)) {
            $script['source'] = $item->source;
        }

        if (null === $script['src'] && null === $script['source']) {
            return;
        }

        $this->_static[] = $script;
    }

    /**
     * Retrieve scripts file path
     *
     * @return stdClass
     */
    protected function _getCombinedItem ()
    {
        $content  = '';
        $isMinify = !self::$_minifyDisabled ? '_minify' : '';

        $files = $this->_static;
        if (!self::$_combineInlineJs) {
            foreach ($files as $idx => $file) {
                if (null != $file['source']) unset($files[$idx]);
            }
        }

        $filename = self::$_cacheDir . '/' . md5(serialize($files)) . $isMinify . '.js';
        
        if (!is_file($filename) || !is_readable($filename)) {
            foreach ($files as $file) {
                if (is_array($file)) {
                    if (null != $file['src']) {
                        if (!self::$_printSourcePathDisabled) {
                            $content .= "\n\n/** File: " . substr($file['src'], strlen(ROOT_PATH) + 1) . " **/\n\n";
                        }
                        $content .= file_get_contents($file['src']);
                    } elseif (null != $file['source']) {
                        $content .= "\n\n/** Inline! **/\n\n";
                        $content .= $file['source'];
                    }
                } else {
                    $content .= "\n\n/** Inline! **/\n\n";
                    $content .= $file;
                }
            }

            if (!self::$_minifyDisabled) {
                $filter = new Singular_Filter_Minify_Scripts();
                $content = $filter->filter($content);
            }
            file_put_contents($filename, $content, LOCK_EX);
        }

        unset($files);

        $src  = str_replace(ROOT_PATH, $this->_baseUrl, $filename);
        $data = $this->createData('text/javascript', array('src' => $src));

        return $data;
    }

    /**
     * Disables the combine static
     *
     * @param unknown_type $flag
     */
    public static function disableCombine ($flag)
    {
        self::$_combineDisabled = (boolean) $flag;
    }

    /**
     * Disables the minify static
     *
     * @param unknown_type $flag
     */
    public static function disableMinify ($flag)
    {
        self::$_minifyDisabled = (boolean) $flag;
    }
    
    /**
     * Disables print source path inside combined file
     *
     * @param bool $flag
     */
    public static function disablePrintSourcePath ($flag)
    {
        self::$_printSourcePathDisabled = (boolean) $flag;
    }

    /**
     * Disables inline js combining
     *
     * @static
     * @param bool $flag
     */
    public static function disableCombineInlineJs ($flag = true)
    {
        self::$_combineInlineJs = (bool) !$flag;
    }

    /**
     * Set cache direcory
     *
     * @param string $cacheDir
     */
    public static function setCacheDir ($cacheDir)
    {
        if (!is_dir($cacheDir) || !is_writable($cacheDir)) {
            throw new Singular_Exception('Direcory "' . $cacheDir . '" does not exist or do not have permission to write.');
        }
        self::$_cacheDir = rtrim((string) $cacheDir, '/');
    }
}
