<?php
/**
 * Singular
 *
 * LICENSE
 *
 * The SOFTWARE PRODUCT is protected by copyright treaties,
 * as well as other intellectual property laws and treaties.
 * The SOFTWARE PRODUCT is licensed, not sold.
 * Software.
 * You may install and use one copy of the SOFTWARE PRODUCT
 * for each license purchased or if a site license then you
 * may install any number of copies at the site where the
 * site license is defined.
 * Your rights to use the SOFTWARE PRODUCT are not transferable.
 *
 * @category   Singular
 * @package    Singular_View
 * @subpackage Helper
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */

/**
 * @category Singular
 * @package Singular_View
 * @subpackage Singular_View_Helper
 * @copyright  Copyright (c) 2010 Zenonis. (http://www.zenonis.com)
 */
class Singular_View_Helper_Notify extends Zend_View_Helper_Abstract
{
    const SESSION_NAMESPACE = 'Notifications';

    const STORAGE_PROPERTY = 'messages';

    const DATA_STACK_ERROR = 'error';

    const DATA_STACK_SUCCESS = 'success';

    const DATA_STACK_INFO = 'info';

    protected static $_storage = null;

    private $_storageStructure = array(
        self::DATA_STACK_ERROR   => array(),
        self::DATA_STACK_SUCCESS => array(),
        self::DATA_STACK_INFO    => array()
    );

    protected static $_ticksProcessed = false;

    public function notify ()
    {
        if (null === self::$_storage) {
            $this->_initStorageHandler();
        }
        return $this;
    }

    public function error ($messages, $tiks = 0)
    {
        $type = self::DATA_STACK_ERROR;
        (is_array($messages))
                ? $this->addMessages($type, $messages, $tiks)
                : $this->addMessage($type, $messages, $tiks);
    }
    
    public function success ($messages, $tiks = 0)
    {
        $type = self::DATA_STACK_SUCCESS;
        (is_array($messages))
                ? $this->addMessages($type, $messages, $tiks)
                : $this->addMessage($type, $messages, $tiks);
    }

    public function info ($messages, $tiks = 0)
    {
        $type = self::DATA_STACK_INFO;
        (is_array($messages))
                ? $this->addMessages($type, $messages, $tiks)
                : $this->addMessage($type, $messages, $tiks);
    }

    public function addMessages ($type, $messages, $tiks = 0)
    {
        if (!is_array($messages)) {
            throw new Singular_Exception('Parameter "$messages" must be an array, ' . gettype($messages) . ' was given.');
        }

        foreach ($messages as $message) {
            $this->addMessage($type, $message, $tiks);
        }
    }

    public function addMessage ($type, $message, $tiks = 0)
    {
        if (!is_string($message)) {
            throw new Singular_Exception('Parameter "$message" must be an string, ' . gettype($message) . ' was given.');
        }

        switch ($type) {
            case self::DATA_STACK_ERROR:
            case self::DATA_STACK_SUCCESS:
            case self::DATA_STACK_INFO:
                $this->_addMessage($type, $message, $tiks);
                break;
            default:
                throw new Singular_Exception("Invalid data stack type \"$type\".");
                break;
        }
    }

    private function _addMessage ($type, $message, $tiks = 0)
    {
        $stack = $this->_getDataStack($type);
        $stack[] = array(
            'message' => trim(ucfirst($this->view->t($message))),
            'ticks'   => (int) $tiks
        );
        $this->_storeDataStack($type, $stack);
    }

    public function getErrorMessages ()
    {
        $type = self::DATA_STACK_ERROR;
        $messages = $this->_sanitizeMessages($this->_getDataStack($type));
        $this->clearMessages($type);

        return $messages;
    }

    public function getSuccessMessages()
    {
        $type = self::DATA_STACK_SUCCESS;
        $messages = $this->_sanitizeMessages($this->_getDataStack($type));
        $this->clearMessages($type);

        return $messages;
    }

    public function getInfoMessages()
    {
        $type = self::DATA_STACK_INFO;
        $messages = $this->_sanitizeMessages($this->_getDataStack($type));
        $this->clearMessages($type);

        return $messages;
    }

    public function getAllMessages ()
    {
        $stacks = $this->_getAllDataStacks();
        foreach ($stacks as $stack => $messages) {
            $messages = $this->_sanitizeMessages($messages);
            $stacks[$stack] = $messages;
        }
        $this->clearMessages();

        return $stacks;
    }

    public function clearMessages ($type = null, $force = false)
    {
        if (null === $type) {
            if ($force) {
                return $this->_setStorageStructure(true);
            }
            $stacks = $this->_getAllDataStacks();
            foreach ($stacks as $stack => $messages) {
                $this->_clearZeroTicksMessages($stack);
            }
            return;
        } elseif (is_string($type)) {
            if ($force) {
                $this->_getStorageHandler()->{self::STORAGE_PROPERTY}[$type] = array();
            }
            $this->_clearZeroTicksMessages($type);
            return;
        }

        throw new Singular_Exception("Invalid data stack type \"$type\".");
    }

    private function _clearZeroTicksMessages ($type)
    {
        $messages = $this->_getDataStack($type);

        foreach ($messages as $idx => $message) {
            if (0 >= $messages[$idx]['ticks']) {
                unset($messages[$idx]);
            }
        }
        $this->_getStorageHandler()->{self::STORAGE_PROPERTY}[$type] = $messages;
    }

    private function _storeDataStack ($type, $data)
    {
        if (!is_array($data)) {
            throw new Singular_Exception('Parameter "$data" must be an string, ' . gettype($message) . ' was given.');
        }
        
        $this->_getStorageHandler()
            ->{self::STORAGE_PROPERTY}[$type] = $data;
    }

    protected function _getDataStack ($type)
    {
        if (isset($this->_getStorageHandler()->{self::STORAGE_PROPERTY}[$type])) {
            $this->_processTicks();
            return $this->_getStorageHandler()->{self::STORAGE_PROPERTY}[$type];
        }

        throw new Singular_Exception("The data stack \"{$type}\" is invalid.");
    }
    
    protected function _getAllDataStacks ()
    {
        $this->_processTicks();
        return $this->_getStorageHandler()->{self::STORAGE_PROPERTY};
    }

    protected function _initStorageHandler ()
    {
        self::$_storage = new Zend_Session_Namespace(self::SESSION_NAMESPACE);
        $this->_setStorageStructure();
    }

    protected function _getStorageHandler ()
    {
        if (null === self::$_storage) {
            $this->_initStorageHandler();
        }

        return self::$_storage;
    }

    protected function _setStorageStructure ($force = false)
    {
        if (!isset($this->_getStorageHandler()->{self::STORAGE_PROPERTY})
            || true === (bool) $force) {
            $this->_getStorageHandler()->{self::STORAGE_PROPERTY} = $this->_storageStructure;
        }
    }

    protected function _processTicks ()
    {
        if (self::$_ticksProcessed) return;

        $stacks = $this->_getStorageHandler()->{self::STORAGE_PROPERTY};
        foreach ($stacks as $stack => $messages) {
            foreach ($messages as $idx => $message) {
                if (0 < $messages[$idx]['ticks']) {
                    $messages[$idx]['ticks']--;
                }
            }
            $stacks[$stack] = $messages;
        }
        $this->_getStorageHandler()->{self::STORAGE_PROPERTY} = $stacks;
        self::$_ticksProcessed = true;
    }

    private function _sanitizeMessages ($messages)
    {
        $sanitized = array();
        foreach ($messages as $message) {
            if (0 == $message['ticks']) {
                $sanitized[] = $message['message'];
            }
        }

        return $sanitized;
    }
}
