<?php

class Singular_View_HtmlElement
{
    
    const MODULE_SELF = 'module_self';

    /**
     * @var Zend_Config
     */
    private $_config = null;

    /**
     * @var null|string
     */
    private $_module = null;

    /**
     * @var bool
     */
    private $_useTheme = false;

    /**
     * @var null|string
     */
    private $_type = null;

    /**
     * @var null|string
     */
    private $_src = null;

    public function __construct()
    {
        $this->_setConfig();
    }

    /**
     * @throws Singular_Exception
     * @return void
     */
    private function _setConfig ()
    {
        if (Singular_Runtime::inStorage('config')) {
            $this->_config = Singular_Runtime::extract('config');
        } else {
            throw new Singular_Exception('Configuration is not loaded');
        }
    }

    /**
     * @throws Singular_Exception
     * @return null|Zend_Config
     */
    public function getConfig ()
    {
        if ($this->_config instanceof Zend_Config) {
            return $this->_config;
        } else {
            throw new Singular_Exception('Configuration is can\'t be loaded');
        }
    }

    /**
     * @param  null $name
     * @return Singular_View_HtmlElement
     */
    public function setModuleName ($name = null)
    {
        if (null !== $name || !is_bool($name) && '' != $name) {
            $this->_module = $name;
        }
        return $this;
    }

    /**
     * @param  $flag
     * @return Singular_View_HtmlElement
     */
    public function useTheme ($flag)
    {
        $this->_useTheme = (bool) $flag;
        return $this;
    }

    /**
     * @return bool
     */
    public function isTheme ()
    {
        return $this->_useTheme;
    }

    /**
     * @return null
     */
    public function getModuleName ()
    {
        return $this->_module;
    }

    /**
     * @param null $type
     * @return Singular_View_HtmlElement
     */
    public function setElementType ($type = null)
    {
        if (null !== $type && '' != $type) {
            $this->_type = $type;
        }
        return $this;
    }

    /**
     * @return null
     */
    public function getElementType ()
    {
        return $this->_type;
    }

    /**
     * @throws Singular_Exception
     * @param null $src
     * @return Singular_View_HtmlElement
     */
    public function setElementSrc ($src = null)
    {
        if (null !== $src && '' != $src) {
            if (substr($src, 0, 1) == '/') {
                $src = substr($src, -strlen($src) + 1);
            }
            $this->_src = $src;
            return $this;
        } else {
            throw new Singular_Exception('Src for element type '
                    . $this->getElementType()
                    . ' is empty');
        }
    }

    /**
     * @return null
     */
    public function getElementSrc ()
    {
        return $this->_src;
    }

    /**
     * @throws Singular_Exception
     * @return string
     */
    public function compile ()
    {
        // get module name
        $moduleName = Zend_Controller_Front::getInstance()->getRequest()->getModuleName();
        
        if ($this->isTheme()) {
            // get view
            $view = Zend_Layout::getMvcInstance()->getView();
            
            // get theme name
            $themeName = (Singular_Core::_('Environment')->isAdminRequest())
                    ? Singular_Core::_('Theme')->getAdmin()
                    : Singular_Core::_('Theme')->getDefault();

            // get assets path
            $assetsPath = self::trimSlashes($this->getConfig()->resources->themes->path)
                . "/{$themeName}/" . self::trimSlashes($this->getConfig()->htmlElement->assetsPathThemes);
        } else {
            if (self::MODULE_SELF == $this->getModuleName()) {
                $assetsPath = self::trimSlashes($this->getConfig()->htmlElement->assetsPathModules)
                    . '/'
                    . $moduleName;
            } elseif (null !== $this->getModuleName() && !is_bool($this->getModuleName())) {
                $assetsPath = self::trimSlashes($this->getConfig()->htmlElement->assetsPathModules)
                    . '/'
                    . $this->getModuleName();
            } else {
                $assetsPath = self::trimSlashes($this->getConfig()->htmlElement->assetsPath);
            }
        }
        
        switch ($this->getElementType()) {
            case 'image':
                $elementDir = self::trimSlashes($this->getConfig()->htmlElement->imageDirName);
                break;
            case 'javascript':
                $elementDir = self::trimSlashes($this->getConfig()->htmlElement->javascriptDirName);
                break;
            case 'stylesheet':
                $elementDir = self::trimSlashes($this->getConfig()->htmlElement->stylesheetDirName);
                break;
            case 'flash':
                $elementDir = self::trimSlashes($this->getConfig()->htmlElement->flashDirName);
                break;
            case 'common':
            case 'default':
                $elementDir = self::trimSlashes($this->getConfig()->htmlElement->commonDirName);
                break;
        }

        $baseUrl = $this->getConfig()->resources->frontController->baseUrl;
        $baseUrl = (substr($baseUrl, strlen($baseUrl) - 1) != '/') ? "{$baseUrl}/" : $baseUrl;

        $elementPath = $baseUrl . "{$assetsPath}/$elementDir/" . $this->getElementSrc();

        $view = Zend_Layout::getMvcInstance()->getView();
        if (true == $this->getConfig()->htmlElement->checkExistence) {
            if (!$this->_checkExistence($elementPath)) {
                throw new Singular_Exception('Element ' . $this->view->baseUrl($elementPath) . ' is not exists');
            }
        }
        
        return $elementPath;
    }

    /**
     * @param  $path
     * @return bool
     */
    private function _checkExistence ($path)
    {
        $path = !(substr($path, 0, 1) == '/' | '\\') ? DS . $path : $path;
        return (file_exists(ROOT_PATH . $path) &&
                is_readable(ROOT_PATH . $path)) ? true : false;
    }

    public static function trimSlashes ($str)
    {
        return trim($str, '/');
    }
}
