<?php
/**
 *=BEGIN SINGULAR CORE NEW BSD
 *
 * This file is part of the SingularCore.
 *
 * Copyright(c) 2010-2011, USWebStyle, LLC.
 * http://www.singularcore.com/
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.singularcore.com/license
 *
 *=END SINGULAR CORE NEW BSD
 *
 * @category   Singular
 * @package    Singular_Widget
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle)
 * @license    http://www.singularcore.com/license     New BSD License
 */

/**
 * FormTabs widget
 *
 * @abstract
 * @category   Singular
 * @package    Singular_Widget
 * @copyright  Copyright (c) 2011 USWebStyle, LLC. (http://www.uswebstyle.com)
 * @license    http://www.singularcore.com/license     New BSD License
 */
class Singular_Widget_FormTabs extends Singular_Widget_Abstract
{
    /**
     * Sub-forms collector
     * 
     * @var array
     */
    protected $_forms = array();

    /**
     * Initialize object
     *
     * Called from {@link __construct()} as final step of object instantiation.
     *
     * @abstract
     * @return void
     */
    public function init ()
    {
        // add client javascript control file
        $this->view->headScript()->appendFile(
            $this->view->javascriptSrc('widgets/form_tabs/widget.js', null)
        );
    }

    /**
     * Runs before the widget has been rendered
     *
     * @return void
     */
    public function preRender ()
    {
        // set defined forms
        $this->setVar('forms', $this->getForms());
        // set forms num
        $this->setVar('formsNum', $this->countForms());
    }

    /**
     * Sets array with sub-forms
     * 
     * @throws Singular_Widget_Exception
     * @param  array $forms
     * @return Singular_Widget_FormTabs
     */
    public function setForms ($forms)
    {
        if (!is_array($forms)) {
            throw new Singular_Widget_Exception(
                'Invalid argument: $forms must be an array.'
            );
        }
        foreach ($forms as $form) {
            if (!$form instanceof Singular_Form_SubForm) {
                throw new Singular_Widget_Exception(
                    'Form $form must be an instance of Singular_Form_SubForm'
                );
            }
        }

        $this->_forms = $forms;

        return $this;
    }

    /**
     * Gets array with sub-forms
     * 
     * @return array
     */
    public function getForms ()
    {
        return $this->_forms;
    }

    /**
     * Returns forms num
     * 
     * @return int
     */
    public function countForms ()
    {
        return count($this->getForms());
    }

    /**
     * Renders widget
     *
     * @return void
     */
    public function renderWidget ()
    {
        $this->_execPreRender();

        // TODO: Make renderWidget basics
        throw new Singular_Widget_Exception(
            'Can\'t render this widget without partial'
        );
    }
}
