/**
 * Singular CORE
 *
 * Client application
 */
var App = {
    version: '2.0',

    config: {
        basePath: '/',
        environment: 'production',
        //environment: 'development',

        ajax: {
            useDefaultEventHandlers: true,

            before: function (xhr, settings) {
                var preloader = $('<div id="default-preloader">Loading...</div>');

                $(preloader).css({
                    display: 'none',
                    position: 'fixed',
                    zIndex: '100000000000',
                    width: '200px',
                    border: '1px solid #777',
                    backgroundImage: 'url(' + App.assets.theme.image('/preloader/default-preloader-bg.gif') + ')',
                    textAlign: 'center',
                    padding: '5px 0 5px 0',
                    color: '#fff',
                    fontSize: '14px',
                    fontWeight: 'bold'
                });
                
                $(preloader).css({
                    top: $(window).height() / 2 - $(preloader).height() / 2,
                    left: $(window).width() / 2 - $(preloader).width() / 2
                });

                $(preloader).appendTo('body').fadeIn('slow');
            },

            after: function (xhr, status) {
                $('#default-preloader').fadeOut('slow', function(){$(this).remove()});
            },

            error: function (xhr, textStatus, errorThrown) {
                $('#default-preloader').fadeOut('slow', function(){$(this).remove()});
                App.dialog.error('Remote request error', 'Reason: ' + textStatus);
            }
        },

        theme: null,

        assets: {
            paths: {
                module: ':basePath/public/assets/modules/:moduleName/:resource/:path',
                theme:  ':basePath/public/themes/:theme/assets/:resource/:path',
                common: ':basePath/public/assets/:resource/:path',
                file:   ':basePath/public/files/:path',
                avatar: ':basePath/public/files/avatars/:path'
            }
        }
    },

    translations: {},

    params: {},

    moduleName: null,

    controllerName: null,

    actionName: null,

    baseUrl: function (path) {
        return App.config.basePath + path;
    },

    isInDevelopment: function () {
        return ('development' === App.config.environment);
    },

    t: function() {
        var regex = /%%|%(\d+\$)?([-+#0 ]*)(\*\d+\$|\*|\d+)?(\.(\*\d+\$|\*|\d+))?([scboxXuidfegEG])/g;
        var a = arguments, i = 0, format = a[i++];

        if (typeof App.translations[format] !== 'undefined') {
            format = App.translations[format];
        }

        if (typeof a[1] == 'object') {
            for (key in a[1]) {
                format = format.replace(':' + key, a[1][key]);
            }
            return format;
        }

        // pad()
        var pad = function(str, len, chr, leftJustify) {
            var padding = (str.length >= len) ? '' : Array(1 + len - str.length >>> 0).join(chr);
            return leftJustify ? str + padding : padding + str;
        };

        // justify()
        var justify = function(value, prefix, leftJustify, minWidth, zeroPad) {
            var diff = minWidth - value.length;
            if (diff > 0) {
                if (leftJustify || !zeroPad) {
                    value = pad(value, minWidth, ' ', leftJustify);
                } else {
                    value = value.slice(0, prefix.length) + pad('', diff, '0', true) + value.slice(prefix.length);
                }
            }
            return value;
        };

        // formatBaseX()
        var formatBaseX = function(value, base, prefix, leftJustify, minWidth, precision, zeroPad) {
            // Note: casts negative numbers to positive ones
            var number = value >>> 0;
            prefix = prefix && number && {'2': '0b', '8': '0', '16': '0x'}[base] || '';
            value = prefix + pad(number.toString(base), precision || 0, '0', false);
            return justify(value, prefix, leftJustify, minWidth, zeroPad);
        };

        // formatString()
        var formatString = function(value, leftJustify, minWidth, precision, zeroPad) {
            if (precision != null) {
                value = value.slice(0, precision);
            }
            return justify(value, '', leftJustify, minWidth, zeroPad);
        };

        // finalFormat()
        var doFormat = function(substring, valueIndex, flags, minWidth, _, precision, type) {
            if (substring == '%%') return '%';

            // parse flags
            var leftJustify = false, positivePrefix = '', zeroPad = false, prefixBaseX = false;
            for (var j = 0; flags && j < flags.length; j++) switch (flags.charAt(j)) {
                case ' ':
                    positivePrefix = ' ';
                    break;
                case '+':
                    positivePrefix = '+';
                    break;
                case '-':
                    leftJustify = true;
                    break;
                case '0':
                    zeroPad = true;
                    break;
                case '#':
                    prefixBaseX = true;
                    break;
            }

            // parameters may be null, undefined, empty-string or real valued
            // we want to ignore null, undefined and empty-string values
            if (!minWidth) {
                minWidth = 0;
            } else if (minWidth == '*') {
                minWidth = +a[i++];
            } else if (minWidth.charAt(0) == '*') {
                minWidth = +a[minWidth.slice(1, -1)];
            } else {
                minWidth = +minWidth;
            }

            // Note: undocumented perl feature:
            if (minWidth < 0) {
                minWidth = -minWidth;
                leftJustify = true;
            }

            if (!isFinite(minWidth)) {
                throw new Error('sprintf: (minimum-)width must be finite');
            }

            if (!precision) {
                precision = 'fFeE'.indexOf(type) > -1 ? 6 : (type == 'd') ? 0 : void(0);
            } else if (precision == '*') {
                precision = +a[i++];
            } else if (precision.charAt(0) == '*') {
                precision = +a[precision.slice(1, -1)];
            } else {
                precision = +precision;
            }

            // grab value using valueIndex if required?
            var value = valueIndex ? a[valueIndex.slice(0, -1)] : a[i++];

            switch (type) {
                case 's':
                    return formatString(String(value), leftJustify, minWidth, precision, zeroPad);
                case 'c':
                    return formatString(String.fromCharCode(+value), leftJustify, minWidth, precision, zeroPad);
                case 'b':
                    return formatBaseX(value, 2, prefixBaseX, leftJustify, minWidth, precision, zeroPad);
                case 'o':
                    return formatBaseX(value, 8, prefixBaseX, leftJustify, minWidth, precision, zeroPad);
                case 'x':
                    return formatBaseX(value, 16, prefixBaseX, leftJustify, minWidth, precision, zeroPad);
                case 'X':
                    return formatBaseX(value, 16, prefixBaseX, leftJustify, minWidth, precision, zeroPad).toUpperCase();
                case 'u':
                    return formatBaseX(value, 10, prefixBaseX, leftJustify, minWidth, precision, zeroPad);
                case 'i':
                case 'd':
                {
                    var number = parseInt(+value);
                    var prefix = number < 0 ? '-' : positivePrefix;
                    value = prefix + pad(String(Math.abs(number)), precision, '0', false);
                    return justify(value, prefix, leftJustify, minWidth, zeroPad);
                }
                case 'e':
                case 'E':
                case 'f':
                case 'F':
                case 'g':
                case 'G':
                {
                    var number = +value;
                    var prefix = number < 0 ? '-' : positivePrefix;
                    var method = ['toExponential', 'toFixed', 'toPrecision']['efg'.indexOf(type.toLowerCase())];
                    var textTransform = ['toString', 'toUpperCase']['eEfFgG'.indexOf(type) % 2];
                    value = prefix + Math.abs(number)[method](precision);
                    return justify(value, prefix, leftJustify, minWidth, zeroPad)[textTransform]();
                }
                default:
                    return substring;
            }
        };

        return format.replace(regex, doFormat);
    },

    addParam: function (name, value) {
        App.params[name] = value;
    },
    
    addParams: function (params) {
        if ($.isPlainObject(params)) {
            $.each(params, function(key, value) {
                App.params[key] = value;
            });
        }
    },

    getParam: function (name, _default) {
        if (App.hasParam(name)) {
            return App.params[name];
        }
        if (_default) {
            return _default;
        }

        return null;
    },

    getAllParams: function () {
        return App.params;
    },

    hasParam: function (name) {
        if (typeof(App.params[name]) == 'undefined') {
            return false;
        }

        return true;
    },

    getModuleName: function () {
        return App.moduleName;
    },

    getControllerName: function () {
        return App.controllerName;
    },

    getActionName: function () {
        return App.actionName;
    },

    getVersion: function () {
        return App.version;
    },

    isMobileTheme: function () {
        return App.getParam('mobileTheme');
    },

    route: function () {
        return {
            is: function (route) {
                if ($.isArray(route)) {
                    return -1 != $.inArray(App.routeName, route);
                }
                return App.routeName == route;
            },
            not: function (route) {
                return !this.is(route);
            }
        }
    },

    auth: function () {
        return {
            userIsLoggedIn: function () {
                return App.getParam('userIsLoggedIn');
            }
        }
    },

    loggedUser: function () {
        var loggedUser = App.getParam('loggedUser', {});

        return {
            getId: function () {
                if (!loggedUser.id) loggedUser.id = 2;
                return loggedUser.id;
            },
            getFullName: function () {
                return loggedUser.full_name;
            },
            getEmail: function () {
                return loggedUser.email;
            },
            getPhoto: function () {
                if (!loggedUser.photo) loggedUser.photo = 'avatar-default.png';
                return App.assets.avatar(loggedUser.photo);
            },
            getRole: function () {
                return loggedUser.role;
            },
            isTrusted: function () {
                return loggedUser.is_trusted;
            },
            isAdmin: function () {
                return 'administrator' === loggedUser.role;
            },
            isRepresentative: function () {
                return 'administrator' === loggedUser.role || 'official-representative' === loggedUser.role;
            },
            isUser: function () {
                return 'user' === loggedUser.role;
            },
            isGuest: function () {
                return 'guest' === loggedUser.role;
            },
            getProfileLink: function () {
                return App.baseUrl('/profile/' + this.getId());
            },
            getSettingsLink: function () {
                return App.baseUrl('/profile/' + this.getId() + '/settings');
            }
        };
    },

    agent: function () {

        var ua = navigator.userAgent.toLowerCase();
        var check = function (r) {
            return r.test(ua);
        };
        var DOC = document;
        var isStrict = DOC.compatMode == "CSS1Compat";

        return {
            isOpera:     function () { return check(/opera/)},
            isChrome:    function () { return check(/chrome/)},
            isWebKit:    function () { return check(/webkit/)},
            isSafari:    function () { return !this.isChrome() && check(/safari/)},
            isSafari2:   function () { return this.isSafari() && check(/applewebkit\/4/)},
            isSafari3:   function () { return this.isSafari() && check(/version\/3/)},
            isSafari4:   function () { return this.isSafari() && check(/version\/4/)},
            isSafari5:   function () { return this.isSafari() && check(/version\/5/)},
            isIE:        function () { return !this.isOpera() && check(/msie/)},
            isIE7:       function () { return this.isIE() && check(/msie 7/)},
            isIE8:       function () { return this.isIE() && check(/msie 8/)},
            isIE9:       function () { return this.isIE() && check(/msie 9/)},
            isIE6:       function () { return this.isIE() && !this.isIE7() && !this.isIE8() && !this.isIE9()},
            isGecko:     function () { return !this.isWebKit() && check(/gecko/)},
            isGecko2:    function () { return this.isGecko() && check(/rv:1\.8/)},
            isGecko3:    function () { return this.isGecko() && check(/rv:1\.9/)},
            isFirefox:   function () { return this.isGecko() && check(/firefox/)},
            isBorderBox: function () { return this.isIE && !isStrict},
            isWindows:   function () { return check(/windows|win32/)},
            isMac:       function () { return check(/macintosh|mac os x/)},
            isAir:       function () { return check(/adobeair/)},
            isLinux:     function () { return check(/linux/)},
            isSecure:    function () { return /^https/i.test(window.location.protocol)},
            isIE7InIE8:  function () { return this.isIE7() && DOC.documentMode == 7},
            getInfo:     function () {
                var jsType = '', browserType = '', browserVersion = '', osName = '';
                var ua = navigator.userAgent.toLowerCase();
                var check = function (r) {
                    return r.test(ua);
                };

                if (this.isWindows()) {
                    osName = 'Windows';

                    if (check(/windows nt/)) {
                        var start = ua.indexOf('windows nt');
                        var end = ua.indexOf(';', start);
                        osName = ua.substring(start, end);
                    }
                } else {
                    osName = this.isMac() ? 'Mac' : this.isLinux() ? 'Linux' : 'Other';
                }

                if (this.isIE()) {
                    browserType = 'IE';
                    jsType = 'IE';

                    var versionStart = ua.indexOf('msie') + 5;
                    var versionEnd = ua.indexOf(';', versionStart);
                    browserVersion = ua.substring(versionStart, versionEnd);

                    jsType = this.isIE6() ? 'IE6' : this.isIE7() ? 'IE7' : this.isIE8() ? 'IE8' : 'IE';
                } else if (this.isGecko()) {
                    var isFF = check(/firefox/);
                    browserType = isFF ? 'Firefox' : 'Others';
                    ;
                    jsType = this.isGecko2() ? 'Gecko2' : this.isGecko3() ? 'Gecko3' : 'Gecko';

                    if (isFF) {
                        var versionStart = ua.indexOf('firefox') + 8;
                        var versionEnd = ua.indexOf(' ', versionStart);
                        if (versionEnd == -1) {
                            versionEnd = ua.length;
                        }
                        browserVersion = ua.substring(versionStart, versionEnd);
                    }
                } else if (this.isChrome()) {
                    browserType = 'Chrome';
                    jsType = this.isWebKit() ? 'Web Kit' : 'Other';

                    var versionStart = ua.indexOf('chrome') + 7;
                    var versionEnd = ua.indexOf(' ', versionStart);
                    browserVersion = ua.substring(versionStart, versionEnd);
                } else {
                    browserType = this.isOpera() ? 'Opera' : this.isSafari() ? 'Safari' : '';
                }

                return {
                    jsType: jsType.toLowerCase(),
                    browserName: browserType.toLowerCase(),
                    browserVersion: browserVersion,
                    osName: osName.toLowerCase()
                }
            }
        }
    },

    utils: {
        md5: function (string) {

            function RotateLeft(lValue, iShiftBits) {
                return (lValue << iShiftBits) | (lValue >>> (32 - iShiftBits));
            }

            function AddUnsigned(lX, lY) {
                var lX4, lY4, lX8, lY8, lResult;
                lX8 = (lX & 0x80000000);
                lY8 = (lY & 0x80000000);
                lX4 = (lX & 0x40000000);
                lY4 = (lY & 0x40000000);
                lResult = (lX & 0x3FFFFFFF) + (lY & 0x3FFFFFFF);
                if (lX4 & lY4) {
                    return (lResult ^ 0x80000000 ^ lX8 ^ lY8);
                }
                if (lX4 | lY4) {
                    if (lResult & 0x40000000) {
                        return (lResult ^ 0xC0000000 ^ lX8 ^ lY8);
                    } else {
                        return (lResult ^ 0x40000000 ^ lX8 ^ lY8);
                    }
                } else {
                    return (lResult ^ lX8 ^ lY8);
                }
            }

            function F(x, y, z) {
                return (x & y) | ((~x) & z);
            }

            function G(x, y, z) {
                return (x & z) | (y & (~z));
            }

            function H(x, y, z) {
                return (x ^ y ^ z);
            }

            function I(x, y, z) {
                return (y ^ (x | (~z)));
            }

            function FF(a, b, c, d, x, s, ac) {
                a = AddUnsigned(a, AddUnsigned(AddUnsigned(F(b, c, d), x), ac));
                return AddUnsigned(RotateLeft(a, s), b);
            };

            function GG(a, b, c, d, x, s, ac) {
                a = AddUnsigned(a, AddUnsigned(AddUnsigned(G(b, c, d), x), ac));
                return AddUnsigned(RotateLeft(a, s), b);
            };

            function HH(a, b, c, d, x, s, ac) {
                a = AddUnsigned(a, AddUnsigned(AddUnsigned(H(b, c, d), x), ac));
                return AddUnsigned(RotateLeft(a, s), b);
            };

            function II(a, b, c, d, x, s, ac) {
                a = AddUnsigned(a, AddUnsigned(AddUnsigned(I(b, c, d), x), ac));
                return AddUnsigned(RotateLeft(a, s), b);
            };

            function ConvertToWordArray(string) {
                var lWordCount;
                var lMessageLength = string.length;
                var lNumberOfWords_temp1 = lMessageLength + 8;
                var lNumberOfWords_temp2 = (lNumberOfWords_temp1 - (lNumberOfWords_temp1 % 64)) / 64;
                var lNumberOfWords = (lNumberOfWords_temp2 + 1) * 16;
                var lWordArray = Array(lNumberOfWords - 1);
                var lBytePosition = 0;
                var lByteCount = 0;
                while (lByteCount < lMessageLength) {
                    lWordCount = (lByteCount - (lByteCount % 4)) / 4;
                    lBytePosition = (lByteCount % 4) * 8;
                    lWordArray[lWordCount] = (lWordArray[lWordCount] | (string.charCodeAt(lByteCount) << lBytePosition));
                    lByteCount++;
                }
                lWordCount = (lByteCount - (lByteCount % 4)) / 4;
                lBytePosition = (lByteCount % 4) * 8;
                lWordArray[lWordCount] = lWordArray[lWordCount] | (0x80 << lBytePosition);
                lWordArray[lNumberOfWords - 2] = lMessageLength << 3;
                lWordArray[lNumberOfWords - 1] = lMessageLength >>> 29;
                return lWordArray;
            };

            function WordToHex(lValue) {
                var WordToHexValue = "", WordToHexValue_temp = "", lByte, lCount;
                for (lCount = 0; lCount <= 3; lCount++) {
                    lByte = (lValue >>> (lCount * 8)) & 255;
                    WordToHexValue_temp = "0" + lByte.toString(16);
                    WordToHexValue = WordToHexValue + WordToHexValue_temp.substr(WordToHexValue_temp.length - 2, 2);
                }
                return WordToHexValue;
            };

            function Utf8Encode(string) {
                string = string.replace(/\r\n/g, "\n");
                var utftext = "";

                for (var n = 0; n < string.length; n++) {

                    var c = string.charCodeAt(n);

                    if (c < 128) {
                        utftext += String.fromCharCode(c);
                    }
                    else if ((c > 127) && (c < 2048)) {
                        utftext += String.fromCharCode((c >> 6) | 192);
                        utftext += String.fromCharCode((c & 63) | 128);
                    }
                    else {
                        utftext += String.fromCharCode((c >> 12) | 224);
                        utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                        utftext += String.fromCharCode((c & 63) | 128);
                    }

                }

                return utftext;
            };

            var x = Array();
            var k, AA, BB, CC, DD, a, b, c, d;
            var S11 = 7, S12 = 12, S13 = 17, S14 = 22;
            var S21 = 5, S22 = 9 , S23 = 14, S24 = 20;
            var S31 = 4, S32 = 11, S33 = 16, S34 = 23;
            var S41 = 6, S42 = 10, S43 = 15, S44 = 21;

            string = Utf8Encode(string);

            x = ConvertToWordArray(string);

            a = 0x67452301;
            b = 0xEFCDAB89;
            c = 0x98BADCFE;
            d = 0x10325476;

            for (k = 0; k < x.length; k += 16) {
                AA = a;
                BB = b;
                CC = c;
                DD = d;
                a = FF(a, b, c, d, x[k + 0], S11, 0xD76AA478);
                d = FF(d, a, b, c, x[k + 1], S12, 0xE8C7B756);
                c = FF(c, d, a, b, x[k + 2], S13, 0x242070DB);
                b = FF(b, c, d, a, x[k + 3], S14, 0xC1BDCEEE);
                a = FF(a, b, c, d, x[k + 4], S11, 0xF57C0FAF);
                d = FF(d, a, b, c, x[k + 5], S12, 0x4787C62A);
                c = FF(c, d, a, b, x[k + 6], S13, 0xA8304613);
                b = FF(b, c, d, a, x[k + 7], S14, 0xFD469501);
                a = FF(a, b, c, d, x[k + 8], S11, 0x698098D8);
                d = FF(d, a, b, c, x[k + 9], S12, 0x8B44F7AF);
                c = FF(c, d, a, b, x[k + 10], S13, 0xFFFF5BB1);
                b = FF(b, c, d, a, x[k + 11], S14, 0x895CD7BE);
                a = FF(a, b, c, d, x[k + 12], S11, 0x6B901122);
                d = FF(d, a, b, c, x[k + 13], S12, 0xFD987193);
                c = FF(c, d, a, b, x[k + 14], S13, 0xA679438E);
                b = FF(b, c, d, a, x[k + 15], S14, 0x49B40821);
                a = GG(a, b, c, d, x[k + 1], S21, 0xF61E2562);
                d = GG(d, a, b, c, x[k + 6], S22, 0xC040B340);
                c = GG(c, d, a, b, x[k + 11], S23, 0x265E5A51);
                b = GG(b, c, d, a, x[k + 0], S24, 0xE9B6C7AA);
                a = GG(a, b, c, d, x[k + 5], S21, 0xD62F105D);
                d = GG(d, a, b, c, x[k + 10], S22, 0x2441453);
                c = GG(c, d, a, b, x[k + 15], S23, 0xD8A1E681);
                b = GG(b, c, d, a, x[k + 4], S24, 0xE7D3FBC8);
                a = GG(a, b, c, d, x[k + 9], S21, 0x21E1CDE6);
                d = GG(d, a, b, c, x[k + 14], S22, 0xC33707D6);
                c = GG(c, d, a, b, x[k + 3], S23, 0xF4D50D87);
                b = GG(b, c, d, a, x[k + 8], S24, 0x455A14ED);
                a = GG(a, b, c, d, x[k + 13], S21, 0xA9E3E905);
                d = GG(d, a, b, c, x[k + 2], S22, 0xFCEFA3F8);
                c = GG(c, d, a, b, x[k + 7], S23, 0x676F02D9);
                b = GG(b, c, d, a, x[k + 12], S24, 0x8D2A4C8A);
                a = HH(a, b, c, d, x[k + 5], S31, 0xFFFA3942);
                d = HH(d, a, b, c, x[k + 8], S32, 0x8771F681);
                c = HH(c, d, a, b, x[k + 11], S33, 0x6D9D6122);
                b = HH(b, c, d, a, x[k + 14], S34, 0xFDE5380C);
                a = HH(a, b, c, d, x[k + 1], S31, 0xA4BEEA44);
                d = HH(d, a, b, c, x[k + 4], S32, 0x4BDECFA9);
                c = HH(c, d, a, b, x[k + 7], S33, 0xF6BB4B60);
                b = HH(b, c, d, a, x[k + 10], S34, 0xBEBFBC70);
                a = HH(a, b, c, d, x[k + 13], S31, 0x289B7EC6);
                d = HH(d, a, b, c, x[k + 0], S32, 0xEAA127FA);
                c = HH(c, d, a, b, x[k + 3], S33, 0xD4EF3085);
                b = HH(b, c, d, a, x[k + 6], S34, 0x4881D05);
                a = HH(a, b, c, d, x[k + 9], S31, 0xD9D4D039);
                d = HH(d, a, b, c, x[k + 12], S32, 0xE6DB99E5);
                c = HH(c, d, a, b, x[k + 15], S33, 0x1FA27CF8);
                b = HH(b, c, d, a, x[k + 2], S34, 0xC4AC5665);
                a = II(a, b, c, d, x[k + 0], S41, 0xF4292244);
                d = II(d, a, b, c, x[k + 7], S42, 0x432AFF97);
                c = II(c, d, a, b, x[k + 14], S43, 0xAB9423A7);
                b = II(b, c, d, a, x[k + 5], S44, 0xFC93A039);
                a = II(a, b, c, d, x[k + 12], S41, 0x655B59C3);
                d = II(d, a, b, c, x[k + 3], S42, 0x8F0CCC92);
                c = II(c, d, a, b, x[k + 10], S43, 0xFFEFF47D);
                b = II(b, c, d, a, x[k + 1], S44, 0x85845DD1);
                a = II(a, b, c, d, x[k + 8], S41, 0x6FA87E4F);
                d = II(d, a, b, c, x[k + 15], S42, 0xFE2CE6E0);
                c = II(c, d, a, b, x[k + 6], S43, 0xA3014314);
                b = II(b, c, d, a, x[k + 13], S44, 0x4E0811A1);
                a = II(a, b, c, d, x[k + 4], S41, 0xF7537E82);
                d = II(d, a, b, c, x[k + 11], S42, 0xBD3AF235);
                c = II(c, d, a, b, x[k + 2], S43, 0x2AD7D2BB);
                b = II(b, c, d, a, x[k + 9], S44, 0xEB86D391);
                a = AddUnsigned(a, AA);
                b = AddUnsigned(b, BB);
                c = AddUnsigned(c, CC);
                d = AddUnsigned(d, DD);
            }

            var temp = WordToHex(a) + WordToHex(b) + WordToHex(c) + WordToHex(d);

            return temp.toLowerCase();
        },

        xml2Json: function(xml, tab) {
            var X = {
                toObj: function(xml) {
                    var o = {};
                    //debugger;
                    if (xml.nodeType == 1) {   // element node ..
                        if (xml.attributes.length)   // element with attributes  ..
                            for (var i = 0; i < xml.attributes.length; i++)
                                o["@" + xml.attributes[i].nodeName] = (xml.attributes[i].nodeValue || "").toString();
                        if (xml.firstChild) { // element has child nodes ..
                            var textChild = 0, cdataChild = 0, hasElementChild = false;
                            for (var n = xml.firstChild; n; n = n.nextSibling) {
                                if (n.nodeType == 1) hasElementChild = true;
                                else if (n.nodeType == 3 && n.nodeValue.match(/[^ \f\n\r\t\v]/)) textChild++; // non-whitespace text
                                else if (n.nodeType == 4) cdataChild++; // cdata section node
                            }
                            if (hasElementChild) {
                                if (textChild < 2 && cdataChild < 2) { // structured element with evtl. a single text or/and cdata node ..
                                    X.removeWhite(xml);
                                    for (var n = xml.firstChild; n; n = n.nextSibling) {
                                        if (n.nodeType == 3)  // text node
                                            o["#text"] = X.escape(n.nodeValue);
                                        else if (n.nodeType == 4)  // cdata node
                                            o["#cdata"] = X.escape(n.nodeValue);
                                        else if (o[n.nodeName]) {  // multiple occurence of element ..
                                            if (o[n.nodeName] instanceof Array)
                                                o[n.nodeName][o[n.nodeName].length] = X.toObj(n);
                                            else
                                                o[n.nodeName] = [o[n.nodeName], X.toObj(n)];
                                        }
                                        else  // first occurence of element..
                                            o[n.nodeName] = X.toObj(n);
                                    }
                                }
                                else { // mixed content
                                    if (!xml.attributes.length)
                                        o = X.escape(X.innerXml(xml));
                                    else
                                        o["#text"] = X.escape(X.innerXml(xml));
                                }
                            }
                            else if (textChild) { // pure text
                                if (!xml.attributes.length)
                                    o = X.escape(X.innerXml(xml));
                                else
                                    o["#text"] = X.escape(X.innerXml(xml));
                            }
                            else if (cdataChild) { // cdata
                                if (cdataChild > 1)
                                    o = X.escape(X.innerXml(xml));
                                else
                                    for (var n = xml.firstChild; n; n = n.nextSibling)
                                        o["#cdata"] = X.escape(n.nodeValue);
                            }
                        }
                        if (!xml.attributes.length && !xml.firstChild) o = null;
                    }
                    else if (xml.nodeType == 9) { // document.node
                        o = X.toObj(xml.documentElement);
                    }
                    else
                        alert("unhandled node type: " + xml.nodeType);
                    return o;
                },
                toJson: function(o, name, ind) {
                    var json = name ? ("\"" + name + "\"") : "";
                    if (o instanceof Array) {
                        for (var i = 0,n = o.length; i < n; i++)
                            o[i] = X.toJson(o[i], "", ind + "\t");
                        json += (name ? ":[" : "[") + (o.length > 1 ? ("\n" + ind + "\t" + o.join(",\n" + ind + "\t") + "\n" + ind) : o.join("")) + "]";
                    }
                    else if (o == null)
                        json += (name && ":") + "null";
                    else if (typeof(o) == "object") {
                        var arr = [];
                        for (var m in o)
                            arr[arr.length] = X.toJson(o[m], m, ind + "\t");
                        json += (name ? ":{" : "{") + (arr.length > 1 ? ("\n" + ind + "\t" + arr.join(",\n" + ind + "\t") + "\n" + ind) : arr.join("")) + "}";
                    }
                    else if (typeof(o) == "string")
                        json += (name && ":") + "\"" + o.toString() + "\"";
                    else
                        json += (name && ":") + o.toString();
                    return json;
                },
                innerXml: function(node) {
                    var s = ""
                    if ("innerHTML" in node)
                        s = node.innerHTML;
                    else {
                        var asXml = function(n) {
                            var s = "";
                            if (n.nodeType == 1) {
                                s += "<" + n.nodeName;
                                for (var i = 0; i < n.attributes.length; i++)
                                    s += " " + n.attributes[i].nodeName + "=\"" + (n.attributes[i].nodeValue || "").toString() + "\"";
                                if (n.firstChild) {
                                    s += ">";
                                    for (var c = n.firstChild; c; c = c.nextSibling)
                                        s += asXml(c);
                                    s += "</" + n.nodeName + ">";
                                }
                                else
                                    s += "/>";
                            }
                            else if (n.nodeType == 3)
                                s += n.nodeValue;
                            else if (n.nodeType == 4)
                                s += "<![CDATA[" + n.nodeValue + "]]>";
                            return s;
                        };
                        for (var c = node.firstChild; c; c = c.nextSibling)
                            s += asXml(c);
                    }
                    return s;
                },
                escape: function(txt) {
                    return txt.replace(/[\\]/g, "\\\\")
                            .replace(/[\"]/g, '\\"')
                            .replace(/[\n]/g, '\\n')
                            .replace(/[\r]/g, '\\r');
                },
                removeWhite: function(e) {
                    e.normalize();
                    for (var n = e.firstChild; n;) {
                        if (n.nodeType == 3) {  // text node
                            if (!n.nodeValue.match(/[^ \f\n\r\t\v]/)) { // pure whitespace text node
                                var nxt = n.nextSibling;
                                e.removeChild(n);
                                n = nxt;
                            }
                            else
                                n = n.nextSibling;
                        }
                        else if (n.nodeType == 1) {  // element node
                            X.removeWhite(n);
                            n = n.nextSibling;
                        }
                        else                      // any other node
                            n = n.nextSibling;
                    }
                    return e;
                }
            };
            if (xml.nodeType == 9) // document node
                xml = xml.documentElement;
            var nws = X.removeWhite(xml);
            var obj = X.toObj(nws);
            var json = X.toJson(obj, xml.nodeName, "\t");
            //debugger;
            return "{\n" + (tab ? json.replace(/\t/g, tab) : json.replace(/\t|\n/g, "")) + "\n}";
        },

        scrollTo: function (element, duration, addPixels) {
            duration = (duration && typeof(duration) == 'number') ? duration : 300;
            addPixels = (addPixels && typeof(addPixels) == 'number') ? addPixels : 0;
            $('html, body').stop().animate({
                scrollTop: $(element).offset().top + addPixels
            }, {queue: false, duration: duration});
        },

        timer: function (funct, msec) {
            if (typeof(funct) == 'function' && msec && typeof(msec) == 'number') {
                setTimeout(funct, msec);
            }
        },

        camelToDash: function (str) {
            return str.replace(/\W+/g, '-')
                    .replace(/([a-z\d])([A-Z])/g, '$1-$2');
        },

        dashToCamel: function(str) {
            return str.replace(/\W+(.)/g, function (x, chr) {
                return chr.toUpperCase();
            });
        },
        
        isMobile: {
            Android: function() {
                return navigator.userAgent.match(/Android/i) ? true : false;
            },
            BlackBerry: function() {
                return navigator.userAgent.match(/BlackBerry/i) ? true : false;
            },
            iOS: function() {
                return navigator.userAgent.match(/iPhone|iPad|iPod/i) ? true : false;
            },
            Windows: function() {
                return navigator.userAgent.match(/IEMobile/i) ? true : false;
            },
            any: function() {
                return (
                    App.utils.isMobile.Android()
                    || App.utils.isMobile.BlackBerry()
                    || App.utils.isMobile.iOS()
                    || App.utils.isMobile.Windows()
                );
            }
        }
    },

    cookie: {
        set: function (name, value, expires, path, domain, secure) {
            document.cookie = name + "=" + value +
                ((expires) ? "; expires=" + expires : "") +
                ((path) ? "; path=" + path : "") +
                ((domain) ? "; domain=" + domain : "") +
                ((secure) ? "; secure" : "");
        },

        get: function (name) {
            var cookie = " " + document.cookie;
            var search = " " + name + "=";
            var setStr = null;
            var offset = 0;
            var end = 0;
            if (cookie.length > 0) {
                offset = cookie.indexOf(search);
                if (offset != -1) {
                    offset += search.length;
                    end = cookie.indexOf(";", offset)
                    if (end == -1) {
                        end = cookie.length;
                    }
                    setStr = cookie.substring(offset, end);
                }
            }

            return(setStr);
        }
    },

    ZIndex: {
        value: 10000,

        get: function () {
            App.ZIndex.value++;
            return App.ZIndex.value - 1;
        }
    },

    form: {
        populate: function (frm, data) {
            $.each(data, function (key, value) {
                var $ctrl = $('[name=' + key + ']', frm);
                switch ($ctrl.attr("type")) {
                    case "text" :
                    case "hidden":
                    case "textarea":
                        $ctrl.val(value);
                        break;
                    case "radio" :
                    case "checkbox":
                        $ctrl.each(function () {
                            if ($(this).attr('value') == value) {
                                $(this).attr("checked", value);
                            }
                        });
                        break;
                    default:
                        $ctrl.val(value);
                        break;
                }
            });
        },

        reset: function (frm) {
            frm = $(frm);
            frm[0].reset();
        }
    },
    
    event: {
        click: function() {
            return (!App.utils.isMobile.iOS()) ? 'click' : 'touchstart';
        }
    },
    
    init: function (config) {


        // Override config via init()
        if (config && typeof(config) == 'object') {
            $.extend(App.config, config);
        }

        if (null !== App.moduleName && null !== App.controllerName && null !== App.actionName) {
            mvcModule = eval('App.module.' + App.utils.dashToCamel(App.moduleName));

            if (mvcModule && typeof(mvcModule) == 'object') {
                mvcController = eval('mvcModule.' + 'controllers[\'' + App.utils.dashToCamel(App.controllerName) + '\']');
                if (mvcController && typeof(mvcController) == 'object') {
                    if (mvcController._init && typeof(mvcController._init) == 'function') {
                        mvcController._init();
                    }
                    mvcAction = eval('mvcController[\'' + App.utils.dashToCamel(App.actionName) + '\']');
                    if (mvcAction && typeof(mvcAction) == 'function') {
                        mvcAction();
                    }
                }
            }
        }

        // Fire events on document ready
        App.onReady(function(){
            // Auto hide notifications
            window.setTimeout(function(){
                $('div.b-info-line:visible').fadeOut(300, function(){
                    $('div.b-info-line:visible').remove();
                });
            }, 5000);
            // Notification events
            $('div.b-info-line:visible').find('div.mid').live('click', function(event){
                $(this).parents('div.b-info-line').fadeOut(300, function(){
                    $(this).remove();
                });
                event.preventDefault();
            });
        });
    },

    // Wrapper for init() without config
    reinit: function () {
        App.init();
    },

    onReady: function (eventFunction) {
        if (typeof(eventFunction) == 'function') {
            $(eventFunction);
        } else if (typeof(eventFunction) == 'string' && eventFunction in window) {
            if (_ef = eval(eventFunction.replace('()', ''))) {
                _ef();
            }
        }
    },

    // Logger
    log: function () {
        if (window.console) return console.log(Array.prototype.slice.call(arguments));
    },

    assets: {
        module: {
            image: function (moduleName, path) {
                return App.assets.module._assemblePath('images', moduleName, path);
            },

            javascript: function (moduleName, path) {
                return App.assets.module._assemblePath('js', moduleName, path);
            },

            stylesheet: function (moduleName, path) {
                return App.assets.module._assemblePath('css', moduleName, path);
            },

            flash: function (moduleName, path) {
                return App.assets.module._assemblePath('flash', moduleName, path);
            },

            common: function (moduleName, path) {
                return App.assets.module._assemblePath('common', moduleName, path);
            },

            _assemblePath: function (resType, moduleName, path) {
                _path = App.config.assets.paths.module;

                return _path.replace(':basePath', App.config.basePath)
                        .replace(':moduleName', moduleName)
                        .replace(':resource', resType)
                        .replace(':path', path)
                        .replace(new RegExp(/\/+/g), '/')
                        .replace(':/', '://');
            }
        },

        theme: {
            image: function (path) {
                return App.assets.theme._assemblePath('images', path);
            },

            javascript: function (path) {
                return App.assets.theme._assemblePath('js', path);
            },

            stylesheet: function (path) {
                return App.assets.theme._assemblePath('css', path);
            },

            flash: function (path) {
                return App.assets.theme._assemblePath('flash', path);
            },

            common: function (path) {
                return App.assets.theme._assemblePath('common', path);
            },

            _assemblePath: function (resType, path) {
                _path = App.config.assets.paths.theme;

                return _path.replace(':basePath', App.config.basePath)
                        .replace(':theme', App.config.theme)
                        .replace(':resource', resType)
                        .replace(':path', path)
                        .replace(new RegExp(/\/+/g), '/')
                        .replace(':/', '://');
            }
        },

        common: {
            image: function (path) {
                return App.assets.common._assemblePath('images', path);
            },

            javascript: function (path) {
                return App.assets.common._assemblePath('js', path);
            },

            stylesheet: function (path) {
                return App.assets.common._assemblePath('css', path);
            },

            flash: function (path) {
                return App.assets.common._assemblePath('flash', path);
            },

            common: function (path) {
                return App.assets.common._assemblePath('common', path);
            },

            _assemblePath: function (resType, path) {
                _path = App.config.assets.paths.common;

                return _path.replace(':basePath', App.config.basePath)
                        .replace(':resource', resType)
                        .replace(':path', path)
                        .replace(new RegExp(/\/+/g), '/')
                        .replace(':/', '://');
            }
        },
        avatar: function (file) {
            var _path = App.config.assets.paths.avatar;

            return _path.replace(':basePath', App.config.basePath)
                .replace(':path', file)
                .replace(new RegExp(/\/+/g), '/')
                .replace(':/', '://');
        }
    },

    // Dialog
    dialog: {
        // App.dialog.notice
        notice: function (title, message, options) {
            type = 'notice';
            return App.dialog._typePrepare(title, message, options, type);
        },

        // App.dialog.info
        info: function (title, message, options) {
            type = 'info';
            return App.dialog._typePrepare(title, message, options, type);
        },

        // App.dialog.error
        error: function (title, message, options) {
            type = 'error';
            return App.dialog._typePrepare(title, message, options, type);
        },

        _typePrepare: function (title, message, options, type) {
            if (!$.isPlainObject(title)) {
                if ('object' != typeof(options)) {
                    options = {};
                }
                options.title = title;
                options.message = message;
            }
            if (!type) {
                type = 'notice';
            }
            options.type = type;

            return App.dialog.open(options);
        },

        // App.dialog.yesNo
        yesNo: function (title, message, onYes, onNo, options) {
            if (!$.isPlainObject(title)) {
                if ('object' != typeof(options)) {
                    options = {};
                }
                options.title = title;
                options.message = message;
                if ('onYes' in options) {
                    onYes = options.onYes;
                }
                if ('onNo' in options) {
                    onNo = options.onNo;
                }
            }
            options.type = 'notice';
            options.buttons = {
                Yes: function() {
                    if (jQuery.isFunction(onYes)) {
                        onYes();
                    }
                    $(this).dialog("close");
                },
                No: function() {
                    if (jQuery.isFunction(onNo)) {
                        onNo();
                    }
                    $(this).dialog("close");
                }
            }

            return App.dialog.open(options);
        },

        // App.dialog.open
        open: function (title, message, options) {
            if ($.isPlainObject(title)) {
                options = title;
            } else {
                if ('object' != typeof(options)) {
                    options = {};
                }
                
                // append "title" options property
                if (typeof(title) == 'string') {
                    if (!("title" in options)) {
                        options.title = title;
                    }
                }
                // append "message" options property
                if (typeof(message) == 'string') {
                    if (!("message" in options)) {
                        options.message = message;
                    }
                }
            }

            // dialog type
            if (!("type" in options)) {
                options.type = 'notice';
            }
            
            switch (options.type) {
                case 'notice':
                case 'info':
                case 'error':
                    imgName = 'dialog-' + options.type;
                    break;

                default:
                    imgName = 'dialog-notice';
                    break;
            }

            titleImg = '<img style="padding-right:5px; float:left; display:inline;" src="' + App.assets.common.image('icons/' + imgName + '.png') + '" />';

            // modal defaults
            defaults = {
                resizable: false,
                modal:     true,
                draggable: true,
                title:     titleImg + 'Dialog:',
                message:   '<p>Empty message...</p>',
                buttons:   {
                    Ok: function() {
                        $(this).dialog("close");
                    }
                }
            }

            // combine options
            options = $.extend(defaults, options);

            options.title = '<div style="' + 'line-height:16px;">' + titleImg + options.title + '</div>';

            // checking for right xpath or html in message or leave message as plain text
            msgContent = $(options.message).html();
            if (null === msgContent) {
                options.message = '<p>' + options.message + '</p>'
            }

            $(options.message).dialog(options);
        }
    },

    // Notification types
    notify: {
        error: function (message) {App.notify.show('error', message)},

        success: function (message) {App.notify.show('success', message)},

        info: function (message) {App.notify.show('info', message)},

        show: function (type, message) {
            switch (type) {
                case 'error':
                    _subClass = 'error-line';
                    break;
                case 'success':
                    _subClass = 'success-line';
                    break;
                case 'info':
                    _subClass = '';
                    break;
            }
            _blankBlock = $('div.b-info-line:hidden');
            _cloned = _blankBlock.clone().removeClass('hidden');
            if ('' != _subClass) _cloned.addClass(_subClass);
            _cloned.find('span.notify-msg').html(message);
            _blankBlock.before(_cloned);
            App.utils.scrollTo($('div.b-info-line:first'));

            delete _cloned, _blankBlock;
        }
    },

    // Ajax functions
    remote: {
        load: function (id, url) {
            App.remote.initPreloader();
            $('#' + id).load(url);
        },

        get: function (url, data, callback, type) {
            return App.remote.request('get', url, data, callback, type);
        },

        post: function (url, data, callback, type) {
            return App.remote.request('post', url, data, callback, type);
        },

        request: function (requestType, url, data, callback, type) {
            App.remote.initPreloader();

            var complete = function (result) {};
            var success  = function (result) {};
            var error    = function (result) {};

            if (jQuery.isPlainObject(url)) {
                if (jQuery.isFunction(url.success)) {
                    success = url.success;
                }

                if (jQuery.isFunction(url.error)) {
                    error = url.error;
                }

                if (jQuery.isFunction(url.complete)) {
                    complete = url.complete;
                }

                if (jQuery.isPlainObject(url.data)) {
                    data = url.data;
                }

                if (url.url) {
                    url = url.url;
                }
            }

            var onComplete = function (response) {
                if (jQuery.isPlainObject(response)) {
                    var result = App.remote.parseResult(response.responseText);
                } else {
                    var result = {};
                }

                App.dynamic.event().trigger('remote.request.complete', {
                    result: result
                });

                // fire complete event
                complete(result);

                if (!jQuery.isPlainObject(result.data)) {
                    App.notify.error(response.responseText);
                    return;
                }

                if (result.data.isSuccess()) {
                    success(result);
                } else if (result.data.isError()) {
                    error(result);
                }
            }

            if (jQuery.isFunction(data)) {
                callback = data;
                data     = null;
            }

            switch (requestType) {
                case 'post':
                    App.dynamic.event().trigger('remote.request.post', {
                        url:  url, data: data
                    });
                    return $.post(url, data, callback, type).complete(onComplete);
                break;

                case 'get':
                    App.dynamic.event().trigger('remote.request.get', {
                        url: url, data: data
                    });
                    return $.get(url, data, callback, type).complete(onComplete);
                break;
            }
        },

        parseResult: function (result, asIs) {
            pType = 'unknown';
            parsed = null;

            try {
                if ($.isPlainObject(result)) {
                    parsed = result;
                } else {
                    parsed = $.parseJSON(result);
                }
                pType = 'json';
            } catch (jEx) {
                try {
                    parsed = $.parseXML(result);
                    pType = 'xml';
                } catch (xEx) {
                    parsed = null;
                    pType = 'unknown';
                }
            }

            if (null !== parsed) {
                if ('xml' == pType) {
                    parsed = App.utils.xml2Json(parsed);
                    parsed = $.parseJSON(parsed);
                }

                App.remote._result.setResponse(parsed);
                parsed = App.remote._result;
            }

            if (asIs) {
                parsed = result;
                pType = 'unknown'
            }

            return {
                status: (null !== parsed) ? 'success' : 'error',
                type: pType,
                data: parsed
            }
        },

        _result: {
            _response: {},

            setResponse: function (responseData) {
                if (responseData && typeof(responseData) == 'object') {
                    App.remote._result._response = responseData.response;

                    return App.remote._result;
                }
            },

            getServerTime: function () {return App.remote._result._get('server_time')},

            getType: function () {return App.remote._result._get('response_type')},

            isError: function () {
                if ('error' == App.remote._result.getType()) {
                    return true;
                }

                return false;
            },

            isSuccess: function () {
                if ('success' == App.remote._result.getType()) {
                    return true;
                }

                return false;
            },

            getCode: function () {return App.remote._result._get('code')},

            getMessage: function () {return App.remote._result._get('message')},

            getOutput: function () {return App.remote._result._get('output')},

            getModule: function () {return App.remote._result._get('module')},

            getController: function () {return App.remote._result._get('controller')},

            getAction: function () {return App.remote._result._get('action')},

            getParam: function (paramName) {
                _params = App.remote._result._get('params');

                if (null !== _params) {
                    _param = eval('_params.' + paramName);

                    if (_param) {
                        return _param;
                    }
                }
                
                return null;
            },

            getParams: function () {return App.remote._result._get('params')},

            _get: function (_param) {
                _param = eval('App.remote._result._response.' + _param);
                if (_param) {
                    return _param;
                }

                return null;
            }
        },

        initPreloader: function () {
            // Apply ajax event handlers
            if (true === App.config.ajax.useDefaultEventHandlers) {
                $.ajaxSetup({
                    beforeSend: App.config.ajax.before,
                    complete: App.config.ajax.after,
                    error: App.config.ajax.error
                });
            }
        }
    },

    /**
     * Ajax Iframe method
     *
     * @example:
     * var onStartHandler = function() {
     *     $('div.preloader').show();
     * }
     * var onCompleteHandler = function(response) {
     *     $('div.preloader').hide();
     *     $('div.result').html(response);
     * }
     * $('form.send').bind('submit', function (event) {
     *     return App.remoteIframe.submit(this, {'onStart' : onStartHandler, 'onComplete' : onCompleteHandler})
     * });
     */
    remoteIframe: {

        frame: function (c) {
            var n = 'f' + Math.floor(Math.random() * 99999);
            var d = document.createElement('DIV');
            d.innerHTML = '<iframe style="display:none;" src="about:blank" id="' + n + '" name="' + n + '" onload="App.remoteIframe.loaded(\'' + n + '\')"></iframe>';
            document.body.appendChild(d);

            var i = document.getElementById(n);
            if (c && typeof(c.onComplete) == 'function') {
                i.onComplete = c.onComplete;
            }

            return n;
        },

        form: function (f, name) {
            f.setAttribute('target', name);
        },

        submit: function (f, c) {
            App.remoteIframe.form(f, App.remoteIframe.frame(c));
            if (c && typeof(c.onStart) == 'function') {
                return c.onStart();
            } else {
                return true;
            }
        },

        loaded: function (id) {
            var i = document.getElementById(id);
            if (i.contentDocument) {
                var d = i.contentDocument;
            } else if (i.contentWindow) {
                var d = i.contentWindow.document;
            } else {
                var d = window.frames[id].document;
            }
            if (d.location.href == "about:blank") {
                return;
            }

            if (typeof(i.onComplete) == 'function') {
                i.onComplete(d.body.innerHTML);
            }
        }
    },

    module: {

        extend: function (module) {
            if (module && typeof(module) == 'object') {
                $.extend(App.module, module);
            }
        }
    }
};

/**
 * Dynamic - is new client-side extension for UseResponse 3.0.
 *
 * @type {{error: {system: Function}, helper: {getElement: Function}, modal: {center: Function, openFromUrl: Function, open: Function}, overlay: {show: Function, hide: Function}}}
 */
App.dynamic = {
    /**
     * Handling Errors.
     */
    error: {
        /**
         * Raises system error.
         *
         * @param string text
         * @param int code
         */
        system: function (text, code) {
            var err = "Application Error:\n\n" + text;
            if (code) {
                err += "\n\nCode: " + code;
            }
            alert(err);
        }
    },

    event: function () {
        /**
         * Events container
         *
         * @var jQuery
         */
        var eventContainer = $('body:first');

        /**
         * Application namespace
         *
         * @var String
         */
        var baseNamespace = '.useresponse_events';

        // Public interface
        return {
            /**
             * Bind event handler
             *
             * @param String event
             * @param Function callback
             */
            'bind': function (event, callback) {
                var events = event.split(' ');
                var new_events = [];
                $.each(events, function (index, event_name) {
                    if (event_name = $.trim(event_name)) {
                        new_events.push(event_name + baseNamespace);
                    }
                });

                eventContainer.bind(new_events.join(' '), callback);
            },

            /**
             * Unbind event handlers
             *
             * @param String event
             */
            'unbind': function (event) {
                if (event == undefined) {
                    // unbind all
                    eventContainer.unbind(baseNamespace);
                    return true;
                }
                var events = event.split(' ');
                var new_events = [];
                $.each(events, function (index, event_name) {
                    if (event_name = $.trim(event_name)) {
                        new_events.push(event_name + baseNamespace);
                    }
                });

                eventContainer.unbind(new_events.join(' '));
            },

            /**
             * Trigger specific event
             *
             * @param String event
             * @param mixed params
             */
            'trigger': function (event, params) {
                // if we are in development, and console is availabled
                if (App.isInDevelopment() && typeof(console) != 'undefined' && console && console.log) {
                    if (jQuery.inArray(event, ['history_state_changed', 'content_updated', 'single_content_updated', 'window_resize', 'keydown', 'window_resize_start', 'window_resize_end', 'navigate_away']) == -1) {
                        console.log('EVENT triggered', event, params);
                    }
                }

                if (event == undefined) {
                    event = '';
                }

                eventContainer.trigger(event + baseNamespace, params);
            }
        }
    },

    /**
     * Various helpers.
     */
    helper: {
        /**
         * jQuerify input element.
         *
         * @param   mixed mixed
         * @returns {*|jQuery|HTMLElement}
         */
        getElement: function (mixed) {
            var element = $(mixed);
            if (0 == element.length) {
                return App.dynamic.error.system('Invalid Element selector/object', 1);
            }
            return $(mixed);
        },

        makeUniqId: function (prefix) {
            return prefix + '_' + $.now() + "_" + Math.floor((Math.random() * 100) + 1);
        }
    },

    preloader: {
        show: function () {
            var pClass    = 'dynamicPreloader',
                preloader = $('<div></div>')
                    .attr('id', App.dynamic.helper.makeUniqId('preloader'))
                    .addClass(pClass)
                    .css('z-index', App.ZIndex.get())
                    .html(App.t('Loading &#8230'))
                    .appendTo('body');
            App.dynamic.modal.center(preloader);
            $(window).on('resize', function () {
                App.dynamic.modal.center(preloader);
            });
        },

        hide: function () {

        }
    },

    /**
     * Modal windows.
     */
    modal: {
        defaults: {
            minMarginTop:    20,
            minMarginBottom: 20
        },

        /**
         * Centers modal dialog.
         *
         * @param string|jQuery el  Element to center
         * @param string axis       Center according given axis ordinates. Valid values are:
         *                          'x', 'y' or 'xy'
         */
        center: function (el, axis) {
            if (!axis || '' == axis) axis = 'xy';
            el            = App.dynamic.helper.getElement(el);
            var elWidth   = el.width(),
                elHeight  = el.height(),
                winWidth  = $(window).width(),
                winHeight = $(window).height(),
                leftPos   = Math.round(winWidth / 2) - Math.round(elWidth / 2),
                topPos    = Math.round(winHeight / 2) - Math.round(elHeight / 2),
                coords    = {left: 1, top: App.dynamic.modal.defaults.minMarginTop};
            if (elWidth < winWidth)   coords.left = leftPos;
            if (elHeight < winHeight) coords.top  = (coords.top < topPos) ? topPos : coords.top;
            if ('x' == axis) delete coords.top;
            if ('y' == axis) delete coords.left;
            el.css(coords);
        },

        fitContent: function (el) {
            el = App.dynamic.helper.getElement(el);
            if (el.is('.modal')) {
                var modalData          = el.data(),
                    modalMinHeight     = parseInt(el.css('min-height')),
                    contentBlock       = el.find('.modalContent'),
                    contentBlockHeight = contentBlock[0].scrollHeight,
                    winHeight          = $(window).height(),
                    maxHeight          = winHeight - App.dynamic.modal.defaults.minMarginTop - App.dynamic.modal.defaults.minMarginBottom;

                var contPads = 0;
                if (modalData.useWrapper) {
                    contentBlockHeight = contentBlockHeight - 20;
                    contPads = parseInt(contentBlock.css('margin-top')) + parseInt(contentBlock.css('margin-bottom'))
                               + parseInt(contentBlock.css('padding-top')) + parseInt(contentBlock.css('padding-bottom'));
                }
                var headerAbsHeight = 57 + contPads,
                    calcHeight      = el[0].offsetHeight;
                if (maxHeight < contentBlockHeight) {
                    calcHeight = maxHeight;
                }
                if (modalMinHeight > calcHeight + headerAbsHeight - App.dynamic.modal.defaults.minMarginTop - App.dynamic.modal.defaults.minMarginBottom) {
                    calcHeight = modalMinHeight;
                }
                if (contentBlockHeight + contPads < maxHeight) {
                    return contentBlock.height(contentBlockHeight);
                }
                contentBlock.height(calcHeight - headerAbsHeight);
            }
        },

        /**
         * Opens new Modal Dialog using ajax
         *
         * @param options
         */
        openFromUrl: function (options) {
            var defaults = {
                autoNotifySuccess: true,
                autoNotifyError:   true,
                modal:             {},
                request:           {
                    type: 'get',
                    url:  null,
                    data: {
                        format: 'json'
                    },
                    complete: function (response) {},
                    success:  function (response) {},
                    error:    function (response) {}
                }
            };
            var settings = $.extend(true, {}, defaults, options);

            if (jQuery.inArray(settings.request.type, ['get', 'post']) == -1) {
                settings.request.type = 'get';
            }
            if (jQuery.inArray(settings.request.data.format, ['json', 'xml']) == -1) {
                settings.request.data.format = 'json';
            }

            /**
             * Returns sanitized message from response object.
             *
             * @param Object response
             * @returns {String|null}
             */
            var getMessage = function (response) {
                var msg = $.trim(response.data.getMessage());
                if ('' == msg || 'Undefined' == msg || typeof(msg) == 'undefined' || null === msg || 0 === msg) {
                    return null;
                }
                return msg;
            };

            /**
             * Returns sanitized output from response object.
             *
             * @param Object response
             * @returns {String|null}
             */
            var getContent = function (response) {
                var out = $.trim(response.data.getOutput());
                if ('' == out || typeof(out) == 'undefined' || null === out || 0 === out) {
                    return null;
                }
                return out;
            };

            // Complete callback
            if (!jQuery.isFunction(settings.request.complete)) {
                settings.request.complete = function (response) {};
            }
            var userComplete = settings.request.complete;
            settings.request.complete = function (response) {
                userComplete(response);
            };

            // Success callback
            if (!jQuery.isFunction(settings.request.success)) {
                settings.request.success = function (response) {};
            }
            var userSuccess = settings.request.success;
            settings.request.success = function (response) {
                userSuccess(response);
                if (settings.autoNotifySuccess) {
                    var message = getMessage(response);
                    if (null !== message) App.notify.success(message);
                }
                if (!('title' in settings.modal)) {
                    var title = response.data.getParam('title');
                    if (title) {
                        settings.modal.title = title;
                    }
                }
                settings.modal.content = getContent(response);
                var modalInstance = App.dynamic.modal.open(settings.modal);
            };

            // Error callback
            if (!jQuery.isFunction(settings.request.error)) {
                settings.request.error = function (response) {};
            }
            var userError = settings.request.error;
            settings.request.error = function (response) {
                userError(response);
                if (settings.autoNotifyError) {
                    var message = getMessage(response);
                    if (null !== message) App.notify.error(message);
                }
            };

            App.remote.request(
                settings.request.type,
                settings.request
            );
        },

        open: function (options) {
            var defaults = {
                name:          App.dynamic.helper.makeUniqId('modal'),
                width:         700,
                autoResize:    true,
                useWrapper:    true,
                widthClass:    null,
                wrapperClass:  'modalContentWrapper',
                closeOnEscape: true,
                overlay:       true,
                title:         App.t("Dialog"),
                content:       null,
                success:       function () {}
            };

            var modalId      = App.dynamic.helper.makeUniqId('modal'),
                instance     = $('<div class="modal"></div>').appendTo('body').attr('id', modalId),
                header       = $('<div class="modalHeader unselectable"></div>').appendTo(instance),
                title        = $('<span class="modalTitle"></span>').appendTo(header),
                closeControl = $('<i class="icon16Button icloseModal modalClose"></i>').appendTo(header),
                contentBlock = $('<div class="modalContent scrollable"></div>').appendTo(instance),
                shader       = $('<div class="shader"></div>').appendTo(instance);

            var settings = $.extend({}, defaults, options);

            if (settings.title) title.html(settings.title);
            if (settings.widthClass) {
                instance.addClass(settings.widthClass);
            } else if (settings.width) {
                instance.width(settings.width);
            }

            if (settings.content) {
                var contentWrapper = $('<div></div>');
                if (settings.useWrapper) {
                    contentWrapper.addClass(settings.wrapperClass);
                } else {
                    shader.hide();
                }
                contentBlock.html(
                    contentWrapper.append(settings.content)
                );
            }

            closeControl.on(App.event.click(), function (event) {
                App.dynamic.event().trigger('modal.close', {modal: instance});
                var modal = $(this).closest('.modal');
                if (1 === modal.length) {
                    if (modal.data('overlay')) {
                        App.dynamic.overlay.hide(modal.attr('id'));
                    }
                    modal.detach();
                }
                $(this).closest('.modal').remove();
            });

            $(document).bind('keydown', function (e) {
                if (e.which == 27) {
                    var modal = $('body > div.modal:last');
                    if (modal.is('.modal')) {
                        if (modal.data('overlay')) {
                            App.dynamic.overlay.hide(modal.attr('id'));
                        }
                        App.dynamic.event().trigger('modal.close', {modal: modal});
                        App.dynamic.modal.close(modal);
                    }
                }
            });

            instance.data({
                padded:        settings.padded,
                overlay:       settings.overlay,
                closeOnEscape: settings.closeOnEscape,
                name:          settings.name
            });

            App.dynamic.modal.fitContent(instance);
            App.dynamic.modal.center(instance);
            if (settings.overlay) {
                App.dynamic.overlay.show(modalId);
            }
            instance.css('z-index', App.ZIndex.get());

            if (settings.autoResize) {
                $(window).on('resize', function () {
                    App.dynamic.modal.fitContent(instance);
                    App.dynamic.modal.center(instance);
                });
                contentBlock.on('change', function (event) {
                    App.dynamic.modal.fitContent(instance);
                    App.dynamic.modal.center(instance);
                });
            }

            contentBlock.rollbar({zIndex: 100});

            settings.success(instance);
            App.dynamic.event().trigger('modal.open', {modal: instance});

            return instance;
        },

        close: function (instance) {
            instance = App.dynamic.helper.getElement(instance);
            if (instance.is('div.modal')) {
                instance.removeData();
                instance.detach();
            }
        }
    },

    overlay: {
        show: function (name) {
            if (!name) {
                name = App.dynamic.helper.makeUniqId('overlay');
            } else {
                name = 'overlay_'+ name;
            }
            var attribs = {
                'class': 'overlay',
                id: name
            };

            var overlay = $('<div/>', attribs).appendTo('body').css('z-index', App.ZIndex.get());
            App.dynamic.event().trigger('overlay.show', {overlay: overlay});
        },

        hide: function (name) {
            var overlay = $('div.overlay');
            if (name) {
                name = (0 !== name.indexOf('overlay_')) ? 'overlay_' + name : name;
                overlay = overlay.filter('#' + name);
            }

            if (overlay.is('div:visible')) {
                App.dynamic.event().trigger('overlay.hide', {overlay: overlay});
                overlay.detach();
            }
        }
    }
};

/* reinit application */
App.init();
