/*

 RollBar - jQuery ScrollBar Plugin
 -----------------------------------------------

 @author   flGravity
 @created  4/4/12
 @version  1.5
 @site     http://codecanyon.net/user/flGravity

 */

;
(function ($) {
    function RollBar(c, s) {
        this.container = $(c);
        this.settings = s;
        this.timer = 0;
        this.before = {v: 0, h: 0};
        this.touch = {};
        this.pressed = 0;
        this.vslider = $("<div/>", {"class": "rollbar-handle"});
        this.vpath = $("<div/>", {"class": "rollbar-path-vertical"});
        this.hslider = $("<div/>", {"class": "rollbar-handle"});
        this.hpath = $("<div/>", {"class": "rollbar-path-horizontal"});
        this.sliders = this.vslider.add(this.hslider);
        this.container.css({overflow: "hidden", position: "relative"}).contents().filter(this.settings.contentFilter).wrapAll('<div class="rollbar-content"></div>');
        this.content = this.container.children(".rollbar-content").css({position: "relative", top: 0, left: 0, height: "auto", overflow: "hidden"});
        if (this.settings.scroll == "horizontal") {
            this.container.prepend(this.hpath.append(this.hslider))
        } else {
            if (this.settings.scroll == "vertical") {
                this.container.prepend(this.vpath.append(this.vslider))
            } else {
                this.container.prepend(this.vpath.append(this.vslider), this.hpath.append(this.hslider))
            }
        }
        this.vpath.add(this.hpath).css({"z-index": this.settings.zIndex, display: "none"});
        this.vslider.css({height: this.settings.sliderSize, opacity: this.settings.sliderOpacity});
        this.hslider.css({width: this.settings.sliderSize, opacity: this.settings.sliderOpacity});
        if (this.settings.sliderOpacity) {
            this.sliders.hover(this.fixFn(function () {
                this.sliders.stop().fadeTo(this.settings.sliderOpacityTime, 1)
            }), this.fixFn(function () {
                if (!this.pressed) {
                    this.sliders.stop().fadeTo(this.settings.sliderOpacityTime, this.settings.sliderOpacity)
                }
            }))
        }
        this.init();
        this.pathSize();
        this.bindEvent($(window), "load", function () {
            setTimeout(this.fixFn(this.checkScroll), 10)
        });
        if (this.settings.lazyCheckScroll > 0) {
            setInterval(this.fixFn(function () {
                this.checkScroll();
                this.pathSize()
            }), this.settings.lazyCheckScroll)
        }
    }

    RollBar.prototype.checkScroll = function () {
        this.vtrack = this.vpath.height() - this.vslider.height();
        this.htrack = this.hpath.width() - this.hslider.width();
        this.vdiff = this.content.height() - this.container.height();
        this.hdiff = this.content.width() - this.container.width();
        if (!this.settings.autoHide) {
            return
        }
        if (this.vdiff > 0) {
            this.vpath.fadeIn(this.settings.autoHideTime)
        } else {
            this.vpath.fadeOut(this.settings.autoHideTime)
        }
        if (this.hdiff > 0) {
            this.hpath.fadeIn(this.settings.autoHideTime)
        } else {
            this.hpath.fadeOut(this.settings.autoHideTime)
        }
    };
    RollBar.prototype.pathSize = function () {
        var pad = parseInt(this.settings.pathPadding, 10);
        this.vpath.css({top: pad + "px", height: this.container.height() - 2 * pad + "px"});
        this.hpath.css({left: pad + "px", width: this.container.width() - 2 * pad + "px"})
    };
    RollBar.prototype.scroll = function (v, h, e) {
        var hs = 0;
        var vs = 0;
        if (v < 0) {
            v = 0
        }
        if (v > this.vtrack) {
            v = this.vtrack
        }
        this.vslider.css("top", v + "px");
        if (h < 0) {
            h = 0
        }
        if (h > this.htrack) {
            h = this.htrack
        }
        this.hslider.css("left", h + "px");
        if (this.vdiff > 0) {
            vs = v / this.vtrack;
            this.content.css("top", Math.round(-this.vdiff * vs));
            if (e && (v && v != this.vtrack)) {
                e.stopPropagation();
                e.preventDefault()
            }
        }
        if (this.hdiff > 0) {
            hs = h / this.htrack;
            this.content.css("left", Math.round(-this.hdiff * hs));
            if (e && (h && h != this.htrack)) {
                e.stopPropagation();
                e.preventDefault()
            }
        }
        if (this.before.v != vs || this.before.h != hs) {
            if (typeof this.settings.onscroll == "function") {
                this.settings.onscroll.call(this.container.get(0), vs, hs)
            }
            this.before.v = vs;
            this.before.h = hs
        }
    };
    RollBar.prototype.easeScroll = function (v, h) {
        var n = 0;
        var steps = Math.floor(this.settings.scrollTime / this.settings.scrollInterval);
        var vs = this.vslider.position().top;
        var hs = this.hslider.position().left;
        var easing = $.easing[this.settings.scrollEasing] || $.easing.linear;
        this.sliders.stop().fadeTo(this.settings.sliderOpacityTime, 1);
        window.clearInterval(this.timer);
        this.timer = window.setInterval(this.fixFn(function () {
            this.scroll(vs + easing(n / steps, n, 0, 1, steps) * v, hs + easing(n / steps, n, 0, 1, steps) * h);
            if (++n > steps) {
                window.clearInterval(this.timer);
                this.sliders.stop().fadeTo(this.settings.sliderOpacityTime, this.settings.sliderOpacity)
            }
        }), this.settings.scrollInterval)
    };
    RollBar.prototype.fixFn = function (f, s) {
        var scope = this;
        return function () {
            f.apply(s || scope, Array.prototype.slice.call(arguments))
        }
    };
    RollBar.prototype.bindEvent = function (t, e, f, s) {
        return t.bind(e, this.fixFn(f, s))
    };
    RollBar.prototype.init = function () {
        var document = $(window.document);
        this.bindEvent(this.sliders, "mousedown", function (e) {
            this.pressed = (e.target === this.vslider.get(0)) ? 1 : 2;
            var hclick = e.pageX;
            var vclick = e.pageY;
            var vtop = this.vslider.position().top;
            var hleft = this.hslider.position().left;
            this.bindEvent(document, "mousemove", function (e) {
                if (this.pressed == 1) {
                    this.scroll(vtop + (e.pageY - vclick), hleft)
                } else {
                    this.scroll(vtop, hleft + (e.pageX - hclick))
                }
            });
            this.bindEvent(document, "selectstart", function (e) {
                e.preventDefault()
            })
        });
        this.bindEvent(document, "mouseup", function (e) {
            if (this.pressed == 1 && e.target !== this.vslider.get(0)) {
                this.vslider.fadeTo(this.settings.sliderOpacityTime, this.settings.sliderOpacity)
            } else {
                if (this.pressed == 2 && e.target !== this.hslider.get(0)) {
                    this.hslider.fadeTo(this.settings.sliderOpacityTime, this.settings.sliderOpacity)
                }
            }
            this.pressed = 0;
            document.unbind("mousemove");
            document.unbind("selectstart")
        });
        this.bindEvent(this.container, "touchstart", function (e) {
            var event = e.originalEvent;
            var touch = event.changedTouches[0];
            this.touch.sx = touch.pageX;
            this.touch.sy = touch.pageY;
            this.touch.sv = this.vslider.position().top;
            this.touch.sh = this.hslider.position().left;
            this.sliders.stop().fadeTo(this.settings.sliderOpacityTime, 1);
            if (this.settings.blockGlobalScroll && (this.vdiff || this.hdiff)) {
                event.stopPropagation()
            }
        });
        this.bindEvent(this.container, "touchmove", function (e) {
            var event = e.originalEvent;
            var touch = event.targetTouches[0];
            this.scroll(this.touch.sv + (this.touch.sy - touch.pageY) * this.settings.touchSpeed, this.touch.sh + (this.touch.sx - touch.pageX) * this.settings.touchSpeed, e);
            if (this.settings.blockGlobalScroll && (this.vdiff || this.hdiff)) {
                event.preventDefault();
                event.stopPropagation()
            }
        });
        this.bindEvent(this.container, "touchend touchcancel", function (e) {
            var event = e.originalEvent;
            var touch = event.changedTouches[0];
            this.sliders.stop().fadeTo(this.settings.sliderOpacityTime, this.settings.sliderOpacity);
            if (this.settings.blockGlobalScroll && (this.vdiff || this.hdiff)) {
                event.stopPropagation()
            }
        });
        var vtrack = this.vpath.height(), htrack = this.hpath.width();
        this.bindEvent($(window), "resize", function () {
            this.pathSize();
            this.checkScroll();
            if (this.vdiff <= 0) {
                this.content.css("top", 0)
            }
            if (this.hdiff <= 0) {
                this.content.css("left", 0)
            }
            this.scroll(Math.round(parseInt(this.vslider.css("top"), 10) * this.vpath.height() / vtrack), Math.round(parseInt(this.hslider.css("left"), 10) * this.hpath.width() / htrack));
            vtrack = this.vpath.height();
            htrack = this.hpath.width()
        });
        this.bindEvent(this.container, "mousewheel", function (e, delta, deltaX, deltaY) {
            var targetNode = e.target.nodeName;
            if (targetNode == "TEXTAREA" || (targetNode == "SELECT" || targetNode == "OPTION")) {
                e.stopPropagation();
                return
            }
            this.scroll(this.vslider.position().top - this.settings.wheelSpeed * deltaY, this.hslider.position().left + this.settings.wheelSpeed * deltaX, e);
            this.sliders.stop().fadeTo(this.settings.sliderOpacityTime, 1);
            window.clearTimeout(this.timer);
            this.timer = window.setTimeout(this.fixFn(function () {
                this.sliders.stop().fadeTo(this.settings.sliderOpacityTime, this.settings.sliderOpacity)
            }), this.settings.sliderOpacityDelay);
            if (this.settings.blockGlobalScroll && (this.vdiff || this.hdiff)) {
                e.preventDefault();
                e.stopPropagation()
            }
        });
        this.bindEvent(document, "keydown", function (e) {
            var hkey = 0, vkey = 0;
            vkey = (e.keyCode == 38) ? -this.settings.keyScroll : vkey;
            vkey = (e.keyCode == 40) ? this.settings.keyScroll : vkey;
            hkey = (e.keyCode == 37) ? -this.settings.keyScroll : hkey;
            hkey = (e.keyCode == 39) ? this.settings.keyScroll : hkey;
            if (vkey || hkey) {
                this.easeScroll(vkey, hkey)
            }
        });
        this.bindEvent(this.container, "dragstart", function (e) {
            e.preventDefault()
        });
        this.bindEvent(this.container, "rollbar", function (e, v, h, px) {
            e.stopPropagation();
            if (v === "reset") {
                this.container.find(".rollbar-content, .rollbar-handle").css({top: 0, left: 0});
                return
            }
            v = v || 0;
            h = h || 0;
            if (/^[-\d\.]+$/.test(v)) {
                v = parseFloat(v);
                if (Math.abs(v) <= 1 && !px) {
                    v *= this.vtrack
                } else {
                    v = v + v * (this.vtrack / this.vdiff - 1)
                }
            }
            if (/^[-\d\.]+$/.test(h)) {
                h = parseFloat(h);
                if (Math.abs(h) <= 1 && !px) {
                    h *= this.htrack
                } else {
                    h = h + h * (this.htrack / this.hdiff - 1)
                }
            }
            this.easeScroll(v, h)
        })
    };
    $.fn.rollbar = function (s) {
        var settings = {scroll: "both", autoHide: true, autoHideTime: "fast", lazyCheckScroll: 1000, blockGlobalScroll: false, contentFilter: "*", sliderSize: "30%", sliderOpacity: 0.5, sliderOpacityTime: 200, sliderOpacityDelay: 1000, wheelSpeed: 20, touchSpeed: 0.3, pathPadding: "20px", keyScroll: 100, scrollTime: 500, scrollInterval: 15, scrollEasing: "swing", zIndex: 100, onscroll: function () {
        }};
        $.extend(settings, s);
        return this.each(function () {
            new RollBar(this, settings)
        })
    }
})(jQuery);
